@@ -1,21 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* BGP dump to ascii converter
  * Copyright (C) 1999 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -68,7 +53,7 @@ enum MRT_MSG_TYPES {
 	MSG_TABLE_DUMP		  /* routing table dump */
 };
 
-static int attr_parse(struct stream *s, uint16_t len)
+static void attr_parse(struct stream *s, uint16_t len)
 {
 	unsigned int flag;
 	unsigned int type;
@@ -77,7 +62,7 @@ static int attr_parse(struct stream *s, uint16_t len)
 
 	lim = s->getp + len;
 
-	printf("attr_parse s->getp %zd, len %d, lim %d\n", s->getp, len, lim);
+	printf("%s s->getp %zd, len %d, lim %d\n", __func__, s->getp, len, lim);
 
 	while (s->getp < lim) {
 		flag = stream_getc(s);
@@ -101,28 +86,27 @@ static int attr_parse(struct stream *s, uint16_t len)
 		case BGP_ATTR_AS_PATH: {
 			struct aspath *aspath;
 
-			aspath = aspath_parse(s, length, 1);
+			aspath = aspath_parse(s, length, 1,
+					      bgp_get_asnotation(NULL));
 			printf("ASPATH: %s\n", aspath->str);
 			aspath_free(aspath);
 		} break;
 		case BGP_ATTR_NEXT_HOP: {
 			struct in_addr nexthop;
 			nexthop.s_addr = stream_get_ipv4(s);
-			printf("NEXTHOP: %s\n", inet_ntoa(nexthop));
+			printf("NEXTHOP: %pI4\n", &nexthop);
 		} break;
 		default:
 			stream_getw_from(s, length);
 			break;
 		}
 	}
-
-	return 0;
 }
 
 int main(int argc, char **argv)
 {
 	int ret;
-	FILE *fp;
+	int fd;
 	struct stream *s;
 	time_t now;
 	int type;
@@ -136,6 +120,7 @@ int main(int argc, char **argv)
 	struct in_addr dip;
 	uint16_t viewno, seq_num;
 	struct prefix_ipv4 p;
+	char tbuf[32];
 
 	s = stream_new(10000);
 
@@ -143,8 +128,8 @@ int main(int argc, char **argv)
 		fprintf(stderr, "Usage: %s FILENAME\n", argv[0]);
 		exit(1);
 	}
-	fp = fopen(argv[1], "r");
-	if (!fp) {
+	fd = open(argv[1], O_RDONLY);
+	if (fd < 0) {
 		fprintf(stdout,
 			"%% Can't open configuration file %s due to '%s'.\n",
 			argv[1], safe_strerror(errno));
@@ -154,13 +139,14 @@ int main(int argc, char **argv)
 	while (1) {
 		stream_reset(s);
 
-		ret = fread(s->data, 12, 1, fp);
-		if (!ret || feof(fp)) {
-			printf("END OF FILE\n");
-			break;
-		}
-		if (ferror(fp)) {
-			printf("ERROR OF FREAD\n");
+		ret = stream_read(s, fd, 12);
+		if (ret != 12) {
+			if (!ret)
+				printf("END OF FILE\n");
+			else if (ret < 0)
+				printf("ERROR OF READ\n");
+			else
+				printf("UNDERFLOW\n");
 			break;
 		}
 
@@ -170,7 +156,7 @@ int main(int argc, char **argv)
 		subtype = stream_getw(s);
 		len = stream_getl(s);
 
-		printf("TIME: %s", ctime(&now));
+		printf("TIME: %s", ctime_r(&now, tbuf));
 
 		/* printf ("TYPE: %d/%d\n", type, subtype); */
 
@@ -217,13 +203,14 @@ int main(int argc, char **argv)
 
 		printf("len: %zd\n", len);
 
-		fread(s->data + 12, len, 1, fp);
-		if (feof(fp)) {
-			printf("ENDOF FILE 2\n");
-			break;
-		}
-		if (ferror(fp)) {
-			printf("ERROR OF FREAD 2\n");
+		ret = stream_read(s, fd, len);
+		if (ret != (int)len) {
+			if (!ret)
+				printf("END OF FILE 2\n");
+			else if (ret < 0)
+				printf("ERROR OF READ 2\n");
+			else
+				printf("UNDERFLOW 2\n");
 			break;
 		}
 
@@ -244,7 +231,7 @@ int main(int argc, char **argv)
 			while (s->getp < len - 16) {
 				p.prefix.s_addr = stream_get_ipv4(s);
 				p.prefixlen = stream_getc(s);
-				printf("PREFIX: %s/%d\n", inet_ntoa(p.prefix),
+				printf("PREFIX: %pI4/%d\n", &p.prefix,
 				       p.prefixlen);
 
 				status = stream_getc(s);
@@ -252,9 +239,9 @@ int main(int argc, char **argv)
 				peer.s_addr = stream_get_ipv4(s);
 				source_as = stream_getw(s);
 
-				printf("FROM: %s AS%d\n", inet_ntoa(peer),
-				       source_as);
-				printf("ORIGINATED: %s", ctime(&originated));
+				printf("FROM: %pI4 AS%d\n", &peer, source_as);
+				printf("ORIGINATED: %s", ctime_r(&originated,
+								 tbuf));
 
 				attrlen = stream_getw(s);
 				printf("ATTRLEN: %d\n", attrlen);
@@ -278,12 +265,12 @@ int main(int argc, char **argv)
 			sip.s_addr = stream_get_ipv4(s);
 			dip.s_addr = stream_get_ipv4(s);
 
-			printf("saddr: %s\n", inet_ntoa(sip));
-			printf("daddr: %s\n", inet_ntoa(dip));
+			printf("saddr: %pI4\n", &sip);
+			printf("daddr: %pI4\n", &dip);
 
 			printf("\n");
 		}
 	}
-	fclose(fp);
+	close(fd);
 	return 0;
 }
