@@ -1,21 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* BGP community-list and extcommunity-list.
  * Copyright (C) 1999 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -33,6 +18,7 @@
 #include "bgpd/bgp_community.h"
 #include "bgpd/bgp_ecommunity.h"
 #include "bgpd/bgp_lcommunity.h"
+#include "bgpd/bgp_community_alias.h"
 #include "bgpd/bgp_aspath.h"
 #include "bgpd/bgp_regex.h"
 #include "bgpd/bgp_clist.h"
@@ -183,7 +169,7 @@ community_list_insert(struct community_list_handler *ch, const char *name,
 	new->name_hash = bgp_clist_hash_key_community_list(new);
 
 	/* Save for later */
-	hash_get(cm->hash, new, hash_alloc_intern);
+	(void)hash_get(cm->hash, new, hash_alloc_intern);
 
 	/* If name is made by all digit character.  We treat it as
 	   number. */
@@ -337,18 +323,42 @@ static void community_list_entry_delete(struct community_list_master *cm,
 		community_list_delete(cm, list);
 }
 
+/*
+ * Replace community-list entry in the list. Note that entry is the new one
+ * and replace is one one being replaced.
+ */
+static void community_list_entry_replace(struct community_list *list,
+					 struct community_entry *replace,
+					 struct community_entry *entry)
+{
+	if (replace->next) {
+		entry->next = replace->next;
+		replace->next->prev = entry;
+	} else {
+		entry->next = NULL;
+		list->tail = entry;
+	}
+
+	if (replace->prev) {
+		entry->prev = replace->prev;
+		replace->prev->next = entry;
+	} else {
+		entry->prev = NULL;
+		list->head = entry;
+	}
+
+	community_entry_free(replace);
+}
+
 /* Add community-list entry to the list.  */
 static void community_list_entry_add(struct community_list *list,
 				     struct community_entry *entry,
 				     struct community_list_handler *ch,
 				     int master)
 {
-	struct community_list_master *cm = NULL;
 	struct community_entry *replace;
 	struct community_entry *point;
 
-	cm = community_list_master_lookup(ch, master);
-
 	/* Automatic assignment of seq no. */
 	if (entry->seq == COMMUNITY_SEQ_NUMBER_AUTO)
 		entry->seq = bgp_clist_new_seq_get(list);
@@ -357,8 +367,10 @@ static void community_list_entry_add(struct community_list *list,
 		point = NULL;
 	else {
 		replace = bgp_clist_seq_check(list, entry->seq);
-		if (replace)
-			community_list_entry_delete(cm, list, entry);
+		if (replace) {
+			community_list_entry_replace(list, replace, entry);
+			return;
+		}
 
 		/* Check insert point. */
 		for (point = list->head; point; point = point->next)
@@ -437,9 +449,6 @@ static char *community_str_get(struct community *com, int i)
 	comval = ntohl(comval);
 
 	switch (comval) {
-	case COMMUNITY_INTERNET:
-		str = XSTRDUP(MTYPE_COMMUNITY_STR, "internet");
-		break;
 	case COMMUNITY_GSHUT:
 		str = XSTRDUP(MTYPE_COMMUNITY_STR, "graceful-shutdown");
 		break;
@@ -522,20 +531,24 @@ static bool community_regexp_include(regex_t *reg, struct community *com, int i)
 static bool community_regexp_match(struct community *com, regex_t *reg)
 {
 	const char *str;
+	char *regstr;
+	int rv;
 
 	/* When there is no communities attribute it is treated as empty
 	   string.  */
 	if (com == NULL || com->size == 0)
 		str = "";
 	else
-		str = community_str(com, false);
+		str = community_str(com, false, true);
+
+	regstr = bgp_alias2community_str(str);
 
 	/* Regular expression match.  */
-	if (regexec(reg, str, 0, NULL, 0) == 0)
-		return true;
+	rv = regexec(reg, regstr, 0, NULL, 0);
 
-	/* No match.  */
-	return false;
+	XFREE(MTYPE_TMP, regstr);
+
+	return rv == 0;
 }
 
 static char *lcommunity_str_get(struct lcommunity *lcom, int i)
@@ -592,20 +605,24 @@ static bool lcommunity_regexp_include(regex_t *reg, struct lcommunity *lcom,
 static bool lcommunity_regexp_match(struct lcommunity *com, regex_t *reg)
 {
 	const char *str;
+	char *regstr;
+	int rv;
 
 	/* When there is no communities attribute it is treated as empty
 	   string.  */
 	if (com == NULL || com->size == 0)
 		str = "";
 	else
-		str = lcommunity_str(com, false);
+		str = lcommunity_str(com, false, true);
+
+	regstr = bgp_alias2community_str(str);
 
 	/* Regular expression match.  */
-	if (regexec(reg, str, 0, NULL, 0) == 0)
-		return true;
+	rv = regexec(reg, regstr, 0, NULL, 0);
 
-	/* No match.  */
-	return false;
+	XFREE(MTYPE_TMP, regstr);
+
+	return rv == 0;
 }
 
 
@@ -628,86 +645,6 @@ static bool ecommunity_regexp_match(struct ecommunity *ecom, regex_t *reg)
 	return false;
 }
 
-#if 0
-/* Delete community attribute using regular expression match.  Return
-   modified communites attribute.  */
-static struct community *
-community_regexp_delete (struct community *com, regex_t * reg)
-{
-	int i;
-	uint32_t comval;
-	/* Maximum is "65535:65535" + '\0'. */
-	char c[12];
-	const char *str;
-
-	if (!com)
-		return NULL;
-
-	i = 0;
-	while (i < com->size)
-	{
-		memcpy (&comval, com_nthval (com, i), sizeof(uint32_t));
-		comval = ntohl (comval);
-
-		switch (comval) {
-		case COMMUNITY_INTERNET:
-			str = "internet";
-			break;
-		case COMMUNITY_ACCEPT_OWN:
-			str = "accept-own";
-			break;
-		case COMMUNITY_ROUTE_FILTER_TRANSLATED_v4:
-			str = "route-filter-translated-v4";
-			break;
-		case COMMUNITY_ROUTE_FILTER_v4:
-			str = "route-filter-v4";
-			break;
-		case COMMUNITY_ROUTE_FILTER_TRANSLATED_v6:
-			str = "route-filter-translated-v6";
-			break;
-		case COMMUNITY_ROUTE_FILTER_v6:
-			str = "route-filter-v6";
-			break;
-		case COMMUNITY_LLGR_STALE:
-			str = "llgr-stale";
-			break;
-		case COMMUNITY_NO_LLGR:
-			str = "no-llgr";
-			break;
-		case COMMUNITY_ACCEPT_OWN_NEXTHOP:
-			str = "accept-own-nexthop";
-			break;
-		case COMMUNITY_BLACKHOLE:
-			str = "blackhole";
-			break;
-		case COMMUNITY_NO_EXPORT:
-			str = "no-export";
-			break;
-		case COMMUNITY_NO_ADVERTISE:
-			str = "no-advertise";
-			break;
-		case COMMUNITY_LOCAL_AS:
-			str = "local-AS";
-			break;
-		case COMMUNITY_NO_PEER:
-			str = "no-peer";
-			break;
-		default:
-			sprintf (c, "%d:%d", (comval >> 16) & 0xFFFF,
-			 comval & 0xFFFF);
-			str = c;
-			break;
-		}
-
-		if (regexec (reg, str, 0, NULL, 0) == 0)
-			community_del_val (com, com_nthval (com, i));
-		else
-			i++;
-	}
-	return com;
-}
-#endif
-
 /* When given community attribute matches to the community-list return
    1 else return 0.  */
 bool community_list_match(struct community *com, struct community_list *list)
@@ -715,13 +652,7 @@ bool community_list_match(struct community *com, struct community_list *list)
 	struct community_entry *entry;
 
 	for (entry = list->head; entry; entry = entry->next) {
-		if (entry->any)
-			return entry->direct == COMMUNITY_PERMIT;
-
 		if (entry->style == COMMUNITY_LIST_STANDARD) {
-			if (community_include(entry->u.com, COMMUNITY_INTERNET))
-				return entry->direct == COMMUNITY_PERMIT;
-
 			if (community_match(com, entry->u.com))
 				return entry->direct == COMMUNITY_PERMIT;
 		} else if (entry->style == COMMUNITY_LIST_EXPANDED) {
@@ -737,9 +668,6 @@ bool lcommunity_list_match(struct lcommunity *lcom, struct community_list *list)
 	struct community_entry *entry;
 
 	for (entry = list->head; entry; entry = entry->next) {
-		if (entry->any)
-			return entry->direct == COMMUNITY_PERMIT;
-
 		if (entry->style == LARGE_COMMUNITY_LIST_STANDARD) {
 			if (lcommunity_match(lcom, entry->u.lcom))
 				return entry->direct == COMMUNITY_PERMIT;
@@ -761,11 +689,8 @@ bool lcommunity_list_exact_match(struct lcommunity *lcom,
 	struct community_entry *entry;
 
 	for (entry = list->head; entry; entry = entry->next) {
-		if (entry->any)
-			return entry->direct == COMMUNITY_PERMIT;
-
 		if (entry->style == LARGE_COMMUNITY_LIST_STANDARD) {
-			if (lcommunity_cmp(lcom, entry->u.com))
+			if (lcommunity_cmp(lcom, entry->u.lcom))
 				return entry->direct == COMMUNITY_PERMIT;
 		} else if (entry->style == LARGE_COMMUNITY_LIST_EXPANDED) {
 			if (lcommunity_regexp_match(lcom, entry->reg))
@@ -780,9 +705,6 @@ bool ecommunity_list_match(struct ecommunity *ecom, struct community_list *list)
 	struct community_entry *entry;
 
 	for (entry = list->head; entry; entry = entry->next) {
-		if (entry->any)
-			return entry->direct == COMMUNITY_PERMIT;
-
 		if (entry->style == EXTCOMMUNITY_LIST_STANDARD) {
 			if (ecommunity_match(ecom, entry->u.ecom))
 				return entry->direct == COMMUNITY_PERMIT;
@@ -802,13 +724,7 @@ bool community_list_exact_match(struct community *com,
 	struct community_entry *entry;
 
 	for (entry = list->head; entry; entry = entry->next) {
-		if (entry->any)
-			return entry->direct == COMMUNITY_PERMIT;
-
 		if (entry->style == COMMUNITY_LIST_STANDARD) {
-			if (community_include(entry->u.com, COMMUNITY_INTERNET))
-				return entry->direct == COMMUNITY_PERMIT;
-
 			if (community_cmp(com, entry->u.com))
 				return entry->direct == COMMUNITY_PERMIT;
 		} else if (entry->style == COMMUNITY_LIST_EXPANDED) {
@@ -819,6 +735,27 @@ bool community_list_exact_match(struct community *com,
 	return false;
 }
 
+bool community_list_any_match(struct community *com, struct community_list *list)
+{
+	struct community_entry *entry;
+	uint32_t val;
+	int i;
+
+	for (i = 0; i < com->size; i++) {
+		val = community_val_get(com, i);
+
+		for (entry = list->head; entry; entry = entry->next) {
+			if (entry->style == COMMUNITY_LIST_STANDARD &&
+			    community_include(entry->u.com, val))
+				return entry->direct == COMMUNITY_PERMIT;
+			if ((entry->style == COMMUNITY_LIST_EXPANDED) &&
+			    community_regexp_include(entry->reg, com, i))
+				return entry->direct == COMMUNITY_PERMIT;
+		}
+	}
+	return false;
+}
+
 /* Delete all permitted communities in the list from com.  */
 struct community *community_list_match_delete(struct community *com,
 					      struct community_list *list)
@@ -837,28 +774,15 @@ struct community *community_list_match_delete(struct community *com,
 		val = community_val_get(com, i);
 
 		for (entry = list->head; entry; entry = entry->next) {
-			if (entry->any) {
+			if ((entry->style == COMMUNITY_LIST_STANDARD) &&
+			    community_include(entry->u.com, val)) {
 				if (entry->direct == COMMUNITY_PERMIT) {
 					com_index_to_delete[delete_index] = i;
 					delete_index++;
 				}
 				break;
-			}
-
-			else if ((entry->style == COMMUNITY_LIST_STANDARD)
-				 && (community_include(entry->u.com,
-						       COMMUNITY_INTERNET)
-				     || community_include(entry->u.com, val))) {
-				if (entry->direct == COMMUNITY_PERMIT) {
-					com_index_to_delete[delete_index] = i;
-					delete_index++;
-				}
-				break;
-			}
-
-			else if ((entry->style == COMMUNITY_LIST_EXPANDED)
-				 && community_regexp_include(entry->reg, com,
-							     i)) {
+			} else if ((entry->style == COMMUNITY_LIST_EXPANDED) &&
+				   community_regexp_include(entry->reg, com, i)) {
 				if (entry->direct == COMMUNITY_PERMIT) {
 					com_index_to_delete[delete_index] = i;
 					delete_index++;
@@ -892,12 +816,6 @@ static bool community_list_dup_check(struct community_list *list,
 		if (entry->direct != new->direct)
 			continue;
 
-		if (entry->any != new->any)
-			continue;
-
-		if (entry->any)
-			return true;
-
 		switch (entry->style) {
 		case COMMUNITY_LIST_STANDARD:
 			if (community_cmp(entry->u.com, new->u.com))
@@ -955,20 +873,17 @@ int community_list_set(struct community_list_handler *ch, const char *name,
 		}
 	}
 
-	if (str) {
-		if (style == COMMUNITY_LIST_STANDARD)
-			com = community_str2com(str);
-		else
-			regex = bgp_regcomp(str);
+	if (style == COMMUNITY_LIST_STANDARD)
+		com = community_str2com(str);
+	else
+		regex = bgp_regcomp(str);
 
-		if (!com && !regex)
-			return COMMUNITY_LIST_ERR_MALFORMED_VAL;
-	}
+	if (!com && !regex)
+		return COMMUNITY_LIST_ERR_MALFORMED_VAL;
 
 	entry = community_entry_new();
 	entry->direct = direct;
 	entry->style = style;
-	entry->any = (str ? false : true);
 	entry->u.com = com;
 	entry->reg = regex;
 	entry->seq = seqnum;
@@ -1028,6 +943,28 @@ int community_list_unset(struct community_list_handler *ch, const char *name,
 	return 0;
 }
 
+bool lcommunity_list_any_match(struct lcommunity *lcom,
+			       struct community_list *list)
+{
+	struct community_entry *entry;
+	uint8_t *ptr;
+	int i;
+
+	for (i = 0; i < lcom->size; i++) {
+		ptr = lcom->val + (i * LCOMMUNITY_SIZE);
+
+		for (entry = list->head; entry; entry = entry->next) {
+			if ((entry->style == LARGE_COMMUNITY_LIST_STANDARD) &&
+			    lcommunity_include(entry->u.lcom, ptr))
+				return entry->direct == COMMUNITY_PERMIT;
+			if ((entry->style == LARGE_COMMUNITY_LIST_EXPANDED) &&
+			    lcommunity_regexp_include(entry->reg, lcom, i))
+				return entry->direct == COMMUNITY_PERMIT;
+		}
+	}
+	return false;
+}
+
 /* Delete all permitted large communities in the list from com.  */
 struct lcommunity *lcommunity_list_match_delete(struct lcommunity *lcom,
 						struct community_list *list)
@@ -1045,7 +982,8 @@ struct lcommunity *lcommunity_list_match_delete(struct lcommunity *lcom,
 	for (i = 0; i < lcom->size; i++) {
 		ptr = lcom->val + (i * LCOMMUNITY_SIZE);
 		for (entry = list->head; entry; entry = entry->next) {
-			if (entry->any) {
+			if ((entry->style == LARGE_COMMUNITY_LIST_STANDARD) &&
+			    lcommunity_include(entry->u.lcom, ptr)) {
 				if (entry->direct == COMMUNITY_PERMIT) {
 					com_index_to_delete[delete_index] = i;
 					delete_index++;
@@ -1053,18 +991,10 @@ struct lcommunity *lcommunity_list_match_delete(struct lcommunity *lcom,
 				break;
 			}
 
-			else if ((entry->style == LARGE_COMMUNITY_LIST_STANDARD)
-				 && lcommunity_include(entry->u.lcom, ptr)) {
-				if (entry->direct == COMMUNITY_PERMIT) {
-					com_index_to_delete[delete_index] = i;
-					delete_index++;
-				}
-				break;
-			}
-
-			else if ((entry->style == LARGE_COMMUNITY_LIST_EXPANDED)
-				 && lcommunity_regexp_include(entry->reg, lcom,
-							      i)) {
+			else if ((entry->style ==
+				  LARGE_COMMUNITY_LIST_EXPANDED) &&
+				 lcommunity_regexp_include(entry->reg, lcom,
+							   i)) {
 				if (entry->direct == COMMUNITY_PERMIT) {
 					com_index_to_delete[delete_index] = i;
 					delete_index++;
@@ -1083,12 +1013,53 @@ struct lcommunity *lcommunity_list_match_delete(struct lcommunity *lcom,
 	return lcom;
 }
 
+/* Delete all permitted extended communities in the list from ecom.*/
+struct ecommunity *ecommunity_list_match_delete(struct ecommunity *ecom,
+						struct community_list *list)
+{
+	struct community_entry *entry;
+	uint32_t com_index_to_delete[ecom->size];
+	uint8_t *ptr;
+	uint32_t delete_index = 0;
+	uint32_t i;
+	struct ecommunity local_ecom = {.size = 1};
+	struct ecommunity_val local_eval = {0};
+
+	for (i = 0; i < ecom->size; i++) {
+		local_ecom.val = ecom->val + (i * ECOMMUNITY_SIZE);
+		for (entry = list->head; entry; entry = entry->next) {
+			if (((entry->style == EXTCOMMUNITY_LIST_STANDARD) &&
+			     ecommunity_include(entry->u.ecom, &local_ecom)) ||
+			   ((entry->style == EXTCOMMUNITY_LIST_EXPANDED) &&
+			    ecommunity_regexp_match(ecom, entry->reg))) {
+				if (entry->direct == COMMUNITY_PERMIT) {
+					com_index_to_delete[delete_index] = i;
+					delete_index++;
+				}
+				break;
+			}
+		}
+	}
+
+	/* Delete all of the extended communities we flagged for deletion */
+	for (i = delete_index; i > 0; i--) {
+		ptr = ecom->val + (com_index_to_delete[i-1] * ECOMMUNITY_SIZE);
+		memcpy(&local_eval.val, ptr, sizeof(local_eval.val));
+		ecommunity_del_val(ecom, &local_eval);
+	}
+
+	return ecom;
+}
+
 /* Helper to check if every octet do not exceed UINT_MAX */
-static bool lcommunity_list_valid(const char *community)
+bool lcommunity_list_valid(const char *community, int style)
 {
 	int octets;
 	char **splits, **communities;
+	char *endptr;
 	int num, num_communities;
+	regex_t *regres;
+	int invalid = 0;
 
 	frrstr_split(community, " ", &communities, &num_communities);
 
@@ -1097,25 +1068,43 @@ static bool lcommunity_list_valid(const char *community)
 		frrstr_split(communities[j], ":", &splits, &num);
 
 		for (int i = 0; i < num; i++) {
-			if (strtoul(splits[i], NULL, 10) > UINT_MAX)
-				return false;
-
 			if (strlen(splits[i]) == 0)
-				return false;
+				/* There is no digit to check */
+				invalid++;
+
+			if (style == LARGE_COMMUNITY_LIST_STANDARD) {
+				if (*splits[i] == '-')
+					/* Must not be negative */
+					invalid++;
+				else if (strtoul(splits[i], &endptr, 10)
+					 > UINT_MAX)
+					/* Larger than 4 octets */
+					invalid++;
+				else if (*endptr)
+					/* Not all characters were digits */
+					invalid++;
+			} else {
+				regres = bgp_regcomp(communities[j]);
+				if (!regres)
+					/* malformed regex */
+					invalid++;
+				else
+					bgp_regex_free(regres);
+			}
 
 			octets++;
 			XFREE(MTYPE_TMP, splits[i]);
 		}
 		XFREE(MTYPE_TMP, splits);
 
-		if (octets < 3)
-			return false;
+		if (octets != 3)
+			invalid++;
 
 		XFREE(MTYPE_TMP, communities[j]);
 	}
 	XFREE(MTYPE_TMP, communities);
 
-	return true;
+	return (invalid > 0) ? false : true;
 }
 
 /* Set lcommunity-list.  */
@@ -1150,9 +1139,6 @@ int lcommunity_list_set(struct community_list_handler *ch, const char *name,
 	}
 
 	if (str) {
-		if (!lcommunity_list_valid(str))
-			return COMMUNITY_LIST_ERR_MALFORMED_VAL;
-
 		if (style == LARGE_COMMUNITY_LIST_STANDARD)
 			lcom = lcommunity_str2com(str);
 		else
@@ -1165,7 +1151,6 @@ int lcommunity_list_set(struct community_list_handler *ch, const char *name,
 	entry = community_entry_new();
 	entry->direct = direct;
 	entry->style = style;
-	entry->any = (str ? false : true);
 	entry->u.lcom = lcom;
 	entry->reg = regex;
 	entry->seq = seqnum;
@@ -1286,7 +1271,6 @@ int extcommunity_list_set(struct community_list_handler *ch, const char *name,
 	entry = community_entry_new();
 	entry->direct = direct;
 	entry->style = style;
-	entry->any = false;
 	if (ecom)
 		entry->config = ecommunity_ecom2str(
 			ecom, ECOMMUNITY_FORMAT_COMMUNITY_LIST, 0);
@@ -1408,3 +1392,67 @@ void community_list_terminate(struct community_list_handler *ch)
 
 	XFREE(MTYPE_COMMUNITY_LIST_HANDLER, ch);
 }
+
+static int bgp_community_list_vector_walker(struct hash_bucket *bucket,
+					    void *data)
+{
+	vector *comps = data;
+	struct community_list *list = bucket->data;
+
+	vector_set(*comps, XSTRDUP(MTYPE_COMPLETION, list->name));
+
+	return 1;
+}
+
+static void bgp_community_list_cmd_completion(vector comps,
+					      struct cmd_token *token)
+{
+	struct community_list_master *cm;
+
+	cm = community_list_master_lookup(bgp_clist, COMMUNITY_LIST_MASTER);
+
+	hash_walk(cm->hash, bgp_community_list_vector_walker, &comps);
+}
+
+static void bgp_lcommunity_list_cmd_completion(vector comps,
+					       struct cmd_token *token)
+{
+	struct community_list_master *cm;
+
+	cm = community_list_master_lookup(bgp_clist,
+					  LARGE_COMMUNITY_LIST_MASTER);
+
+	hash_walk(cm->hash, bgp_community_list_vector_walker, &comps);
+}
+
+static void bgp_extcommunity_list_cmd_completion(vector comps,
+						 struct cmd_token *token)
+{
+	struct community_list_master *cm;
+
+	cm = community_list_master_lookup(bgp_clist, EXTCOMMUNITY_LIST_MASTER);
+
+	hash_walk(cm->hash, bgp_community_list_vector_walker, &comps);
+}
+
+static const struct cmd_variable_handler community_list_handlers[] = {
+	{.tokenname = "COMMUNITY_LIST_NAME",
+	 .completions = bgp_community_list_cmd_completion},
+	{.completions = NULL}};
+
+static const struct cmd_variable_handler lcommunity_list_handlers[] = {
+	{.tokenname = "LCOMMUNITY_LIST_NAME",
+	 .completions = bgp_lcommunity_list_cmd_completion},
+	{.completions = NULL}};
+
+static const struct cmd_variable_handler extcommunity_list_handlers[] = {
+	{.tokenname = "EXTCOMMUNITY_LIST_NAME",
+	 .completions = bgp_extcommunity_list_cmd_completion},
+	{.completions = NULL}};
+
+void bgp_community_list_command_completion_setup(void)
+{
+	cmd_variable_handler_register(community_list_handlers);
+	cmd_variable_handler_register(lcommunity_list_handlers);
+	cmd_variable_handler_register(extcommunity_list_handlers);
+}
