@@ -1,22 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* BGP-4 Finite State Machine
  * From RFC1771 [A Border Gateway Protocol 4 (BGP-4)]
  * Copyright (C) 1998 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef _QUAGGA_BGP_FSM_H
@@ -32,35 +17,30 @@ enum bgp_fsm_state_progress {
 /* Macro for BGP read, write and timer thread.  */
 #define BGP_TIMER_ON(T, F, V)                                                  \
 	do {                                                                   \
-		if ((peer->status != Deleted))                                 \
-			thread_add_timer(bm->master, (F), peer, (V), &(T));    \
+		if ((connection->status != Deleted))                           \
+			event_add_timer(bm->master, (F), connection, (V),      \
+					&(T));                                 \
 	} while (0)
 
-#define BGP_EVENT_ADD(P, E)                                                    \
-	do {                                                                   \
-		if ((P)->status != Deleted)                                    \
-			thread_add_event(bm->master, bgp_event, (P), (E),      \
-					 NULL);                                \
+#define BGP_EVENT_ADD(C, E)                                                     \
+	do {                                                                    \
+		if ((C)->status != Deleted)                                     \
+			event_add_event(bm->master, bgp_event, (C), (E), NULL); \
 	} while (0)
 
-#define BGP_EVENT_FLUSH(P)                                                     \
-	do {                                                                   \
-		assert(peer);                                                  \
-		thread_cancel_event_ready(bm->master, (P));                    \
+#define BGP_UPDATE_GROUP_TIMER_ON(T, F)                                               \
+	do {                                                                          \
+		if (BGP_SUPPRESS_FIB_ENABLED(peer->bgp) &&                            \
+		    PEER_ROUTE_ADV_DELAY(peer))                                       \
+			event_add_timer_msec(bm->master, (F), connection,             \
+					     (BGP_DEFAULT_UPDATE_ADVERTISEMENT_TIME * \
+					      1000),                                  \
+					     (T));                                    \
+		else                                                                  \
+			event_add_timer_msec(bm->master, (F), connection, 0,          \
+					     (T));                                    \
 	} while (0)
 
-#define BGP_UPDATE_GROUP_TIMER_ON(T, F)					       \
-	do {								       \
-		if (BGP_SUPPRESS_FIB_ENABLED(peer->bgp) &&		       \
-		    PEER_ROUTE_ADV_DELAY(peer))				       \
-			thread_add_timer_msec(bm->master, (F), peer,	       \
-				(BGP_DEFAULT_UPDATE_ADVERTISEMENT_TIME * 1000),\
-				(T));					       \
-		else							       \
-			thread_add_timer_msec(bm->master, (F), peer,	       \
-					      0, (T));			       \
-	} while (0)							       \
-
 #define BGP_MSEC_JITTER 10
 
 /* Status codes for bgp_event_update() */
@@ -126,13 +106,15 @@ enum bgp_fsm_state_progress {
 /*
  * Update FSM for peer based on whether we have valid nexthops or not.
  */
-extern void bgp_fsm_nht_update(struct peer *peer, bool has_valid_nexthops);
-extern void bgp_event(struct thread *);
-extern int bgp_event_update(struct peer *, enum bgp_fsm_events event);
-extern enum bgp_fsm_state_progress bgp_stop(struct peer *peer);
-extern void bgp_timer_set(struct peer *);
-extern void bgp_routeadv_timer(struct thread *);
-extern void bgp_fsm_change_status(struct peer *peer,
+extern void bgp_fsm_nht_update(struct peer_connection *connection,
+			       struct peer *peer, bool has_valid_nexthops);
+extern void bgp_event(struct event *event);
+extern int bgp_event_update(struct peer_connection *connection,
+			    enum bgp_fsm_events event);
+extern enum bgp_fsm_state_progress bgp_stop(struct peer_connection *connection);
+extern void bgp_timer_set(struct peer_connection *connection);
+extern void bgp_routeadv_timer(struct event *event);
+extern void bgp_fsm_change_status(struct peer_connection *connection,
 				  enum bgp_fsm_status status);
 extern const char *const peer_down_str[];
 extern void bgp_update_delay_end(struct bgp *);
@@ -161,11 +143,10 @@ extern void bgp_adjust_routeadv(struct peer *);
 DECLARE_HOOK(peer_backward_transition, (struct peer *peer), (peer));
 DECLARE_HOOK(peer_established, (struct peer *peer), (peer));
 
-int bgp_gr_update_all(struct bgp *bgp, enum global_gr_command global_gr_cmd);
-int bgp_neighbor_graceful_restart(struct peer *peer,
-				  enum peer_gr_command peer_gr_cmd);
-unsigned int bgp_peer_gr_action(struct peer *peer, enum peer_mode old_peer_state,
-				enum peer_mode new_peer_state);
+int bgp_gr_update_all(struct bgp *bgp, int global_gr_cmd);
+int bgp_neighbor_graceful_restart(struct peer *peer, int peer_gr_cmd);
+unsigned int bgp_peer_gr_action(struct peer *peer,
+		int old_peer_state, int new_peer_state);
 void bgp_peer_move_to_gr_mode(struct peer *peer, int new_state);
 unsigned int bgp_peer_gr_helper_enable(struct peer *peer);
 unsigned int bgp_peer_gr_enable(struct peer *peer);
