@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * BGP pbr
  * Copyright (C) 6WIND
- *
- * FRR is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * FRR is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #ifndef __BGP_PBR_H__
 #define __BGP_PBR_H__
@@ -79,6 +66,7 @@ struct bgp_pbr_entry_action {
 		vrf_id_t redirect_vrf;
 		struct _pbr_redirect_ip {
 			struct in_addr redirect_ip_v4;
+			struct in6_addr redirect_ip_v6;
 			uint8_t duplicate;
 		} zr;
 		uint8_t marking_dscp;
@@ -114,13 +102,17 @@ struct bgp_pbr_entry_main {
 	uint8_t match_dscp_num;
 	uint8_t match_tcpflags_num;
 	uint8_t match_fragment_num;
+	uint8_t match_flowlabel_num;
 
 	struct prefix src_prefix;
 	struct prefix dst_prefix;
+	uint8_t src_prefix_offset;
+	uint8_t dst_prefix_offset;
 
 #define PROTOCOL_UDP 17
 #define PROTOCOL_TCP 6
 #define PROTOCOL_ICMP 1
+#define PROTOCOL_ICMPV6 58
 	struct bgp_pbr_match_val protocol[BGP_PBR_MATCH_VAL_MAX];
 	struct bgp_pbr_match_val src_port[BGP_PBR_MATCH_VAL_MAX];
 	struct bgp_pbr_match_val dst_port[BGP_PBR_MATCH_VAL_MAX];
@@ -129,6 +121,7 @@ struct bgp_pbr_entry_main {
 	struct bgp_pbr_match_val icmp_code[BGP_PBR_MATCH_VAL_MAX];
 	struct bgp_pbr_match_val packet_length[BGP_PBR_MATCH_VAL_MAX];
 	struct bgp_pbr_match_val dscp[BGP_PBR_MATCH_VAL_MAX];
+	struct bgp_pbr_match_val flow_label[BGP_PBR_MATCH_VAL_MAX];
 
 	struct bgp_pbr_match_val tcpflags[BGP_PBR_MATCH_VAL_MAX];
 	struct bgp_pbr_match_val fragment[BGP_PBR_MATCH_VAL_MAX];
@@ -154,6 +147,8 @@ extern int bgp_pbr_interface_compare(const struct bgp_pbr_interface *a,
 struct bgp_pbr_config {
 	struct bgp_pbr_interface_head ifaces_by_name_ipv4;
 	bool pbr_interface_any_ipv4;
+	struct bgp_pbr_interface_head ifaces_by_name_ipv6;
+	bool pbr_interface_any_ipv6;
 };
 
 extern struct bgp_pbr_config *bgp_pbr_cfg;
@@ -179,6 +174,7 @@ struct bgp_pbr_match {
 	uint32_t type;
 
 	uint32_t flags;
+	uint8_t family;
 
 	uint16_t pkt_len_min;
 	uint16_t pkt_len_max;
@@ -187,6 +183,7 @@ struct bgp_pbr_match {
 	uint8_t dscp_value;
 	uint8_t fragment;
 	uint8_t protocol;
+	uint16_t flow_label;
 
 	vrf_id_t vrf_id;
 
@@ -254,6 +251,7 @@ struct bgp_pbr_action {
 	bool install_in_progress;
 	uint32_t refcnt;
 	struct bgp *bgp;
+	afi_t afi;
 };
 
 extern struct bgp_pbr_rule *bgp_pbr_rule_lookup(vrf_id_t vrf_id,
@@ -289,7 +287,6 @@ extern bool bgp_pbr_match_hash_equal(const void *arg1,
 
 void bgp_pbr_print_policy_route(struct bgp_pbr_entry_main *api);
 
-struct bgp_node;
 struct bgp_path_info;
 extern void bgp_pbr_update_entry(struct bgp *bgp, const struct prefix *p,
 				 struct bgp_path_info *new_select, afi_t afi,
