@@ -1,22 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* BGP4V2-MIB SNMP support
  *
  * Copyright (C) 2022 Donatas Abraitis <donatas@opensourcerouting.org>
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -28,12 +13,11 @@
 #include "log.h"
 #include "prefix.h"
 #include "command.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "smux.h"
 #include "filter.h"
 #include "hook.h"
 #include "libfrr.h"
-#include "lib/version.h"
 
 #include "bgpd/bgpd.h"
 #include "bgpd/bgp_table.h"
@@ -59,14 +43,16 @@ static struct peer *peer_lookup_all_vrf(struct ipaddr *addr)
 
 	for (ALL_LIST_ELEMENTS_RO(bm->bgp, bgpnode, bgp)) {
 		for (ALL_LIST_ELEMENTS_RO(bgp->peer, node, peer)) {
-			switch (sockunion_family(&peer->su)) {
+			switch (sockunion_family(&peer->connection->su)) {
 			case AF_INET:
-				if (IPV4_ADDR_SAME(&peer->su.sin.sin_addr,
+				if (IPV4_ADDR_SAME(&peer->connection->su.sin
+							    .sin_addr,
 						   &addr->ip._v4_addr))
 					return peer;
 				break;
 			case AF_INET6:
-				if (IPV6_ADDR_SAME(&peer->su.sin6.sin6_addr,
+				if (IPV6_ADDR_SAME(&peer->connection->su.sin6
+							    .sin6_addr,
 						   &addr->ip._v6_addr))
 					return peer;
 				break;
@@ -90,38 +76,47 @@ static struct peer *peer_lookup_all_vrf_next(struct ipaddr *addr, oid *offset,
 
 	for (ALL_LIST_ELEMENTS_RO(bm->bgp, bgpnode, bgp)) {
 		for (ALL_LIST_ELEMENTS_RO(bgp->peer, node, peer)) {
-			sa_family_t peer_family = sockunion_family(&peer->su);
+			sa_family_t peer_family =
+				sockunion_family(&peer->connection->su);
 
 			if (peer_family != family)
 				continue;
 
-			switch (sockunion_family(&peer->su)) {
+			switch (peer_family) {
 			case AF_INET:
 				oid2in_addr(offset, IN_ADDR_SIZE,
 					    &addr->ip._v4_addr);
-				if (IPV4_ADDR_CMP(&peer->su.sin.sin_addr,
+				if (IPV4_ADDR_CMP(&peer->connection->su.sin
+							   .sin_addr,
 						  &addr->ip._v4_addr) < 0 ||
-				    IPV4_ADDR_SAME(&peer->su.sin.sin_addr,
+				    IPV4_ADDR_SAME(&peer->connection->su.sin
+							    .sin_addr,
 						   &addr->ip._v4_addr))
 					continue;
 
 				if (!next_peer ||
-				    IPV4_ADDR_CMP(&next_peer->su.sin.sin_addr,
-						  &peer->su.sin.sin_addr) > 0)
+				    IPV4_ADDR_CMP(&next_peer->connection->su.sin
+							   .sin_addr,
+						  &peer->connection->su.sin
+							   .sin_addr) > 0)
 					next_peer = peer;
 
 				break;
 			case AF_INET6:
 				oid2in6_addr(offset, &addr->ip._v6_addr);
-				if (IPV6_ADDR_CMP(&peer->su.sin6.sin6_addr,
+				if (IPV6_ADDR_CMP(&peer->connection->su.sin6
+							   .sin6_addr,
 						  &addr->ip._v6_addr) < 0 ||
-				    IPV6_ADDR_SAME(&peer->su.sin6.sin6_addr,
+				    IPV6_ADDR_SAME(&peer->connection->su.sin6
+							    .sin6_addr,
 						   &addr->ip._v6_addr))
 					continue;
 
 				if (!next_peer ||
-				    IPV6_ADDR_CMP(&next_peer->su.sin6.sin6_addr,
-						  &peer->su.sin6.sin6_addr) > 0)
+				    IPV6_ADDR_CMP(&next_peer->connection->su
+							   .sin6.sin6_addr,
+						  &peer->connection->su.sin6
+							   .sin6_addr) > 0)
 					next_peer = peer;
 
 				break;
@@ -144,7 +139,7 @@ static struct peer *bgpv2PeerTable_lookup(struct variable *v, oid name[],
 	struct peer *peer = NULL;
 	size_t namelen = v ? v->namelen : BGP4V2_PEER_ENTRY_OFFSET;
 	oid *offset = name + namelen;
-	sa_family_t family = name[namelen - 1] == 4 ? AF_INET : AF_INET6;
+	sa_family_t family = name[namelen - 1] == 1 ? AF_INET : AF_INET6;
 	int afi_len = IN_ADDR_SIZE;
 	size_t offsetlen = *length - namelen;
 
@@ -174,13 +169,15 @@ static struct peer *bgpv2PeerTable_lookup(struct variable *v, oid name[],
 		if (peer == NULL)
 			return NULL;
 
-		switch (sockunion_family(&peer->su)) {
+		switch (sockunion_family(&peer->connection->su)) {
 		case AF_INET:
-			oid_copy_in_addr(offset, &peer->su.sin.sin_addr);
+			oid_copy_in_addr(offset,
+					 &peer->connection->su.sin.sin_addr);
 			*length = afi_len + namelen;
 			return peer;
 		case AF_INET6:
-			oid_copy_in6_addr(offset, &peer->su.sin6.sin6_addr);
+			oid_copy_in6_addr(offset,
+					  &peer->connection->su.sin6.sin6_addr);
 			*length = afi_len + namelen;
 			return peer;
 		default:
@@ -281,7 +278,7 @@ static uint8_t *bgpv2PeerTable(struct variable *v, oid name[], size_t *length,
 		else
 			return SNMP_INTEGER(BGP_PEER_ADMIN_STATUS_RUNNING);
 	case BGP4V2_PEER_STATE:
-		return SNMP_INTEGER(peer->status);
+		return SNMP_INTEGER(peer->connection->status);
 	case BGP4V2_PEER_DESCRIPTION:
 		if (peer->desc)
 			return SNMP_STRING(peer->desc);
@@ -441,16 +438,22 @@ bgp4v2PathAttrLookup(struct variable *v, oid name[], size_t *length,
 	unsigned int len;
 	struct ipaddr paddr = {};
 	size_t namelen = v ? v->namelen : BGP4V2_NLRI_ENTRY_OFFSET;
-	sa_family_t family = name[namelen - 1] == 4 ? AF_INET : AF_INET6;
-	afi_t afi = AFI_IP;
-	size_t afi_len = IN_ADDR_SIZE;
+	sa_family_t family;
+	sa_family_t min_family = 0; /* family of the selected min path */
+	afi_t afi;
+	safi_t safi;
+	size_t afi_len;
+	long prefix_type = 0;
+	long peer_addr_type = 0;
 
-	if (family == AF_INET6) {
-		afi = AFI_IP6;
-		afi_len = IN6_ADDR_SIZE;
-	}
+	/* Bgp4V2AddressFamilyIdentifierTC limited to IPv6 */
+	if (name[namelen - 1] > IANA_AFI_IPV6)
+		return NULL;
+	afi = afi_iana2int(name[namelen - 1]);
+	afi_len = afi == AFI_IP ? IN_ADDR_SIZE : IN6_ADDR_SIZE;
+
+#define BGP_NLRI_ENTRY_OFFSET namelen
 
-#define BGP_NLRI_ENTRY_OFFSET (afi_len + 1 + afi_len)
 
 	sockunion_init(&su);
 
@@ -458,32 +461,61 @@ bgp4v2PathAttrLookup(struct variable *v, oid name[], size_t *length,
 		if (*length - namelen != BGP_NLRI_ENTRY_OFFSET)
 			return NULL;
 
-		/* Set OID offset for prefix */
+		/* Set OID offset for prefix type */
 		offset = name + namelen;
-		if (family == AF_INET)
-			oid2in_addr(offset, afi_len, &addr->u.prefix4);
-		else
-			oid2in6_addr(offset, &addr->u.prefix6);
-		offset += afi_len;
 
-		/* Prefix length */
+		/* Bgp4V2SubsequentAddressFamilyIdentifierTC  */
+		/* limited to Labeled unicast */
+		if (*offset > IANA_SAFI_LABELED_UNICAST)
+			return NULL;
+		safi = safi_iana2int(*offset);
+		offset++;
+
+		/* get bgp4V2NlriPrefixType */
+		prefix_type = *offset;
+		offset++;
+
+		/* get bgp4V2NlriPrefix */
+		if (prefix_type == IANA_AFI_IPV4) {
+			oid2in_addr(offset, IN_ADDR_SIZE, &addr->u.prefix4);
+			addr->family = AF_INET;
+			offset += IN_ADDR_SIZE;
+		} else if (prefix_type == IANA_AFI_IPV6) {
+			oid2in6_addr(offset, &addr->u.prefix6);
+			addr->family = AF_INET6;
+			offset += IN6_ADDR_SIZE;
+		}
+
+		/* get bgp4V2NlriPrefixLen */
 		addr->prefixlen = *offset;
-		addr->family = family;
+		offset++;
+
+		/* get bgp4V2PeerRemoteAddrType */
+		peer_addr_type = *offset;
+		if (peer_addr_type == IANA_AFI_IPV4)
+			family = AF_INET;
+		else
+			family = AF_INET6;
 		offset++;
 
 		/* Peer address */
 		su.sin.sin_family = family;
+
+		/* get bgp4V2PeerRemoteAddr*/
 		if (family == AF_INET)
-			oid2in_addr(offset, afi_len, &su.sin.sin_addr);
+			oid2in_addr(offset, IN_ADDR_SIZE, &su.sin.sin_addr);
 		else
 			oid2in6_addr(offset, &su.sin6.sin6_addr);
 
+		/* bgp4V2NlriIndex currently ignored */
+
 		/* Lookup node */
-		dest = bgp_node_lookup(bgp->rib[afi][SAFI_UNICAST], addr);
+		dest = bgp_node_lookup(bgp->rib[afi][safi], addr);
 		if (dest) {
 			for (path = bgp_dest_get_bgp_path_info(dest); path;
 			     path = path->next)
-				if (sockunion_same(&path->peer->su, &su))
+				if (sockunion_same(&path->peer->connection->su,
+						   &su))
 					return path;
 
 			bgp_dest_unlock_node(dest);
@@ -492,116 +524,191 @@ bgp4v2PathAttrLookup(struct variable *v, oid name[], size_t *length,
 		return NULL;
 	}
 
+	/* Set OID offset for prefix type */
 	offset = name + namelen;
 	offsetlen = *length - namelen;
 	len = offsetlen;
 
 	if (offsetlen == 0) {
 		dest = bgp_table_top(bgp->rib[afi][SAFI_UNICAST]);
+		safi = SAFI_UNICAST;
 	} else {
-		if (len > afi_len)
-			len = afi_len;
 
-		if (family == AF_INET)
-			oid2in_addr(offset, len, &addr->u.prefix4);
-		else
+		/* bgp4V2NlriAfi  is already get  */
+		/* it is comming from the name parameter */
+
+		/* get bgp4V2NlriSafi */
+		/* Bgp4V2SubsequentAddressFamilyIdentifierTC */
+		/* limited to Labeled unicast */
+		if (*offset > IANA_SAFI_LABELED_UNICAST)
+			return NULL;
+		safi = safi_iana2int(*offset);
+		offset++;
+
+		/* get bgp4V2NlriPrefixType */
+		prefix_type = *offset;
+		offset++;
+		/* get bgp4V2NlriPrefix */
+		if (prefix_type == IANA_AFI_IPV4) {
+			oid2in_addr(offset, IN_ADDR_SIZE, &addr->u.prefix4);
+			addr->family = AF_INET;
+			offset += IN_ADDR_SIZE;
+			offsetlen -= IN_ADDR_SIZE;
+		} else if (prefix_type == IANA_AFI_IPV6) {
 			oid2in6_addr(offset, &addr->u.prefix6);
+			addr->family = AF_INET6;
+			offset += IN6_ADDR_SIZE;
+			offsetlen -= IN6_ADDR_SIZE;
+		}
 
-		offset += afi_len;
-		offsetlen -= afi_len;
-
+		/* get bgp4V2NlriPrefixLen */
 		if (offsetlen > 0)
 			addr->prefixlen = *offset;
 		else
-			addr->prefixlen = len * 8;
-
-		addr->family = family;
-
-		dest = bgp_node_get(bgp->rib[afi][SAFI_UNICAST], addr);
+			addr->prefixlen = afi_len * 8;
 
 		offset++;
 		offsetlen--;
+
+		/* get node */
+		dest = bgp_node_get(bgp->rib[afi][safi], addr);
 	}
 
+	if (!dest)
+		return NULL;
+
 	if (offsetlen > 0) {
 		len = offsetlen;
 		if (len > afi_len)
 			len = afi_len;
 
+
+		/* get bgp4V2PeerRemoteAddrType */
+		peer_addr_type = *offset;
+		if (peer_addr_type == IANA_AFI_IPV4)
+			family = AF_INET;
+		else
+			family = AF_INET6;
+		offset++;
+
 		if (family == AF_INET)
-			oid2in_addr(offset, len, &paddr.ip._v4_addr);
+			oid2in_addr(offset, IN_ADDR_SIZE, &paddr.ip._v4_addr);
 		else
 			oid2in6_addr(offset, &paddr.ip._v6_addr);
 	} else {
-		if (family == AF_INET)
-			memset(&paddr.ip._v4_addr, 0, afi_len);
+		/* default case  start with ipv4*/
+		if (afi == AFI_IP)
+			family = AF_INET;
 		else
-			memset(&paddr.ip._v6_addr, 0, afi_len);
+			family = AF_INET6;
+		memset(&paddr.ip, 0, sizeof(paddr.ip));
 	}
 
-	if (!dest)
-		return NULL;
-
 	do {
 		min = NULL;
+		min_family = 0;
 
 		for (path = bgp_dest_get_bgp_path_info(dest); path;
 		     path = path->next) {
 			sa_family_t path_family =
-				sockunion_family(&path->peer->su);
+				sockunion_family(&path->peer->connection->su);
+			/* the next addr must be > to the current */
+			if (path_family < family)
+				continue;
 
-			if (path_family == AF_INET &&
-			    IPV4_ADDR_CMP(&paddr.ip._v4_addr,
-					  &path->peer->su.sin.sin_addr) < 0) {
-				if (!min ||
-				    (min &&
-				     IPV4_ADDR_CMP(
-					     &path->peer->su.sin.sin_addr,
-					     &min->peer->su.sin.sin_addr) < 0))
-					min = path;
-			} else if (path_family == AF_INET6 &&
-				   IPV6_ADDR_CMP(
-					   &paddr.ip._v6_addr,
-					   &path->peer->su.sin6.sin6_addr) <
-					   0) {
-				if (!min ||
-				    (min &&
-				     IPV6_ADDR_CMP(
-					     &path->peer->su.sin6.sin6_addr,
-					     &min->peer->su.sin6.sin6_addr) <
-					     0))
-					min = path;
+			if (family == AF_INET
+			    && IPV4_ADDR_CMP(&paddr.ip._v4_addr,
+					     &path->peer->connection->su.sin.sin_addr)
+				       >= 0)
+				continue;
+			else if (family == AF_INET6
+				 && IPV6_ADDR_CMP(
+					    &paddr.ip._v6_addr,
+					    &path->peer->connection->su.sin6.sin6_addr)
+					    >= 0)
+				continue;
+
+			/* first valid path its the  min*/
+			if (!min) {
+				min = path;
+				min_family = path_family;
+				continue;
+			}
+
+			/* consider path < min */
+			if (path_family < min_family) {
+				min = path;
+				min_family = path_family;
+				continue;
+			}
+
+			if (path_family == AF_INET
+			    && IPV4_ADDR_CMP(&path->peer->connection->su.sin.sin_addr,
+					     &min->peer->connection->su.sin.sin_addr)
+				       < 0) {
+				min = path;
+				min_family = path_family;
+
+			} else if (path_family == AF_INET6
+				   && IPV6_ADDR_CMP(
+					      &path->peer->connection->su.sin6.sin6_addr,
+					      &min->peer->connection->su.sin6.sin6_addr)
+					      < 0) {
+				min = path;
+				min_family = path_family;
 			}
 		}
 
 		if (min) {
 			const struct prefix *rn_p = bgp_dest_get_prefix(dest);
 
-			*length = namelen + BGP_NLRI_ENTRY_OFFSET;
-
 			offset = name + namelen;
 
-			/* Encode prefix into OID */
-			if (family == AF_INET)
+			/* encode bgp4V2NlriSafi*/
+			*offset = SAFI_UNICAST;
+			offset++;
+			/* encode bgp4V2NlriPrefixType into index*/
+			/* encode  bgp4V2NlriPrefix into index */
+			if (rn_p->family == AF_INET) {
+				*offset = IANA_AFI_IPV4;
+				offset++;
 				oid_copy_in_addr(offset, &rn_p->u.prefix4);
-			else
+				offset += IN_ADDR_SIZE;
+			} else {
+				*offset = IANA_AFI_IPV6;
+				offset++;
 				oid_copy_in6_addr(offset, &rn_p->u.prefix6);
-
-			offset += afi_len;
+				offset += IN6_ADDR_SIZE;
+			}
+			/* encode bgp4V2NlriPrefixLen into index*/
 			*offset = rn_p->prefixlen;
 			offset++;
 
-			/* Encode peer's IP into OID */
-			if (family == AF_INET) {
+			/* Encode bgp4V2PeerRemoteAddrType */
+			/* Encode bgp4V2PeerRemoteAddr */
+			if (min_family == AF_INET) {
+				*offset = IANA_AFI_IPV4;
+				offset++;
 				oid_copy_in_addr(offset,
-						 &min->peer->su.sin.sin_addr);
+						 &min->peer->connection->su.sin.sin_addr);
+				offset += IN_ADDR_SIZE;
 				addr->u.prefix4 = rn_p->u.prefix4;
 			} else {
+				*offset = IANA_AFI_IPV6;
+				offset++;
 				oid_copy_in6_addr(
-					offset, &min->peer->su.sin6.sin6_addr);
+					offset, &min->peer->connection->su.sin6.sin6_addr);
+				offset += IN6_ADDR_SIZE;
 				addr->u.prefix6 = rn_p->u.prefix6;
 			}
 
+			/* Encode bgp4V2NlriIndex*/
+
+			*offset = 1;
+			offset++;
+
+			*length = offset - name;
+
 			addr->prefixlen = rn_p->prefixlen;
 			addr->family = rn_p->family;
 
@@ -610,10 +717,8 @@ bgp4v2PathAttrLookup(struct variable *v, oid name[], size_t *length,
 			return min;
 		}
 
-		if (family == AF_INET)
-			memset(&paddr.ip._v4_addr, 0, afi_len);
-		else
-			memset(&paddr.ip._v6_addr, 0, afi_len);
+		memset(&paddr.ip, 0, sizeof(paddr.ip));
+
 	} while ((dest = bgp_route_next(dest)));
 
 	return NULL;
@@ -798,619 +903,619 @@ static struct variable bgpv2_variables[] = {
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_INSTANCE, 1, 4}},
+	 {1, 2, 1, BGP4V2_PEER_INSTANCE, 1, 1}},
 	{BGP4V2_PEER_INSTANCE,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_INSTANCE, 2, 16}},
+	 {1, 2, 1, BGP4V2_PEER_INSTANCE, 1, 2}},
 	{BGP4V2_PEER_LOCAL_ADDR_TYPE,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_LOCAL_ADDR_TYPE, 1, 4}},
+	 {1, 2, 1, BGP4V2_PEER_LOCAL_ADDR_TYPE, 1, 1}},
 	{BGP4V2_PEER_LOCAL_ADDR_TYPE,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_LOCAL_ADDR_TYPE, 2, 16}},
+	 {1, 2, 1, BGP4V2_PEER_LOCAL_ADDR_TYPE, 1, 2}},
 	{BGP4V2_PEER_LOCAL_ADDR,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_LOCAL_ADDR, 1, 4}},
+	 {1, 2, 1, BGP4V2_PEER_LOCAL_ADDR, 1, 1}},
 	{BGP4V2_PEER_LOCAL_ADDR,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_LOCAL_ADDR, 2, 16}},
+	 {1, 2, 1, BGP4V2_PEER_LOCAL_ADDR, 1, 2}},
 	{BGP4V2_PEER_REMOTE_ADDR_TYPE,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_REMOTE_ADDR_TYPE, 1, 4}},
+	 {1, 2, 1, BGP4V2_PEER_REMOTE_ADDR_TYPE, 1, 1}},
 	{BGP4V2_PEER_REMOTE_ADDR_TYPE,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_REMOTE_ADDR_TYPE, 2, 16}},
+	 {1, 2, 1, BGP4V2_PEER_REMOTE_ADDR_TYPE, 1, 2}},
 	{BGP4V2_PEER_REMOTE_ADDR,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_REMOTE_ADDR, 1, 4}},
+	 {1, 2, 1, BGP4V2_PEER_REMOTE_ADDR, 1, 1}},
 	{BGP4V2_PEER_REMOTE_ADDR,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_REMOTE_ADDR, 2, 16}},
+	 {1, 2, 1, BGP4V2_PEER_REMOTE_ADDR, 1, 2}},
 	{BGP4V2_PEER_LOCAL_PORT,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_LOCAL_PORT, 1, 4}},
+	 {1, 2, 1, BGP4V2_PEER_LOCAL_PORT, 1, 1}},
 	{BGP4V2_PEER_LOCAL_PORT,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_LOCAL_PORT, 2, 16}},
+	 {1, 2, 1, BGP4V2_PEER_LOCAL_PORT, 1, 2}},
 	{BGP4V2_PEER_LOCAL_AS,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_LOCAL_AS, 1, 4}},
+	 {1, 2, 1, BGP4V2_PEER_LOCAL_AS, 1, 1}},
 	{BGP4V2_PEER_LOCAL_AS,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_LOCAL_AS, 2, 16}},
+	 {1, 2, 1, BGP4V2_PEER_LOCAL_AS, 1, 2}},
 	{BGP4V2_PEER_LOCAL_IDENTIFIER,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_LOCAL_IDENTIFIER, 1, 4}},
+	 {1, 2, 1, BGP4V2_PEER_LOCAL_IDENTIFIER, 1, 1}},
 	{BGP4V2_PEER_LOCAL_IDENTIFIER,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_LOCAL_IDENTIFIER, 2, 16}},
+	 {1, 2, 1, BGP4V2_PEER_LOCAL_IDENTIFIER, 1, 2}},
 	{BGP4V2_PEER_REMOTE_PORT,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_REMOTE_PORT, 1, 4}},
+	 {1, 2, 1, BGP4V2_PEER_REMOTE_PORT, 1, 1}},
 	{BGP4V2_PEER_REMOTE_PORT,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_REMOTE_PORT, 2, 16}},
+	 {1, 2, 1, BGP4V2_PEER_REMOTE_PORT, 1, 2}},
 	{BGP4V2_PEER_REMOTE_AS,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_REMOTE_AS, 1, 4}},
+	 {1, 2, 1, BGP4V2_PEER_REMOTE_AS, 1, 1}},
 	{BGP4V2_PEER_REMOTE_AS,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_REMOTE_AS, 2, 16}},
+	 {1, 2, 1, BGP4V2_PEER_REMOTE_AS, 1, 2}},
 	{BGP4V2_PEER_REMOTE_IDENTIFIER,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_REMOTE_IDENTIFIER, 1, 4}},
+	 {1, 2, 1, BGP4V2_PEER_REMOTE_IDENTIFIER, 1, 1}},
 	{BGP4V2_PEER_REMOTE_IDENTIFIER,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_REMOTE_IDENTIFIER, 2, 16}},
+	 {1, 2, 1, BGP4V2_PEER_REMOTE_IDENTIFIER, 1, 2}},
 	{BGP4V2_PEER_ADMIN_STATUS,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_ADMIN_STATUS, 1, 4}},
+	 {1, 2, 1, BGP4V2_PEER_ADMIN_STATUS, 1, 1}},
 	{BGP4V2_PEER_ADMIN_STATUS,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_ADMIN_STATUS, 2, 16}},
+	 {1, 2, 1, BGP4V2_PEER_ADMIN_STATUS, 1, 2}},
 	{BGP4V2_PEER_STATE,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_STATE, 1, 4}},
+	 {1, 2, 1, BGP4V2_PEER_STATE, 1, 1}},
 	{BGP4V2_PEER_STATE,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_STATE, 2, 16}},
+	 {1, 2, 1, BGP4V2_PEER_STATE, 1, 2}},
 	{BGP4V2_PEER_DESCRIPTION,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_DESCRIPTION, 1, 4}},
+	 {1, 2, 1, BGP4V2_PEER_DESCRIPTION, 1, 1}},
 	{BGP4V2_PEER_DESCRIPTION,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerTable,
 	 6,
-	 {1, 2, 1, BGP4V2_PEER_DESCRIPTION, 2, 16}},
+	 {1, 2, 1, BGP4V2_PEER_DESCRIPTION, 1, 2}},
 	/* bgp4V2PeerErrorsEntry */
 	{BGP4V2_PEER_LAST_ERROR_CODE_RECEIVED,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_CODE_RECEIVED, 1, 4}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_CODE_RECEIVED, 1, 1}},
 	{BGP4V2_PEER_LAST_ERROR_CODE_RECEIVED,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_CODE_RECEIVED, 2, 16}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_CODE_RECEIVED, 1, 2}},
 	{BGP4V2_PEER_LAST_ERROR_SUBCODE_RECEIVED,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SUBCODE_RECEIVED, 1, 4}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SUBCODE_RECEIVED, 1, 1}},
 	{BGP4V2_PEER_LAST_ERROR_SUBCODE_RECEIVED,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SUBCODE_RECEIVED, 2, 16}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SUBCODE_RECEIVED, 1, 2}},
 	{BGP4V2_PEER_LAST_ERROR_RECEIVED_TIME,
-	 ASN_UNSIGNED,
+	 ASN_TIMETICKS,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_RECEIVED_TIME, 1, 4}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_RECEIVED_TIME, 1, 1}},
 	{BGP4V2_PEER_LAST_ERROR_RECEIVED_TIME,
-	 ASN_UNSIGNED,
+	 ASN_TIMETICKS,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_RECEIVED_TIME, 2, 16}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_RECEIVED_TIME, 1, 2}},
 	{BGP4V2_PEER_LAST_ERROR_RECEIVED_TEXT,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_RECEIVED_TEXT, 1, 4}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_RECEIVED_TEXT, 1, 1}},
 	{BGP4V2_PEER_LAST_ERROR_RECEIVED_TEXT,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_RECEIVED_TEXT, 2, 16}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_RECEIVED_TEXT, 1, 2}},
 	{BGP4V2_PEER_LAST_ERROR_RECEIVED_DATA,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_RECEIVED_DATA, 1, 4}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_RECEIVED_DATA, 1, 1}},
 	{BGP4V2_PEER_LAST_ERROR_RECEIVED_DATA,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_RECEIVED_DATA, 2, 16}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_RECEIVED_DATA, 1, 2}},
 	{BGP4V2_PEER_LAST_ERROR_CODE_SENT,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_CODE_SENT, 1, 4}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_CODE_SENT, 1, 1}},
 	{BGP4V2_PEER_LAST_ERROR_CODE_SENT,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_CODE_SENT, 2, 16}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_CODE_SENT, 1, 2}},
 	{BGP4V2_PEER_LAST_ERROR_SUBCODE_SENT,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SUBCODE_SENT, 1, 4}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SUBCODE_SENT, 1, 1}},
 	{BGP4V2_PEER_LAST_ERROR_SUBCODE_SENT,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SUBCODE_SENT, 2, 16}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SUBCODE_SENT, 1, 2}},
 	{BGP4V2_PEER_LAST_ERROR_SENT_TIME,
-	 ASN_UNSIGNED,
+	 ASN_TIMETICKS,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SENT_TIME, 1, 4}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SENT_TIME, 1, 1}},
 	{BGP4V2_PEER_LAST_ERROR_SENT_TIME,
-	 ASN_UNSIGNED,
+	 ASN_TIMETICKS,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SENT_TIME, 2, 16}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SENT_TIME, 1, 2}},
 	{BGP4V2_PEER_LAST_ERROR_SENT_TEXT,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SENT_TEXT, 1, 4}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SENT_TEXT, 1, 1}},
 	{BGP4V2_PEER_LAST_ERROR_SENT_TEXT,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SENT_TEXT, 2, 16}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SENT_TEXT, 1, 2}},
 	{BGP4V2_PEER_LAST_ERROR_SENT_DATA,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SENT_DATA, 1, 4}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SENT_DATA, 1, 1}},
 	{BGP4V2_PEER_LAST_ERROR_SENT_DATA,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgpv2PeerErrorsTable,
 	 6,
-	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SENT_DATA, 2, 16}},
+	 {1, 3, 1, BGP4V2_PEER_LAST_ERROR_SENT_DATA, 1, 2}},
 	/* bgp4V2PeerEventTimesEntry */
 	{BGP4V2_PEER_FSM_ESTABLISHED_TIME,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerEventTimesTable,
 	 6,
-	 {1, 4, 1, BGP4V2_PEER_FSM_ESTABLISHED_TIME, 1, 4}},
+	 {1, 4, 1, BGP4V2_PEER_FSM_ESTABLISHED_TIME, 1, 1}},
 	{BGP4V2_PEER_FSM_ESTABLISHED_TIME,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerEventTimesTable,
 	 6,
-	 {1, 4, 1, BGP4V2_PEER_FSM_ESTABLISHED_TIME, 2, 16}},
+	 {1, 4, 1, BGP4V2_PEER_FSM_ESTABLISHED_TIME, 1, 2}},
 	{BGP4V2_PEER_PEER_IN_UPDATES_ELAPSED_TIME,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerEventTimesTable,
 	 6,
-	 {1, 4, 1, BGP4V2_PEER_PEER_IN_UPDATES_ELAPSED_TIME, 1, 4}},
+	 {1, 4, 1, BGP4V2_PEER_PEER_IN_UPDATES_ELAPSED_TIME, 1, 1}},
 	{BGP4V2_PEER_PEER_IN_UPDATES_ELAPSED_TIME,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgpv2PeerEventTimesTable,
 	 6,
-	 {1, 4, 1, BGP4V2_PEER_PEER_IN_UPDATES_ELAPSED_TIME, 2, 16}},
+	 {1, 4, 1, BGP4V2_PEER_PEER_IN_UPDATES_ELAPSED_TIME, 1, 2}},
 	/* bgp4V2NlriTable */
 	{BGP4V2_NLRI_INDEX,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_INDEX, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_INDEX, 1, 1}},
 	{BGP4V2_NLRI_INDEX,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_INDEX, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_INDEX, 1, 2}},
 	{BGP4V2_NLRI_AFI,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_AFI, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_AFI, 1, 1}},
 	{BGP4V2_NLRI_AFI,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_AFI, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_AFI, 1, 2}},
 	{BGP4V2_NLRI_SAFI,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_SAFI, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_SAFI, 1, 1}},
 	{BGP4V2_NLRI_SAFI,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_SAFI, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_SAFI, 1, 2}},
 	{BGP4V2_NLRI_PREFIX_TYPE,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_PREFIX_TYPE, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_PREFIX_TYPE, 1, 1}},
 	{BGP4V2_NLRI_PREFIX_TYPE,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_PREFIX_TYPE, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_PREFIX_TYPE, 1, 2}},
 	{BGP4V2_NLRI_PREFIX,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_PREFIX, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_PREFIX, 1, 1}},
 	{BGP4V2_NLRI_PREFIX,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_PREFIX, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_PREFIX, 1, 2}},
 	{BGP4V2_NLRI_PREFIX_LEN,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_PREFIX_LEN, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_PREFIX_LEN, 1, 1}},
 	{BGP4V2_NLRI_PREFIX_LEN,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_PREFIX_LEN, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_PREFIX_LEN, 1, 2}},
 	{BGP4V2_NLRI_BEST,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_BEST, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_BEST, 1, 1}},
 	{BGP4V2_NLRI_BEST,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_BEST, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_BEST, 1, 2}},
 	{BGP4V2_NLRI_CALC_LOCAL_PREF,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_CALC_LOCAL_PREF, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_CALC_LOCAL_PREF, 1, 1}},
 	{BGP4V2_NLRI_CALC_LOCAL_PREF,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_CALC_LOCAL_PREF, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_CALC_LOCAL_PREF, 1, 2}},
 	{BGP4V2_NLRI_ORIGIN,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_ORIGIN, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_ORIGIN, 1, 1}},
 	{BGP4V2_NLRI_ORIGIN,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_ORIGIN, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_ORIGIN, 1, 2}},
 	{BGP4V2_NLRI_NEXT_HOP_ADDR_TYPE,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_NEXT_HOP_ADDR_TYPE, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_NEXT_HOP_ADDR_TYPE, 1, 1}},
 	{BGP4V2_NLRI_NEXT_HOP_ADDR_TYPE,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_NEXT_HOP_ADDR_TYPE, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_NEXT_HOP_ADDR_TYPE, 1, 2}},
 	{BGP4V2_NLRI_NEXT_HOP_ADDR,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_NEXT_HOP_ADDR, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_NEXT_HOP_ADDR, 1, 1}},
 	{BGP4V2_NLRI_NEXT_HOP_ADDR,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_NEXT_HOP_ADDR, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_NEXT_HOP_ADDR, 1, 2}},
 	{BGP4V2_NLRI_LINK_LOCAL_NEXT_HOP_ADDR_TYPE,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_LINK_LOCAL_NEXT_HOP_ADDR_TYPE, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_LINK_LOCAL_NEXT_HOP_ADDR_TYPE, 1, 1}},
 	{BGP4V2_NLRI_LINK_LOCAL_NEXT_HOP_ADDR_TYPE,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_LINK_LOCAL_NEXT_HOP_ADDR_TYPE, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_LINK_LOCAL_NEXT_HOP_ADDR_TYPE, 1, 2}},
 	{BGP4V2_NLRI_LINK_LOCAL_NEXT_HOP_ADDR,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_LINK_LOCAL_NEXT_HOP_ADDR, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_LINK_LOCAL_NEXT_HOP_ADDR, 1, 1}},
 	{BGP4V2_NLRI_LINK_LOCAL_NEXT_HOP_ADDR,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_LINK_LOCAL_NEXT_HOP_ADDR, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_LINK_LOCAL_NEXT_HOP_ADDR, 1, 2}},
 	{BGP4V2_NLRI_LOCAL_PREF_PRESENT,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_LOCAL_PREF_PRESENT, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_LOCAL_PREF_PRESENT, 1, 1}},
 	{BGP4V2_NLRI_LOCAL_PREF_PRESENT,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_LOCAL_PREF_PRESENT, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_LOCAL_PREF_PRESENT, 1, 2}},
 	{BGP4V2_NLRI_LOCAL_PREF,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_LOCAL_PREF, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_LOCAL_PREF, 1, 1}},
 	{BGP4V2_NLRI_LOCAL_PREF,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_LOCAL_PREF, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_LOCAL_PREF, 1, 2}},
 	{BGP4V2_NLRI_MED_PRESENT,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_MED_PRESENT, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_MED_PRESENT, 1, 1}},
 	{BGP4V2_NLRI_MED_PRESENT,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_MED_PRESENT, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_MED_PRESENT, 1, 2}},
 	{BGP4V2_NLRI_MED,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_MED, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_MED, 1, 1}},
 	{BGP4V2_NLRI_MED,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_MED, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_MED, 1, 2}},
 	{BGP4V2_NLRI_ATOMIC_AGGREGATE,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_ATOMIC_AGGREGATE, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_ATOMIC_AGGREGATE, 1, 1}},
 	{BGP4V2_NLRI_ATOMIC_AGGREGATE,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_ATOMIC_AGGREGATE, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_ATOMIC_AGGREGATE, 1, 2}},
 	{BGP4V2_NLRI_AGGREGATOR_PRESENT,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_AGGREGATOR_PRESENT, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_AGGREGATOR_PRESENT, 1, 1}},
 	{BGP4V2_NLRI_AGGREGATOR_PRESENT,
 	 ASN_INTEGER,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_AGGREGATOR_PRESENT, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_AGGREGATOR_PRESENT, 1, 2}},
 	{BGP4V2_NLRI_AGGREGATOR_AS,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_AGGREGATOR_AS, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_AGGREGATOR_AS, 1, 1}},
 	{BGP4V2_NLRI_AGGREGATOR_AS,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_AGGREGATOR_AS, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_AGGREGATOR_AS, 1, 2}},
 	{BGP4V2_NLRI_AGGREGATOR_ADDR,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_AGGREGATOR_ADDR, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_AGGREGATOR_ADDR, 1, 1}},
 	{BGP4V2_NLRI_AGGREGATOR_ADDR,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_AGGREGATOR_ADDR, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_AGGREGATOR_ADDR, 1, 2}},
 	{BGP4V2_NLRI_AS_PATH_CALC_LENGTH,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_AS_PATH_CALC_LENGTH, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_AS_PATH_CALC_LENGTH, 1, 1}},
 	{BGP4V2_NLRI_AS_PATH_CALC_LENGTH,
 	 ASN_UNSIGNED,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_AS_PATH_CALC_LENGTH, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_AS_PATH_CALC_LENGTH, 1, 2}},
 	{BGP4V2_NLRI_AS_PATH_STRING,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_AS_PATH_STRING, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_AS_PATH_STRING, 1, 1}},
 	{BGP4V2_NLRI_AS_PATH_STRING,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_AS_PATH_STRING, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_AS_PATH_STRING, 1, 2}},
 	{BGP4V2_NLRI_AS_PATH,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_AS_PATH, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_AS_PATH, 1, 1}},
 	{BGP4V2_NLRI_AS_PATH,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_AS_PATH, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_AS_PATH, 1, 2}},
 	{BGP4V2_NLRI_PATH_ATTR_UNKNOWN,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_PATH_ATTR_UNKNOWN, 1, 4}},
+	 {1, 9, 1, BGP4V2_NLRI_PATH_ATTR_UNKNOWN, 1, 1}},
 	{BGP4V2_NLRI_PATH_ATTR_UNKNOWN,
 	 ASN_OCTET_STR,
 	 RONLY,
 	 bgp4v2PathAttrTable,
 	 6,
-	 {1, 9, 1, BGP4V2_NLRI_PATH_ATTR_UNKNOWN, 2, 16}},
+	 {1, 9, 1, BGP4V2_NLRI_PATH_ATTR_UNKNOWN, 1, 2}},
 };
 
-int bgp_snmp_bgp4v2_init(struct thread_master *tm)
+int bgp_snmp_bgp4v2_init(struct event_loop *tm)
 {
 	REGISTER_MIB("mibII/bgpv2", bgpv2_variables, variable, bgpv2_oid);
 	return 0;
