@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /**
  * bgp_updgrp.c: BGP update group structures
  *
@@ -6,28 +7,12 @@
  * @author Avneesh Sachdev <avneesh@sproute.net>
  * @author Rajesh Varadarajan <rajesh@sproute.net>
  * @author Pradosh Mohapatra <pradosh@sproute.net>
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
 #include "prefix.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "buffer.h"
 #include "stream.h"
 #include "command.h"
@@ -89,7 +74,7 @@ static void sync_init(struct update_subgroup *subgrp,
 		XCALLOC(MTYPE_BGP_SYNCHRONISE, sizeof(struct bgp_synchronize));
 	bgp_adv_fifo_init(&subgrp->sync->update);
 	bgp_adv_fifo_init(&subgrp->sync->withdraw);
-	bgp_adv_fifo_init(&subgrp->sync->withdraw_low);
+
 	subgrp->hash =
 		hash_create(bgp_advertise_attr_hash_key,
 			    bgp_advertise_attr_hash_cmp, "BGP SubGroup Hash");
@@ -116,12 +101,9 @@ static void sync_init(struct update_subgroup *subgrp,
 static void sync_delete(struct update_subgroup *subgrp)
 {
 	XFREE(MTYPE_BGP_SYNCHRONISE, subgrp->sync);
-	if (subgrp->hash) {
-		hash_clean(subgrp->hash,
-			   (void (*)(void *))bgp_advertise_attr_free);
-		hash_free(subgrp->hash);
-	}
-	subgrp->hash = NULL;
+	hash_clean_and_free(&subgrp->hash,
+			    (void (*)(void *))bgp_advertise_attr_free);
+
 	if (subgrp->work)
 		stream_free(subgrp->work);
 	subgrp->work = NULL;
@@ -160,6 +142,8 @@ static void conf_copy(struct peer *dst, struct peer *src, afi_t afi,
 	dst->afc_nego[afi][safi] = src->afc_nego[afi][safi];
 	dst->orf_plist[afi][safi] = src->orf_plist[afi][safi];
 	dst->addpath_type[afi][safi] = src->addpath_type[afi][safi];
+	dst->addpath_best_selected[afi][safi] =
+		src->addpath_best_selected[afi][safi];
 	dst->local_as = src->local_as;
 	dst->change_local_as = src->change_local_as;
 	dst->shared_network = src->shared_network;
@@ -295,6 +279,8 @@ static void *updgrp_hash_alloc(void *p)
 	updgrp = XCALLOC(MTYPE_BGP_UPDGRP, sizeof(struct update_group));
 	memcpy(updgrp, in, sizeof(struct update_group));
 	updgrp->conf = XCALLOC(MTYPE_BGP_PEER, sizeof(struct peer));
+	updgrp->conf->connection = XCALLOC(MTYPE_BGP_PEER_CONNECTION,
+					   sizeof(struct peer_connection));
 	conf_copy(updgrp->conf, in->conf, in->afi, in->safi);
 	return updgrp;
 }
@@ -325,6 +311,7 @@ static void *updgrp_hash_alloc(void *p)
  *       16. Local-as should match, if configured.
  *       17. maximum-prefix-out
  *       18. Local-role should also match, if configured.
+ *       19. Add-Path best selected paths count should match as well
  *      )
  */
 static unsigned int updgrp_hash_key_make(const void *p)
@@ -337,6 +324,11 @@ static unsigned int updgrp_hash_key_make(const void *p)
 	afi_t afi;
 	safi_t safi;
 
+	/*
+	 * IF YOU ADD AN ADDITION TO THE HASH KEY TO ENSURE
+	 * THAT THE UPDATE GROUP CALCULATION IS CORRECT THEN
+	 * PLEASE ADD IT TO THE DEBUG OUTPUT TOO AT THE BOTTOM
+	 */
 #define SEED1 999331
 #define SEED2 2147483647
 
@@ -353,6 +345,7 @@ static unsigned int updgrp_hash_key_make(const void *p)
 	key = jhash_1word((peer->flags & PEER_UPDGRP_FLAGS), key);
 	key = jhash_1word((flags & PEER_UPDGRP_AF_FLAGS), key);
 	key = jhash_1word((uint32_t)peer->addpath_type[afi][safi], key);
+	key = jhash_1word(peer->addpath_best_selected[afi][safi], key);
 	key = jhash_1word((peer->cap & PEER_UPDGRP_CAP_FLAGS), key);
 	key = jhash_1word((peer->af_cap[afi][safi] & PEER_UPDGRP_AF_CAP_FLAGS),
 			  key);
@@ -429,8 +422,6 @@ static unsigned int updgrp_hash_key_make(const void *p)
 	 */
 	if (CHECK_FLAG(peer->flags, PEER_FLAG_LONESOUL)
 	    || CHECK_FLAG(peer->af_cap[afi][safi], PEER_CAP_ORF_PREFIX_SM_RCV)
-	    || CHECK_FLAG(peer->af_cap[afi][safi],
-			  PEER_CAP_ORF_PREFIX_SM_OLD_RCV)
 	    || CHECK_FLAG(peer->af_flags[afi][safi], PEER_FLAG_MAX_PREFIX_OUT))
 		key = jhash_1word(jhash(peer->host, strlen(peer->host), SEED2),
 				  key);
@@ -451,6 +442,10 @@ static unsigned int updgrp_hash_key_make(const void *p)
 		key = jhash_1word(jhash(soo_str, strlen(soo_str), SEED1), key);
 	}
 
+	/*
+	 * ANY NEW ITEMS THAT ARE ADDED TO THE key, ENSURE DEBUG
+	 * STATEMENT STAYS UP TO DATE
+	 */
 	if (bgp_debug_neighbor_events(peer)) {
 		zlog_debug(
 			"%pBP Update Group Hash: sort: %d UpdGrpFlags: %ju UpdGrpAFFlags: %ju",
@@ -472,7 +467,7 @@ static unsigned int updgrp_hash_key_make(const void *p)
 			ROUTE_MAP_OUT_NAME(filter) ? ROUTE_MAP_OUT_NAME(filter)
 						   : "(NONE)");
 		zlog_debug(
-			"%pBP Update Group Hash: dlist out: %s plist out: %s aslist out: %s usmap out: %s advmap: %s",
+			"%pBP Update Group Hash: dlist out: %s plist out: %s aslist out: %s usmap out: %s advmap: %s %d",
 			peer,
 			DISTRIBUTE_OUT_NAME(filter)
 				? DISTRIBUTE_OUT_NAME(filter)
@@ -487,7 +482,8 @@ static unsigned int updgrp_hash_key_make(const void *p)
 				? UNSUPPRESS_MAP_NAME(filter)
 				: "(NONE)",
 			ADVERTISE_MAP_NAME(filter) ? ADVERTISE_MAP_NAME(filter)
-						   : "(NONE)");
+						   : "(NONE)",
+			filter->advmap.update_type);
 		zlog_debug(
 			"%pBP Update Group Hash: default rmap: %s shared network and afi active network: %d",
 			peer,
@@ -496,15 +492,19 @@ static unsigned int updgrp_hash_key_make(const void *p)
 				: "(NONE)",
 			peer->shared_network &&
 				peer_afi_active_nego(peer, AFI_IP6));
+		zlog_debug("%pBP Update Group Hash: Lonesoul: %d ORF prefix: %u max prefix out: %ju",
+			   peer, !!CHECK_FLAG(peer->flags, PEER_FLAG_LONESOUL),
+			   CHECK_FLAG(peer->af_cap[afi][safi],
+				      PEER_CAP_ORF_PREFIX_SM_RCV),
+			   (intmax_t)CHECK_FLAG(peer->af_flags[afi][safi],
+						PEER_FLAG_MAX_PREFIX_OUT));
 		zlog_debug(
-			"%pBP Update Group Hash: Lonesoul: %d ORF prefix: %u ORF old: %u max prefix out: %ju",
-			peer, !!CHECK_FLAG(peer->flags, PEER_FLAG_LONESOUL),
-			CHECK_FLAG(peer->af_cap[afi][safi],
-				   PEER_CAP_ORF_PREFIX_SM_RCV),
-			CHECK_FLAG(peer->af_cap[afi][safi],
-				   PEER_CAP_ORF_PREFIX_SM_OLD_RCV),
-			(intmax_t)CHECK_FLAG(peer->af_flags[afi][safi],
-					     PEER_FLAG_MAX_PREFIX_OUT));
+			"%pBP Update Group Hash: local role: %u AIGP: %d SOO: %s",
+			peer, peer->local_role,
+			!!CHECK_FLAG(peer->flags, PEER_FLAG_AIGP),
+			peer->soo[afi][safi]
+				? ecommunity_str(peer->soo[afi][safi])
+				: "(NONE)");
 		zlog_debug("%pBP Update Group Hash key: %u", peer, key);
 	}
 	return key;
@@ -634,11 +634,9 @@ static bool updgrp_hash_cmp(const void *p1, const void *p2)
 	if ((afi == AFI_IP6) && (pe1->shared_network != pe2->shared_network))
 		return false;
 
-	if ((CHECK_FLAG(pe1->flags, PEER_FLAG_LONESOUL)
-	     || CHECK_FLAG(pe1->af_cap[afi][safi], PEER_CAP_ORF_PREFIX_SM_RCV)
-	     || CHECK_FLAG(pe1->af_cap[afi][safi],
-			   PEER_CAP_ORF_PREFIX_SM_OLD_RCV))
-	    && !sockunion_same(&pe1->su, &pe2->su))
+	if ((CHECK_FLAG(pe1->flags, PEER_FLAG_LONESOUL) ||
+	     CHECK_FLAG(pe1->af_cap[afi][safi], PEER_CAP_ORF_PREFIX_SM_RCV)) &&
+	    !sockunion_same(&pe1->connection->su, &pe2->connection->su))
 		return false;
 
 	return true;
@@ -691,6 +689,7 @@ static int update_group_show_walkcb(struct update_group *updgrp, void *arg)
 	json_object *json_peers = NULL;
 	json_object *json_pkt_info = NULL;
 	time_t epoch_tbuf, tbuf;
+	char timebuf[32];
 
 	if (!ctx)
 		return CMD_SUCCESS;
@@ -726,7 +725,7 @@ static int update_group_show_walkcb(struct update_group *updgrp, void *arg)
 		json_time = json_object_new_object();
 		json_object_int_add(json_time, "epoch", epoch_tbuf);
 		json_object_string_add(json_time, "epochString",
-				       ctime(&epoch_tbuf));
+				       ctime_r(&epoch_tbuf, timebuf));
 		json_object_object_add(json_updgrp, "groupCreateTime",
 				       json_time);
 		json_object_string_add(json_updgrp, "afi",
@@ -735,7 +734,8 @@ static int update_group_show_walkcb(struct update_group *updgrp, void *arg)
 				       safi2str(updgrp->safi));
 	} else {
 		vty_out(vty, "Update-group %" PRIu64 ":\n", updgrp->id);
-		vty_out(vty, "  Created: %s", timestamp_string(updgrp->uptime));
+		vty_out(vty, "  Created: %s",
+			timestamp_string(updgrp->uptime, timebuf));
 	}
 
 	filter = &updgrp->conf->filter[updgrp->afi][updgrp->safi];
@@ -796,7 +796,7 @@ static int update_group_show_walkcb(struct update_group *updgrp, void *arg)
 			json_object_int_add(json_subgrp_time, "epoch",
 					    epoch_tbuf);
 			json_object_string_add(json_subgrp_time, "epochString",
-					       ctime(&epoch_tbuf));
+					       ctime_r(&epoch_tbuf, timebuf));
 			json_object_object_add(json_subgrp, "groupCreateTime",
 					       json_subgrp_time);
 		} else {
@@ -804,7 +804,7 @@ static int update_group_show_walkcb(struct update_group *updgrp, void *arg)
 			vty_out(vty, "  Update-subgroup %" PRIu64 ":\n",
 				subgrp->id);
 			vty_out(vty, "    Created: %s",
-				timestamp_string(subgrp->uptime));
+				timestamp_string(subgrp->uptime, timebuf));
 		}
 
 		if (subgrp->split_from.update_group_id
@@ -991,13 +991,18 @@ static struct update_group *update_group_find(struct peer_af *paf)
 	struct update_group *updgrp;
 	struct update_group tmp;
 	struct peer tmp_conf;
+	struct peer_connection tmp_connection;
 
-	if (!peer_established(PAF_PEER(paf)))
+	if (!peer_established((PAF_PEER(paf))->connection))
 		return NULL;
 
 	memset(&tmp, 0, sizeof(tmp));
 	memset(&tmp_conf, 0, sizeof(tmp_conf));
+	memset(&tmp_connection, 0, sizeof(struct peer_connection));
+
 	tmp.conf = &tmp_conf;
+	tmp_conf.connection = &tmp_connection;
+
 	peer2_updgrp_copy(&tmp, paf);
 
 	updgrp = hash_lookup(paf->peer->bgp->update_groups[paf->afid], &tmp);
@@ -1010,10 +1015,14 @@ static struct update_group *update_group_create(struct peer_af *paf)
 	struct update_group *updgrp;
 	struct update_group tmp;
 	struct peer tmp_conf;
+	struct peer_connection tmp_connection;
 
 	memset(&tmp, 0, sizeof(tmp));
 	memset(&tmp_conf, 0, sizeof(tmp_conf));
+	memset(&tmp_connection, 0, sizeof(tmp_connection));
+
 	tmp.conf = &tmp_conf;
+	tmp_conf.connection = &tmp_connection;
 	peer2_updgrp_copy(&tmp, paf);
 
 	updgrp = hash_get(paf->peer->bgp->update_groups[paf->afid], &tmp,
@@ -1043,6 +1052,7 @@ static void update_group_delete(struct update_group *updgrp)
 
 	XFREE(MTYPE_BGP_PEER_IFNAME, updgrp->conf->ifname);
 
+	XFREE(MTYPE_BGP_PEER_CONNECTION, updgrp->conf->connection);
 	XFREE(MTYPE_BGP_PEER, updgrp->conf);
 	XFREE(MTYPE_BGP_UPDGRP, updgrp);
 }
@@ -1100,8 +1110,8 @@ static void update_subgroup_delete(struct update_subgroup *subgrp)
 	if (subgrp->update_group)
 		UPDGRP_INCR_STAT(subgrp->update_group, subgrps_deleted);
 
-	THREAD_OFF(subgrp->t_merge_check);
-	THREAD_OFF(subgrp->t_coalesce);
+	EVENT_OFF(subgrp->t_merge_check);
+	EVENT_OFF(subgrp->t_coalesce);
 
 	bpacket_queue_cleanup(SUBGRP_PKTQ(subgrp));
 	subgroup_clear_table(subgrp);
@@ -1255,7 +1265,7 @@ static struct update_subgroup *update_subgroup_find(struct update_group *updgrp,
 	} else
 		version = 0;
 
-	if (!peer_established(PAF_PEER(paf)))
+	if (!peer_established(PAF_PEER(paf)->connection))
 		return NULL;
 
 	UPDGRP_FOREACH_SUBGRP (updgrp, subgrp) {
@@ -1419,11 +1429,11 @@ bool update_subgroup_check_merge(struct update_subgroup *subgrp,
 /*
 * update_subgroup_merge_check_thread_cb
 */
-static void update_subgroup_merge_check_thread_cb(struct thread *thread)
+static void update_subgroup_merge_check_thread_cb(struct event *thread)
 {
 	struct update_subgroup *subgrp;
 
-	subgrp = THREAD_ARG(thread);
+	subgrp = EVENT_ARG(thread);
 
 	subgrp->t_merge_check = NULL;
 
@@ -1450,8 +1460,8 @@ bool update_subgroup_trigger_merge_check(struct update_subgroup *subgrp,
 		return false;
 
 	subgrp->t_merge_check = NULL;
-	thread_add_timer_msec(bm->master, update_subgroup_merge_check_thread_cb,
-			      subgrp, 0, &subgrp->t_merge_check);
+	event_add_timer_msec(bm->master, update_subgroup_merge_check_thread_cb,
+			     subgrp, 0, &subgrp->t_merge_check);
 
 	SUBGRP_INCR_STAT(subgrp, merge_checks_triggered);
 
@@ -1949,7 +1959,7 @@ void update_group_adjust_peer(struct peer_af *paf)
 		return;
 
 	peer = PAF_PEER(paf);
-	if (!peer_established(peer)) {
+	if (!peer_established(peer->connection)) {
 		return;
 	}
 
@@ -1962,15 +1972,8 @@ void update_group_adjust_peer(struct peer_af *paf)
 	}
 
 	updgrp = update_group_find(paf);
-	if (!updgrp) {
+	if (!updgrp)
 		updgrp = update_group_create(paf);
-		if (!updgrp) {
-			flog_err(EC_BGP_UPDGRP_CREATE,
-				 "couldn't create update group for peer %s",
-				 paf->peer->host);
-			return;
-		}
-	}
 
 	old_subgrp = paf->subgroup;
 
@@ -1993,11 +1996,8 @@ void update_group_adjust_peer(struct peer_af *paf)
 	}
 
 	subgrp = update_subgroup_find(updgrp, paf);
-	if (!subgrp) {
+	if (!subgrp)
 		subgrp = update_subgroup_create(updgrp);
-		if (!subgrp)
-			return;
-	}
 
 	update_subgroup_add_peer(subgrp, paf, 1);
 	if (BGP_DEBUG(update_groups, UPDATE_GROUPS))
@@ -2014,13 +2014,13 @@ int update_group_adjust_soloness(struct peer *peer, int set)
 
 	if (!CHECK_FLAG(peer->sflags, PEER_STATUS_GROUP)) {
 		peer_lonesoul_or_not(peer, set);
-		if (peer_established(peer))
+		if (peer_established(peer->connection))
 			bgp_announce_route_all(peer);
 	} else {
 		group = peer->group;
 		for (ALL_LIST_ELEMENTS(group->peer, node, nnode, peer)) {
 			peer_lonesoul_or_not(peer, set);
-			if (peer_established(peer))
+			if (peer_established(peer->connection))
 				bgp_announce_route_all(peer);
 		}
 	}
@@ -2108,16 +2108,15 @@ update_group_default_originate_route_map_walkcb(struct update_group *updgrp,
 	return UPDWALK_CONTINUE;
 }
 
-void update_group_refresh_default_originate_route_map(struct thread *thread)
+void update_group_refresh_default_originate_route_map(struct event *thread)
 {
 	struct bgp *bgp;
 	char reason[] = "refresh default-originate route-map";
 
-	bgp = THREAD_ARG(thread);
+	bgp = EVENT_ARG(thread);
 	update_group_walk(bgp, update_group_default_originate_route_map_walkcb,
 			  reason);
-	THREAD_OFF(bgp->t_rmap_def_originate_eval);
-	bgp_unlock(bgp);
+	EVENT_OFF(bgp->t_rmap_def_originate_eval);
 }
 
 /*
@@ -2214,12 +2213,15 @@ void subgroup_trigger_write(struct update_subgroup *subgrp)
 	 * the subgroup output queue into their own output queue. This action
 	 * will trigger a write job on the I/O thread.
 	 */
-	SUBGRP_FOREACH_PEER (subgrp, paf)
-		if (peer_established(paf->peer))
-			thread_add_timer_msec(
-				bm->master, bgp_generate_updgrp_packets,
-				paf->peer, 0,
-				&paf->peer->t_generate_updgrp_packets);
+	SUBGRP_FOREACH_PEER (subgrp, paf) {
+		struct peer_connection *connection = paf->peer->connection;
+
+		if (peer_established(connection))
+			event_add_timer_msec(bm->master,
+					     bgp_generate_updgrp_packets,
+					     connection, 0,
+					     &connection->t_generate_updgrp_packets);
+	}
 }
 
 int update_group_clear_update_dbg(struct update_group *updgrp, void *arg)
