@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  *
  * Copyright 2009-2016, LabN Consulting, L.L.C.
  *
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /*
@@ -24,11 +11,9 @@
 
 /* TBD remove unneeded includes */
 
-#include <errno.h>
-
 #include "lib/zebra.h"
 #include "lib/prefix.h"
-#include "lib/table.h"
+#include "lib/agg_table.h"
 #include "lib/vty.h"
 #include "lib/memory.h"
 #include "lib/log.h"
@@ -68,10 +53,10 @@ static void rfapiMonitorEthDetachImport(struct bgp *bgp,
 /*
  * Debug function, special case
  */
-void rfapiMonitorEthSlCheck(struct route_node *rn, const char *tag1,
+void rfapiMonitorEthSlCheck(struct agg_node *rn, const char *tag1,
 			    const char *tag2)
 {
-	struct route_node *rn_saved = NULL;
+	struct agg_node *rn_saved = NULL;
 	static struct skiplist *sl_saved = NULL;
 	struct skiplist *sl;
 
@@ -118,12 +103,13 @@ void rfapiMonitorDupCheck(struct bgp *bgp)
 	struct rfapi_descriptor *rfd;
 
 	for (ALL_LIST_ELEMENTS_RO(&bgp->rfapi->descriptors, hnode, rfd)) {
-		struct route_node *mrn;
+		struct agg_node *mrn;
 
 		if (!rfd->mon)
 			continue;
 
-		for (mrn = route_top(rfd->mon); mrn; mrn = route_next(mrn)) {
+		for (mrn = agg_route_top(rfd->mon); mrn;
+		     mrn = agg_route_next(mrn)) {
 			struct rfapi_monitor_vpn *m;
 			for (m = (struct rfapi_monitor_vpn *)(mrn->info); m;
 			     m = m->next)
@@ -132,12 +118,13 @@ void rfapiMonitorDupCheck(struct bgp *bgp)
 	}
 
 	for (ALL_LIST_ELEMENTS_RO(&bgp->rfapi->descriptors, hnode, rfd)) {
-		struct route_node *mrn;
+		struct agg_node *mrn;
 
 		if (!rfd->mon)
 			continue;
 
-		for (mrn = route_top(rfd->mon); mrn; mrn = route_next(mrn)) {
+		for (mrn = agg_route_top(rfd->mon); mrn;
+		     mrn = agg_route_next(mrn)) {
 			struct rfapi_monitor_vpn *m;
 
 			for (m = (struct rfapi_monitor_vpn *)(mrn->info); m;
@@ -158,15 +145,17 @@ void rfapiMonitorCleanCheck(struct bgp *bgp)
 		assert(!rfd->import_table->vpn0_queries[AFI_IP]);
 		assert(!rfd->import_table->vpn0_queries[AFI_IP6]);
 
-		struct route_node *rn;
+		struct agg_node *rn;
 
-		for (rn = route_top(rfd->import_table->imported_vpn[AFI_IP]);
-		     rn; rn = route_next(rn)) {
+		for (rn = agg_route_top(
+			     rfd->import_table->imported_vpn[AFI_IP]);
+		     rn; rn = agg_route_next(rn)) {
 
 			assert(!RFAPI_MONITOR_VPN(rn));
 		}
-		for (rn = route_top(rfd->import_table->imported_vpn[AFI_IP6]);
-		     rn; rn = route_next(rn)) {
+		for (rn = agg_route_top(
+			     rfd->import_table->imported_vpn[AFI_IP6]);
+		     rn; rn = agg_route_next(rn)) {
 
 			assert(!RFAPI_MONITOR_VPN(rn));
 		}
@@ -180,7 +169,7 @@ void rfapiMonitorCheckAttachAllowed(void)
 	assert(!(bgp->rfapi_cfg->flags & BGP_VNC_CONFIG_CALLBACK_DISABLE));
 }
 
-void rfapiMonitorExtraFlush(safi_t safi, struct route_node *rn)
+void rfapiMonitorExtraFlush(safi_t safi, struct agg_node *rn)
 {
 	struct rfapi_it_extra *hie;
 	struct rfapi_monitor_vpn *v;
@@ -202,7 +191,7 @@ void rfapiMonitorExtraFlush(safi_t safi, struct route_node *rn)
 			e_next = e->next;
 			e->next = NULL;
 			XFREE(MTYPE_RFAPI_MONITOR_ENCAP, e);
-			route_unlock_node(rn);
+			agg_unlock_node(rn);
 		}
 		hie->u.encap.e = NULL;
 		break;
@@ -212,48 +201,54 @@ void rfapiMonitorExtraFlush(safi_t safi, struct route_node *rn)
 			v_next = v->next;
 			v->next = NULL;
 			XFREE(MTYPE_RFAPI_MONITOR, e);
-			route_unlock_node(rn);
+			agg_unlock_node(rn);
 		}
 		hie->u.vpn.v = NULL;
 		if (hie->u.vpn.e.source) {
 			while (!skiplist_delete_first(hie->u.vpn.e.source)) {
-				route_unlock_node(rn);
+				agg_unlock_node(rn);
 			}
 			skiplist_free(hie->u.vpn.e.source);
 			hie->u.vpn.e.source = NULL;
-			route_unlock_node(rn);
+			agg_unlock_node(rn);
 		}
 		if (hie->u.vpn.idx_rd) {
-			/* looping through bi->extra->vnc.import.rd is tbd */
+			/* looping through bpi->extra->vnc.import.rd is tbd */
 			while (!skiplist_delete_first(hie->u.vpn.idx_rd)) {
-				route_unlock_node(rn);
+				agg_unlock_node(rn);
 			}
 			skiplist_free(hie->u.vpn.idx_rd);
 			hie->u.vpn.idx_rd = NULL;
-			route_unlock_node(rn);
+			agg_unlock_node(rn);
 		}
 		if (hie->u.vpn.mon_eth) {
 			while (!skiplist_delete_first(hie->u.vpn.mon_eth)) {
-				route_unlock_node(rn);
+				agg_unlock_node(rn);
 			}
 			skiplist_free(hie->u.vpn.mon_eth);
 			hie->u.vpn.mon_eth = NULL;
-			route_unlock_node(rn);
+			agg_unlock_node(rn);
 		}
 		break;
 
-	default:
+	case SAFI_UNSPEC:
+	case SAFI_UNICAST:
+	case SAFI_MULTICAST:
+	case SAFI_EVPN:
+	case SAFI_LABELED_UNICAST:
+	case SAFI_FLOWSPEC:
+	case SAFI_MAX:
 		assert(0);
 	}
 	XFREE(MTYPE_RFAPI_IT_EXTRA, hie);
 	rn->aggregate = NULL;
-	route_unlock_node(rn);
+	agg_unlock_node(rn);
 }
 
 /*
  * If the child lists are empty, release the rfapi_it_extra struct
  */
-void rfapiMonitorExtraPrune(safi_t safi, struct route_node *rn)
+void rfapiMonitorExtraPrune(safi_t safi, struct agg_node *rn)
 {
 	struct rfapi_it_extra *hie;
 
@@ -279,47 +274,53 @@ void rfapiMonitorExtraPrune(safi_t safi, struct route_node *rn)
 				return;
 			skiplist_free(hie->u.vpn.mon_eth);
 			hie->u.vpn.mon_eth = NULL;
-			route_unlock_node(rn); /* uncount skiplist */
+			agg_unlock_node(rn); /* uncount skiplist */
 		}
 		if (hie->u.vpn.e.source) {
 			if (skiplist_count(hie->u.vpn.e.source))
 				return;
 			skiplist_free(hie->u.vpn.e.source);
 			hie->u.vpn.e.source = NULL;
-			route_unlock_node(rn);
+			agg_unlock_node(rn);
 		}
 		if (hie->u.vpn.idx_rd) {
 			if (skiplist_count(hie->u.vpn.idx_rd))
 				return;
 			skiplist_free(hie->u.vpn.idx_rd);
 			hie->u.vpn.idx_rd = NULL;
-			route_unlock_node(rn);
+			agg_unlock_node(rn);
 		}
 		if (hie->u.vpn.mon_eth) {
 			if (skiplist_count(hie->u.vpn.mon_eth))
 				return;
 			skiplist_free(hie->u.vpn.mon_eth);
 			hie->u.vpn.mon_eth = NULL;
-			route_unlock_node(rn);
+			agg_unlock_node(rn);
 		}
 		break;
 
-	default:
+	case SAFI_UNSPEC:
+	case SAFI_UNICAST:
+	case SAFI_MULTICAST:
+	case SAFI_EVPN:
+	case SAFI_LABELED_UNICAST:
+	case SAFI_FLOWSPEC:
+	case SAFI_MAX:
 		assert(0);
 	}
 	XFREE(MTYPE_RFAPI_IT_EXTRA, hie);
 	rn->aggregate = NULL;
-	route_unlock_node(rn);
+	agg_unlock_node(rn);
 }
 
 /*
  * returns locked node
  */
-struct route_node *rfapiMonitorGetAttachNode(struct rfapi_descriptor *rfd,
-					     struct prefix *p)
+struct agg_node *rfapiMonitorGetAttachNode(struct rfapi_descriptor *rfd,
+					   struct prefix *p)
 {
 	afi_t afi;
-	struct route_node *rn;
+	struct agg_node *rn;
 
 	if (RFAPI_0_PREFIX(p)) {
 		assert(1);
@@ -341,19 +342,19 @@ struct route_node *rfapiMonitorGetAttachNode(struct rfapi_descriptor *rfd,
 	 * if a monitor is moved to another node, there must be
 	 * corresponding unlock/locks
 	 */
-	for (rn = route_node_match(rfd->import_table->imported_vpn[afi], p);
+	for (rn = agg_node_match(rfd->import_table->imported_vpn[afi], p);
 	     rn;) {
 
-		struct bgp_info *bi;
+		struct bgp_path_info *bpi;
 		struct prefix pfx_dummy;
 
 		/* TBD update this code to use new valid_interior_count */
-		for (bi = rn->info; bi; bi = bi->next) {
+		for (bpi = rn->info; bpi; bpi = bpi->next) {
 			/*
 			 * If there is a cached ENCAP UN address, it's a usable
 			 * VPN route
 			 */
-			if (bi->extra && bi->extra->vnc.import.un_family) {
+			if (bpi->extra && bpi->extra->vnc.import.un_family) {
 				break;
 			}
 
@@ -362,16 +363,16 @@ struct route_node *rfapiMonitorGetAttachNode(struct rfapi_descriptor *rfd,
 			 * address,
 			 * it's a usable VPN route.
 			 */
-			if (!rfapiGetVncTunnelUnAddr(bi->attr, &pfx_dummy)) {
+			if (!rfapiGetVncTunnelUnAddr(bpi->attr, &pfx_dummy)) {
 				break;
 			}
 		}
-		if (bi)
+		if (bpi)
 			break;
 
-		route_unlock_node(rn);
-		if ((rn = rn->parent)) {
-			route_lock_node(rn);
+		agg_unlock_node(rn);
+		if ((rn = agg_node_parent(rn))) {
+			agg_lock_node(rn);
 		}
 	}
 
@@ -383,8 +384,8 @@ struct route_node *rfapiMonitorGetAttachNode(struct rfapi_descriptor *rfd,
 
 		/* creates default node if none exists, and increments ref count
 		 */
-		rn = route_node_get(rfd->import_table->imported_vpn[afi],
-				    &pfx_default);
+		rn = agg_node_get(rfd->import_table->imported_vpn[afi],
+				  &pfx_default);
 	}
 
 	return rn;
@@ -396,10 +397,10 @@ struct route_node *rfapiMonitorGetAttachNode(struct rfapi_descriptor *rfd,
  * returned (for the benefit of caller which might like to use it
  * to generate an immediate query response).
  */
-static struct route_node *rfapiMonitorAttachImport(struct rfapi_descriptor *rfd,
-						   struct rfapi_monitor_vpn *m)
+static struct agg_node *rfapiMonitorAttachImport(struct rfapi_descriptor *rfd,
+						 struct rfapi_monitor_vpn *m)
 {
-	struct route_node *rn;
+	struct agg_node *rn;
 
 	rfapiMonitorCheckAttachAllowed();
 
@@ -438,7 +439,7 @@ static struct route_node *rfapiMonitorAttachImport(struct rfapi_descriptor *rfd,
  */
 void rfapiMonitorAttachImportHd(struct rfapi_descriptor *rfd)
 {
-	struct route_node *mrn;
+	struct agg_node *mrn;
 
 	if (!rfd->mon) {
 		/*
@@ -447,7 +448,7 @@ void rfapiMonitorAttachImportHd(struct rfapi_descriptor *rfd)
 		return;
 	}
 
-	for (mrn = route_top(rfd->mon); mrn; mrn = route_next(mrn)) {
+	for (mrn = agg_route_top(rfd->mon); mrn; mrn = agg_route_next(mrn)) {
 
 		if (!mrn->info)
 			continue;
@@ -467,11 +468,11 @@ void rfapiMonitorAttachImportHd(struct rfapi_descriptor *rfd)
  * are disabled, this function will not perform a lookup, and the
  * caller will have to do its own lookup.
  */
-struct route_node *
-rfapiMonitorAdd(struct bgp *bgp, struct rfapi_descriptor *rfd, struct prefix *p)
+struct agg_node *rfapiMonitorAdd(struct bgp *bgp, struct rfapi_descriptor *rfd,
+				 struct prefix *p)
 {
 	struct rfapi_monitor_vpn *m;
-	struct route_node *rn;
+	struct agg_node *rn;
 
 	/*
 	 * Initialize nve's monitor list if needed
@@ -480,15 +481,15 @@ rfapiMonitorAdd(struct bgp *bgp, struct rfapi_descriptor *rfd, struct prefix *p)
 	 * or be 0/0 so they won't get mixed up.
 	 */
 	if (!rfd->mon) {
-		rfd->mon = route_table_init();
+		rfd->mon = agg_table_init();
 	}
-	rn = route_node_get(rfd->mon, p);
+	rn = agg_node_get(rfd->mon, p);
 	if (rn->info) {
 		/*
 		 * received this query before, no further action needed
 		 */
 		rfapiMonitorTimerRestart((struct rfapi_monitor_vpn *)rn->info);
-		route_unlock_node(rn);
+		agg_unlock_node(rn);
 		return NULL;
 	}
 
@@ -575,7 +576,7 @@ rfapiMonitorDetachImport(struct rfapi_monitor_vpn *m)
 						this->next;
 				}
 				RFAPI_CHECK_REFCOUNT(m->node, SAFI_MPLS_VPN, 1);
-				route_unlock_node(m->node);
+				agg_unlock_node(m->node);
 			}
 			m->node = NULL;
 		}
@@ -586,12 +587,12 @@ rfapiMonitorDetachImport(struct rfapi_monitor_vpn *m)
 
 void rfapiMonitorDetachImportHd(struct rfapi_descriptor *rfd)
 {
-	struct route_node *rn;
+	struct agg_node *rn;
 
 	if (!rfd->mon)
 		return;
 
-	for (rn = route_top(rfd->mon); rn; rn = route_next(rn)) {
+	for (rn = agg_route_top(rfd->mon); rn; rn = agg_route_next(rn)) {
 		if (rn->info) {
 			rfapiMonitorDetachImport(
 				(struct rfapi_monitor_vpn *)(rn->info));
@@ -602,11 +603,11 @@ void rfapiMonitorDetachImportHd(struct rfapi_descriptor *rfd)
 void rfapiMonitorDel(struct bgp *bgp, struct rfapi_descriptor *rfd,
 		     struct prefix *p)
 {
-	struct route_node *rn;
+	struct agg_node *rn;
 	struct rfapi_monitor_vpn *m;
 
 	assert(rfd->mon);
-	rn = route_node_get(rfd->mon, p); /* locks node */
+	rn = agg_node_get(rfd->mon, p); /* locks node */
 	m = rn->info;
 
 	assert(m);
@@ -618,18 +619,15 @@ void rfapiMonitorDel(struct bgp *bgp, struct rfapi_descriptor *rfd,
 		rfapiMonitorDetachImport(m);
 	}
 
-	if (m->timer) {
-		thread_cancel(m->timer);
-		m->timer = NULL;
-	}
+	EVENT_OFF(m->timer);
 
 	/*
 	 * remove from rfd list
 	 */
 	XFREE(MTYPE_RFAPI_MONITOR, m);
 	rn->info = NULL;
-	route_unlock_node(rn); /* undo original lock when created */
-	route_unlock_node(rn); /* undo lock in route_node_get */
+	agg_unlock_node(rn); /* undo original lock when created */
+	agg_unlock_node(rn); /* undo lock in agg_node_get */
 
 	--rfd->monitor_count;
 	--bgp->rfapi->monitor_count;
@@ -640,7 +638,7 @@ void rfapiMonitorDel(struct bgp *bgp, struct rfapi_descriptor *rfd,
  */
 int rfapiMonitorDelHd(struct rfapi_descriptor *rfd)
 {
-	struct route_node *rn;
+	struct agg_node *rn;
 	struct bgp *bgp;
 	int count = 0;
 
@@ -649,7 +647,8 @@ int rfapiMonitorDelHd(struct rfapi_descriptor *rfd)
 	bgp = bgp_get_default();
 
 	if (rfd->mon) {
-		for (rn = route_top(rfd->mon); rn; rn = route_next(rn)) {
+		for (rn = agg_route_top(rfd->mon); rn;
+		     rn = agg_route_next(rn)) {
 			struct rfapi_monitor_vpn *m;
 			if ((m = rn->info)) {
 				if (!(bgp->rfapi_cfg->flags
@@ -657,21 +656,18 @@ int rfapiMonitorDelHd(struct rfapi_descriptor *rfd)
 					rfapiMonitorDetachImport(m);
 				}
 
-				if (m->timer) {
-					thread_cancel(m->timer);
-					m->timer = NULL;
-				}
+				EVENT_OFF(m->timer);
 
 				XFREE(MTYPE_RFAPI_MONITOR, m);
 				rn->info = NULL;
-				route_unlock_node(rn); /* undo original lock
+				agg_unlock_node(rn); /* undo original lock
 							  when created */
 				++count;
 				--rfd->monitor_count;
 				--bgp->rfapi->monitor_count;
 			}
 		}
-		route_table_finish(rfd->mon);
+		agg_table_finish(rfd->mon);
 		rfd->mon = NULL;
 	}
 
@@ -694,10 +690,7 @@ int rfapiMonitorDelHd(struct rfapi_descriptor *rfd)
 #endif
 			}
 
-			if (mon_eth->timer) {
-				thread_cancel(mon_eth->timer);
-				mon_eth->timer = NULL;
-			}
+			EVENT_OFF(mon_eth->timer);
 
 			/*
 			 * remove from rfd list
@@ -737,35 +730,30 @@ void rfapiMonitorResponseRemovalOn(struct bgp *bgp)
 	bgp->rfapi_cfg->flags &= ~BGP_VNC_CONFIG_RESPONSE_REMOVAL_DISABLE;
 }
 
-static int rfapiMonitorTimerExpire(struct thread *t)
+static void rfapiMonitorTimerExpire(struct event *t)
 {
-	struct rfapi_monitor_vpn *m = t->arg;
+	struct rfapi_monitor_vpn *m = EVENT_ARG(t);
 
 	/* forget reference to thread, it's gone */
 	m->timer = NULL;
 
 	/* delete the monitor */
 	rfapiMonitorDel(bgp_get_default(), m->rfd, &m->p);
-
-	return 0;
 }
 
 static void rfapiMonitorTimerRestart(struct rfapi_monitor_vpn *m)
 {
-	if (m->timer) {
-		unsigned long remain = thread_timer_remain_second(m->timer);
+	unsigned long remain = event_timer_remain_second(m->timer);
 
-		/* unexpected case, but avoid wraparound problems below */
-		if (remain > m->rfd->response_lifetime)
-			return;
+	/* unexpected case, but avoid wraparound problems below */
+	if (remain > m->rfd->response_lifetime)
+		return;
 
-		/* don't restart if we just restarted recently */
-		if (m->rfd->response_lifetime - remain < 2)
-			return;
+	/* don't restart if we just restarted recently */
+	if (m->rfd->response_lifetime - remain < 2)
+		return;
 
-		thread_cancel(m->timer);
-		m->timer = NULL;
-	}
+	EVENT_OFF(m->timer);
 
 	{
 		char buf[BUFSIZ];
@@ -775,9 +763,9 @@ static void rfapiMonitorTimerRestart(struct rfapi_monitor_vpn *m)
 			rfapi_ntop(m->p.family, m->p.u.val, buf, BUFSIZ),
 			m->rfd->response_lifetime);
 	}
-	m->timer = NULL;
-	thread_add_timer(bm->master, rfapiMonitorTimerExpire, m,
-			 m->rfd->response_lifetime, &m->timer);
+
+	event_add_timer(bm->master, rfapiMonitorTimerExpire, m,
+			m->rfd->response_lifetime, &m->timer);
 }
 
 /*
@@ -786,9 +774,10 @@ static void rfapiMonitorTimerRestart(struct rfapi_monitor_vpn *m)
  * been responsible for the response, i.e., any monitors for
  * the exact prefix or a parent of it.
  */
-void rfapiMonitorTimersRestart(struct rfapi_descriptor *rfd, struct prefix *p)
+void rfapiMonitorTimersRestart(struct rfapi_descriptor *rfd,
+			       const struct prefix *p)
 {
-	struct route_node *rn;
+	struct agg_node *rn;
 
 	if (AF_ETHERNET == p->family) {
 		struct rfapi_monitor_eth *mon_eth;
@@ -812,14 +801,17 @@ void rfapiMonitorTimersRestart(struct rfapi_descriptor *rfd, struct prefix *p)
 		}
 
 	} else {
-		for (rn = route_top(rfd->mon); rn; rn = route_next(rn)) {
+		for (rn = agg_route_top(rfd->mon); rn;
+		     rn = agg_route_next(rn)) {
 			struct rfapi_monitor_vpn *m;
+			const struct prefix *p_node;
 
 			if (!((m = rn->info)))
 				continue;
 
+			p_node = agg_node_get_prefix(m->node);
 			/* NB order of test is significant ! */
-			if (!m->node || prefix_match(&m->node->p, p)) {
+			if (!m->node || prefix_match(p_node, p)) {
 				rfapiMonitorTimerRestart(m);
 			}
 		}
@@ -831,16 +823,14 @@ void rfapiMonitorTimersRestart(struct rfapi_descriptor *rfd, struct prefix *p)
  * rfapiRibUpdatePendingNode with this node and all corresponding NVEs.
  */
 void rfapiMonitorItNodeChanged(
-	struct rfapi_import_table *import_table, struct route_node *it_node,
+	struct rfapi_import_table *import_table, struct agg_node *it_node,
 	struct rfapi_monitor_vpn *monitor_list) /* for base it node, NULL=all */
 {
 	struct skiplist *nves_seen;
-	struct route_node *rn = it_node;
+	struct agg_node *rn = it_node;
 	struct bgp *bgp = bgp_get_default();
-	afi_t afi = family2afi(rn->p.family);
-#if DEBUG_L2_EXTRA
-	char buf_prefix[PREFIX_STRLEN];
-#endif
+	const struct prefix *p = agg_node_get_prefix(rn);
+	afi_t afi = family2afi(p->family);
 
 	assert(bgp);
 	assert(import_table);
@@ -848,9 +838,8 @@ void rfapiMonitorItNodeChanged(
 	nves_seen = skiplist_new(0, NULL, NULL);
 
 #if DEBUG_L2_EXTRA
-	prefix2str(&it_node->p, buf_prefix, sizeof(buf_prefix));
-	vnc_zlog_debug_verbose("%s: it=%p, it_node=%p, it_node->prefix=%s",
-			       __func__, import_table, it_node, buf_prefix);
+	vnc_zlog_debug_verbose("%s: it=%p, it_node=%p, it_node->prefix=%pFX",
+			       __func__, import_table, it_node, &it_node->p);
 #endif
 
 	if (AFI_L2VPN == afi) {
@@ -862,10 +851,9 @@ void rfapiMonitorItNodeChanged(
 		if ((sl = RFAPI_MONITOR_ETH(rn))) {
 
 			for (cursor = NULL,
-			    rc = skiplist_next(sl, NULL, (void **)&m,
-					       (void **)&cursor);
+			    rc = skiplist_next(sl, NULL, (void **)&m, &cursor);
 			     !rc; rc = skiplist_next(sl, NULL, (void **)&m,
-						     (void **)&cursor)) {
+						     &cursor)) {
 
 				if (skiplist_search(nves_seen, m->rfd, NULL)) {
 					/*
@@ -909,7 +897,8 @@ void rfapiMonitorItNodeChanged(
 			 * to them
 			 * because we haven't sent them an initial route.
 			 */
-			if (!rn->parent && !rn->info && it_node->parent)
+			if (!agg_node_parent(rn) && !rn->info
+			    && it_node->parent)
 				break;
 
 			for (; m; m = m->next) {
@@ -926,17 +915,10 @@ void rfapiMonitorItNodeChanged(
 					assert(!skiplist_insert(nves_seen,
 								m->rfd, NULL));
 
-					char buf_attach_pfx[PREFIX_STRLEN];
-					char buf_target_pfx[PREFIX_STRLEN];
-
-					prefix2str(&m->node->p, buf_attach_pfx,
-						   sizeof(buf_attach_pfx));
-					prefix2str(&m->p, buf_target_pfx,
-						   sizeof(buf_target_pfx));
 					vnc_zlog_debug_verbose(
-						"%s: update rfd %p attached to pfx %s (targ=%s)",
-						__func__, m->rfd,
-						buf_attach_pfx, buf_target_pfx);
+						"%s: update rfd %p attached to pfx %pRN (targ=%pFX)",
+						__func__, m->rfd, m->node,
+						&m->p);
 
 					/*
 					 * update its RIB
@@ -947,7 +929,7 @@ void rfapiMonitorItNodeChanged(
 						m->rfd->response_lifetime);
 				}
 			}
-			rn = rn->parent;
+			rn = agg_node_parent(rn);
 			if (rn)
 				m = RFAPI_MONITOR_VPN(rn);
 		} while (rn);
@@ -1023,8 +1005,7 @@ void rfapiMonitorItNodeChanged(
  * omit old node and its subtree
  */
 void rfapiMonitorMovedUp(struct rfapi_import_table *import_table,
-			 struct route_node *old_node,
-			 struct route_node *new_node,
+			 struct agg_node *old_node, struct agg_node *new_node,
 			 struct rfapi_monitor_vpn *monitor_list)
 {
 	struct bgp *bgp = bgp_get_default();
@@ -1055,9 +1036,9 @@ void rfapiMonitorMovedUp(struct rfapi_import_table *import_table,
 	}
 }
 
-static int rfapiMonitorEthTimerExpire(struct thread *t)
+static void rfapiMonitorEthTimerExpire(struct event *t)
 {
-	struct rfapi_monitor_eth *m = t->arg;
+	struct rfapi_monitor_eth *m = EVENT_ARG(t);
 
 	/* forget reference to thread, it's gone */
 	m->timer = NULL;
@@ -1066,25 +1047,21 @@ static int rfapiMonitorEthTimerExpire(struct thread *t)
 	rfapiMonitorEthDel(bgp_get_default(), m->rfd, &m->macaddr,
 			   m->logical_net_id);
 
-	return 0;
 }
 
 static void rfapiMonitorEthTimerRestart(struct rfapi_monitor_eth *m)
 {
-	if (m->timer) {
-		unsigned long remain = thread_timer_remain_second(m->timer);
+	unsigned long remain = event_timer_remain_second(m->timer);
 
-		/* unexpected case, but avoid wraparound problems below */
-		if (remain > m->rfd->response_lifetime)
-			return;
+	/* unexpected case, but avoid wraparound problems below */
+	if (remain > m->rfd->response_lifetime)
+		return;
 
-		/* don't restart if we just restarted recently */
-		if (m->rfd->response_lifetime - remain < 2)
-			return;
+	/* don't restart if we just restarted recently */
+	if (m->rfd->response_lifetime - remain < 2)
+		return;
 
-		thread_cancel(m->timer);
-		m->timer = NULL;
-	}
+	EVENT_OFF(m->timer);
 
 	{
 		char buf[BUFSIZ];
@@ -1094,15 +1071,15 @@ static void rfapiMonitorEthTimerRestart(struct rfapi_monitor_eth *m)
 			rfapiEthAddr2Str(&m->macaddr, buf, BUFSIZ),
 			m->rfd->response_lifetime);
 	}
-	m->timer = NULL;
-	thread_add_timer(bm->master, rfapiMonitorEthTimerExpire, m,
-			 m->rfd->response_lifetime, &m->timer);
+
+	event_add_timer(bm->master, rfapiMonitorEthTimerExpire, m,
+			m->rfd->response_lifetime, &m->timer);
 }
 
-static int mon_eth_cmp(void *a, void *b)
+static int mon_eth_cmp(const void *a, const void *b)
 {
-	struct rfapi_monitor_eth *m1;
-	struct rfapi_monitor_eth *m2;
+	const struct rfapi_monitor_eth *m1;
+	const struct rfapi_monitor_eth *m2;
 
 	int i;
 
@@ -1125,7 +1102,7 @@ static int mon_eth_cmp(void *a, void *b)
 
 static void rfapiMonitorEthAttachImport(
 	struct rfapi_import_table *it,
-	struct route_node *rn,	 /* it node attach point if non-0 */
+	struct agg_node *rn,	   /* it node attach point if non-0 */
 	struct rfapi_monitor_eth *mon) /* monitor struct to attach */
 {
 	struct skiplist *sl;
@@ -1162,7 +1139,7 @@ static void rfapiMonitorEthAttachImport(
 	if (!sl) {
 		sl = RFAPI_MONITOR_ETH_W_ALLOC(rn) =
 			skiplist_new(0, NULL, NULL);
-		route_lock_node(rn); /* count skiplist mon_eth */
+		agg_lock_node(rn); /* count skiplist mon_eth */
 	}
 
 #if DEBUG_L2_EXTRA
@@ -1175,7 +1152,7 @@ static void rfapiMonitorEthAttachImport(
 	assert(!rc);
 
 	/* count eth monitor */
-	route_lock_node(rn);
+	agg_lock_node(rn);
 }
 
 /*
@@ -1202,7 +1179,7 @@ static void rfapiMonitorEthAttachImportHd(struct bgp *bgp,
 
 		struct rfapi_import_table *it;
 		struct prefix pfx_mac_buf;
-		struct route_node *rn;
+		struct agg_node *rn;
 
 		it = rfapiMacImportTableGet(bgp, mon->logical_net_id);
 		assert(it);
@@ -1212,7 +1189,7 @@ static void rfapiMonitorEthAttachImportHd(struct bgp *bgp,
 		pfx_mac_buf.prefixlen = 48;
 		pfx_mac_buf.u.prefix_eth = mon->macaddr;
 
-		rn = route_node_get(it->imported_vpn[AFI_L2VPN], &pfx_mac_buf);
+		rn = agg_node_get(it->imported_vpn[AFI_L2VPN], &pfx_mac_buf);
 		assert(rn);
 
 		(void)rfapiMonitorEthAttachImport(it, rn, mon);
@@ -1226,7 +1203,7 @@ static void rfapiMonitorEthDetachImport(
 	struct rfapi_import_table *it;
 	struct prefix pfx_mac_buf;
 	struct skiplist *sl;
-	struct route_node *rn;
+	struct agg_node *rn;
 	int rc;
 
 	it = rfapiMacImportTableGet(bgp, mon->logical_net_id);
@@ -1262,24 +1239,18 @@ static void rfapiMonitorEthDetachImport(
 	pfx_mac_buf.prefixlen = 48;
 	pfx_mac_buf.u.prefix_eth = mon->macaddr;
 
-	rn = route_node_get(it->imported_vpn[AFI_L2VPN], &pfx_mac_buf);
+	rn = agg_node_get(it->imported_vpn[AFI_L2VPN], &pfx_mac_buf);
 	assert(rn);
 
-#if DEBUG_L2_EXTRA
-	char buf_prefix[PREFIX_STRLEN];
-
-	prefix2str(&rn->p, buf_prefix, sizeof(buf_prefix));
-#endif
-
 	/*
 	 * Get sl to detach from
 	 */
 	sl = RFAPI_MONITOR_ETH(rn);
 #if DEBUG_L2_EXTRA
 	vnc_zlog_debug_verbose(
-		"%s: it=%p, rn=%p, rn->lock=%d, sl=%p, pfx=%s, LNI=%d, detaching eth mon %p",
-		__func__, it, rn, rn->lock, sl, buf_prefix, mon->logical_net_id,
-		mon);
+		"%s: it=%p, rn=%p, rn->lock=%d, sl=%p, pfx=%pFX, LNI=%d, detaching eth mon %p",
+		__func__, it, rn, rn->lock, sl, agg_node_get_prefix(rn),
+		mon->logical_net_id, mon);
 #endif
 	assert(sl);
 
@@ -1288,19 +1259,19 @@ static void rfapiMonitorEthDetachImport(
 	assert(!rc);
 
 	/* uncount eth monitor */
-	route_unlock_node(rn);
+	agg_unlock_node(rn);
 }
 
-struct route_node *rfapiMonitorEthAdd(struct bgp *bgp,
-				      struct rfapi_descriptor *rfd,
-				      struct ethaddr *macaddr,
-				      uint32_t logical_net_id)
+struct agg_node *rfapiMonitorEthAdd(struct bgp *bgp,
+				    struct rfapi_descriptor *rfd,
+				    struct ethaddr *macaddr,
+				    uint32_t logical_net_id)
 {
 	int rc;
 	struct rfapi_monitor_eth mon_buf;
 	struct rfapi_monitor_eth *val;
 	struct rfapi_import_table *it;
-	struct route_node *rn = NULL;
+	struct agg_node *rn = NULL;
 	struct prefix pfx_mac_buf;
 
 	if (!rfd->mon_eth) {
@@ -1323,7 +1294,7 @@ struct route_node *rfapiMonitorEthAdd(struct bgp *bgp,
 	pfx_mac_buf.u.prefix_eth = *macaddr;
 
 	if (!RFAPI_0_ETHERADDR(macaddr)) {
-		rn = route_node_get(it->imported_vpn[AFI_L2VPN], &pfx_mac_buf);
+		rn = agg_node_get(it->imported_vpn[AFI_L2VPN], &pfx_mac_buf);
 		assert(rn);
 	}
 
@@ -1428,10 +1399,7 @@ void rfapiMonitorEthDel(struct bgp *bgp, struct rfapi_descriptor *rfd,
 		rfapiMonitorEthDetachImport(bgp, val);
 	}
 
-	if (val->timer) {
-		thread_cancel(val->timer);
-		val->timer = NULL;
-	}
+	EVENT_OFF(val->timer);
 
 	/*
 	 * remove from rfd list
@@ -1453,8 +1421,8 @@ void rfapiMonitorCallbacksOff(struct bgp *bgp)
 {
 	struct rfapi_import_table *it;
 	afi_t afi;
-	struct route_table *rt;
-	struct route_node *rn;
+	struct agg_table *rt;
+	struct agg_node *rn;
 	void *cursor;
 	int rc;
 	struct rfapi *h = bgp->rfapi;
@@ -1485,7 +1453,8 @@ void rfapiMonitorCallbacksOff(struct bgp *bgp)
 
 			rt = it->imported_vpn[afi];
 
-			for (rn = route_top(rt); rn; rn = route_next(rn)) {
+			for (rn = agg_route_top(rt); rn;
+			     rn = agg_route_next(rn)) {
 				m = RFAPI_MONITOR_VPN(rn);
 				if (RFAPI_MONITOR_VPN(rn))
 					RFAPI_MONITOR_VPN_W_ALLOC(rn) = NULL;
@@ -1494,7 +1463,7 @@ void rfapiMonitorCallbacksOff(struct bgp *bgp)
 					m->next =
 						NULL; /* gratuitous safeness */
 					m->node = NULL;
-					route_unlock_node(rn); /* uncount */
+					agg_unlock_node(rn); /* uncount */
 				}
 			}
 
@@ -1531,12 +1500,12 @@ void rfapiMonitorCallbacksOff(struct bgp *bgp)
 		 * Find non-0 monitors (i.e., actual addresses, not FTD
 		 * monitors)
 		 */
-		for (rn = route_top(rt); rn; rn = route_next(rn)) {
+		for (rn = agg_route_top(rt); rn; rn = agg_route_next(rn)) {
 			struct skiplist *sl;
 
 			sl = RFAPI_MONITOR_ETH(rn);
 			while (!skiplist_delete_first(sl)) {
-				route_unlock_node(rn); /* uncount monitor */
+				agg_unlock_node(rn); /* uncount monitor */
 			}
 		}
 
