@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  *
  * Copyright 2009-2016, LabN Consulting, L.L.C.
  *
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /*
@@ -108,9 +95,9 @@ static int is_host_prefix(const struct prefix *p)
 {
 	switch (p->family) {
 	case AF_INET:
-		return (p->prefixlen == 32);
+		return (p->prefixlen == IPV4_MAX_BITLEN);
 	case AF_INET6:
-		return (p->prefixlen == 128);
+		return (p->prefixlen == IPV6_MAX_BITLEN);
 	}
 	return 0;
 }
@@ -206,19 +193,11 @@ static void print_rhn_list(const char *tag1, const char *tag2)
 
 	/* XXX uses secret knowledge of skiplist structure */
 	for (p = sl->header->forward[0]; p; p = p->forward[0]) {
-		char kbuf[PREFIX_STRLEN];
-		char hbuf[PREFIX_STRLEN];
-		char ubuf[PREFIX_STRLEN];
-
 		pb = p->value;
 
-		prefix2str(p->key, kbuf, sizeof(kbuf));
-		prefix2str(&pb->hpfx, hbuf, sizeof(hbuf));
-		prefix2str(&pb->upfx, ubuf, sizeof(ubuf));
-
 		vnc_zlog_debug_verbose(
-			"RHN Entry %d (q=%p): kpfx=%s, upfx=%s, hpfx=%s, ubpi=%p",
-			++count, p, kbuf, ubuf, hbuf, pb->ubpi);
+			"RHN Entry %d (q=%p): kpfx=%pFX, upfx=%pFX, hpfx=%pFX, ubpi=%p",
+			++count, p, p->key, &pb->upfx, &pb->hpfx, pb->ubpi);
 	}
 }
 #endif
@@ -246,7 +225,7 @@ static void vnc_rhnck(char *tag)
 		struct prefix pfx_orig_nexthop;
 
 		memset(&pfx_orig_nexthop, 0,
-		       sizeof(struct prefix)); /* keep valgrind happy */
+		       sizeof(pfx_orig_nexthop)); /* keep valgrind happy */
 
 		pkey = p->key;
 		pb = p->value;
@@ -260,15 +239,9 @@ static void vnc_rhnck(char *tag)
 		 * pfx */
 		assert(!vnc_prefix_cmp(&pb->hpfx, pkey));
 		if (vnc_prefix_cmp(&pb->hpfx, &pfx_orig_nexthop)) {
-			char str_onh[PREFIX_STRLEN];
-			char str_nve_pfx[PREFIX_STRLEN];
-
-			prefix2str(&pfx_orig_nexthop, str_onh, sizeof(str_onh));
-			prefix2str(&pb->hpfx, str_nve_pfx, sizeof(str_nve_pfx));
-
 			vnc_zlog_debug_verbose(
-				"%s: %s: FATAL: resolve_nve_nexthop list item bpi nexthop %s != nve pfx %s",
-				__func__, tag, str_onh, str_nve_pfx);
+				"%s: %s: FATAL: resolve_nve_nexthop list item bpi nexthop %pFX != nve pfx %pFX",
+				__func__, tag, &pfx_orig_nexthop, &pb->hpfx);
 			assert(0);
 		}
 	}
@@ -317,7 +290,7 @@ static int process_unicast_route(struct bgp *bgp,		 /* in */
 	struct prefix pfx_orig_nexthop;
 
 	memset(&pfx_orig_nexthop, 0,
-	       sizeof(struct prefix)); /* keep valgrind happy */
+	       sizeof(pfx_orig_nexthop)); /* keep valgrind happy */
 
 	/*
 	 * prefix list check
@@ -360,7 +333,7 @@ static int process_unicast_route(struct bgp *bgp,		 /* in */
 	 * must be freed before we return. It's easier to put it after
 	 * all of the possible returns above.
 	 */
-	memset(&hattr, 0, sizeof(struct attr));
+	memset(&hattr, 0, sizeof(hattr));
 	/* hattr becomes a ghost attr */
 	hattr = *attr;
 
@@ -371,7 +344,7 @@ static int process_unicast_route(struct bgp *bgp,		 /* in */
 		memset(&info, 0, sizeof(info));
 		info.peer = peer;
 		info.attr = &hattr;
-		ret = route_map_apply(rmap, prefix, RMAP_BGP, &info);
+		ret = route_map_apply(rmap, prefix, &info);
 		if (ret == RMAP_DENYMATCH) {
 			bgp_attr_flush(&hattr);
 			vnc_zlog_debug_verbose(
@@ -387,8 +360,8 @@ static int process_unicast_route(struct bgp *bgp,		 /* in */
 	 */
 	rfapiUnicastNexthop2Prefix(afi, &hattr, unicast_nexthop);
 
-	if (hattr.ecommunity)
-		*ecom = ecommunity_dup(hattr.ecommunity);
+	if (bgp_attr_get_ecommunity(&hattr))
+		*ecom = ecommunity_dup(bgp_attr_get_ecommunity(&hattr));
 	else
 		*ecom = ecommunity_new();
 
@@ -482,7 +455,7 @@ static void vnc_import_bgp_add_route_mode_resolve_nve_one_bi(
 		plifetime = &lifetime;
 	}
 
-	encaptlvs = bpi->attr->vnc_subtlvs;
+	encaptlvs = bgp_attr_get_vnc_subtlvs(bpi->attr);
 	if (bpi->attr->encap_tunneltype != BGP_ENCAP_TYPE_RESERVED
 	    && bpi->attr->encap_tunneltype != BGP_ENCAP_TYPE_MPLS) {
 		opt = &optary[cur_opt++];
@@ -494,8 +467,8 @@ static void vnc_import_bgp_add_route_mode_resolve_nve_one_bi(
 
 	struct ecommunity *new_ecom = ecommunity_dup(ecom);
 
-	if (bpi->attr->ecommunity)
-		ecommunity_merge(new_ecom, bpi->attr->ecommunity);
+	if (bgp_attr_get_ecommunity(bpi->attr))
+		ecommunity_merge(new_ecom, bgp_attr_get_ecommunity(bpi->attr));
 
 	if (bpi->extra)
 		label = decode_label(&bpi->extra->label[0]);
@@ -523,23 +496,17 @@ static void vnc_import_bgp_add_route_mode_resolve_nve_one_rd(
 	uint32_t *med,		     /* NULL = no med */
 	struct prefix *ubpi_nexthop) /* unicast nexthop */
 {
-	struct bgp_node *bn;
+	struct bgp_dest *bd;
 	struct bgp_path_info *bpi;
 
 	if (!table_rd)
 		return;
 
-	{
-		char str_nh[PREFIX_STRLEN];
-
-		prefix2str(ubpi_nexthop, str_nh, sizeof(str_nh));
-
-		vnc_zlog_debug_verbose("%s: ubpi_nexthop=%s", __func__, str_nh);
-	}
+	vnc_zlog_debug_verbose("%s: ubpi_nexthop=%pFX", __func__, ubpi_nexthop);
 
 	/* exact match */
-	bn = bgp_node_lookup(table_rd, ubpi_nexthop);
-	if (!bn) {
+	bd = bgp_node_lookup(table_rd, ubpi_nexthop);
+	if (!bd) {
 		vnc_zlog_debug_verbose(
 			"%s: no match in RD's table for ubpi_nexthop",
 			__func__);
@@ -547,14 +514,14 @@ static void vnc_import_bgp_add_route_mode_resolve_nve_one_rd(
 	}
 
 	/* Iterate over bgp_info items at this node */
-	for (bpi = bgp_node_get_bgp_path_info(bn); bpi; bpi = bpi->next) {
+	for (bpi = bgp_dest_get_bgp_path_info(bd); bpi; bpi = bpi->next) {
 
 		vnc_import_bgp_add_route_mode_resolve_nve_one_bi(
 			bgp, afi, bpi, /* VPN bpi */
 			prd, prefix, local_pref, med, ecom);
 	}
 
-	bgp_unlock_node(bn);
+	bgp_dest_unlock_node(bd);
 }
 
 static void vnc_import_bgp_add_route_mode_resolve_nve(
@@ -570,16 +537,13 @@ static void vnc_import_bgp_add_route_mode_resolve_nve(
 	uint32_t *med = NULL;
 
 	struct prefix_bag *pb;
-	struct bgp_node *bnp; /* prd table node */
+	struct bgp_dest *bdp; /* prd table node */
 
 	/*debugging */
 	if (VNC_DEBUG(VERBOSE)) {
-		char str_pfx[PREFIX_STRLEN];
 		char str_nh[PREFIX_STRLEN];
 		struct prefix nh;
 
-		prefix2str(prefix, str_pfx, sizeof(str_pfx));
-
 		nh.prefixlen = 0;
 		rfapiUnicastNexthop2Prefix(afi, info->attr, &nh);
 		if (nh.prefixlen) {
@@ -590,8 +554,8 @@ static void vnc_import_bgp_add_route_mode_resolve_nve(
 		}
 
 		vnc_zlog_debug_verbose(
-			"%s(bgp=%p, unicast prefix=%s, unicast nh=%s)",
-			__func__, bgp, str_pfx, str_nh);
+			"%s(bgp=%p, unicast prefix=%pFX, unicast nh=%s)",
+			__func__, bgp, prefix, str_nh);
 	}
 
 	if (info->type != ZEBRA_ROUTE_BGP) {
@@ -668,18 +632,18 @@ static void vnc_import_bgp_add_route_mode_resolve_nve(
 	 * (exact match, /32). If an exact match is found, call add_vnc_route.
 	 */
 
-	for (bnp = bgp_table_top(bgp->rib[afi][SAFI_MPLS_VPN]); bnp;
-	     bnp = bgp_route_next(bnp)) {
+	for (bdp = bgp_table_top(bgp->rib[afi][SAFI_MPLS_VPN]); bdp;
+	     bdp = bgp_route_next(bdp)) {
 
 		struct bgp_table *table;
 
-		table = bgp_node_get_bgp_table_info(bnp);
+		table = bgp_dest_get_bgp_table_info(bdp);
 
 		if (!table)
 			continue;
 
 		vnc_import_bgp_add_route_mode_resolve_nve_one_rd(
-			(struct prefix_rd *)bgp_node_get_prefix(bnp), table,
+			(struct prefix_rd *)bgp_dest_get_prefix(bdp), table,
 			afi, bgp, prefix, ecom, &local_pref, med,
 			&pfx_unicast_nexthop);
 	}
@@ -713,12 +677,7 @@ static void vnc_import_bgp_add_route_mode_plain(struct bgp *bgp,
 	uint32_t local_pref;
 	uint32_t *med = NULL;
 
-	{
-		char buf[PREFIX_STRLEN];
-
-		prefix2str(prefix, buf, sizeof(buf));
-		vnc_zlog_debug_verbose("%s(prefix=%s) entry", __func__, buf);
-	}
+	vnc_zlog_debug_verbose("%s(prefix=%pFX) entry", __func__, prefix);
 
 	if (!afi) {
 		flog_err(EC_LIB_DEVELOPMENT, "%s: can't get afi of prefix",
@@ -783,12 +742,8 @@ static void vnc_import_bgp_add_route_mode_plain(struct bgp *bgp,
 		ahr_flags |= RFAPI_AHR_NO_TUNNEL_SUBTLV;
 	}
 
-	if (VNC_DEBUG(IMPORT_BGP_ADD_ROUTE)) {
-		char buf[PREFIX_STRLEN];
-
-		prefix2str(vn_pfx, buf, sizeof(buf));
-		vnc_zlog_debug_any("%s vn_pfx=%s", __func__, buf);
-	}
+	if (VNC_DEBUG(IMPORT_BGP_ADD_ROUTE))
+		vnc_zlog_debug_any("%s vn_pfx=%pFX", __func__, vn_pfx);
 
 	/*
 	 * Compute VN address
@@ -805,7 +760,7 @@ static void vnc_import_bgp_add_route_mode_plain(struct bgp *bgp,
 	 * must be freed before we return. It's easier to put it after
 	 * all of the possible returns above.
 	 */
-	memset(&hattr, 0, sizeof(struct attr));
+	memset(&hattr, 0, sizeof(hattr));
 	/* hattr becomes a ghost attr */
 	hattr = *attr;
 
@@ -816,7 +771,7 @@ static void vnc_import_bgp_add_route_mode_plain(struct bgp *bgp,
 		memset(&info, 0, sizeof(info));
 		info.peer = peer;
 		info.attr = &hattr;
-		ret = route_map_apply(rmap, prefix, RMAP_BGP, &info);
+		ret = route_map_apply(rmap, prefix, &info);
 		if (ret == RMAP_DENYMATCH) {
 			bgp_attr_flush(&hattr);
 			vnc_zlog_debug_verbose(
@@ -850,8 +805,8 @@ static void vnc_import_bgp_add_route_mode_plain(struct bgp *bgp,
 		memset(&prd, 0, sizeof(prd));
 		rfapi_set_autord_from_vn(&prd, &vnaddr);
 
-		if (iattr && iattr->ecommunity)
-			ecom = ecommunity_dup(iattr->ecommunity);
+		if (iattr && bgp_attr_get_ecommunity(iattr))
+			ecom = ecommunity_dup(bgp_attr_get_ecommunity(iattr));
 	}
 
 	local_pref = calc_local_pref(iattr, peer);
@@ -879,6 +834,8 @@ static void vnc_import_bgp_add_route_mode_plain(struct bgp *bgp,
 
 	if (ecom)
 		ecommunity_free(&ecom);
+	if (iattr)
+		bgp_attr_unintern(&iattr);
 }
 
 static void vnc_import_bgp_add_route_mode_nvegroup(
@@ -899,12 +856,7 @@ static void vnc_import_bgp_add_route_mode_nvegroup(
 	struct route_map *rmap = NULL;
 	uint32_t local_pref;
 
-	{
-		char buf[PREFIX_STRLEN];
-
-		prefix2str(prefix, buf, sizeof(buf));
-		vnc_zlog_debug_verbose("%s(prefix=%s) entry", __func__, buf);
-	}
+	vnc_zlog_debug_verbose("%s(prefix=%pFX) entry", __func__, prefix);
 
 	assert(rfg);
 
@@ -985,12 +937,8 @@ static void vnc_import_bgp_add_route_mode_nvegroup(
 		vncHDBgpDirect.un_addr = pfx_un.prefix;
 	}
 
-	if (VNC_DEBUG(IMPORT_BGP_ADD_ROUTE)) {
-		char buf[PREFIX_STRLEN];
-
-		prefix2str(vn_pfx, buf, sizeof(buf));
-		vnc_zlog_debug_any("%s vn_pfx=%s", __func__, buf);
-	}
+	if (VNC_DEBUG(IMPORT_BGP_ADD_ROUTE))
+		vnc_zlog_debug_any("%s vn_pfx=%pFX", __func__, vn_pfx);
 
 	/*
 	 * Compute VN address
@@ -1007,7 +955,7 @@ static void vnc_import_bgp_add_route_mode_nvegroup(
 	 * must be freed before we return. It's easier to put it after
 	 * all of the possible returns above.
 	 */
-	memset(&hattr, 0, sizeof(struct attr));
+	memset(&hattr, 0, sizeof(hattr));
 	/* hattr becomes a ghost attr */
 	hattr = *attr;
 
@@ -1018,7 +966,7 @@ static void vnc_import_bgp_add_route_mode_nvegroup(
 		memset(&path, 0, sizeof(path));
 		path.peer = peer;
 		path.attr = &hattr;
-		ret = route_map_apply(rmap, prefix, RMAP_BGP, &path);
+		ret = route_map_apply(rmap, prefix, &path);
 		if (ret == RMAP_DENYMATCH) {
 			bgp_attr_flush(&hattr);
 			vnc_zlog_debug_verbose(
@@ -1056,8 +1004,9 @@ static void vnc_import_bgp_add_route_mode_nvegroup(
 		else
 			ecom = ecommunity_new();
 
-		if (iattr && iattr->ecommunity)
-			ecom = ecommunity_merge(ecom, iattr->ecommunity);
+		if (iattr && bgp_attr_get_ecommunity(iattr))
+			ecom = ecommunity_merge(ecom,
+						bgp_attr_get_ecommunity(iattr));
 	}
 
 	local_pref = calc_local_pref(iattr, peer);
@@ -1083,6 +1032,8 @@ static void vnc_import_bgp_add_route_mode_nvegroup(
 
 	if (ecom)
 		ecommunity_free(&ecom);
+	if (iattr)
+		bgp_attr_unintern(&iattr);
 }
 
 static void vnc_import_bgp_del_route_mode_plain(struct bgp *bgp,
@@ -1114,7 +1065,7 @@ static void vnc_import_bgp_del_route_mode_plain(struct bgp *bgp,
 	vnaddr.addr_family = vn_pfx->family;
 	switch (vn_pfx->family) {
 	case AF_INET:
-		if (vn_pfx->prefixlen != 32) {
+		if (vn_pfx->prefixlen != IPV4_MAX_BITLEN) {
 			vnc_zlog_debug_verbose(
 				"%s: redist VN plen (%d) != 32, skipping",
 				__func__, vn_pfx->prefixlen);
@@ -1124,7 +1075,7 @@ static void vnc_import_bgp_del_route_mode_plain(struct bgp *bgp,
 		break;
 
 	case AF_INET6:
-		if (vn_pfx->prefixlen != 128) {
+		if (vn_pfx->prefixlen != IPV6_MAX_BITLEN) {
 			vnc_zlog_debug_verbose(
 				"%s: redist VN plen (%d) != 128, skipping",
 				__func__, vn_pfx->prefixlen);
@@ -1188,7 +1139,7 @@ static void vnc_import_bgp_del_route_mode_nvegroup(struct bgp *bgp,
 	vnaddr.addr_family = vn_pfx->family;
 	switch (vn_pfx->family) {
 	case AF_INET:
-		if (vn_pfx->prefixlen != 32) {
+		if (vn_pfx->prefixlen != IPV4_MAX_BITLEN) {
 			vnc_zlog_debug_verbose(
 				"%s: redist VN plen (%d) != 32, skipping",
 				__func__, vn_pfx->prefixlen);
@@ -1198,7 +1149,7 @@ static void vnc_import_bgp_del_route_mode_nvegroup(struct bgp *bgp,
 		break;
 
 	case AF_INET6:
-		if (vn_pfx->prefixlen != 128) {
+		if (vn_pfx->prefixlen != IPV6_MAX_BITLEN) {
 			vnc_zlog_debug_verbose(
 				"%s: redist VN plen (%d) != 128, skipping",
 				__func__, vn_pfx->prefixlen);
@@ -1282,23 +1233,18 @@ static void vnc_import_bgp_del_route_mode_resolve_nve_one_rd(
 	const struct prefix *prefix,	   /* unicast prefix */
 	const struct prefix *ubpi_nexthop) /* unicast bpi's nexthop */
 {
-	struct bgp_node *bn;
+	struct bgp_dest *bd;
 	struct bgp_path_info *bpi;
 
 	if (!table_rd)
 		return;
 
-	{
-		char str_nh[PREFIX_STRLEN];
-
-		prefix2str(ubpi_nexthop, str_nh, sizeof(str_nh));
-		vnc_zlog_debug_verbose("%s: ubpi_nexthop=%s", __func__, str_nh);
-	}
+	vnc_zlog_debug_verbose("%s: ubpi_nexthop=%pFX", __func__, ubpi_nexthop);
 
 
 	/* exact match */
-	bn = bgp_node_lookup(table_rd, ubpi_nexthop);
-	if (!bn) {
+	bd = bgp_node_lookup(table_rd, ubpi_nexthop);
+	if (!bd) {
 		vnc_zlog_debug_verbose(
 			"%s: no match in RD's table for ubpi_nexthop",
 			__func__);
@@ -1306,7 +1252,7 @@ static void vnc_import_bgp_del_route_mode_resolve_nve_one_rd(
 	}
 
 	/* Iterate over bgp_info items at this node */
-	for (bpi = bgp_node_get_bgp_path_info(bn); bpi; bpi = bpi->next) {
+	for (bpi = bgp_dest_get_bgp_path_info(bd); bpi; bpi = bpi->next) {
 
 		vnc_import_bgp_del_route_mode_resolve_nve_one_bi(
 			bgp, afi, bpi, /* VPN bpi */
@@ -1314,7 +1260,7 @@ static void vnc_import_bgp_del_route_mode_resolve_nve_one_rd(
 			prefix);       /* unicast route prefix */
 	}
 
-	bgp_unlock_node(bn);
+	bgp_dest_unlock_node(bd);
 }
 
 static void
@@ -1331,7 +1277,7 @@ vnc_import_bgp_del_route_mode_resolve_nve(struct bgp *bgp, afi_t afi,
 	void *cursor;
 	struct skiplist *sl = bgp->rfapi->resolve_nve_nexthop;
 	int rc;
-	struct bgp_node *bnp; /* prd table node */
+	struct bgp_dest *bdp; /* prd table node */
 
 	if (!sl) {
 		vnc_zlog_debug_verbose("%s: no RHN entries, skipping",
@@ -1372,18 +1318,18 @@ vnc_import_bgp_del_route_mode_resolve_nve(struct bgp *bgp, afi_t afi,
 	 * (exact match, /32). If an exact match is found, call add_vnc_route.
 	 */
 
-	for (bnp = bgp_table_top(bgp->rib[afi][SAFI_MPLS_VPN]); bnp;
-	     bnp = bgp_route_next(bnp)) {
+	for (bdp = bgp_table_top(bgp->rib[afi][SAFI_MPLS_VPN]); bdp;
+	     bdp = bgp_route_next(bdp)) {
 
 		struct bgp_table *table;
 
-		table = bgp_node_get_bgp_table_info(bnp);
+		table = bgp_dest_get_bgp_table_info(bdp);
 
 		if (!table)
 			continue;
 
 		vnc_import_bgp_del_route_mode_resolve_nve_one_rd(
-			(struct prefix_rd *)bgp_node_get_prefix(bnp), table,
+			(struct prefix_rd *)bgp_dest_get_prefix(bdp), table,
 			afi, bgp, prefix, &pfx_unicast_nexthop);
 	}
 
@@ -1465,19 +1411,13 @@ void vnc_import_bgp_add_vnc_host_route_mode_resolve_nve(
 		uint32_t local_pref;
 
 		memset(&pfx_unicast_nexthop, 0,
-		       sizeof(struct prefix)); /* keep valgrind happy */
-
-		if (VNC_DEBUG(IMPORT_BGP_ADD_ROUTE)) {
-			char hbuf[PREFIX_STRLEN];
-			char ubuf[PREFIX_STRLEN];
-
-			prefix2str(&pb->hpfx, hbuf, sizeof(hbuf));
-			prefix2str(&pb->upfx, ubuf, sizeof(ubuf));
+		       sizeof(pfx_unicast_nexthop)); /* keep valgrind happy */
 
+		if (VNC_DEBUG(IMPORT_BGP_ADD_ROUTE))
 			vnc_zlog_debug_any(
-				"%s: examining RHN Entry (q=%p): upfx=%s, hpfx=%s, ubpi=%p",
-				__func__, cursor, ubuf, hbuf, pb->ubpi);
-		}
+				"%s: examining RHN Entry (q=%p): upfx=%pFX, hpfx=%pFX, ubpi=%p",
+				__func__, cursor, &pb->upfx, &pb->hpfx,
+				pb->ubpi);
 
 		if (process_unicast_route(bgp, afi, &pb->upfx, pb->ubpi, &ecom,
 					  &pfx_unicast_nexthop)) {
@@ -1497,16 +1437,9 @@ void vnc_import_bgp_add_vnc_host_route_mode_resolve_nve(
 		 * Sanity check
 		 */
 		if (vnc_prefix_cmp(&pfx_unicast_nexthop, prefix)) {
-			char str_unh[PREFIX_STRLEN];
-			char str_nve_pfx[PREFIX_STRLEN];
-
-			prefix2str(&pfx_unicast_nexthop, str_unh,
-				   sizeof(str_unh));
-			prefix2str(prefix, str_nve_pfx, sizeof(str_nve_pfx));
-
 			vnc_zlog_debug_verbose(
-				"%s: FATAL: resolve_nve_nexthop list item bpi nexthop %s != nve pfx %s",
-				__func__, str_unh, str_nve_pfx);
+				"%s: FATAL: resolve_nve_nexthop list item bpi nexthop %pFX != nve pfx %pFX",
+				__func__, &pfx_unicast_nexthop, prefix);
 			assert(0);
 		}
 
@@ -1521,13 +1454,9 @@ void vnc_import_bgp_add_vnc_host_route_mode_resolve_nve(
 #if DEBUG_RHN_LIST
 		/* debug */
 		{
-			char pbuf[PREFIX_STRLEN];
-
-			prefix2str(prefix, pbuf, sizeof(pbuf));
-
 			vnc_zlog_debug_verbose(
-				"%s: advancing past RHN Entry (q=%p): with prefix %s",
-				__func__, cursor, pbuf);
+				"%s: advancing past RHN Entry (q=%p): with prefix %pFX",
+				__func__, cursor, prefix);
 			print_rhn_list(__func__, NULL); /* debug */
 		}
 #endif
@@ -1550,14 +1479,8 @@ void vnc_import_bgp_del_vnc_host_route_mode_resolve_nve(
 	struct rfapi_cfg *hc = NULL;
 	int rc;
 
-	{
-		char str_pfx[PREFIX_STRLEN];
-
-		prefix2str(prefix, str_pfx, sizeof(str_pfx));
-
-		vnc_zlog_debug_verbose("%s(bgp=%p, nve prefix=%s)", __func__,
-				       bgp, str_pfx);
-	}
+	vnc_zlog_debug_verbose("%s(bgp=%p, nve prefix=%pFX)", __func__, bgp,
+			       prefix);
 
 	if (afi != AFI_IP && afi != AFI_IP6)
 		return;
@@ -1606,7 +1529,7 @@ void vnc_import_bgp_del_vnc_host_route_mode_resolve_nve(
 		struct prefix pfx_unicast_nexthop;
 
 		memset(&pfx_unicast_nexthop, 0,
-		       sizeof(struct prefix)); /* keep valgrind happy */
+		       sizeof(pfx_unicast_nexthop)); /* keep valgrind happy */
 
 		if (process_unicast_route(bgp, afi, &pb->upfx, pb->ubpi, &ecom,
 					  &pfx_unicast_nexthop)) {
@@ -1621,16 +1544,9 @@ void vnc_import_bgp_del_vnc_host_route_mode_resolve_nve(
 		 * Sanity check
 		 */
 		if (vnc_prefix_cmp(&pfx_unicast_nexthop, prefix)) {
-			char str_unh[PREFIX_STRLEN];
-			char str_nve_pfx[PREFIX_STRLEN];
-
-			prefix2str(&pfx_unicast_nexthop, str_unh,
-				   sizeof(str_unh));
-			prefix2str(prefix, str_nve_pfx, sizeof(str_nve_pfx));
-
 			vnc_zlog_debug_verbose(
-				"%s: FATAL: resolve_nve_nexthop list item bpi nexthop %s != nve pfx %s",
-				__func__, str_unh, str_nve_pfx);
+				"%s: FATAL: resolve_nve_nexthop list item bpi nexthop %pFX != nve pfx %pFX",
+				__func__, &pfx_unicast_nexthop, prefix);
 			assert(0);
 		}
 
@@ -1788,7 +1704,7 @@ static void vnc_import_bgp_exterior_add_route_it(
 					prd = NULL;
 
 				/* use local_pref from unicast route */
-				memset(&new_attr, 0, sizeof(struct attr));
+				memset(&new_attr, 0, sizeof(new_attr));
 				new_attr = *bpi_interior->attr;
 				if (info->attr->flag
 				    & ATTR_FLAG_BIT(BGP_ATTR_LOCAL_PREF)) {
@@ -1882,7 +1798,7 @@ void vnc_import_bgp_exterior_del_route(
 		return;
 
 	memset(&pfx_orig_nexthop, 0,
-	       sizeof(struct prefix)); /* keep valgrind happy */
+	       sizeof(pfx_orig_nexthop)); /* keep valgrind happy */
 
 	h = bgp_default->rfapi;
 	hc = bgp_default->rfapi_cfg;
@@ -2070,8 +1986,6 @@ void vnc_import_bgp_exterior_add_route_interior(
 
 	if (RFAPI_HAS_MONITOR_EXTERIOR(rn_interior)) {
 
-		int count = 0; /* debugging */
-
 		vnc_zlog_debug_verbose(
 			"%s: has exterior monitor; ext src: %p", __func__,
 			RFAPI_MONITOR_EXTERIOR(rn_interior)->source);
@@ -2095,9 +2009,6 @@ void vnc_import_bgp_exterior_add_route_interior(
 			struct attr new_attr;
 			uint32_t label = 0;
 
-
-			++count; /* debugging */
-
 			assert(bpi_exterior);
 			assert(pfx_exterior);
 
@@ -2305,13 +2216,11 @@ void vnc_import_bgp_exterior_add_route_interior(
 				     (void **)&pfx_exterior, &cursor)) {
 
 		struct prefix pfx_nexthop;
-		char buf[PREFIX_STRLEN];
 		afi_t afi_exterior = family2afi(pfx_exterior->family);
 
-		prefix2str(pfx_exterior, buf, sizeof(buf));
 		vnc_zlog_debug_verbose(
-			"%s: checking exterior orphan at prefix %s", __func__,
-			buf);
+			"%s: checking exterior orphan at prefix %pFX", __func__,
+			pfx_exterior);
 
 		if (afi_exterior != afi) {
 			vnc_zlog_debug_verbose(
@@ -2550,7 +2459,7 @@ void vnc_import_bgp_exterior_del_route_interior(
 					prd = NULL;
 
 				/* use local_pref from unicast route */
-				memset(&new_attr, 0, sizeof(struct attr));
+				memset(&new_attr, 0, sizeof(new_attr));
 				new_attr = *bpi->attr;
 				if (bpi_exterior
 				    && (bpi_exterior->attr->flag
@@ -2602,15 +2511,10 @@ void vnc_import_bgp_add_route(struct bgp *bgp, const struct prefix *prefix,
 
 	if (VNC_DEBUG(VERBOSE)) {
 		struct prefix pfx_nexthop;
-		char buf[PREFIX_STRLEN];
-		char buf_nh[PREFIX_STRLEN];
 
-		prefix2str(prefix, buf, sizeof(buf));
 		rfapiUnicastNexthop2Prefix(afi, info->attr, &pfx_nexthop);
-		prefix2str(&pfx_nexthop, buf_nh, sizeof(buf_nh));
-
-		vnc_zlog_debug_verbose("%s: pfx %s, nh %s", __func__, buf,
-				       buf_nh);
+		vnc_zlog_debug_verbose("%s: pfx %pFX, nh %pFX", __func__,
+				       prefix, &pfx_nexthop);
 	}
 #if DEBUG_RHN_LIST
 	print_rhn_list(__func__, "ENTER ");
@@ -2673,15 +2577,10 @@ void vnc_import_bgp_del_route(struct bgp *bgp, const struct prefix *prefix,
 
 	{
 		struct prefix pfx_nexthop;
-		char buf[PREFIX_STRLEN];
-		char buf_nh[PREFIX_STRLEN];
 
-		prefix2str(prefix, buf, sizeof(buf));
 		rfapiUnicastNexthop2Prefix(afi, info->attr, &pfx_nexthop);
-		prefix2str(&pfx_nexthop, buf_nh, sizeof(buf_nh));
-
-		vnc_zlog_debug_verbose("%s: pfx %s, nh %s", __func__, buf,
-				       buf_nh);
+		vnc_zlog_debug_verbose("%s: pfx %pFX, nh %pFX", __func__,
+				       prefix, &pfx_nexthop);
 	}
 #if DEBUG_RHN_LIST
 	print_rhn_list(__func__, "ENTER ");
@@ -2737,7 +2636,7 @@ void vnc_import_bgp_redist_enable(struct bgp *bgp, afi_t afi)
 	/* iterate over bgp unicast v4 and v6 routes, call
 	 * vnc_import_bgp_add_route */
 
-	struct bgp_node *rn;
+	struct bgp_dest *dest;
 
 	vnc_zlog_debug_verbose("%s: entry, afi=%d", __func__, afi);
 
@@ -2749,18 +2648,18 @@ void vnc_import_bgp_redist_enable(struct bgp *bgp, afi_t afi)
 	}
 	bgp->rfapi_cfg->redist[afi][ZEBRA_ROUTE_BGP_DIRECT] = 1;
 
-	for (rn = bgp_table_top(bgp->rib[afi][SAFI_UNICAST]); rn;
-	     rn = bgp_route_next(rn)) {
+	for (dest = bgp_table_top(bgp->rib[afi][SAFI_UNICAST]); dest;
+	     dest = bgp_route_next(dest)) {
 
 		struct bgp_path_info *bpi;
 
-		for (bpi = bgp_node_get_bgp_path_info(rn); bpi;
+		for (bpi = bgp_dest_get_bgp_path_info(dest); bpi;
 		     bpi = bpi->next) {
 
 			if (CHECK_FLAG(bpi->flags, BGP_PATH_REMOVED))
 				continue;
 
-			vnc_import_bgp_add_route(bgp, bgp_node_get_prefix(rn),
+			vnc_import_bgp_add_route(bgp, bgp_dest_get_prefix(dest),
 						 bpi);
 		}
 	}
@@ -2772,7 +2671,7 @@ void vnc_import_bgp_redist_enable(struct bgp *bgp, afi_t afi)
 void vnc_import_bgp_exterior_redist_enable(struct bgp *bgp, afi_t afi)
 {
 	struct bgp *bgp_exterior;
-	struct bgp_node *rn;
+	struct bgp_dest *dest;
 
 	bgp_exterior = bgp->rfapi_cfg->redist_bgp_exterior_view;
 
@@ -2791,19 +2690,19 @@ void vnc_import_bgp_exterior_redist_enable(struct bgp *bgp, afi_t afi)
 		return;
 	}
 
-	for (rn = bgp_table_top(bgp_exterior->rib[afi][SAFI_UNICAST]); rn;
-	     rn = bgp_route_next(rn)) {
+	for (dest = bgp_table_top(bgp_exterior->rib[afi][SAFI_UNICAST]); dest;
+	     dest = bgp_route_next(dest)) {
 
 		struct bgp_path_info *bpi;
 
-		for (bpi = bgp_node_get_bgp_path_info(rn); bpi;
+		for (bpi = bgp_dest_get_bgp_path_info(dest); bpi;
 		     bpi = bpi->next) {
 
 			if (CHECK_FLAG(bpi->flags, BGP_PATH_REMOVED))
 				continue;
 
 			vnc_import_bgp_exterior_add_route(
-				bgp_exterior, bgp_node_get_prefix(rn), bpi);
+				bgp_exterior, bgp_dest_get_prefix(dest), bpi);
 		}
 	}
 	vnc_zlog_debug_verbose(
@@ -2818,7 +2717,7 @@ void vnc_import_bgp_exterior_redist_enable_it(
 	struct bgp *bgp, afi_t afi, struct rfapi_import_table *it_only)
 {
 	struct bgp *bgp_exterior;
-	struct bgp_node *rn;
+	struct bgp_dest *dest;
 
 	vnc_zlog_debug_verbose("%s: entry", __func__);
 
@@ -2837,19 +2736,19 @@ void vnc_import_bgp_exterior_redist_enable_it(
 		return;
 	}
 
-	for (rn = bgp_table_top(bgp_exterior->rib[afi][SAFI_UNICAST]); rn;
-	     rn = bgp_route_next(rn)) {
+	for (dest = bgp_table_top(bgp_exterior->rib[afi][SAFI_UNICAST]); dest;
+	     dest = bgp_route_next(dest)) {
 
 		struct bgp_path_info *bpi;
 
-		for (bpi = bgp_node_get_bgp_path_info(rn); bpi;
+		for (bpi = bgp_dest_get_bgp_path_info(dest); bpi;
 		     bpi = bpi->next) {
 
 			if (CHECK_FLAG(bpi->flags, BGP_PATH_REMOVED))
 				continue;
 
 			vnc_import_bgp_exterior_add_route_it(
-				bgp_exterior, bgp_node_get_prefix(rn), bpi,
+				bgp_exterior, bgp_dest_get_prefix(dest), bpi,
 				it_only);
 		}
 	}
@@ -2862,8 +2761,8 @@ void vnc_import_bgp_redist_disable(struct bgp *bgp, afi_t afi)
 	 * iterate over vpn routes, find routes of type ZEBRA_ROUTE_BGP_DIRECT,
 	 * delete (call timer expire immediately)
 	 */
-	struct bgp_node *rn1;
-	struct bgp_node *rn2;
+	struct bgp_dest *dest1;
+	struct bgp_dest *dest2;
 
 	vnc_zlog_debug_verbose("%s: entry", __func__);
 
@@ -2878,21 +2777,22 @@ void vnc_import_bgp_redist_disable(struct bgp *bgp, afi_t afi)
 	 * Two-level table for SAFI_MPLS_VPN
 	 * Be careful when changing the things we iterate over
 	 */
-	for (rn1 = bgp_table_top(bgp->rib[afi][SAFI_MPLS_VPN]); rn1;
-	     rn1 = bgp_route_next(rn1)) {
-		const struct prefix *rn1_p;
+	for (dest1 = bgp_table_top(bgp->rib[afi][SAFI_MPLS_VPN]); dest1;
+	     dest1 = bgp_route_next(dest1)) {
+		const struct prefix *dest1_p;
 
-		if (!bgp_node_has_bgp_path_info_data(rn1))
+		if (!bgp_dest_has_bgp_path_info_data(dest1))
 			continue;
 
-		rn1_p = bgp_node_get_prefix(rn1);
-		for (rn2 = bgp_table_top(bgp_node_get_bgp_table_info(rn1)); rn2;
-		     rn2 = bgp_route_next(rn2)) {
-			const struct prefix *rn2_p = bgp_node_get_prefix(rn2);
+		dest1_p = bgp_dest_get_prefix(dest1);
+		for (dest2 = bgp_table_top(bgp_dest_get_bgp_table_info(dest1));
+		     dest2; dest2 = bgp_route_next(dest2)) {
+			const struct prefix *dest2_p =
+				bgp_dest_get_prefix(dest2);
 			struct bgp_path_info *bpi;
 			struct bgp_path_info *nextbpi;
 
-			for (bpi = bgp_node_get_bgp_path_info(rn2); bpi;
+			for (bpi = bgp_dest_get_bgp_path_info(dest2); bpi;
 			     bpi = nextbpi) {
 
 				nextbpi = bpi->next;
@@ -2917,8 +2817,8 @@ void vnc_import_bgp_redist_disable(struct bgp *bgp, afi_t afi)
 					rfd);
 
 				del_vnc_route(rfd, bpi->peer, bgp,
-					      SAFI_MPLS_VPN, rn2_p,
-					      (struct prefix_rd *)rn1_p,
+					      SAFI_MPLS_VPN, dest2_p,
+					      (struct prefix_rd *)dest1_p,
 					      bpi->type, bpi->sub_type, NULL,
 					      1); /* kill */
 
@@ -2966,20 +2866,20 @@ void vnc_import_bgp_exterior_redist_disable(struct bgp *bgp, afi_t afi)
 
 
 	{
-		struct bgp_node *rn;
-		for (rn = bgp_table_top(bgp_exterior->rib[afi][SAFI_UNICAST]);
-		     rn; rn = bgp_route_next(rn)) {
+		struct bgp_dest *dest;
+		for (dest = bgp_table_top(bgp_exterior->rib[afi][SAFI_UNICAST]);
+		     dest; dest = bgp_route_next(dest)) {
 
 			struct bgp_path_info *bpi;
 
-			for (bpi = bgp_node_get_bgp_path_info(rn); bpi;
+			for (bpi = bgp_dest_get_bgp_path_info(dest); bpi;
 			     bpi = bpi->next) {
 
 				if (CHECK_FLAG(bpi->flags, BGP_PATH_REMOVED))
 					continue;
 
 				vnc_import_bgp_exterior_del_route(
-					bgp_exterior, bgp_node_get_prefix(rn),
+					bgp_exterior, bgp_dest_get_prefix(dest),
 					bpi);
 			}
 		}
