@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * EIGRP Network Related Functions.
  * Copyright (C) 2013-2014
@@ -7,27 +8,11 @@
  *   Matej Perina
  *   Peter Orsag
  *   Peter Paluch
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
-#include "thread.h"
+#include "frrevent.h"
 #include "linklist.h"
 #include "prefix.h"
 #include "if.h"
@@ -69,8 +54,8 @@ int eigrp_sock_init(struct vrf *vrf)
 			AF_INET, SOCK_RAW, IPPROTO_EIGRPIGP, vrf->vrf_id,
 			vrf->vrf_id != VRF_DEFAULT ? vrf->name : NULL);
 		if (eigrp_sock < 0) {
-			zlog_err("eigrp_read_sock_init: socket: %s",
-				 safe_strerror(errno));
+			zlog_err("%s: socket: %s",
+				 __func__, safe_strerror(errno));
 			exit(1);
 		}
 
@@ -160,10 +145,8 @@ int eigrp_if_ipmulticast(struct eigrp *top, struct prefix *p,
 	ret = setsockopt_ipv4_multicast_if(top->fd, p->u.prefix4, ifindex);
 	if (ret < 0)
 		zlog_warn(
-			"can't setsockopt IP_MULTICAST_IF (fd %d, addr %s, "
-			"ifindex %u): %s",
-			top->fd, inet_ntoa(p->u.prefix4), ifindex,
-			safe_strerror(errno));
+			"can't setsockopt IP_MULTICAST_IF (fd %d, addr %pI4, ifindex %u): %s",
+			top->fd, &p->u.prefix4, ifindex, safe_strerror(errno));
 
 	return ret;
 }
@@ -179,14 +162,11 @@ int eigrp_if_add_allspfrouters(struct eigrp *top, struct prefix *p,
 		htonl(EIGRP_MULTICAST_ADDRESS), ifindex);
 	if (ret < 0)
 		zlog_warn(
-			"can't setsockopt IP_ADD_MEMBERSHIP (fd %d, addr %s, "
-			"ifindex %u, AllSPFRouters): %s; perhaps a kernel limit "
-			"on # of multicast group memberships has been exceeded?",
-			top->fd, inet_ntoa(p->u.prefix4), ifindex,
-			safe_strerror(errno));
+			"can't setsockopt IP_ADD_MEMBERSHIP (fd %d, addr %pI4, ifindex %u, AllSPFRouters): %s; perhaps a kernel limit on # of multicast group memberships has been exceeded?",
+			top->fd, &p->u.prefix4, ifindex, safe_strerror(errno));
 	else
-		zlog_debug("interface %s [%u] join EIGRP Multicast group.",
-			   inet_ntoa(p->u.prefix4), ifindex);
+		zlog_debug("interface %pI4 [%u] join EIGRP Multicast group.",
+			   &p->u.prefix4, ifindex);
 
 	return ret;
 }
@@ -201,13 +181,11 @@ int eigrp_if_drop_allspfrouters(struct eigrp *top, struct prefix *p,
 		htonl(EIGRP_MULTICAST_ADDRESS), ifindex);
 	if (ret < 0)
 		zlog_warn(
-			"can't setsockopt IP_DROP_MEMBERSHIP (fd %d, addr %s, "
-			"ifindex %u, AllSPFRouters): %s",
-			top->fd, inet_ntoa(p->u.prefix4), ifindex,
-			safe_strerror(errno));
+			"can't setsockopt IP_DROP_MEMBERSHIP (fd %d, addr %pI4, ifindex %u, AllSPFRouters): %s",
+			top->fd, &p->u.prefix4, ifindex, safe_strerror(errno));
 	else
-		zlog_debug("interface %s [%u] leave EIGRP Multicast group.",
-			   inet_ntoa(p->u.prefix4), ifindex);
+		zlog_debug("interface %pI4 [%u] leave EIGRP Multicast group.",
+			   &p->u.prefix4, ifindex);
 
 	return ret;
 }
@@ -226,7 +204,7 @@ int eigrp_network_set(struct eigrp *eigrp, struct prefix *p)
 	}
 
 	struct prefix *pref = prefix_new();
-	PREFIX_COPY_IPV4(pref, p);
+	prefix_copy(pref, p);
 	rn->info = (void *)pref;
 
 	/* Schedule Router ID Update. */
@@ -295,11 +273,11 @@ void eigrp_if_update(struct interface *ifp)
 	 * we need to check eac one and add the interface as approperate
 	 */
 	for (ALL_LIST_ELEMENTS(eigrp_om->eigrp, node, nnode, eigrp)) {
-		if (ifp->vrf_id != eigrp->vrf_id)
+		if (ifp->vrf->vrf_id != eigrp->vrf_id)
 			continue;
 
 		/* EIGRP must be on and Router-ID must be configured. */
-		if (eigrp->router_id.s_addr == 0)
+		if (eigrp->router_id.s_addr == INADDR_ANY)
 			continue;
 
 		/* Run each network for this interface. */
@@ -353,76 +331,6 @@ int eigrp_network_unset(struct eigrp *eigrp, struct prefix *p)
 	return 1;
 }
 
-uint32_t eigrp_calculate_metrics(struct eigrp *eigrp,
-				 struct eigrp_metrics metric)
-{
-	uint64_t temp_metric;
-	temp_metric = 0;
-
-	if (metric.delay == EIGRP_MAX_METRIC)
-		return EIGRP_MAX_METRIC;
-
-	// EIGRP Metric =
-	// {K1*BW+[(K2*BW)/(256-load)]+(K3*delay)}*{K5/(reliability+K4)}
-
-	if (eigrp->k_values[0])
-		temp_metric += (eigrp->k_values[0] * metric.bandwidth);
-	if (eigrp->k_values[1])
-		temp_metric += ((eigrp->k_values[1] * metric.bandwidth)
-				/ (256 - metric.load));
-	if (eigrp->k_values[2])
-		temp_metric += (eigrp->k_values[2] * metric.delay);
-	if (eigrp->k_values[3] && !eigrp->k_values[4])
-		temp_metric *= eigrp->k_values[3];
-	if (!eigrp->k_values[3] && eigrp->k_values[4])
-		temp_metric *= (eigrp->k_values[4] / metric.reliability);
-	if (eigrp->k_values[3] && eigrp->k_values[4])
-		temp_metric *= ((eigrp->k_values[4] / metric.reliability)
-				+ eigrp->k_values[3]);
-
-	if (temp_metric <= EIGRP_MAX_METRIC)
-		return (uint32_t)temp_metric;
-	else
-		return EIGRP_MAX_METRIC;
-}
-
-uint32_t eigrp_calculate_total_metrics(struct eigrp *eigrp,
-				       struct eigrp_nexthop_entry *entry)
-{
-	struct eigrp_interface *ei = entry->ei;
-
-	entry->total_metric = entry->reported_metric;
-	uint64_t temp_delay =
-		(uint64_t)entry->total_metric.delay
-		+ (uint64_t)eigrp_delay_to_scaled(ei->params.delay);
-	entry->total_metric.delay = temp_delay > EIGRP_MAX_METRIC
-					    ? EIGRP_MAX_METRIC
-					    : (uint32_t)temp_delay;
-
-	uint32_t bw = eigrp_bandwidth_to_scaled(ei->params.bandwidth);
-	entry->total_metric.bandwidth = entry->total_metric.bandwidth > bw
-						? bw
-						: entry->total_metric.bandwidth;
-
-	return eigrp_calculate_metrics(eigrp, entry->total_metric);
-}
-
-uint8_t eigrp_metrics_is_same(struct eigrp_metrics metric1,
-			      struct eigrp_metrics metric2)
-{
-	if ((metric1.bandwidth == metric2.bandwidth)
-	    && (metric1.delay == metric2.delay)
-	    && (metric1.hop_count == metric2.hop_count)
-	    && (metric1.load == metric2.load)
-	    && (metric1.reliability == metric2.reliability)
-	    && (metric1.mtu[0] == metric2.mtu[0])
-	    && (metric1.mtu[1] == metric2.mtu[1])
-	    && (metric1.mtu[2] == metric2.mtu[2]))
-		return 1;
-
-	return 0; // if different
-}
-
 void eigrp_external_routes_refresh(struct eigrp *eigrp, int type)
 {
 }
