@@ -1,29 +1,16 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * IS-IS Rout(e)ing protocol - isis_main.c
  *
  * Copyright (C) 2001,2002   Sampo Saaristo
  *                           Tampere University of Technology
  *                           Institute of Communications Engineering
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public Licenseas published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
 #include "getopt.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "log.h"
 #include <lib/version.h>
 #include "command.h"
@@ -40,7 +27,9 @@
 #include "qobj.h"
 #include "libfrr.h"
 #include "routemap.h"
+#include "affinitymap.h"
 
+#include "isisd/isis_affinitymap.h"
 #include "isisd/isis_constants.h"
 #include "isisd/isis_common.h"
 #include "isisd/isis_flags.h"
@@ -58,6 +47,7 @@
 #include "isisd/isis_mt.h"
 #include "isisd/fabricd.h"
 #include "isisd/isis_nb.h"
+#include "isisd/isis_ldp_sync.h"
 
 /* Default configuration file name */
 #define ISISD_DEFAULT_CONFIG "isisd.conf"
@@ -66,7 +56,7 @@
 #define FABRICD_VTY_PORT     2618
 
 /* isisd privileges */
-zebra_capabilities_t _caps_p[] = {ZCAP_NET_RAW, ZCAP_BIND};
+zebra_capabilities_t _caps_p[] = {ZCAP_NET_RAW, ZCAP_BIND, ZCAP_SYS_ADMIN};
 
 struct zebra_privs_t isisd_privs = {
 #if defined(FRR_USER)
@@ -88,7 +78,7 @@ static const struct option longopts[] = {
 	{0}};
 
 /* Master of threads. */
-struct thread_master *master;
+struct event_loop *master;
 
 /*
  * Prototypes.
@@ -101,7 +91,9 @@ void sigusr1(void);
 
 static __attribute__((__noreturn__)) void terminate(int i)
 {
+	isis_terminate();
 	isis_sr_term();
+	isis_srv6_term();
 	isis_zebra_stop();
 	exit(i);
 }
@@ -145,7 +137,7 @@ void sigusr1(void)
 	zlog_rotate();
 }
 
-struct quagga_signal_t isisd_signals[] = {
+struct frr_signal_t isisd_signals[] = {
 	{
 		.signal = SIGHUP,
 		.handler = &sighup,
@@ -165,14 +157,60 @@ struct quagga_signal_t isisd_signals[] = {
 };
 
 
+/* clang-format off */
 static const struct frr_yang_module_info *const isisd_yang_modules[] = {
+	&frr_filter_info,
 	&frr_interface_info,
 #ifndef FABRICD
 	&frr_isisd_info,
 #endif /* ifndef FABRICD */
 	&frr_route_map_info,
+	&frr_affinity_map_info,
 	&frr_vrf_info,
 };
+/* clang-format on */
+
+
+/* Max wait time for config to load before generating LSPs */
+#define ISIS_PRE_CONFIG_MAX_WAIT_SECONDS 600
+
+static void isis_config_finish(struct event *t)
+{
+	struct listnode *node, *inode;
+	struct isis *isis;
+	struct isis_area *area;
+
+	for (ALL_LIST_ELEMENTS_RO(im->isis, inode, isis)) {
+		for (ALL_LIST_ELEMENTS_RO(isis->area_list, node, area))
+			config_end_lsp_generate(area);
+	}
+}
+
+static void isis_config_end_timeout(struct event *t)
+{
+	zlog_err("IS-IS configuration end timer expired after %d seconds.",
+		 ISIS_PRE_CONFIG_MAX_WAIT_SECONDS);
+	isis_config_finish(t);
+}
+
+static void isis_config_start(void)
+{
+	EVENT_OFF(t_isis_cfg);
+	event_add_timer(im->master, isis_config_end_timeout, NULL,
+			ISIS_PRE_CONFIG_MAX_WAIT_SECONDS, &t_isis_cfg);
+}
+
+static void isis_config_end(void)
+{
+	/* If ISIS config processing thread isn't running, then
+	 * we can return and rely it's properly handled.
+	 */
+	if (!event_is_scheduled(t_isis_cfg))
+		return;
+
+	EVENT_OFF(t_isis_cfg);
+	isis_config_finish(t_isis_cfg);
+}
 
 #ifdef FABRICD
 FRR_DAEMON_INFO(fabricd, OPEN_FABRIC, .vty_port = FABRICD_VTY_PORT,
@@ -184,14 +222,14 @@ FRR_DAEMON_INFO(isisd, ISIS, .vty_port = ISISD_VTY_PORT,
 		.proghelp = "Implementation of the IS-IS routing protocol.",
 #endif
 		.copyright =
-			"Copyright (c) 2001-2002 Sampo Saaristo,"
-			" Ofer Wald and Hannes Gredler",
+			"Copyright (c) 2001-2002 Sampo Saaristo, Ofer Wald and Hannes Gredler",
 
 		.signals = isisd_signals,
 		.n_signals = array_size(isisd_signals),
 
 		.privs = &isisd_privs, .yang_modules = isisd_yang_modules,
-		.n_yang_modules = array_size(isisd_yang_modules), )
+		.n_yang_modules = array_size(isisd_yang_modules),
+);
 
 /*
  * Main routine of isisd. Parse arguments and handle IS-IS state machine.
@@ -228,20 +266,24 @@ int main(int argc, char **argv, char **envp)
 			break;
 		default:
 			frr_help_exit(1);
-			break;
 		}
 	}
 
 	/* thread master */
-	master = frr_init();
-
+	isis_master_init(frr_init());
+	master = im->master;
 	/*
 	 *  initializations
 	 */
+	cmd_init_config_callbacks(isis_config_start, isis_config_end);
 	isis_error_init();
 	access_list_init();
-	vrf_init(NULL, NULL, NULL, NULL, NULL);
+	access_list_add_hook(isis_filter_update);
+	access_list_delete_hook(isis_filter_update);
+	isis_vrf_init();
 	prefix_list_init();
+	prefix_list_add_hook(isis_prefix_list_update);
+	prefix_list_delete_hook(isis_prefix_list_update);
 	isis_init();
 	isis_circuit_init();
 #ifdef FABRICD
@@ -249,20 +291,23 @@ int main(int argc, char **argv, char **envp)
 #endif /* FABRICD */
 #ifndef FABRICD
 	isis_cli_init();
-#endif /* ifdef FABRICD */
-	isis_spf_cmds_init();
+#endif /* ifndef FABRICD */
+	isis_spf_init();
 	isis_redist_init();
 	isis_route_map_init();
 	isis_mpls_te_init();
 	isis_sr_init();
+	isis_srv6_init();
 	lsp_init();
 	mt_init();
 
-	/* create the global 'isis' instance */
-	isis_new(1, VRF_DEFAULT);
+#ifndef FABRICD
+	isis_affinity_map_init();
+#endif /* ifndef FABRICD */
 
 	isis_zebra_init(master, instance);
-	isis_bfd_init();
+	isis_bfd_init(master);
+	isis_ldp_sync_init();
 	fabricd_init();
 
 	frr_config_fork();
