@@ -1,20 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 2016 by Open Source Routing.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
- * MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -31,12 +17,12 @@
 #include "ldpd.h"
 #include "ldpe.h"
 #include "lde.h"
+#include "ldp_sync.h"
 #include "log.h"
 #include "ldp_debug.h"
 
 static void	 ifp2kif(struct interface *, struct kif *);
-static void	 ifc2kaddr(struct interface *, struct connected *,
-		    struct kaddr *);
+static void	 ifc2kaddr(struct interface *, struct connected *, struct kaddr *);
 static int	 ldp_zebra_send_mpls_labels(int, struct kroute *);
 static int	 ldp_router_id_update(ZAPI_CALLBACK_ARGS);
 static int	 ldp_interface_address_add(ZAPI_CALLBACK_ARGS);
@@ -46,7 +32,15 @@ static int	 ldp_zebra_read_pw_status_update(ZAPI_CALLBACK_ARGS);
 static void	 ldp_zebra_connected(struct zclient *);
 static void	 ldp_zebra_filter_update(struct access_list *access);
 
+static void 	ldp_zebra_opaque_register(void);
+static void 	ldp_zebra_opaque_unregister(void);
+static int 	ldp_sync_zebra_send_announce(void);
+static int 	ldp_zebra_opaque_msg_handler(ZAPI_CALLBACK_ARGS);
+static void 	ldp_sync_zebra_init(void);
+
 static struct zclient	*zclient;
+extern struct zclient *zclient_sync;
+static bool zebra_registered = false;
 
 static void
 ifp2kif(struct interface *ifp, struct kif *kif)
@@ -95,7 +89,7 @@ pw2zpw(struct l2vpn_pw *pw, struct zapi_pw *zpw)
 	zpw->nexthop.ipv6 = pw->addr.v6;
 	zpw->local_label = NO_LABEL;
 	zpw->remote_label = NO_LABEL;
-	if (pw->flags & F_PW_CWORD)
+	if (CHECK_FLAG(pw->flags, F_PW_CWORD))
 		zpw->flags = F_PSEUDOWIRE_CWORD;
 	zpw->data.ldp.lsr_id = pw->lsr_id;
 	zpw->data.ldp.pwid = pw->pwid;
@@ -103,6 +97,104 @@ pw2zpw(struct l2vpn_pw *pw, struct zapi_pw *zpw)
 	    sizeof(zpw->data.ldp.vpn_name));
 }
 
+static void
+ldp_zebra_opaque_register(void)
+{
+	zclient_register_opaque(zclient, LDP_IGP_SYNC_IF_STATE_REQUEST);
+	zclient_register_opaque(zclient, LDP_RLFA_REGISTER);
+	zclient_register_opaque(zclient, LDP_RLFA_UNREGISTER_ALL);
+}
+
+static void
+ldp_zebra_opaque_unregister(void)
+{
+	zclient_unregister_opaque(zclient, LDP_IGP_SYNC_IF_STATE_REQUEST);
+	zclient_unregister_opaque(zclient, LDP_RLFA_REGISTER);
+	zclient_unregister_opaque(zclient, LDP_RLFA_UNREGISTER_ALL);
+}
+
+int
+ldp_sync_zebra_send_state_update(struct ldp_igp_sync_if_state *state)
+{
+	if (zclient_send_opaque(zclient, LDP_IGP_SYNC_IF_STATE_UPDATE,
+				(const uint8_t *)state, sizeof(*state))
+	    == ZCLIENT_SEND_FAILURE)
+		return -1;
+	else
+		return 0;
+}
+
+static int
+ldp_sync_zebra_send_announce(void)
+{
+	struct ldp_igp_sync_announce announce;
+	announce.proto = ZEBRA_ROUTE_LDP;
+
+	if (zclient_send_opaque(zclient, LDP_IGP_SYNC_ANNOUNCE_UPDATE,
+				(const uint8_t *)&announce, sizeof(announce))
+	    == ZCLIENT_SEND_FAILURE)
+		return -1;
+	else
+		return 0;
+}
+
+int ldp_zebra_send_rlfa_labels(struct zapi_rlfa_response *rlfa_labels)
+{
+	int ret;
+
+	ret = zclient_send_opaque(zclient, LDP_RLFA_LABELS,
+				  (const uint8_t *)rlfa_labels,
+				  sizeof(*rlfa_labels));
+	if (ret == ZCLIENT_SEND_FAILURE) {
+		log_warn("failed to send RLFA labels to IGP");
+		return -1;
+	}
+
+	return 0;
+}
+
+static int
+ldp_zebra_opaque_msg_handler(ZAPI_CALLBACK_ARGS)
+{
+	struct stream *s;
+	struct zapi_opaque_msg info;
+	struct ldp_igp_sync_if_state_req state_req;
+	struct zapi_rlfa_igp igp;
+	struct zapi_rlfa_request rlfa;
+
+	s = zclient->ibuf;
+
+	if(zclient_opaque_decode(s, &info) != 0)
+		return -1;
+
+	switch (info.type) {
+	case LDP_IGP_SYNC_IF_STATE_REQUEST:
+		STREAM_GET(&state_req, s, sizeof(state_req));
+		main_imsg_compose_ldpe(IMSG_LDP_SYNC_IF_STATE_REQUEST, 0, &state_req,
+			    sizeof(state_req));
+		break;
+	case LDP_RLFA_REGISTER:
+		STREAM_GET(&rlfa, s, sizeof(rlfa));
+		main_imsg_compose_both(IMSG_RLFA_REG, &rlfa, sizeof(rlfa));
+		break;
+	case LDP_RLFA_UNREGISTER_ALL:
+		STREAM_GET(&igp, s, sizeof(igp));
+		main_imsg_compose_both(IMSG_RLFA_UNREG_ALL, &igp, sizeof(igp));
+		break;
+	default:
+		break;
+	}
+
+stream_failure:
+        return 0;
+}
+
+static void
+ldp_sync_zebra_init(void)
+{
+	ldp_sync_zebra_send_announce();
+}
+
 static int
 ldp_zebra_send_mpls_labels(int cmd, struct kroute *kr)
 {
@@ -140,12 +232,17 @@ ldp_zebra_send_mpls_labels(int cmd, struct kroute *kr)
 		zl.route.instance = kr->route_instance;
 	}
 
-	/*
-	 * For broken LSPs, instruct the forwarding plane to pop the top-level
+	/* If allow-broken-lsps is enabled then if an lsp is received with
+	 * no remote label, instruct the forwarding plane to pop the top-level
 	 * label and forward packets normally. This is a best-effort attempt
 	 * to deliver labeled IP packets to their final destination (instead of
 	 * dropping them).
 	 */
+	if (kr->remote_label == NO_LABEL
+	    && !CHECK_FLAG(ldpd_conf->flags, F_LDPD_ALLOW_BROKEN_LSP)
+	    && cmd == ZEBRA_MPLS_LABELS_ADD)
+		return 0;
+
 	if (kr->remote_label == NO_LABEL)
 		kr->remote_label = MPLS_LABEL_IMPLICIT_NULL;
 
@@ -174,7 +271,10 @@ ldp_zebra_send_mpls_labels(int cmd, struct kroute *kr)
 	znh->label_num = 1;
 	znh->labels[0] = kr->remote_label;
 
-	return zebra_send_mpls_labels(zclient, cmd, &zl);
+	if (zebra_send_mpls_labels(zclient, cmd, &zl) == ZCLIENT_SEND_FAILURE)
+		return -1;
+
+	return 0;
 }
 
 int
@@ -195,7 +295,7 @@ kmpw_add(struct zapi_pw *zpw)
 	debug_zebra_out("pseudowire %s nexthop %s (add)",
 	    zpw->ifname, log_addr(zpw->af, (union ldpd_addr *)&zpw->nexthop));
 
-	return (zebra_send_pw(zclient, ZEBRA_PW_ADD, zpw));
+	return zebra_send_pw(zclient, ZEBRA_PW_ADD, zpw) == ZCLIENT_SEND_FAILURE;
 }
 
 int
@@ -204,7 +304,7 @@ kmpw_del(struct zapi_pw *zpw)
 	debug_zebra_out("pseudowire %s nexthop %s (del)",
 	    zpw->ifname, log_addr(zpw->af, (union ldpd_addr *)&zpw->nexthop));
 
-	return (zebra_send_pw(zclient, ZEBRA_PW_DELETE, zpw));
+	return zebra_send_pw(zclient, ZEBRA_PW_DELETE, zpw) == ZCLIENT_SEND_FAILURE;
 }
 
 int
@@ -214,7 +314,7 @@ kmpw_set(struct zapi_pw *zpw)
 	    zpw->ifname, log_addr(zpw->af, (union ldpd_addr *)&zpw->nexthop),
 	    zpw->local_label, zpw->remote_label);
 
-	return (zebra_send_pw(zclient, ZEBRA_PW_SET, zpw));
+	return zebra_send_pw(zclient, ZEBRA_PW_SET, zpw) == ZCLIENT_SEND_FAILURE;
 }
 
 int
@@ -223,7 +323,7 @@ kmpw_unset(struct zapi_pw *zpw)
 	debug_zebra_out("pseudowire %s nexthop %s (unset)",
 	    zpw->ifname, log_addr(zpw->af, (union ldpd_addr *)&zpw->nexthop));
 
-	return (zebra_send_pw(zclient, ZEBRA_PW_UNSET, zpw));
+	return zebra_send_pw(zclient, ZEBRA_PW_UNSET, zpw) == ZCLIENT_SEND_FAILURE;
 }
 
 void
@@ -245,8 +345,7 @@ kif_redistribute(const char *ifname)
 
 		for (ALL_LIST_ELEMENTS_RO(ifp->connected, cnode, ifc)) {
 			ifc2kaddr(ifp, ifc, &ka);
-			main_imsg_compose_ldpe(IMSG_NEWADDR, 0, &ka,
-			    sizeof(ka));
+			main_imsg_compose_ldpe(IMSG_NEWADDR, 0, &ka, sizeof(ka));
 		}
 	}
 }
@@ -261,7 +360,7 @@ ldp_router_id_update(ZAPI_CALLBACK_ARGS)
 	if (bad_addr_v4(router_id.u.prefix4))
 		return (0);
 
-	debug_zebra_in("router-id update %s", inet_ntoa(router_id.u.prefix4));
+	debug_zebra_in("router-id update %pI4", &router_id.u.prefix4);
 
 	global.rtr_id.s_addr = router_id.u.prefix4.s_addr;
 	main_imsg_compose_ldpe(IMSG_RTRID_UPDATE, 0, &global.rtr_id,
@@ -299,7 +398,7 @@ ldp_ifp_destroy(struct interface *ifp)
 }
 
 static int
-ldp_interface_status_change_helper(struct interface *ifp)
+ldp_interface_status_change(struct interface *ifp)
 {
 	struct listnode		*node;
 	struct connected	*ifc;
@@ -314,14 +413,12 @@ ldp_interface_status_change_helper(struct interface *ifp)
 	if (if_is_operative(ifp)) {
 		for (ALL_LIST_ELEMENTS_RO(ifp->connected, node, ifc)) {
 			ifc2kaddr(ifp, ifc, &ka);
-			main_imsg_compose_ldpe(IMSG_NEWADDR, 0, &ka,
-			    sizeof(ka));
+			main_imsg_compose_ldpe(IMSG_NEWADDR, 0, &ka, sizeof(ka));
 		}
 	} else {
 		for (ALL_LIST_ELEMENTS_RO(ifp->connected, node, ifc)) {
 			ifc2kaddr(ifp, ifc, &ka);
-			main_imsg_compose_ldpe(IMSG_DELADDR, 0, &ka,
-			    sizeof(ka));
+			main_imsg_compose_ldpe(IMSG_DELADDR, 0, &ka, sizeof(ka));
 		}
 	}
 
@@ -330,12 +427,12 @@ ldp_interface_status_change_helper(struct interface *ifp)
 
 static int ldp_ifp_up(struct interface *ifp)
 {
-	return ldp_interface_status_change_helper(ifp);
+	return ldp_interface_status_change(ifp);
 }
 
 static int ldp_ifp_down(struct interface *ifp)
 {
-	return ldp_interface_status_change_helper(ifp);
+	return ldp_interface_status_change(ifp);
 }
 
 static int
@@ -426,7 +523,7 @@ ldp_zebra_read_route(ZAPI_CALLBACK_ARGS)
 
 	switch (api.type) {
 	case ZEBRA_ROUTE_CONNECT:
-		kr.flags |= F_CONNECTED;
+		SET_FLAG(kr.flags, F_CONNECTED);
 		break;
 	case ZEBRA_ROUTE_BGP:
 		/* LDP should follow the IGP and ignore BGP routes */
@@ -476,10 +573,10 @@ ldp_zebra_read_route(ZAPI_CALLBACK_ARGS)
 			kr.ifindex = api_nh->ifindex;
 			break;
 		case NEXTHOP_TYPE_IFINDEX:
-			if (!(kr.flags & F_CONNECTED))
+			if (!CHECK_FLAG(kr.flags, F_CONNECTED))
 				continue;
 			break;
-		default:
+		case NEXTHOP_TYPE_BLACKHOLE:
 			continue;
 		}
 
@@ -489,8 +586,7 @@ ldp_zebra_read_route(ZAPI_CALLBACK_ARGS)
 		    zebra_route_string(api.type));
 
 		if (add)
-			main_imsg_compose_lde(IMSG_NETWORK_ADD, 0, &kr,
-			    sizeof(kr));
+			main_imsg_compose_lde(IMSG_NETWORK_ADD, 0, &kr, sizeof(kr));
 	}
 
 	main_imsg_compose_lde(IMSG_NETWORK_UPDATE, 0, &kr, sizeof(kr));
@@ -508,22 +604,55 @@ ldp_zebra_read_pw_status_update(ZAPI_CALLBACK_ARGS)
 
 	zebra_read_pw_status_update(cmd, zclient, length, vrf_id, &zpw);
 
-	debug_zebra_in("pseudowire %s status %s", zpw.ifname,
-	    (zpw.status == PW_STATUS_UP) ? "up" : "down");
+	debug_zebra_in("pseudowire %s status %s 0x%x", zpw.ifname,
+	    (zpw.status == PW_FORWARDING) ? "up" : "down",
+	    zpw.status);
 
 	main_imsg_compose_lde(IMSG_PW_UPDATE, 0, &zpw, sizeof(zpw));
 
 	return (0);
 }
 
+void ldp_zebra_regdereg_zebra_info(bool want_register)
+{
+	if (zebra_registered == want_register)
+		return;
+
+	log_debug("%s to receive default VRF information",
+		  want_register ? "Register" : "De-register");
+
+	if (want_register) {
+		zclient_send_reg_requests(zclient, VRF_DEFAULT);
+		zebra_redistribute_send(ZEBRA_REDISTRIBUTE_ADD, zclient, AFI_IP,
+					ZEBRA_ROUTE_ALL, 0, VRF_DEFAULT);
+		zebra_redistribute_send(ZEBRA_REDISTRIBUTE_ADD, zclient,
+					AFI_IP6, ZEBRA_ROUTE_ALL, 0,
+					VRF_DEFAULT);
+	} else {
+		zclient_send_dereg_requests(zclient, VRF_DEFAULT);
+		zebra_redistribute_send(ZEBRA_REDISTRIBUTE_DELETE, zclient,
+					AFI_IP, ZEBRA_ROUTE_ALL, 0,
+					VRF_DEFAULT);
+		zebra_redistribute_send(ZEBRA_REDISTRIBUTE_DELETE, zclient,
+					AFI_IP6, ZEBRA_ROUTE_ALL, 0,
+					VRF_DEFAULT);
+	}
+	zebra_registered = want_register;
+}
+
 static void
 ldp_zebra_connected(struct zclient *zclient)
 {
-	zclient_send_reg_requests(zclient, VRF_DEFAULT);
-	zebra_redistribute_send(ZEBRA_REDISTRIBUTE_ADD, zclient, AFI_IP,
-	    ZEBRA_ROUTE_ALL, 0, VRF_DEFAULT);
-	zebra_redistribute_send(ZEBRA_REDISTRIBUTE_ADD, zclient, AFI_IP6,
-	    ZEBRA_ROUTE_ALL, 0, VRF_DEFAULT);
+	zebra_registered = false;
+
+	/* if MPLS was already enabled and we are re-connecting, register again
+	 */
+	if (CHECK_FLAG(vty_conf->flags, F_LDPD_ENABLED))
+		ldp_zebra_regdereg_zebra_info(true);
+
+	ldp_zebra_opaque_register();
+
+	ldp_sync_zebra_init();
 }
 
 static void
@@ -533,35 +662,35 @@ ldp_zebra_filter_update(struct access_list *access)
 
 	if (access && access->name[0] != '\0') {
 		strlcpy(laccess.name, access->name, sizeof(laccess.name));
-		laccess.type = access->type;
-		debug_evt("%s ACL update filter name %s type %d", __func__,
-		    access->name, access->type);
+		debug_evt("%s ACL update filter name %s", __func__, access->name);
 
-		main_imsg_compose_both(IMSG_FILTER_UPDATE, &laccess,
-			sizeof(laccess));
+		main_imsg_compose_both(IMSG_FILTER_UPDATE, &laccess, sizeof(laccess));
 	}
 }
 
 extern struct zebra_privs_t ldpd_privs;
 
-void
-ldp_zebra_init(struct thread_master *master)
+static zclient_handler *const ldp_handlers[] = {
+	[ZEBRA_ROUTER_ID_UPDATE] = ldp_router_id_update,
+	[ZEBRA_INTERFACE_ADDRESS_ADD] = ldp_interface_address_add,
+	[ZEBRA_INTERFACE_ADDRESS_DELETE] = ldp_interface_address_delete,
+	[ZEBRA_REDISTRIBUTE_ROUTE_ADD] = ldp_zebra_read_route,
+	[ZEBRA_REDISTRIBUTE_ROUTE_DEL] = ldp_zebra_read_route,
+	[ZEBRA_PW_STATUS_UPDATE] = ldp_zebra_read_pw_status_update,
+	[ZEBRA_OPAQUE_MESSAGE] = ldp_zebra_opaque_msg_handler,
+};
+
+void ldp_zebra_init(struct event_loop *master)
 {
-	if_zapi_callbacks(ldp_ifp_create, ldp_ifp_up,
-			  ldp_ifp_down, ldp_ifp_destroy);
+	if_zapi_callbacks(ldp_ifp_create, ldp_ifp_up, ldp_ifp_down, ldp_ifp_destroy);
 
 	/* Set default values. */
-	zclient = zclient_new(master, &zclient_options_default);
+	zclient = zclient_new(master, &zclient_options_default, ldp_handlers,
+			      array_size(ldp_handlers));
 	zclient_init(zclient, ZEBRA_ROUTE_LDP, 0, &ldpd_privs);
 
 	/* set callbacks */
 	zclient->zebra_connected = ldp_zebra_connected;
-	zclient->router_id_update = ldp_router_id_update;
-	zclient->interface_address_add = ldp_interface_address_add;
-	zclient->interface_address_delete = ldp_interface_address_delete;
-	zclient->redistribute_route_add = ldp_zebra_read_route;
-	zclient->redistribute_route_del = ldp_zebra_read_route;
-	zclient->pw_status_update = ldp_zebra_read_pw_status_update;
 
 	/* Access list initialize. */
 	access_list_add_hook(ldp_zebra_filter_update);
@@ -571,7 +700,14 @@ ldp_zebra_init(struct thread_master *master)
 void
 ldp_zebra_destroy(void)
 {
+	ldp_zebra_opaque_unregister();
 	zclient_stop(zclient);
 	zclient_free(zclient);
 	zclient = NULL;
+
+	if (zclient_sync == NULL)
+		return;
+	zclient_stop(zclient_sync);
+	zclient_free(zclient_sync);
+	zclient_sync = NULL;
 }
