@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: ISC
 /*	$OpenBSD$ */
 
 /*
@@ -5,18 +6,6 @@
  * Copyright (c) 2005 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2004, 2008 Esben Norby <norby@openbsd.org>
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
- * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
- * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
- * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
- * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
 #include <zebra.h>
@@ -46,8 +35,8 @@
 
 static void		 ldpd_shutdown(void);
 static pid_t		 start_child(enum ldpd_process, char *, int, int);
-static int		 main_dispatch_ldpe(struct thread *);
-static int		 main_dispatch_lde(struct thread *);
+static void main_dispatch_ldpe(struct event *thread);
+static void main_dispatch_lde(struct event *thread);
 static int		 main_imsg_send_ipc_sockets(struct imsgbuf *,
 			    struct imsgbuf *);
 static void		 main_imsg_send_net_sockets(int);
@@ -69,13 +58,13 @@ static void		 merge_l2vpns(struct ldpd_conf *, struct ldpd_conf *);
 static void		 merge_l2vpn(struct ldpd_conf *, struct l2vpn *,
 			    struct l2vpn *);
 
-DEFINE_QOBJ_TYPE(iface)
-DEFINE_QOBJ_TYPE(tnbr)
-DEFINE_QOBJ_TYPE(nbr_params)
-DEFINE_QOBJ_TYPE(l2vpn_if)
-DEFINE_QOBJ_TYPE(l2vpn_pw)
-DEFINE_QOBJ_TYPE(l2vpn)
-DEFINE_QOBJ_TYPE(ldpd_conf)
+DEFINE_QOBJ_TYPE(iface);
+DEFINE_QOBJ_TYPE(tnbr);
+DEFINE_QOBJ_TYPE(nbr_params);
+DEFINE_QOBJ_TYPE(l2vpn_if);
+DEFINE_QOBJ_TYPE(l2vpn_pw);
+DEFINE_QOBJ_TYPE(l2vpn);
+DEFINE_QOBJ_TYPE(ldpd_conf);
 
 struct ldpd_global	 global;
 struct ldpd_init	 init;
@@ -86,15 +75,36 @@ static struct imsgev	*iev_lde, *iev_lde_sync;
 static pid_t		 ldpe_pid;
 static pid_t		 lde_pid;
 
+static struct frr_daemon_info ldpd_di;
+
+DEFINE_HOOK(ldp_register_mib, (struct event_loop * tm), (tm));
+
+static void ldp_load_module(const char *name)
+{
+	const char *dir;
+	dir = ldpd_di.module_path ? ldpd_di.module_path : frr_moduledir;
+	struct frrmod_runtime *module;
+
+	module = frrmod_load(name, dir, NULL,NULL);
+	if (!module) {
+		fprintf(stderr, "%s: failed to load %s", __func__, name);
+		log_warnx("%s: failed to load %s", __func__, name);
+	}
+}
+
+void ldp_agentx_enabled(void)
+{
+	ldp_load_module("snmp");
+	hook_call(ldp_register_mib, master);
+}
+
 enum ldpd_process ldpd_process;
 
 #define LDP_DEFAULT_CONFIG	"ldpd.conf"
 #define LDP_VTY_PORT		2612
 
 /* Master of threads. */
-struct thread_master *master;
-
-static struct frr_daemon_info ldpd_di;
+struct event_loop *master;
 
 /* ldpd privileges */
 static zebra_capabilities_t _caps_p [] =
@@ -159,7 +169,7 @@ sigusr1(void)
 	zlog_rotate();
 }
 
-static struct quagga_signal_t ldp_signals[] =
+static struct frr_signal_t ldp_signals[] =
 {
 	{
 		.signal = SIGHUP,
@@ -180,6 +190,7 @@ static struct quagga_signal_t ldp_signals[] =
 };
 
 static const struct frr_yang_module_info *const ldpd_yang_modules[] = {
+	&frr_filter_info,
 	&frr_vrf_info,
 };
 
@@ -195,9 +206,9 @@ FRR_DAEMON_INFO(ldpd, LDP,
 
 	.yang_modules = ldpd_yang_modules,
 	.n_yang_modules = array_size(ldpd_yang_modules),
-)
+);
 
-static int ldp_config_fork_apply(struct thread *t)
+static void ldp_config_fork_apply(struct event *t)
 {
 	/*
 	 * So the frr_config_fork() function schedules
@@ -209,8 +220,6 @@ static int ldp_config_fork_apply(struct thread *t)
 	 * after the read in of the config.
 	 */
 	ldp_config_apply(NULL, vty_conf);
-
-	return 0;
 }
 
 int
@@ -221,7 +230,6 @@ main(int argc, char *argv[])
 	int			 pipe_parent2ldpe[2], pipe_parent2ldpe_sync[2];
 	int			 pipe_parent2lde[2], pipe_parent2lde_sync[2];
 	char			*ctl_sock_name;
-	struct thread           *thread = NULL;
 	bool                    ctl_sock_used = false;
 
 	snprintf(ctl_sock_path, sizeof(ctl_sock_path), LDPD_SOCKET,
@@ -281,7 +289,6 @@ main(int argc, char *argv[])
 			break;
 		default:
 			frr_help_exit(1);
-			break;
 		}
 	}
 
@@ -322,14 +329,18 @@ main(int argc, char *argv[])
 
 	if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC, pipe_parent2ldpe) == -1)
 		fatal("socketpair");
+
 	if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC,
 	    pipe_parent2ldpe_sync) == -1)
 		fatal("socketpair");
+
 	if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC, pipe_parent2lde) == -1)
 		fatal("socketpair");
+
 	if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC,
 	    pipe_parent2lde_sync) == -1)
 		fatal("socketpair");
+
 	sock_set_nonblock(pipe_parent2ldpe[0]);
 	sock_set_cloexec(pipe_parent2ldpe[0]);
 	sock_set_nonblock(pipe_parent2ldpe[1]);
@@ -353,7 +364,7 @@ main(int argc, char *argv[])
 
 	master = frr_init();
 
-	vrf_init(NULL, NULL, NULL, NULL, NULL);
+	vrf_init(NULL, NULL, NULL, NULL);
 	access_list_init();
 	ldp_vty_init();
 	ldp_zebra_init(master);
@@ -372,7 +383,7 @@ main(int argc, char *argv[])
 	frr_config_fork();
 
 	/* apply configuration */
-	thread_add_event(master, ldp_config_fork_apply, NULL, 0, &thread);
+	event_add_event(master, ldp_config_fork_apply, NULL, 0, NULL);
 
 	/* setup pipes to children */
 	if ((iev_ldpe = calloc(1, sizeof(struct imsgev))) == NULL ||
@@ -380,44 +391,42 @@ main(int argc, char *argv[])
 	    (iev_lde = calloc(1, sizeof(struct imsgev))) == NULL ||
 	    (iev_lde_sync = calloc(1, sizeof(struct imsgev))) == NULL)
 		fatal(NULL);
+
 	imsg_init(&iev_ldpe->ibuf, pipe_parent2ldpe[0]);
 	iev_ldpe->handler_read = main_dispatch_ldpe;
-	iev_ldpe->ev_read = NULL;
-	thread_add_read(master, iev_ldpe->handler_read, iev_ldpe, iev_ldpe->ibuf.fd,
-			&iev_ldpe->ev_read);
+	event_add_read(master, iev_ldpe->handler_read, iev_ldpe,
+		       iev_ldpe->ibuf.fd, &iev_ldpe->ev_read);
 	iev_ldpe->handler_write = ldp_write_handler;
 
 	imsg_init(&iev_ldpe_sync->ibuf, pipe_parent2ldpe_sync[0]);
 	iev_ldpe_sync->handler_read = main_dispatch_ldpe;
-	iev_ldpe_sync->ev_read = NULL;
-	thread_add_read(master, iev_ldpe_sync->handler_read, iev_ldpe_sync, iev_ldpe_sync->ibuf.fd,
-			&iev_ldpe_sync->ev_read);
+	event_add_read(master, iev_ldpe_sync->handler_read, iev_ldpe_sync,
+		       iev_ldpe_sync->ibuf.fd, &iev_ldpe_sync->ev_read);
 	iev_ldpe_sync->handler_write = ldp_write_handler;
 
 	imsg_init(&iev_lde->ibuf, pipe_parent2lde[0]);
 	iev_lde->handler_read = main_dispatch_lde;
-	iev_lde->ev_read = NULL;
-	thread_add_read(master, iev_lde->handler_read, iev_lde, iev_lde->ibuf.fd,
-			&iev_lde->ev_read);
+	event_add_read(master, iev_lde->handler_read, iev_lde, iev_lde->ibuf.fd,
+		       &iev_lde->ev_read);
 	iev_lde->handler_write = ldp_write_handler;
 
 	imsg_init(&iev_lde_sync->ibuf, pipe_parent2lde_sync[0]);
 	iev_lde_sync->handler_read = main_dispatch_lde;
-	iev_lde_sync->ev_read = NULL;
-	thread_add_read(master, iev_lde_sync->handler_read, iev_lde_sync, iev_lde_sync->ibuf.fd,
-			&iev_lde_sync->ev_read);
+	event_add_read(master, iev_lde_sync->handler_read, iev_lde_sync,
+		       iev_lde_sync->ibuf.fd, &iev_lde_sync->ev_read);
 	iev_lde_sync->handler_write = ldp_write_handler;
 
 	if (main_imsg_send_ipc_sockets(&iev_ldpe->ibuf, &iev_lde->ibuf))
 		fatal("could not establish imsg links");
-	main_imsg_compose_both(IMSG_DEBUG_UPDATE, &ldp_debug,
-	    sizeof(ldp_debug));
+
+	main_imsg_compose_both(IMSG_DEBUG_UPDATE, &ldp_debug, sizeof(ldp_debug));
 	main_imsg_compose_both(IMSG_INIT, &init, sizeof(init));
 	main_imsg_send_config(ldpd_conf);
 
-	if (ldpd_conf->ipv4.flags & F_LDPD_AF_ENABLED)
+	if (CHECK_FLAG(ldpd_conf->ipv4.flags, F_LDPD_AF_ENABLED))
 		main_imsg_send_net_sockets(AF_INET);
-	if (ldpd_conf->ipv6.flags & F_LDPD_AF_ENABLED)
+
+	if (CHECK_FLAG(ldpd_conf->ipv6.flags, F_LDPD_AF_ENABLED))
 		main_imsg_send_net_sockets(AF_INET6);
 
 	frr_run(master);
@@ -494,7 +503,8 @@ start_child(enum ldpd_process p, char *argv0, int fd_async, int fd_sync)
 	int	 argc = 0, nullfd;
 	pid_t	 pid;
 
-	switch (pid = fork()) {
+	pid = fork();
+	switch (pid) {
 	case -1:
 		fatal("cannot fork");
 	case 0:
@@ -519,6 +529,7 @@ start_child(enum ldpd_process p, char *argv0, int fd_async, int fd_sync)
 
 	if (dup2(fd_async, LDPD_FD_ASYNC) == -1)
 		fatal("cannot setup imsg async fd");
+
 	if (dup2(fd_sync, LDPD_FD_SYNC) == -1)
 		fatal("cannot setup imsg sync fd");
 
@@ -546,10 +557,9 @@ start_child(enum ldpd_process p, char *argv0, int fd_async, int fd_sync)
 
 /* imsg handling */
 /* ARGSUSED */
-static int
-main_dispatch_ldpe(struct thread *thread)
+static void main_dispatch_ldpe(struct event *thread)
 {
-	struct imsgev		*iev = THREAD_ARG(thread);
+	struct imsgev *iev = EVENT_ARG(thread);
 	struct imsgbuf		*ibuf = &iev->ibuf;
 	struct imsg		 imsg;
 	int			 af;
@@ -560,6 +570,7 @@ main_dispatch_ldpe(struct thread *thread)
 
 	if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
 		fatal("imsg_read error");
+
 	if (n == 0)	/* connection closed */
 		shut = 1;
 
@@ -584,6 +595,13 @@ main_dispatch_ldpe(struct thread *thread)
 				fatalx("IMSG_ACL_CHECK imsg with wrong len");
 			ldp_acl_reply(iev, (struct acl_check *)imsg.data);
 			break;
+		case IMSG_LDP_SYNC_IF_STATE_UPDATE:
+			if (imsg.hdr.len != IMSG_HEADER_SIZE +
+			    sizeof(struct ldp_igp_sync_if_state))
+				fatalx("IMSG_LDP_SYNC_IF_STATE_UPDATE imsg with wrong len");
+
+			ldp_sync_zebra_send_state_update((struct ldp_igp_sync_if_state *)imsg.data);
+			break;
 		default:
 			log_debug("%s: error handling imsg %d", __func__,
 			    imsg.hdr.type);
@@ -595,32 +613,32 @@ main_dispatch_ldpe(struct thread *thread)
 		imsg_event_add(iev);
 	else {
 		/* this pipe is dead, so remove the event handlers and exit */
-		THREAD_READ_OFF(iev->ev_read);
-		THREAD_WRITE_OFF(iev->ev_write);
+		EVENT_OFF(iev->ev_read);
+		EVENT_OFF(iev->ev_write);
 		ldpe_pid = 0;
+
 		if (lde_pid == 0)
 			ldpd_shutdown();
 		else
 			kill(lde_pid, SIGTERM);
 	}
-
-	return (0);
 }
 
 /* ARGSUSED */
-static int
-main_dispatch_lde(struct thread *thread)
+static void main_dispatch_lde(struct event *thread)
 {
-	struct imsgev	*iev = THREAD_ARG(thread);
+	struct imsgev *iev = EVENT_ARG(thread);
 	struct imsgbuf	*ibuf = &iev->ibuf;
 	struct imsg	 imsg;
 	ssize_t		 n;
 	int		 shut = 0;
+	struct zapi_rlfa_response *rlfa_labels;
 
 	iev->ev_read = NULL;
 
 	if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
 		fatal("imsg_read error");
+
 	if (n == 0)	/* connection closed */
 		shut = 1;
 
@@ -660,23 +678,19 @@ main_dispatch_lde(struct thread *thread)
 			switch (imsg.hdr.type) {
 			case IMSG_KPW_ADD:
 				if (kmpw_add(imsg.data))
-					log_warnx("%s: error adding "
-					    "pseudowire", __func__);
+					log_warnx("%s: error adding pseudowire", __func__);
 				break;
 			case IMSG_KPW_DELETE:
 				if (kmpw_del(imsg.data))
-					log_warnx("%s: error deleting "
-					    "pseudowire", __func__);
+					log_warnx("%s: error deleting pseudowire", __func__);
 				break;
 			case IMSG_KPW_SET:
 				if (kmpw_set(imsg.data))
-					log_warnx("%s: error setting "
-					    "pseudowire", __func__);
+					log_warnx("%s: error setting pseudowire", __func__);
 				break;
 			case IMSG_KPW_UNSET:
 				if (kmpw_unset(imsg.data))
-					log_warnx("%s: error unsetting "
-					    "pseudowire", __func__);
+					log_warnx("%s: error unsetting pseudowire", __func__);
 				break;
 			}
 			break;
@@ -686,34 +700,42 @@ main_dispatch_lde(struct thread *thread)
 				fatalx("IMSG_ACL_CHECK imsg with wrong len");
 			ldp_acl_reply(iev, (struct acl_check *)imsg.data);
 			break;
+		case IMSG_RLFA_LABELS:
+			if (imsg.hdr.len != IMSG_HEADER_SIZE +
+			    sizeof(struct zapi_rlfa_response)) {
+				log_warnx("%s: wrong imsg len", __func__);
+				break;
+			}
+			rlfa_labels = imsg.data;
+			ldp_zebra_send_rlfa_labels(rlfa_labels);
+			break;
 		default:
 			log_debug("%s: error handling imsg %d", __func__,
 			    imsg.hdr.type);
 			break;
 		}
+
 		imsg_free(&imsg);
 	}
+
 	if (!shut)
 		imsg_event_add(iev);
 	else {
 		/* this pipe is dead, so remove the event handlers and exit */
-		THREAD_READ_OFF(iev->ev_read);
-		THREAD_WRITE_OFF(iev->ev_write);
+		EVENT_OFF(iev->ev_read);
+		EVENT_OFF(iev->ev_write);
 		lde_pid = 0;
 		if (ldpe_pid == 0)
 			ldpd_shutdown();
 		else
 			kill(ldpe_pid, SIGTERM);
 	}
-
-	return (0);
 }
 
 /* ARGSUSED */
-int
-ldp_write_handler(struct thread *thread)
+void ldp_write_handler(struct event *thread)
 {
-	struct imsgev	*iev = THREAD_ARG(thread);
+	struct imsgev *iev = EVENT_ARG(thread);
 	struct imsgbuf	*ibuf = &iev->ibuf;
 	ssize_t		 n;
 
@@ -723,14 +745,12 @@ ldp_write_handler(struct thread *thread)
 		fatal("msgbuf_write");
 	if (n == 0) {
 		/* this pipe is dead, so remove the event handlers */
-		THREAD_READ_OFF(iev->ev_read);
-		THREAD_WRITE_OFF(iev->ev_write);
-		return (0);
+		EVENT_OFF(iev->ev_read);
+		EVENT_OFF(iev->ev_write);
+		return;
 	}
 
 	imsg_event_add(iev);
-
-	return (0);
 }
 
 void
@@ -738,6 +758,7 @@ main_imsg_compose_ldpe(int type, pid_t pid, void *data, uint16_t datalen)
 {
 	if (iev_ldpe == NULL)
 		return;
+
 	imsg_compose_event(iev_ldpe, type, 0, pid, -1, data, datalen);
 }
 
@@ -752,10 +773,13 @@ main_imsg_compose_both(enum imsg_type type, void *buf, uint16_t len)
 {
 	if (iev_ldpe == NULL || iev_lde == NULL)
 		return (0);
+
 	if (imsg_compose_event(iev_ldpe, type, 0, 0, -1, buf, len) == -1)
 		return (-1);
+
 	if (imsg_compose_event(iev_lde, type, 0, 0, -1, buf, len) == -1)
 		return (-1);
+
 	return (0);
 }
 
@@ -763,12 +787,12 @@ void
 imsg_event_add(struct imsgev *iev)
 {
 	if (iev->handler_read)
-		thread_add_read(master, iev->handler_read, iev, iev->ibuf.fd,
-				&iev->ev_read);
+		event_add_read(master, iev->handler_read, iev, iev->ibuf.fd,
+			       &iev->ev_read);
 
 	if (iev->handler_write && iev->ibuf.w.queued)
-		thread_add_write(master, iev->handler_write, iev,
-				 iev->ibuf.fd, &iev->ev_write);
+		event_add_write(master, iev->handler_write, iev, iev->ibuf.fd,
+				&iev->ev_write);
 }
 
 int
@@ -780,6 +804,7 @@ imsg_compose_event(struct imsgev *iev, uint16_t type, uint32_t peerid,
 	if ((ret = imsg_compose(&iev->ibuf, type, peerid,
 	    pid, fd, data, datalen)) != -1)
 		imsg_event_add(iev);
+
 	return (ret);
 }
 
@@ -794,13 +819,12 @@ void
 evbuf_event_add(struct evbuf *eb)
 {
 	if (eb->wbuf.queued)
-		thread_add_write(master, eb->handler, eb->arg, eb->wbuf.fd,
-				 &eb->ev);
+		event_add_write(master, eb->handler, eb->arg, eb->wbuf.fd,
+				&eb->ev);
 }
 
-void
-evbuf_init(struct evbuf *eb, int fd, int (*handler)(struct thread *),
-    void *arg)
+void evbuf_init(struct evbuf *eb, int fd, void (*handler)(struct event *),
+		void *arg)
 {
 	msgbuf_init(&eb->wbuf);
 	eb->wbuf.fd = fd;
@@ -811,7 +835,7 @@ evbuf_init(struct evbuf *eb, int fd, int (*handler)(struct thread *),
 void
 evbuf_clear(struct evbuf *eb)
 {
-	THREAD_WRITE_OFF(eb->ev);
+	EVENT_OFF(eb->ev);
 	msgbuf_clear(&eb->wbuf);
 	eb->wbuf.fd = -1;
 }
@@ -829,6 +853,7 @@ main_imsg_send_ipc_sockets(struct imsgbuf *ldpe_buf, struct imsgbuf *lde_buf)
 	if (imsg_compose(ldpe_buf, IMSG_SOCKET_IPC, 0, 0, pipe_ldpe2lde[0],
 	    NULL, 0) == -1)
 		return (-1);
+
 	if (imsg_compose(lde_buf, IMSG_SOCKET_IPC, 0, 0, pipe_ldpe2lde[1],
 	    NULL, 0) == -1)
 		return (-1);
@@ -855,8 +880,7 @@ main_imsg_send_net_socket(int af, enum socket_type type)
 
 	fd = ldp_create_socket(af, type);
 	if (fd == -1) {
-		log_warnx("%s: failed to create %s socket for address-family "
-		    "%s", __func__, socket_name(type), af_name(af));
+		log_warnx("%s: failed to create %s socket for address-family %s", __func__, socket_name(type), af_name(af));
 		return;
 	}
 
@@ -888,8 +912,10 @@ ldp_acl_request(struct imsgev *iev, char *acl_name, int af,
 	/* receive (blocking) and parse result */
 	if (imsg_read(&iev->ibuf) == -1)
 		fatal("imsg_read error");
+
 	if (imsg_get(&iev->ibuf, &imsg) == -1)
 		fatal("imsg_get");
+
 	if (imsg.hdr.type != IMSG_ACL_CHECK ||
 	    imsg.hdr.len != IMSG_HEADER_SIZE + sizeof(int))
 		fatalx("ldp_acl_request: invalid response");
@@ -956,8 +982,8 @@ ldp_af_global_get(struct ldpd_global *xglobal, int af)
 int
 ldp_is_dual_stack(struct ldpd_conf *xconf)
 {
-	return ((xconf->ipv4.flags & F_LDPD_AF_ENABLED) &&
-	    (xconf->ipv6.flags & F_LDPD_AF_ENABLED));
+	return (CHECK_FLAG(xconf->ipv4.flags, F_LDPD_AF_ENABLED) 
+            && CHECK_FLAG(xconf->ipv6.flags, F_LDPD_AF_ENABLED));
 }
 
 in_addr_t
@@ -1011,11 +1037,13 @@ main_imsg_send_config(struct ldpd_conf *xconf)
 			    sizeof(*lif)) == -1)
 				return (-1);
 		}
+
 		RB_FOREACH(pw, l2vpn_pw_head, &l2vpn->pw_tree) {
 			if (main_imsg_compose_both(IMSG_RECONF_L2VPN_PW, pw,
 			    sizeof(*pw)) == -1)
 				return (-1);
 		}
+
 		RB_FOREACH(pw, l2vpn_pw_head, &l2vpn->pw_inactive_tree) {
 			if (main_imsg_compose_both(IMSG_RECONF_L2VPN_IPW, pw,
 			    sizeof(*pw)) == -1)
@@ -1059,12 +1087,12 @@ ldp_config_normalize(struct ldpd_conf *xconf)
 	struct l2vpn		*l2vpn;
 	struct l2vpn_pw		*pw, *ptmp;
 
-	if (!(xconf->flags & F_LDPD_ENABLED))
+	if (!CHECK_FLAG(xconf->flags, F_LDPD_ENABLED))
 		ldp_config_reset_main(xconf);
 	else {
-		if (!(xconf->ipv4.flags & F_LDPD_AF_ENABLED))
+		if (!CHECK_FLAG(xconf->ipv4.flags, F_LDPD_AF_ENABLED))
 			ldp_config_reset_af(xconf, AF_INET);
-		if (!(xconf->ipv6.flags & F_LDPD_AF_ENABLED))
+		if (!CHECK_FLAG(xconf->ipv6.flags, F_LDPD_AF_ENABLED))
 			ldp_config_reset_af(xconf, AF_INET6);
 
 		RB_FOREACH_SAFE(iface, iface_head, &xconf->iface_tree, itmp) {
@@ -1077,7 +1105,7 @@ ldp_config_normalize(struct ldpd_conf *xconf)
 		}
 
 		RB_FOREACH_SAFE(nbrp, nbrp_head, &xconf->nbrp_tree, ntmp) {
-			if (nbrp->flags & (F_NBRP_KEEPALIVE|F_NBRP_GTSM))
+			if (CHECK_FLAG(nbrp->flags, (F_NBRP_KEEPALIVE|F_NBRP_GTSM)))
 				continue;
 			if (nbrp->auth.method != AUTH_NONE)
 				continue;
@@ -1090,7 +1118,7 @@ ldp_config_normalize(struct ldpd_conf *xconf)
 
 	RB_FOREACH(l2vpn, l2vpn_head, &xconf->l2vpn_tree) {
 		RB_FOREACH_SAFE(pw, l2vpn_pw_head, &l2vpn->pw_tree, ptmp) {
-			if (!(pw->flags & F_PW_STATIC_NBR_ADDR)) {
+			if (!CHECK_FLAG(pw->flags, F_PW_STATIC_NBR_ADDR)) {
 				pw->af = AF_INET;
 				pw->addr.v4 = pw->lsr_id;
 			}
@@ -1100,9 +1128,10 @@ ldp_config_normalize(struct ldpd_conf *xconf)
 			RB_REMOVE(l2vpn_pw_head, &l2vpn->pw_tree, pw);
 			RB_INSERT(l2vpn_pw_head, &l2vpn->pw_inactive_tree, pw);
 		}
+
 		RB_FOREACH_SAFE(pw, l2vpn_pw_head, &l2vpn->pw_inactive_tree,
 		    ptmp) {
-			if (!(pw->flags & F_PW_STATIC_NBR_ADDR)) {
+			if (!CHECK_FLAG(pw->flags, F_PW_STATIC_NBR_ADDR)) {
 				pw->af = AF_INET;
 				pw->addr.v4 = pw->lsr_id;
 			}
@@ -1151,6 +1180,7 @@ ldp_config_reset_main(struct ldpd_conf *conf)
 	conf->lhello_interval = DEFAULT_HELLO_INTERVAL;
 	conf->thello_holdtime = TARGETED_DFLT_HOLDTIME;
 	conf->thello_interval = DEFAULT_HELLO_INTERVAL;
+	conf->wait_for_sync_interval = DFLT_WAIT_FOR_SYNC;
 	conf->trans_pref = DUAL_STACK_LDPOV6;
 	conf->flags = 0;
 }
@@ -1203,6 +1233,7 @@ ldp_config_reset_l2vpns(struct ldpd_conf *conf)
 			RB_REMOVE(l2vpn_if_head, &l2vpn->if_tree, lif);
 			free(lif);
 		}
+
 		while (!RB_EMPTY(l2vpn_pw_head, &l2vpn->pw_tree)) {
 			pw = RB_ROOT(l2vpn_pw_head, &l2vpn->pw_tree);
 
@@ -1210,6 +1241,7 @@ ldp_config_reset_l2vpns(struct ldpd_conf *conf)
 			RB_REMOVE(l2vpn_pw_head, &l2vpn->pw_tree, pw);
 			free(pw);
 		}
+
 		while (!RB_EMPTY(l2vpn_pw_head, &l2vpn->pw_inactive_tree)) {
 			pw = RB_ROOT(l2vpn_pw_head, &l2vpn->pw_inactive_tree);
 
@@ -1217,6 +1249,7 @@ ldp_config_reset_l2vpns(struct ldpd_conf *conf)
 			RB_REMOVE(l2vpn_pw_head, &l2vpn->pw_inactive_tree, pw);
 			free(pw);
 		}
+
 		QOBJ_UNREG(l2vpn);
 		RB_REMOVE(l2vpn_head, &conf->l2vpn_tree, l2vpn);
 		free(l2vpn);
@@ -1237,18 +1270,21 @@ ldp_clear_config(struct ldpd_conf *xconf)
 		RB_REMOVE(iface_head, &xconf->iface_tree, iface);
 		free(iface);
 	}
+
 	while (!RB_EMPTY(tnbr_head, &xconf->tnbr_tree)) {
 		tnbr = RB_ROOT(tnbr_head, &xconf->tnbr_tree);
 
 		RB_REMOVE(tnbr_head, &xconf->tnbr_tree, tnbr);
 		free(tnbr);
 	}
+
 	while (!RB_EMPTY(nbrp_head, &xconf->nbrp_tree)) {
 		nbrp = RB_ROOT(nbrp_head, &xconf->nbrp_tree);
 
 		RB_REMOVE(nbrp_head, &xconf->nbrp_tree, nbrp);
 		free(nbrp);
 	}
+
 	while (!RB_EMPTY(l2vpn_head, &xconf->l2vpn_tree)) {
 		l2vpn = RB_ROOT(l2vpn_head, &xconf->l2vpn_tree);
 
@@ -1281,6 +1317,14 @@ merge_config(struct ldpd_conf *conf, struct ldpd_conf *xconf)
 static void
 merge_global(struct ldpd_conf *conf, struct ldpd_conf *xconf)
 {
+	/* Removing global LDP config requires resetting LDP IGP Sync FSM */
+	if (CHECK_FLAG(conf->flags, F_LDPD_ENABLED) 
+        && (!CHECK_FLAG(xconf->flags, F_LDPD_ENABLED)))
+	{
+		if (ldpd_process == PROC_LDP_ENGINE)
+			ldp_sync_fsm_reset_all();
+	}
+
 	/* change of router-id requires resetting all neighborships */
 	if (conf->rtr_id.s_addr != xconf->rtr_id.s_addr) {
 		if (ldpd_process == PROC_LDP_ENGINE) {
@@ -1298,14 +1342,15 @@ merge_global(struct ldpd_conf *conf, struct ldpd_conf *xconf)
 	 * Configuration of ordered-control or independent-control
 	 * requires resetting all neighborships.
 	 */
-	if ((conf->flags & F_LDPD_ORDERED_CONTROL) !=
-	    (xconf->flags & F_LDPD_ORDERED_CONTROL))
+	if (CHECK_FLAG(conf->flags, F_LDPD_ORDERED_CONTROL) !=
+	    CHECK_FLAG(xconf->flags, F_LDPD_ORDERED_CONTROL))
 		ldpe_reset_nbrs(AF_UNSPEC);
 
 	conf->lhello_holdtime = xconf->lhello_holdtime;
 	conf->lhello_interval = xconf->lhello_interval;
 	conf->thello_holdtime = xconf->thello_holdtime;
 	conf->thello_interval = xconf->thello_interval;
+	conf->wait_for_sync_interval = xconf->wait_for_sync_interval;
 
 	if (conf->trans_pref != xconf->trans_pref) {
 		if (ldpd_process == PROC_LDP_ENGINE)
@@ -1313,12 +1358,25 @@ merge_global(struct ldpd_conf *conf, struct ldpd_conf *xconf)
 		conf->trans_pref = xconf->trans_pref;
 	}
 
-	if ((conf->flags & F_LDPD_DS_CISCO_INTEROP) !=
-	    (xconf->flags & F_LDPD_DS_CISCO_INTEROP)) {
+	if (CHECK_FLAG(conf->flags, F_LDPD_DS_CISCO_INTEROP) !=
+	    CHECK_FLAG(xconf->flags, F_LDPD_DS_CISCO_INTEROP)) {
 		if (ldpd_process == PROC_LDP_ENGINE)
 			ldpe_reset_ds_nbrs();
 	}
 
+	/*
+	 * Configuration of allow-broken-lsp requires reprograming all
+	 * labeled routes
+	 */
+	if (CHECK_FLAG(conf->flags, F_LDPD_ALLOW_BROKEN_LSP) !=
+	    CHECK_FLAG(xconf->flags, F_LDPD_ALLOW_BROKEN_LSP)) {
+		if (ldpd_process == PROC_LDE_ENGINE)
+			lde_allow_broken_lsp_update(xconf->flags);
+	}
+
+	if (ldpd_process == PROC_LDP_ENGINE)
+		ldpe_set_config_change_time();
+
 	conf->flags = xconf->flags;
 }
 
@@ -1332,23 +1390,26 @@ merge_af(int af, struct ldpd_af_conf *af_conf, struct ldpd_af_conf *xa)
 	int		 reset_nbrs_ipv4 = 0;
 	int		 reset_nbrs = 0;
 	int		 update_sockets = 0;
+	int		 change_ldp_disabled = 0;
 
 	/* update timers */
 	if (af_conf->keepalive != xa->keepalive) {
 		af_conf->keepalive = xa->keepalive;
 		stop_init_backoff = 1;
 	}
+
 	af_conf->lhello_holdtime = xa->lhello_holdtime;
 	af_conf->lhello_interval = xa->lhello_interval;
 	af_conf->thello_holdtime = xa->thello_holdtime;
 	af_conf->thello_interval = xa->thello_interval;
 
 	/* update flags */
-	if ((af_conf->flags & F_LDPD_AF_THELLO_ACCEPT) &&
-	    !(xa->flags & F_LDPD_AF_THELLO_ACCEPT))
+	if (CHECK_FLAG(af_conf->flags, F_LDPD_AF_THELLO_ACCEPT) &&
+	    !CHECK_FLAG(xa->flags, F_LDPD_AF_THELLO_ACCEPT))
 		remove_dynamic_tnbrs = 1;
-	if ((af_conf->flags & F_LDPD_AF_NO_GTSM) !=
-	    (xa->flags & F_LDPD_AF_NO_GTSM)) {
+
+	if (CHECK_FLAG(af_conf->flags, F_LDPD_AF_NO_GTSM) !=
+	    CHECK_FLAG(xa->flags, F_LDPD_AF_NO_GTSM)) {
 		if (af == AF_INET6)
 			/* need to set/unset IPV6_MINHOPCOUNT */
 			update_sockets = 1;
@@ -1356,15 +1417,20 @@ merge_af(int af, struct ldpd_af_conf *af_conf, struct ldpd_af_conf *xa)
 			/* for LDPv4 just resetting the neighbors is enough */
 			reset_nbrs_ipv4 = 1;
 	}
-	if ((af_conf->flags & F_LDPD_AF_EXPNULL) !=
-	    (xa->flags & F_LDPD_AF_EXPNULL))
+	if (CHECK_FLAG(af_conf->flags, F_LDPD_AF_EXPNULL) !=
+	    CHECK_FLAG(xa->flags, F_LDPD_AF_EXPNULL))
 		change_egress_label = 1;
 
 	/* changing config of host only fec filtering */
-	if ((af_conf->flags & F_LDPD_AF_ALLOCHOSTONLY)
-	    != (xa->flags & F_LDPD_AF_ALLOCHOSTONLY))
+	if (CHECK_FLAG(af_conf->flags, F_LDPD_AF_ALLOCHOSTONLY)
+	    != CHECK_FLAG(xa->flags, F_LDPD_AF_ALLOCHOSTONLY))
 		change_host_label = 1;
 
+	/* disabling LDP for address family */
+	if (CHECK_FLAG(af_conf->flags, F_LDPD_AF_ENABLED) &&
+	    !CHECK_FLAG(xa->flags, F_LDPD_AF_ENABLED))
+		change_ldp_disabled = 1;
+
 	af_conf->flags = xa->flags;
 
 	/* update the transport address */
@@ -1377,31 +1443,36 @@ merge_af(int af, struct ldpd_af_conf *af_conf, struct ldpd_af_conf *xa)
 	if (strcmp(af_conf->acl_label_allocate_for, xa->acl_label_allocate_for))
 		change_host_label = 1;
 
-	if (strcmp(af_conf->acl_label_advertise_to,
-	    xa->acl_label_advertise_to) ||
-	    strcmp(af_conf->acl_label_advertise_for,
-	    xa->acl_label_advertise_for) ||
-	    strcmp(af_conf->acl_label_accept_from,
-	    xa->acl_label_accept_from) ||
-	    strcmp(af_conf->acl_label_accept_for,
-	    xa->acl_label_accept_for))
+	if (strcmp(af_conf->acl_label_advertise_to, xa->acl_label_advertise_to) ||
+	    strcmp(af_conf->acl_label_advertise_for, xa->acl_label_advertise_for) ||
+	    strcmp(af_conf->acl_label_accept_from, xa->acl_label_accept_from) ||
+	    strcmp(af_conf->acl_label_accept_for, xa->acl_label_accept_for))
 		reset_nbrs = 1;
+
 	if (strcmp(af_conf->acl_thello_accept_from, xa->acl_thello_accept_from))
 		remove_dynamic_tnbrs = 1;
+
 	if (strcmp(af_conf->acl_label_expnull_for, xa->acl_label_expnull_for))
 		change_egress_label = 1;
+
 	strlcpy(af_conf->acl_thello_accept_from, xa->acl_thello_accept_from,
 	    sizeof(af_conf->acl_thello_accept_from));
+
 	strlcpy(af_conf->acl_label_allocate_for, xa->acl_label_allocate_for,
 	    sizeof(af_conf->acl_label_allocate_for));
+
 	strlcpy(af_conf->acl_label_advertise_to, xa->acl_label_advertise_to,
 	    sizeof(af_conf->acl_label_advertise_to));
+
 	strlcpy(af_conf->acl_label_advertise_for, xa->acl_label_advertise_for,
 	    sizeof(af_conf->acl_label_advertise_for));
+
 	strlcpy(af_conf->acl_label_accept_from, xa->acl_label_accept_from,
 	    sizeof(af_conf->acl_label_accept_from));
+
 	strlcpy(af_conf->acl_label_accept_for, xa->acl_label_accept_for,
 	    sizeof(af_conf->acl_label_accept_for));
+
 	strlcpy(af_conf->acl_label_expnull_for, xa->acl_label_expnull_for,
 	    sizeof(af_conf->acl_label_expnull_for));
 
@@ -1410,8 +1481,13 @@ merge_af(int af, struct ldpd_af_conf *af_conf, struct ldpd_af_conf *xa)
 	case PROC_LDE_ENGINE:
 		if (change_egress_label)
 			lde_change_egress_label(af);
+
 		if (change_host_label)
 			lde_change_allocate_filter(af);
+
+		if (change_ldp_disabled)
+			lde_route_update_release_all(af);
+
 		break;
 	case PROC_LDP_ENGINE:
 		if (stop_init_backoff)
@@ -1437,13 +1513,22 @@ merge_ifaces(struct ldpd_conf *conf, struct ldpd_conf *xconf)
 	struct iface		*iface, *itmp, *xi;
 
 	RB_FOREACH_SAFE(iface, iface_head, &conf->iface_tree, itmp) {
-		/* find deleted interfaces */
+		/* find deleted interfaces, which occurs when LDP is removed
+		 * for all address families
+		 */
 		if (if_lookup_name(xconf, iface->name) == NULL) {
 			switch (ldpd_process) {
 			case PROC_LDP_ENGINE:
 				ldpe_if_exit(iface);
 				break;
 			case PROC_LDE_ENGINE:
+				if (iface->ipv4.enabled)
+					lde_route_update_release(iface,
+					    AF_INET);
+				if (iface->ipv6.enabled)
+					lde_route_update_release(iface,
+					    AF_INET6);
+				break;
 			case PROC_MAIN:
 				break;
 			}
@@ -1471,6 +1556,31 @@ merge_ifaces(struct ldpd_conf *conf, struct ldpd_conf *xconf)
 			continue;
 		}
 
+		/* update labels when adding or removing ldp on an
+		 * interface
+		 */
+		if (ldpd_process == PROC_LDE_ENGINE) {
+			/* if we are removing lpd config for an address
+			 * family on an interface then advertise routes
+			 * learned over this interface as if they were
+			 * connected routes
+			 */
+			if (iface->ipv4.enabled && !xi->ipv4.enabled)
+				lde_route_update_release(iface, AF_INET);
+
+			if (iface->ipv6.enabled && !xi->ipv6.enabled)
+				lde_route_update_release(iface, AF_INET6);
+
+			/* if we are adding lpd config for an address
+			 * family on an interface then add proper labels
+			 */
+			if (!iface->ipv4.enabled && xi->ipv4.enabled)
+				lde_route_update(iface, AF_INET);
+
+			if (!iface->ipv6.enabled && xi->ipv6.enabled)
+				lde_route_update(iface, AF_INET6);
+		}
+
 		/* update existing interfaces */
 		merge_iface_af(&iface->ipv4, &xi->ipv4);
 		merge_iface_af(&iface->ipv6, &xi->ipv6);
@@ -1495,14 +1605,14 @@ merge_tnbrs(struct ldpd_conf *conf, struct ldpd_conf *xconf)
 	struct tnbr		*tnbr, *ttmp, *xt;
 
 	RB_FOREACH_SAFE(tnbr, tnbr_head, &conf->tnbr_tree, ttmp) {
-		if (!(tnbr->flags & F_TNBR_CONFIGURED))
+		if (!CHECK_FLAG(tnbr->flags, F_TNBR_CONFIGURED))
 			continue;
 
 		/* find deleted tnbrs */
 		if (tnbr_find(xconf, tnbr->af, &tnbr->addr) == NULL) {
 			switch (ldpd_process) {
 			case PROC_LDP_ENGINE:
-				tnbr->flags &= ~F_TNBR_CONFIGURED;
+				UNSET_FLAG(tnbr->flags, F_TNBR_CONFIGURED);
 				tnbr_check(conf, tnbr);
 				break;
 			case PROC_LDE_ENGINE:
@@ -1531,8 +1641,8 @@ merge_tnbrs(struct ldpd_conf *conf, struct ldpd_conf *xconf)
 		}
 
 		/* update existing tnbrs */
-		if (!(tnbr->flags & F_TNBR_CONFIGURED))
-			tnbr->flags |= F_TNBR_CONFIGURED;
+		if (!CHECK_FLAG(tnbr->flags, F_TNBR_CONFIGURED))
+			SET_FLAG(tnbr->flags, F_TNBR_CONFIGURED);
 	}
 }
 
@@ -1709,7 +1819,7 @@ merge_l2vpn(struct ldpd_conf *xconf, struct l2vpn *l2vpn, struct l2vpn *xl)
 	previous_pw_type = l2vpn->pw_type;
 	previous_mtu = l2vpn->mtu;
 
-	/* merge intefaces */
+	/* merge interfaces */
 	RB_FOREACH_SAFE(lif, l2vpn_if_head, &l2vpn->if_tree, ftmp) {
 		/* find deleted interfaces */
 		if (l2vpn_if_find(xl, lif->ifname) == NULL) {
@@ -1783,8 +1893,8 @@ merge_l2vpn(struct ldpd_conf *xconf, struct l2vpn *l2vpn, struct l2vpn *xl)
 			reinstall_tnbr = 0;
 
 		/* changes that require a session restart */
-		if ((pw->flags & (F_PW_STATUSTLV_CONF|F_PW_CWORD_CONF)) !=
-		    (xp->flags & (F_PW_STATUSTLV_CONF|F_PW_CWORD_CONF)))
+		if (CHECK_FLAG(pw->flags, (F_PW_STATUSTLV_CONF|F_PW_CWORD_CONF)) !=
+		    CHECK_FLAG(xp->flags, (F_PW_STATUSTLV_CONF|F_PW_CWORD_CONF)))
 			reset_nbr = 1;
 		else
 			reset_nbr = 0;
@@ -1813,20 +1923,24 @@ merge_l2vpn(struct ldpd_conf *xconf, struct l2vpn *l2vpn, struct l2vpn *xl)
 		pw->pwid = xp->pwid;
 		strlcpy(pw->ifname, xp->ifname, sizeof(pw->ifname));
 		pw->ifindex = xp->ifindex;
-		if (xp->flags & F_PW_CWORD_CONF)
-			pw->flags |= F_PW_CWORD_CONF;
+		if (CHECK_FLAG(xp->flags, F_PW_CWORD_CONF))
+			SET_FLAG(pw->flags, F_PW_CWORD_CONF);
 		else
-			pw->flags &= ~F_PW_CWORD_CONF;
-		if (xp->flags & F_PW_STATUSTLV_CONF)
-			pw->flags |= F_PW_STATUSTLV_CONF;
+			UNSET_FLAG(pw->flags, F_PW_CWORD_CONF);
+
+		if (CHECK_FLAG(xp->flags, F_PW_STATUSTLV_CONF))
+			SET_FLAG(pw->flags, F_PW_STATUSTLV_CONF);
 		else
-			pw->flags &= ~F_PW_STATUSTLV_CONF;
-		if (xp->flags & F_PW_STATIC_NBR_ADDR)
-			pw->flags |= F_PW_STATIC_NBR_ADDR;
+			UNSET_FLAG(pw->flags, F_PW_STATUSTLV_CONF);
+
+		if (CHECK_FLAG(xp->flags, F_PW_STATIC_NBR_ADDR))
+			SET_FLAG(pw->flags, F_PW_STATIC_NBR_ADDR);
 		else
-			pw->flags &= ~F_PW_STATIC_NBR_ADDR;
+			UNSET_FLAG(pw->flags, F_PW_STATIC_NBR_ADDR);
+
 		if (ldpd_process == PROC_LDP_ENGINE && reinstall_tnbr)
 			ldpe_l2vpn_pw_init(pw);
+
 		if (ldpd_process == PROC_LDE_ENGINE && reinstall_pwfec) {
 			l2vpn->pw_type = xl->pw_type;
 			l2vpn->mtu = xl->mtu;
