@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Logging - VTY code
  * Copyright (C) 2019 Cumulus Networks, Inc.
  *                    Stephen Worley
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -47,18 +34,23 @@ static int log_cmdline_syslog_lvl = ZLOG_DISABLED;
 
 static struct zlog_cfg_file zt_file_cmdline = {
 	.prio_min = ZLOG_DISABLED,
+	.ts_subsec = LOG_TIMESTAMP_PRECISION,
 };
 static struct zlog_cfg_file zt_file = {
 	.prio_min = ZLOG_DISABLED,
+	.ts_subsec = LOG_TIMESTAMP_PRECISION,
 };
 static struct zlog_cfg_filterfile zt_filterfile = {
-	.parent = {
-		.prio_min = ZLOG_DISABLED,
-	},
+	.parent =
+		{
+			.prio_min = ZLOG_DISABLED,
+			.ts_subsec = LOG_TIMESTAMP_PRECISION,
+		},
 };
 
 static struct zlog_cfg_file zt_stdout_file = {
 	.prio_min = ZLOG_DISABLED,
+	.ts_subsec = LOG_TIMESTAMP_PRECISION,
 };
 static struct zlog_cfg_5424 zt_stdout_journald = {
 	.prio_min = ZLOG_DISABLED,
