@@ -1,21 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* Route map function.
  * Copyright (C) 1998, 1999 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -409,6 +394,40 @@ void route_map_no_set_metric_hook(int (*func)(struct route_map_index *index,
 {
 	rmap_match_set_hook.no_set_metric = func;
 }
+/* set min-metric */
+void route_map_set_min_metric_hook(int (*func)(struct route_map_index *index,
+					       const char *command,
+					       const char *arg, char *errmsg,
+					       size_t errmsg_len))
+{
+	rmap_match_set_hook.set_min_metric = func;
+}
+
+/* no set min-metric */
+void route_map_no_set_min_metric_hook(int (*func)(struct route_map_index *index,
+						  const char *command,
+						  const char *arg, char *errmsg,
+						  size_t errmsg_len))
+{
+	rmap_match_set_hook.no_set_min_metric = func;
+}
+/* set max-metric */
+void route_map_set_max_metric_hook(int (*func)(struct route_map_index *index,
+					       const char *command,
+					       const char *arg, char *errmsg,
+					       size_t errmsg_len))
+{
+	rmap_match_set_hook.set_max_metric = func;
+}
+
+/* no set max-metric */
+void route_map_no_set_max_metric_hook(int (*func)(struct route_map_index *index,
+						  const char *command,
+						  const char *arg, char *errmsg,
+						  size_t errmsg_len))
+{
+	rmap_match_set_hook.no_set_max_metric = func;
+}
 
 /* set tag */
 void route_map_set_tag_hook(int (*func)(struct route_map_index *index,
@@ -615,7 +634,8 @@ static unsigned int route_map_dep_hash_make_key(const void *p);
 static void route_map_clear_all_references(char *rmap_name);
 static void route_map_rule_delete(struct route_map_rule_list *,
 				  struct route_map_rule *);
-static bool rmap_debug;
+
+uint32_t rmap_debug;
 
 /* New route map allocation. Please note route map's name must be
    specified. */
@@ -683,7 +703,7 @@ static struct route_map *route_map_add(const char *name)
 	if (!map->ipv6_prefix_table)
 		map->ipv6_prefix_table = route_table_init();
 
-	if (rmap_debug)
+	if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
 		zlog_debug("Add route-map %s", name);
 	return map;
 }
@@ -703,7 +723,7 @@ static void route_map_free_map(struct route_map *map)
 	while ((index = map->head) != NULL)
 		route_map_index_delete(index, 0);
 
-	if (rmap_debug)
+	if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
 		zlog_debug("Deleting route-map %s", map->name);
 
 	list = &route_map_master;
@@ -720,6 +740,9 @@ static void route_map_free_map(struct route_map *map)
 	else
 		list->head = map->next;
 
+	route_table_finish(map->ipv4_prefix_table);
+	route_table_finish(map->ipv6_prefix_table);
+
 	hash_release(route_map_master_hash, map);
 	XFREE(MTYPE_ROUTE_MAP_NAME, map->name);
 	XFREE(MTYPE_ROUTE_MAP, map);
@@ -1047,17 +1070,20 @@ static int vty_show_route_map(struct vty *vty, const char *name, bool use_json)
 {
 	struct route_map *map;
 	json_object *json = NULL;
+	json_object *json_proto = NULL;
 
-	if (use_json)
+	if (use_json) {
 		json = json_object_new_object();
-	else
+		json_proto = json_object_new_object();
+		json_object_object_add(json, frr_protonameinst, json_proto);
+	} else
 		vty_out(vty, "%s:\n", frr_protonameinst);
 
 	if (name) {
 		map = route_map_lookup_by_name(name);
 
 		if (map) {
-			vty_show_route_map_entry(vty, map, json);
+			vty_show_route_map_entry(vty, map, json_proto);
 		} else if (!use_json) {
 			vty_out(vty, "%s: 'route-map %s' not found\n",
 				frr_protonameinst, name);
@@ -1073,7 +1099,7 @@ static int vty_show_route_map(struct vty *vty, const char *name, bool use_json)
 		list_sort(maplist, sort_route_map);
 
 		for (ALL_LIST_ELEMENTS_RO(maplist, ln, map))
-			vty_show_route_map_entry(vty, map, json);
+			vty_show_route_map_entry(vty, map, json_proto);
 
 		list_delete(&maplist);
 	}
@@ -1131,7 +1157,7 @@ void route_map_index_delete(struct route_map_index *index, int notify)
 
 	QOBJ_UNREG(index);
 
-	if (rmap_debug)
+	if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
 		zlog_debug("Deleting route-map %s sequence %d",
 			   index->map->name, index->pref);
 
@@ -1242,7 +1268,7 @@ route_map_index_add(struct route_map *map, enum route_map_type type, int pref)
 		route_map_notify_dependencies(map->name, RMAP_EVENT_CALL_ADDED);
 	}
 
-	if (rmap_debug)
+	if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
 		zlog_debug("Route-map %s add sequence %d, type: %s",
 			   map->name, pref, route_map_type_str(type));
 
@@ -1753,7 +1779,8 @@ route_map_apply_match(struct route_map_rule_list *match_list,
 					ret = RMAP_MATCH;
 				break;
 
-			default:
+			case RMAP_OKAY:
+			case RMAP_ERROR:
 				break;
 			}
 
@@ -1821,10 +1848,8 @@ route_map_get_index(struct route_map *map, const struct prefix *prefix,
 	 * must be AF_INET or AF_INET6 in order for the lookup to succeed. So if
 	 * the AF doesn't line up with the LPM trees, skip the optimization.
 	 */
-	if (map->optimization_disabled ||
-	    (prefix->family == AF_INET && !map->ipv4_prefix_table) ||
-	    (prefix->family == AF_INET6 && !map->ipv6_prefix_table)) {
-		if (rmap_debug)
+	if (map->optimization_disabled) {
+		if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP_DETAIL)))
 			zlog_debug(
 				"Skipping route-map optimization for route-map: %s, pfx: %pFX, family: %d",
 				map->name, prefix, prefix->family);
@@ -1836,9 +1861,6 @@ route_map_get_index(struct route_map *map, const struct prefix *prefix,
 	else
 		table = map->ipv6_prefix_table;
 
-	if (!table)
-		return NULL;
-
 	do {
 		candidate_rmap_list =
 			route_map_get_index_list(&rn, prefix, table);
@@ -1924,19 +1946,10 @@ static void route_map_pfx_table_add_default(afi_t afi,
 	p.family = afi2family(afi);
 	p.prefixlen = 0;
 
-	if (p.family == AF_INET) {
+	if (p.family == AF_INET)
 		table = index->map->ipv4_prefix_table;
-		if (!table)
-			index->map->ipv4_prefix_table = route_table_init();
-
-		table = index->map->ipv4_prefix_table;
-	} else {
+	else
 		table = index->map->ipv6_prefix_table;
-		if (!table)
-			index->map->ipv6_prefix_table = route_table_init();
-
-		table = index->map->ipv6_prefix_table;
-	}
 
 	/* Add default route to table */
 	rn = route_node_get(table, &p);
@@ -2327,8 +2340,6 @@ static void route_map_pfx_tbl_update(route_map_event_t event,
 				     struct route_map_index *index, afi_t afi,
 				     const char *plist_name)
 {
-	struct route_map *rmap = NULL;
-
 	if (!index)
 		return;
 
@@ -2342,19 +2353,6 @@ static void route_map_pfx_tbl_update(route_map_event_t event,
 		route_map_pfx_table_del_default(AFI_IP, index);
 		route_map_pfx_table_del_default(AFI_IP6, index);
 
-		if ((index->map->head == NULL) && (index->map->tail == NULL)) {
-			rmap = index->map;
-
-			if (rmap->ipv4_prefix_table) {
-				route_table_finish(rmap->ipv4_prefix_table);
-				rmap->ipv4_prefix_table = NULL;
-			}
-
-			if (rmap->ipv6_prefix_table) {
-				route_table_finish(rmap->ipv6_prefix_table);
-				rmap->ipv6_prefix_table = NULL;
-			}
-		}
 		return;
 	}
 
@@ -2553,8 +2551,12 @@ route_map_result_t route_map_apply_ext(struct route_map *map,
 	struct route_map_index *index = NULL;
 	struct route_map_rule *set = NULL;
 	bool skip_match_clause = false;
+	struct prefix conv;
 
 	if (recursion > RMAP_RECURSION_LIMIT) {
+		if (map)
+			map->applied++;
+
 		flog_warn(
 			EC_LIB_RMAP_RECURSION_LIMIT,
 			"route-map recursion limit (%d) reached, discarding route",
@@ -2564,29 +2566,48 @@ route_map_result_t route_map_apply_ext(struct route_map *map,
 	}
 
 	if (map == NULL || map->head == NULL) {
+		if (map)
+			map->applied++;
 		ret = RMAP_DENYMATCH;
 		goto route_map_apply_end;
 	}
 
 	map->applied++;
 
+	/*
+	 * Handling for matching evpn_routes in the prefix table.
+	 *
+	 * We convert type2/5 prefix to ipv4/6 prefix to do longest
+	 * prefix matching on.
+	 */
 	if (prefix->family == AF_EVPN) {
-		index = map->head;
-	} else {
-		skip_match_clause = true;
-		index = route_map_get_index(map, prefix, match_object,
-					    &match_ret);
+		if (evpn_prefix2prefix(prefix, &conv) != 0) {
+			if (unlikely(CHECK_FLAG(rmap_debug,
+						DEBUG_ROUTEMAP_DETAIL)))
+				zlog_debug(
+					"Unable to convert EVPN prefix %pFX into IPv4/IPv6 prefix. Falling back to non-optimized route-map lookup",
+					prefix);
+		} else {
+			if (unlikely(CHECK_FLAG(rmap_debug,
+						DEBUG_ROUTEMAP_DETAIL)))
+				zlog_debug(
+					"Converted EVPN prefix %pFX into %pFX for optimized route-map lookup",
+					prefix, &conv);
+
+			prefix = &conv;
+		}
 	}
 
+	index = route_map_get_index(map, prefix, match_object, &match_ret);
 	if (index) {
 		index->applied++;
-		if (rmap_debug)
+		if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
 			zlog_debug(
 				"Best match route-map: %s, sequence: %d for pfx: %pFX, result: %s",
 				map->name, index->pref, prefix,
 				route_map_cmd_result_str(match_ret));
 	} else {
-		if (rmap_debug)
+		if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
 			zlog_debug(
 				"No best match sequence for pfx: %pFX in route-map: %s, result: %s",
 				prefix, map->name,
@@ -2601,6 +2622,7 @@ route_map_result_t route_map_apply_ext(struct route_map *map,
 			ret = RMAP_DENYMATCH;
 		goto route_map_apply_end;
 	}
+	skip_match_clause = true;
 
 	for (; index; index = index->next) {
 		if (!skip_match_clause) {
@@ -2608,7 +2630,7 @@ route_map_result_t route_map_apply_ext(struct route_map *map,
 			/* Apply this index. */
 			match_ret = route_map_apply_match(&index->match_list,
 							  prefix, match_object);
-			if (rmap_debug) {
+			if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP))) {
 				zlog_debug(
 					"Route-map: %s, sequence: %d, prefix: %pFX, result: %s",
 					map->name, index->pref, prefix,
@@ -2721,7 +2743,7 @@ route_map_result_t route_map_apply_ext(struct route_map *map,
 	}
 
 route_map_apply_end:
-	if (rmap_debug)
+	if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
 		zlog_debug("Route-map: %s, prefix: %pFX, result: %s",
 			   (map ? map->name : "null"), prefix,
 			   route_map_result_str(ret));
@@ -2776,7 +2798,7 @@ static void route_map_clear_reference(struct hash_bucket *bucket, void *arg)
 	tmp_dep_data.rname = arg;
 	dep_data = hash_release(dep->dep_rmap_hash, &tmp_dep_data);
 	if (dep_data) {
-		if (rmap_debug)
+		if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
 			zlog_debug("Clearing reference for %s to %s count: %d",
 				   dep->dep_name, tmp_dep_data.rname,
 				   dep_data->refcnt);
@@ -2796,7 +2818,7 @@ static void route_map_clear_all_references(char *rmap_name)
 {
 	int i;
 
-	if (rmap_debug)
+	if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
 		zlog_debug("Clearing references for %s", rmap_name);
 
 	for (i = 1; i < ROUTE_MAP_DEP_MAX; i++) {
@@ -2872,7 +2894,7 @@ static int route_map_dep_update(struct hash *dephash, const char *dep_name,
 	case RMAP_EVENT_LLIST_ADDED:
 	case RMAP_EVENT_CALL_ADDED:
 	case RMAP_EVENT_FILTER_ADDED:
-		if (rmap_debug)
+		if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
 			zlog_debug("Adding dependency for filter %s in route-map %s",
 				   dep_name, rmap_name);
 		dep = (struct route_map_dep *)hash_get(
@@ -2901,7 +2923,7 @@ static int route_map_dep_update(struct hash *dephash, const char *dep_name,
 	case RMAP_EVENT_LLIST_DELETED:
 	case RMAP_EVENT_CALL_DELETED:
 	case RMAP_EVENT_FILTER_DELETED:
-		if (rmap_debug)
+		if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
 			zlog_debug("Deleting dependency for filter %s in route-map %s",
 				   dep_name, rmap_name);
 		dep = (struct route_map_dep *)hash_get(dephash, dname, NULL);
@@ -2955,7 +2977,7 @@ static int route_map_dep_update(struct hash *dephash, const char *dep_name,
 	}
 
 	if (dep) {
-		if (rmap_debug)
+		if (CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP))
 			hash_iterate(dep->dep_rmap_hash,
 				     route_map_print_dependency, dname);
 	}
@@ -3027,7 +3049,7 @@ static void route_map_process_dependency(struct hash_bucket *bucket, void *data)
 	dep_data = bucket->data;
 	rmap_name = dep_data->rname;
 
-	if (rmap_debug)
+	if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
 		zlog_debug("Notifying %s of dependency", rmap_name);
 	if (route_map_master.event_hook)
 		(*route_map_master.event_hook)(rmap_name);
@@ -3075,7 +3097,7 @@ void route_map_notify_dependencies(const char *affected_name,
 		if (!dep->this_hash)
 			dep->this_hash = upd8_hash;
 
-		if (rmap_debug)
+		if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
 			zlog_debug("Filter %s updated", dep->dep_name);
 		hash_iterate(dep->dep_rmap_hash, route_map_process_dependency,
 			     (void *)event);
@@ -3123,13 +3145,13 @@ DEFPY (rmap_clear_counters,
 
 }
 
-DEFUN_NOSH (rmap_show_name,
-            rmap_show_name_cmd,
-            "show route-map [WORD] [json]",
-            SHOW_STR
-            "route-map information\n"
-            "route-map name\n"
-            JSON_STR)
+DEFUN (rmap_show_name,
+       rmap_show_name_cmd,
+       "show route-map [WORD] [json]",
+       SHOW_STR
+       "route-map information\n"
+       "route-map name\n"
+       JSON_STR)
 {
 	bool uj = use_json(argc, argv);
 	int idx = 0;
@@ -3150,24 +3172,34 @@ DEFUN (rmap_show_unused,
 	return vty_show_unused_route_map(vty);
 }
 
-DEFUN (debug_rmap,
+DEFPY (debug_rmap,
        debug_rmap_cmd,
-       "debug route-map",
+       "debug route-map [detail]$detail",
        DEBUG_STR
-       "Debug option set for route-maps\n")
+       "Debug option set for route-maps\n"
+       "Detailed output\n")
 {
-	rmap_debug = true;
+	if (!detail)
+		SET_FLAG(rmap_debug, DEBUG_ROUTEMAP);
+	else
+		SET_FLAG(rmap_debug, DEBUG_ROUTEMAP | DEBUG_ROUTEMAP_DETAIL);
+
 	return CMD_SUCCESS;
 }
 
-DEFUN (no_debug_rmap,
+DEFPY (no_debug_rmap,
        no_debug_rmap_cmd,
-       "no debug route-map",
+       "no debug route-map [detail]$detail",
        NO_STR
        DEBUG_STR
-       "Debug option set for route-maps\n")
+       "Debug option set for route-maps\n"
+       "Detailed output\n")
 {
-	rmap_debug = false;
+	if (!detail)
+		UNSET_FLAG(rmap_debug, DEBUG_ROUTEMAP);
+	else
+		UNSET_FLAG(rmap_debug, DEBUG_ROUTEMAP | DEBUG_ROUTEMAP_DETAIL);
+
 	return CMD_SUCCESS;
 }
 
@@ -3182,7 +3214,7 @@ static struct cmd_node rmap_debug_node = {
 
 void route_map_show_debug(struct vty *vty)
 {
-	if (rmap_debug)
+	if (CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP))
 		vty_out(vty, "debug route-map\n");
 }
 
@@ -3191,7 +3223,7 @@ static int rmap_config_write_debug(struct vty *vty)
 {
 	int write = 0;
 
-	if (rmap_debug) {
+	if (CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)) {
 		vty_out(vty, "debug route-map\n");
 		write++;
 	}
@@ -3393,7 +3425,7 @@ void route_map_init(void)
 			8, route_map_dep_hash_make_key, route_map_dep_hash_cmp,
 			"Route Map Dep Hash");
 
-	rmap_debug = false;
+	UNSET_FLAG(rmap_debug, DEBUG_ROUTEMAP);
 
 	route_map_cli_init();
 
