@@ -1,23 +1,9 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Route map northbound implementation.
  *
  * Copyright (C) 2019 Network Device Education Foundation, Inc. ("NetDEF")
  *                    Rafael Zalamena
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA.
  */
 
 #include <zebra.h>
@@ -45,8 +31,9 @@ int lib_route_map_entry_match_destroy(struct nb_cb_destroy_args *args)
 	if (rhc->rhc_mhook == NULL)
 		return NB_OK;
 
-	rv = rhc->rhc_mhook(NULL, rhc->rhc_rmi, rhc->rhc_rule, NULL,
-			    rhc->rhc_event);
+	rv = rhc->rhc_mhook(rhc->rhc_rmi, rhc->rhc_rule, NULL,
+			    rhc->rhc_event,
+			    args->errmsg, args->errmsg_len);
 	if (rv != CMD_SUCCESS)
 		return NB_ERR_INCONSISTENCY;
 
@@ -65,7 +52,8 @@ int lib_route_map_entry_set_destroy(struct nb_cb_destroy_args *args)
 	if (rhc->rhc_shook == NULL)
 		return NB_OK;
 
-	rv = rhc->rhc_shook(NULL, rhc->rhc_rmi, rhc->rhc_rule, NULL);
+	rv = rhc->rhc_shook(rhc->rhc_rmi, rhc->rhc_rule, NULL,
+			    args->errmsg, args->errmsg_len);
 	if (rv != CMD_SUCCESS)
 		return NB_ERR_INCONSISTENCY;
 
@@ -138,6 +126,30 @@ static int lib_route_map_destroy(struct nb_cb_destroy_args *args)
 	return NB_OK;
 }
 
+/*
+ * XPath: /frr-route-map:lib/route-map/optimization-disabled
+ */
+static int
+lib_route_map_optimization_disabled_modify(struct nb_cb_modify_args *args)
+{
+	struct route_map *rm;
+	bool disabled = yang_dnode_get_bool(args->dnode, NULL);
+
+	switch (args->event) {
+	case NB_EV_VALIDATE:
+	case NB_EV_PREPARE:
+	case NB_EV_ABORT:
+		/* NOTHING */
+		break;
+	case NB_EV_APPLY:
+		rm = nb_running_get_entry(args->dnode, NULL, true);
+		rm->optimization_disabled = disabled;
+		break;
+	}
+
+	return NB_OK;
+}
+
 /*
  * XPath: /frr-route-map:lib/route-map/entry
  */
@@ -245,6 +257,7 @@ lib_route_map_entry_description_destroy(struct nb_cb_destroy_args *args)
 static int lib_route_map_entry_action_modify(struct nb_cb_modify_args *args)
 {
 	struct route_map_index *rmi;
+	struct route_map *map;
 
 	switch (args->event) {
 	case NB_EV_VALIDATE:
@@ -255,7 +268,15 @@ static int lib_route_map_entry_action_modify(struct nb_cb_modify_args *args)
 	case NB_EV_APPLY:
 		rmi = nb_running_get_entry(args->dnode, NULL, true);
 		rmi->type = yang_dnode_get_enum(args->dnode, NULL);
-		/* TODO: notify? */
+		map = rmi->map;
+
+		/* Execute event hook. */
+		if (route_map_master.event_hook) {
+			(*route_map_master.event_hook)(map->name);
+			route_map_notify_dependencies(map->name,
+						      RMAP_EVENT_CALL_ADDED);
+		}
+
 		break;
 	}
 
@@ -332,6 +353,7 @@ static int
 lib_route_map_entry_exit_policy_modify(struct nb_cb_modify_args *args)
 {
 	struct route_map_index *rmi;
+	struct route_map *map;
 	int rm_action;
 	int policy;
 
@@ -361,6 +383,7 @@ lib_route_map_entry_exit_policy_modify(struct nb_cb_modify_args *args)
 		break;
 	case NB_EV_APPLY:
 		rmi = nb_running_get_entry(args->dnode, NULL, true);
+		map = rmi->map;
 		policy = yang_dnode_get_enum(args->dnode, NULL);
 
 		switch (policy) {
@@ -374,6 +397,14 @@ lib_route_map_entry_exit_policy_modify(struct nb_cb_modify_args *args)
 			rmi->exitpolicy = RMAP_GOTO;
 			break;
 		}
+
+		/* Execute event hook. */
+		if (route_map_master.event_hook) {
+			(*route_map_master.event_hook)(map->name);
+			route_map_notify_dependencies(map->name,
+						      RMAP_EVENT_CALL_ADDED);
+		}
+
 		break;
 	}
 
@@ -498,9 +529,10 @@ static int lib_route_map_entry_match_condition_interface_modify(
 	rhc->rhc_rule = "interface";
 	rhc->rhc_event = RMAP_EVENT_MATCH_DELETED;
 
-	rv = rmap_match_set_hook.match_interface(NULL, rhc->rhc_rmi,
+	rv = rmap_match_set_hook.match_interface(rhc->rhc_rmi,
 						 "interface", ifname,
-						 RMAP_EVENT_MATCH_ADDED);
+						 RMAP_EVENT_MATCH_ADDED,
+						 args->errmsg, args->errmsg_len);
 	if (rv != CMD_SUCCESS) {
 		rhc->rhc_mhook = NULL;
 		return NB_ERR_INCONSISTENCY;
@@ -515,77 +547,6 @@ static int lib_route_map_entry_match_condition_interface_destroy(
 	return lib_route_map_entry_match_destroy(args);
 }
 
-/*
- * XPath: /frr-route-map:lib/route-map/entry/match-condition/access-list-num
- */
-static int lib_route_map_entry_match_condition_access_list_num_modify(
-	struct nb_cb_modify_args *args)
-{
-	struct routemap_hook_context *rhc;
-	const char *acl;
-	int condition, rv;
-
-	if (args->event != NB_EV_APPLY)
-		return NB_OK;
-
-	/* Check for hook function. */
-	rv = CMD_SUCCESS;
-	acl = yang_dnode_get_string(args->dnode, NULL);
-	rhc = nb_running_get_entry(args->dnode, NULL, true);
-	condition = yang_dnode_get_enum(args->dnode, "../condition");
-	switch (condition) {
-	case 1: /* ipv4-address-list */
-		if (rmap_match_set_hook.match_ip_address == NULL)
-			break;
-		rhc->rhc_mhook = rmap_match_set_hook.no_match_ip_address;
-		rhc->rhc_rule = "ip address";
-		rhc->rhc_event = RMAP_EVENT_FILTER_DELETED;
-		rv = rmap_match_set_hook.match_ip_address(
-			NULL, rhc->rhc_rmi, "ip address", acl,
-			RMAP_EVENT_FILTER_ADDED);
-		break;
-	case 3: /* ipv4-next-hop-list */
-		if (rmap_match_set_hook.match_ip_next_hop == NULL)
-			break;
-		rhc->rhc_mhook = rmap_match_set_hook.no_match_ip_next_hop;
-		rhc->rhc_rule = "ip next-hop";
-		rhc->rhc_event = RMAP_EVENT_FILTER_DELETED;
-		rv = rmap_match_set_hook.match_ip_next_hop(
-			NULL, rhc->rhc_rmi, "ip next-hop", acl,
-			RMAP_EVENT_FILTER_ADDED);
-		break;
-	}
-	if (rv != CMD_SUCCESS) {
-		rhc->rhc_mhook = NULL;
-		return NB_ERR_INCONSISTENCY;
-	}
-
-	return NB_OK;
-}
-
-static int lib_route_map_entry_match_condition_access_list_num_destroy(
-	struct nb_cb_destroy_args *args)
-{
-	return lib_route_map_entry_match_destroy(args);
-}
-
-/*
- * XPath:
- * /frr-route-map:lib/route-map/entry/match-condition/access-list-num-extended
- */
-static int lib_route_map_entry_match_condition_access_list_num_extended_modify(
-	struct nb_cb_modify_args *args)
-{
-	return lib_route_map_entry_match_condition_access_list_num_modify(args);
-}
-
-static int lib_route_map_entry_match_condition_access_list_num_extended_destroy(
-	struct nb_cb_destroy_args *args)
-{
-	return lib_route_map_entry_match_condition_access_list_num_destroy(
-		args);
-}
-
 /*
  * XPath: /frr-route-map:lib/route-map/entry/match-condition/list-name
  */
@@ -594,7 +555,7 @@ static int lib_route_map_entry_match_condition_list_name_modify(
 {
 	struct routemap_hook_context *rhc;
 	const char *acl;
-	int condition;
+	const char *condition;
 	int rv;
 
 	if (args->event != NB_EV_APPLY)
@@ -603,19 +564,19 @@ static int lib_route_map_entry_match_condition_list_name_modify(
 	/* Check for hook installation, otherwise we can just stop. */
 	acl = yang_dnode_get_string(args->dnode, NULL);
 	rhc = nb_running_get_entry(args->dnode, NULL, true);
-	condition = yang_dnode_get_enum(args->dnode, "../condition");
-	switch (condition) {
-	case 1: /* ipv4-address-list */
+	condition = yang_dnode_get_string(args->dnode, "../../condition");
+
+	if (IS_MATCH_IPv4_ADDRESS_LIST(condition)) {
 		if (rmap_match_set_hook.match_ip_address == NULL)
 			return NB_OK;
 		rhc->rhc_mhook = rmap_match_set_hook.no_match_ip_address;
 		rhc->rhc_rule = "ip address";
 		rhc->rhc_event = RMAP_EVENT_FILTER_DELETED;
 		rv = rmap_match_set_hook.match_ip_address(
-			NULL, rhc->rhc_rmi, "ip address", acl,
-			RMAP_EVENT_FILTER_ADDED);
-		break;
-	case 2: /* ipv4-prefix-list */
+			rhc->rhc_rmi, "ip address", acl,
+			RMAP_EVENT_FILTER_ADDED,
+			args->errmsg, args->errmsg_len);
+	} else if (IS_MATCH_IPv4_PREFIX_LIST(condition)) {
 		if (rmap_match_set_hook.match_ip_address_prefix_list == NULL)
 			return NB_OK;
 		rhc->rhc_mhook =
@@ -623,20 +584,30 @@ static int lib_route_map_entry_match_condition_list_name_modify(
 		rhc->rhc_rule = "ip address prefix-list";
 		rhc->rhc_event = RMAP_EVENT_PLIST_DELETED;
 		rv = rmap_match_set_hook.match_ip_address_prefix_list(
-			NULL, rhc->rhc_rmi, "ip address prefix-list", acl,
-			RMAP_EVENT_PLIST_ADDED);
-		break;
-	case 3: /* ipv4-next-hop-list */
+			rhc->rhc_rmi, "ip address prefix-list", acl,
+			RMAP_EVENT_PLIST_ADDED,
+			args->errmsg, args->errmsg_len);
+	} else if (IS_MATCH_IPv4_NEXTHOP_LIST(condition)) {
 		if (rmap_match_set_hook.match_ip_next_hop == NULL)
 			return NB_OK;
 		rhc->rhc_mhook = rmap_match_set_hook.no_match_ip_next_hop;
 		rhc->rhc_rule = "ip next-hop";
 		rhc->rhc_event = RMAP_EVENT_FILTER_DELETED;
 		rv = rmap_match_set_hook.match_ip_next_hop(
-			NULL, rhc->rhc_rmi, "ip next-hop", acl,
-			RMAP_EVENT_FILTER_ADDED);
-		break;
-	case 4: /* ipv4-next-hop-prefix-list */
+			rhc->rhc_rmi, "ip next-hop", acl,
+			RMAP_EVENT_FILTER_ADDED,
+			args->errmsg, args->errmsg_len);
+	} else if (IS_MATCH_IPv6_NEXTHOP_LIST(condition)) {
+		if (rmap_match_set_hook.match_ipv6_next_hop == NULL)
+			return NB_OK;
+		rhc->rhc_mhook = rmap_match_set_hook.no_match_ipv6_next_hop;
+		rhc->rhc_rule = "ipv6 next-hop";
+		rhc->rhc_event = RMAP_EVENT_FILTER_DELETED;
+		rv = rmap_match_set_hook.match_ipv6_next_hop(
+			rhc->rhc_rmi, "ipv6 next-hop", acl,
+			RMAP_EVENT_FILTER_ADDED, args->errmsg,
+			args->errmsg_len);
+	} else if (IS_MATCH_IPv4_NEXTHOP_PREFIX_LIST(condition)) {
 		if (rmap_match_set_hook.match_ip_next_hop_prefix_list == NULL)
 			return NB_OK;
 		rhc->rhc_mhook =
@@ -644,20 +615,30 @@ static int lib_route_map_entry_match_condition_list_name_modify(
 		rhc->rhc_rule = "ip next-hop prefix-list";
 		rhc->rhc_event = RMAP_EVENT_PLIST_DELETED;
 		rv = rmap_match_set_hook.match_ip_next_hop_prefix_list(
-			NULL, rhc->rhc_rmi, "ip next-hop prefix-list", acl,
-			RMAP_EVENT_PLIST_ADDED);
-		break;
-	case 6: /* ipv6-address-list */
+			rhc->rhc_rmi, "ip next-hop prefix-list", acl,
+			RMAP_EVENT_PLIST_ADDED,
+			args->errmsg, args->errmsg_len);
+	} else if (IS_MATCH_IPv6_NEXTHOP_PREFIX_LIST(condition)) {
+		if (rmap_match_set_hook.match_ipv6_next_hop_prefix_list == NULL)
+			return NB_OK;
+		rhc->rhc_mhook =
+			rmap_match_set_hook.no_match_ipv6_next_hop_prefix_list;
+		rhc->rhc_rule = "ipv6 next-hop prefix-list";
+		rhc->rhc_event = RMAP_EVENT_PLIST_DELETED;
+		rv = rmap_match_set_hook.match_ipv6_next_hop_prefix_list(
+			rhc->rhc_rmi, "ipv6 next-hop prefix-list", acl,
+			RMAP_EVENT_PLIST_ADDED, args->errmsg, args->errmsg_len);
+	} else if (IS_MATCH_IPv6_ADDRESS_LIST(condition)) {
 		if (rmap_match_set_hook.match_ipv6_address == NULL)
 			return NB_OK;
 		rhc->rhc_mhook = rmap_match_set_hook.no_match_ipv6_address;
 		rhc->rhc_rule = "ipv6 address";
 		rhc->rhc_event = RMAP_EVENT_FILTER_DELETED;
 		rv = rmap_match_set_hook.match_ipv6_address(
-			NULL, rhc->rhc_rmi, "ipv6 address", acl,
-			RMAP_EVENT_FILTER_ADDED);
-		break;
-	case 7: /* ipv6-prefix-list */
+			rhc->rhc_rmi, "ipv6 address", acl,
+			RMAP_EVENT_FILTER_ADDED,
+			args->errmsg, args->errmsg_len);
+	} else if (IS_MATCH_IPv6_PREFIX_LIST(condition)) {
 		if (rmap_match_set_hook.match_ipv6_address_prefix_list == NULL)
 			return NB_OK;
 		rhc->rhc_mhook =
@@ -665,13 +646,12 @@ static int lib_route_map_entry_match_condition_list_name_modify(
 		rhc->rhc_rule = "ipv6 address prefix-list";
 		rhc->rhc_event = RMAP_EVENT_PLIST_DELETED;
 		rv = rmap_match_set_hook.match_ipv6_address_prefix_list(
-			NULL, rhc->rhc_rmi, "ipv6 address prefix-list", acl,
-			RMAP_EVENT_PLIST_ADDED);
-		break;
-	default:
+			rhc->rhc_rmi, "ipv6 address prefix-list", acl,
+			RMAP_EVENT_PLIST_ADDED,
+			args->errmsg, args->errmsg_len);
+	} else
 		rv = CMD_ERR_NO_MATCH;
-		break;
-	}
+
 	if (rv != CMD_SUCCESS) {
 		rhc->rhc_mhook = NULL;
 		return NB_ERR_INCONSISTENCY;
@@ -713,8 +693,9 @@ static int lib_route_map_entry_match_condition_ipv4_next_hop_type_modify(
 	rhc->rhc_event = RMAP_EVENT_MATCH_DELETED;
 
 	rv = rmap_match_set_hook.match_ip_next_hop_type(
-		NULL, rhc->rhc_rmi, "ip next-hop type", type,
-		RMAP_EVENT_MATCH_ADDED);
+		rhc->rhc_rmi, "ip next-hop type", type,
+		RMAP_EVENT_MATCH_ADDED,
+		args->errmsg, args->errmsg_len);
 	if (rv != CMD_SUCCESS) {
 		rhc->rhc_mhook = NULL;
 		return NB_ERR_INCONSISTENCY;
@@ -756,8 +737,9 @@ static int lib_route_map_entry_match_condition_ipv6_next_hop_type_modify(
 	rhc->rhc_event = RMAP_EVENT_MATCH_DELETED;
 
 	rv = rmap_match_set_hook.match_ipv6_next_hop_type(
-		NULL, rhc->rhc_rmi, "ipv6 next-hop type", type,
-		RMAP_EVENT_MATCH_ADDED);
+		rhc->rhc_rmi, "ipv6 next-hop type", type,
+		RMAP_EVENT_MATCH_ADDED,
+		args->errmsg, args->errmsg_len);
 	if (rv != CMD_SUCCESS) {
 		rhc->rhc_mhook = NULL;
 		return NB_ERR_INCONSISTENCY;
@@ -798,8 +780,9 @@ static int lib_route_map_entry_match_condition_metric_modify(
 	rhc->rhc_rule = "metric";
 	rhc->rhc_event = RMAP_EVENT_MATCH_DELETED;
 
-	rv = rmap_match_set_hook.match_metric(NULL, rhc->rhc_rmi, "metric",
-					      type, RMAP_EVENT_MATCH_ADDED);
+	rv = rmap_match_set_hook.match_metric(rhc->rhc_rmi, "metric",
+					      type, RMAP_EVENT_MATCH_ADDED,
+					      args->errmsg, args->errmsg_len);
 	if (rv != CMD_SUCCESS) {
 		rhc->rhc_mhook = NULL;
 		return NB_ERR_INCONSISTENCY;
@@ -840,8 +823,9 @@ lib_route_map_entry_match_condition_tag_modify(struct nb_cb_modify_args *args)
 	rhc->rhc_rule = "tag";
 	rhc->rhc_event = RMAP_EVENT_MATCH_DELETED;
 
-	rv = rmap_match_set_hook.match_tag(NULL, rhc->rhc_rmi, "tag", tag,
-					   RMAP_EVENT_MATCH_ADDED);
+	rv = rmap_match_set_hook.match_tag(rhc->rhc_rmi, "tag", tag,
+					   RMAP_EVENT_MATCH_ADDED,
+					   args->errmsg, args->errmsg_len);
 	if (rv != CMD_SUCCESS) {
 		rhc->rhc_mhook = NULL;
 		return NB_ERR_INCONSISTENCY;
@@ -899,7 +883,7 @@ static int lib_route_map_entry_set_action_ipv4_address_modify(
 		 * only implemented action.
 		 */
 		yang_dnode_get_ipv4(&ia, args->dnode, NULL);
-		if (ia.s_addr == INADDR_ANY || IPV4_CLASS_DE(ntohl(ia.s_addr)))
+		if (ia.s_addr == INADDR_ANY || !ipv4_unicast_valid(&ia))
 			return NB_ERR_VALIDATION;
 		/* FALLTHROUGH */
 	case NB_EV_PREPARE:
@@ -921,8 +905,9 @@ static int lib_route_map_entry_set_action_ipv4_address_modify(
 	rhc->rhc_shook = rmap_match_set_hook.no_set_ip_nexthop;
 	rhc->rhc_rule = "ip next-hop";
 
-	rv = rmap_match_set_hook.set_ip_nexthop(NULL, rhc->rhc_rmi,
-						"ip next-hop", address);
+	rv = rmap_match_set_hook.set_ip_nexthop(rhc->rhc_rmi, "ip next-hop",
+						address,
+						args->errmsg, args->errmsg_len);
 	if (rv != CMD_SUCCESS) {
 		rhc->rhc_shook = NULL;
 		return NB_ERR_INCONSISTENCY;
@@ -980,7 +965,8 @@ static int lib_route_map_entry_set_action_ipv6_address_modify(
 	rhc->rhc_rule = "ipv6 next-hop local";
 
 	rv = rmap_match_set_hook.set_ipv6_nexthop_local(
-		NULL, rhc->rhc_rmi, "ipv6 next-hop local", address);
+		rhc->rhc_rmi, "ipv6 next-hop local", address,
+		args->errmsg, args->errmsg_len);
 	if (rv != CMD_SUCCESS) {
 		rhc->rhc_shook = NULL;
 		return NB_ERR_INCONSISTENCY;
@@ -999,7 +985,8 @@ static int lib_route_map_entry_set_action_ipv6_address_destroy(
  * XPath: /frr-route-map:lib/route-map/entry/set-action/value
  */
 static int set_action_modify(enum nb_event event, const struct lyd_node *dnode,
-			     union nb_resource *resource, const char *value)
+			     union nb_resource *resource, const char *value,
+			     char *errmsg, size_t errmsg_len)
 {
 	struct routemap_hook_context *rhc;
 	int rv;
@@ -1023,8 +1010,10 @@ static int set_action_modify(enum nb_event event, const struct lyd_node *dnode,
 	rhc->rhc_shook = rmap_match_set_hook.no_set_metric;
 	rhc->rhc_rule = "metric";
 
-	rv = rmap_match_set_hook.set_metric(NULL, rhc->rhc_rmi, "metric",
-					    value);
+	rv = rmap_match_set_hook.set_metric(rhc->rhc_rmi, "metric",
+					    value,
+					    errmsg, errmsg_len
+					    );
 	if (rv != CMD_SUCCESS) {
 		rhc->rhc_shook = NULL;
 		return NB_ERR_INCONSISTENCY;
@@ -1039,7 +1028,7 @@ lib_route_map_entry_set_action_value_modify(struct nb_cb_modify_args *args)
 	const char *metric = yang_dnode_get_string(args->dnode, NULL);
 
 	return set_action_modify(args->event, args->dnode, args->resource,
-				 metric);
+				 metric, args->errmsg, args->errmsg_len);
 }
 
 static int
@@ -1048,14 +1037,130 @@ lib_route_map_entry_set_action_value_destroy(struct nb_cb_destroy_args *args)
 	return lib_route_map_entry_set_destroy(args);
 }
 
+/*
+ * XPath: /frr-route-map:lib/route-map/entry/set-action/min-metric
+ */
+static int set_action_min_metric_modify(enum nb_event event,
+					const struct lyd_node *dnode,
+					union nb_resource *resource,
+					const char *value, char *errmsg,
+					size_t errmsg_len)
+{
+	struct routemap_hook_context *rhc;
+	int rv;
+
+	if (event != NB_EV_APPLY)
+		return NB_OK;
+
+	/* Check for hook function. */
+	if (rmap_match_set_hook.set_min_metric == NULL)
+		return NB_OK;
+
+	/* Add configuration. */
+	rhc = nb_running_get_entry(dnode, NULL, true);
+
+	/* Set destroy information. */
+	rhc->rhc_shook = rmap_match_set_hook.no_set_min_metric;
+	rhc->rhc_rule = "min-metric";
+
+	rv = rmap_match_set_hook.set_min_metric(rhc->rhc_rmi, "min-metric",
+						value, errmsg, errmsg_len);
+	if (rv != CMD_SUCCESS) {
+		rhc->rhc_shook = NULL;
+		return NB_ERR_INCONSISTENCY;
+	}
+
+	return NB_OK;
+}
+
+static int
+lib_route_map_entry_set_action_min_metric_modify(struct nb_cb_modify_args *args)
+{
+	const char *min_metric = yang_dnode_get_string(args->dnode, NULL);
+
+	return set_action_min_metric_modify(args->event, args->dnode,
+					    args->resource, min_metric,
+					    args->errmsg, args->errmsg_len);
+}
+
+static int lib_route_map_entry_set_action_min_metric_destroy(
+	struct nb_cb_destroy_args *args)
+{
+	return lib_route_map_entry_set_destroy(args);
+}
+
+/*
+ * XPath: /frr-route-map:lib/route-map/entry/set-action/max-metric
+ */
+static int set_action_max_metric_modify(enum nb_event event,
+					const struct lyd_node *dnode,
+					union nb_resource *resource,
+					const char *value, char *errmsg,
+					size_t errmsg_len)
+{
+	struct routemap_hook_context *rhc;
+	int rv;
+
+	if (event != NB_EV_APPLY)
+		return NB_OK;
+
+	/* Check for hook function. */
+	if (rmap_match_set_hook.set_max_metric == NULL)
+		return NB_OK;
+
+	/* Add configuration. */
+	rhc = nb_running_get_entry(dnode, NULL, true);
+
+	/* Set destroy information. */
+	rhc->rhc_shook = rmap_match_set_hook.no_set_max_metric;
+	rhc->rhc_rule = "max-metric";
+
+	rv = rmap_match_set_hook.set_max_metric(rhc->rhc_rmi, "max-metric",
+						value, errmsg, errmsg_len);
+	if (rv != CMD_SUCCESS) {
+		rhc->rhc_shook = NULL;
+		return NB_ERR_INCONSISTENCY;
+	}
+
+	return NB_OK;
+}
+
+static int
+lib_route_map_entry_set_action_max_metric_modify(struct nb_cb_modify_args *args)
+{
+	const char *max_metric = yang_dnode_get_string(args->dnode, NULL);
+
+	return set_action_max_metric_modify(args->event, args->dnode,
+					    args->resource, max_metric,
+					    args->errmsg, args->errmsg_len);
+}
+
+static int lib_route_map_entry_set_action_max_metric_destroy(
+	struct nb_cb_destroy_args *args)
+{
+	return lib_route_map_entry_set_destroy(args);
+}
+
 /*
  * XPath: /frr-route-map:lib/route-map/entry/set-action/add-metric
  */
 static int
 lib_route_map_entry_set_action_add_metric_modify(struct nb_cb_modify_args *args)
 {
+	char metric_str[16];
+
+	if (args->event == NB_EV_VALIDATE
+	    && yang_dnode_get_uint32(args->dnode, NULL) == 0) {
+		snprintf(args->errmsg, args->errmsg_len,
+			 "Can't add zero to metric");
+		return NB_ERR_VALIDATION;
+	}
+
+	snprintf(metric_str, sizeof(metric_str), "+%s",
+		 yang_dnode_get_string(args->dnode, NULL));
 	return set_action_modify(args->event, args->dnode, args->resource,
-				 "+metric");
+				 metric_str,
+				 args->errmsg, args->errmsg_len);
 }
 
 static int lib_route_map_entry_set_action_add_metric_destroy(
@@ -1070,8 +1175,20 @@ static int lib_route_map_entry_set_action_add_metric_destroy(
 static int lib_route_map_entry_set_action_subtract_metric_modify(
 	struct nb_cb_modify_args *args)
 {
+	char metric_str[16];
+
+	if (args->event == NB_EV_VALIDATE
+	    && yang_dnode_get_uint32(args->dnode, NULL) == 0) {
+		snprintf(args->errmsg, args->errmsg_len,
+			 "Can't subtract zero from metric");
+		return NB_ERR_VALIDATION;
+	}
+
+	snprintf(metric_str, sizeof(metric_str), "-%s",
+		 yang_dnode_get_string(args->dnode, NULL));
 	return set_action_modify(args->event, args->dnode, args->resource,
-				 "-metric");
+				 metric_str,
+				 args->errmsg, args->errmsg_len);
 }
 
 static int lib_route_map_entry_set_action_subtract_metric_destroy(
@@ -1087,7 +1204,8 @@ static int lib_route_map_entry_set_action_use_round_trip_time_modify(
 	struct nb_cb_modify_args *args)
 {
 	return set_action_modify(args->event, args->dnode, args->resource,
-				 "rtt");
+				 "rtt",
+				 args->errmsg, args->errmsg_len);
 }
 
 static int lib_route_map_entry_set_action_use_round_trip_time_destroy(
@@ -1103,7 +1221,8 @@ static int lib_route_map_entry_set_action_add_round_trip_time_modify(
 	struct nb_cb_modify_args *args)
 {
 	return set_action_modify(args->event, args->dnode, args->resource,
-				 "+rtt");
+				 "+rtt",
+				 args->errmsg, args->errmsg_len);
 }
 
 static int lib_route_map_entry_set_action_add_round_trip_time_destroy(
@@ -1119,7 +1238,7 @@ static int lib_route_map_entry_set_action_subtract_round_trip_time_modify(
 	struct nb_cb_modify_args *args)
 {
 	return set_action_modify(args->event, args->dnode, args->resource,
-				 "-rtt");
+				 "-rtt", args->errmsg, args->errmsg_len);
 }
 
 static int lib_route_map_entry_set_action_subtract_round_trip_time_destroy(
@@ -1158,7 +1277,8 @@ lib_route_map_entry_set_action_tag_modify(struct nb_cb_modify_args *args)
 	rhc->rhc_shook = rmap_match_set_hook.no_set_tag;
 	rhc->rhc_rule = "tag";
 
-	rv = rmap_match_set_hook.set_tag(NULL, rhc->rhc_rmi, "tag", tag);
+	rv = rmap_match_set_hook.set_tag(rhc->rhc_rmi, "tag", tag,
+					 args->errmsg, args->errmsg_len);
 	if (rv != CMD_SUCCESS) {
 		rhc->rhc_shook = NULL;
 		return NB_ERR_INCONSISTENCY;
@@ -1173,6 +1293,52 @@ lib_route_map_entry_set_action_tag_destroy(struct nb_cb_destroy_args *args)
 	return lib_route_map_entry_set_destroy(args);
 }
 
+/*
+ * XPath: /frr-route-map:lib/route-map/entry/set-action/policy
+ */
+static int
+lib_route_map_entry_set_action_policy_modify(struct nb_cb_modify_args *args)
+{
+	struct routemap_hook_context *rhc;
+	const char *policy;
+	int rv;
+
+	/*
+	 * NOTE: validate if 'action' is 'tag', currently it is not
+	 * necessary because this is the only implemented action. Other
+	 * actions might have different validations.
+	 */
+	if (args->event != NB_EV_APPLY)
+		return NB_OK;
+
+	/* Check for hook function. */
+	if (rmap_match_set_hook.set_srte_color == NULL)
+		return NB_OK;
+
+	/* Add configuration. */
+	rhc = nb_running_get_entry(args->dnode, NULL, true);
+	policy = yang_dnode_get_string(args->dnode, NULL);
+
+	/* Set destroy information. */
+	rhc->rhc_shook = rmap_match_set_hook.no_set_tag;
+	rhc->rhc_rule = "sr-te color";
+
+	rv = rmap_match_set_hook.set_tag(rhc->rhc_rmi, "sr-te color", policy,
+			args->errmsg, args->errmsg_len);
+	if (rv != CMD_SUCCESS) {
+		rhc->rhc_shook = NULL;
+		return NB_ERR_INCONSISTENCY;
+	}
+
+	return NB_OK;
+}
+
+static int
+lib_route_map_entry_set_action_policy_destroy(struct nb_cb_destroy_args *args)
+{
+	return lib_route_map_entry_set_destroy(args);
+}
+
 /* clang-format off */
 const struct frr_yang_module_info frr_route_map_info = {
 	.name = "frr-route-map",
@@ -1184,11 +1350,19 @@ const struct frr_yang_module_info frr_route_map_info = {
 				.destroy = lib_route_map_destroy,
 			}
 		},
+		{
+			.xpath = "/frr-route-map:lib/route-map/optimization-disabled",
+			.cbs = {
+				.modify = lib_route_map_optimization_disabled_modify,
+				.cli_show = route_map_optimization_disabled_show,
+			}
+		},
 		{
 			.xpath = "/frr-route-map:lib/route-map/entry",
 			.cbs = {
 				.create = lib_route_map_entry_create,
 				.destroy = lib_route_map_entry_destroy,
+				.cli_cmp = route_map_instance_cmp,
 				.cli_show = route_map_instance_show,
 				.cli_show_end = route_map_instance_show_end,
 			}
@@ -1238,56 +1412,42 @@ const struct frr_yang_module_info frr_route_map_info = {
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/interface",
+			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/rmap-match-condition/interface",
 			.cbs = {
 				.modify = lib_route_map_entry_match_condition_interface_modify,
 				.destroy = lib_route_map_entry_match_condition_interface_destroy,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/access-list-num",
-			.cbs = {
-				.modify = lib_route_map_entry_match_condition_access_list_num_modify,
-				.destroy = lib_route_map_entry_match_condition_access_list_num_destroy,
-			}
-		},
-		{
-			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/access-list-num-extended",
-			.cbs = {
-				.modify = lib_route_map_entry_match_condition_access_list_num_extended_modify,
-				.destroy = lib_route_map_entry_match_condition_access_list_num_extended_destroy,
-			}
-		},
-		{
-			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/list-name",
+			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/rmap-match-condition/list-name",
 			.cbs = {
 				.modify = lib_route_map_entry_match_condition_list_name_modify,
 				.destroy = lib_route_map_entry_match_condition_list_name_destroy,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/ipv4-next-hop-type",
+			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/rmap-match-condition/ipv4-next-hop-type",
 			.cbs = {
 				.modify = lib_route_map_entry_match_condition_ipv4_next_hop_type_modify,
 				.destroy = lib_route_map_entry_match_condition_ipv4_next_hop_type_destroy,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/ipv6-next-hop-type",
+			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/rmap-match-condition/ipv6-next-hop-type",
 			.cbs = {
 				.modify = lib_route_map_entry_match_condition_ipv6_next_hop_type_modify,
 				.destroy = lib_route_map_entry_match_condition_ipv6_next_hop_type_destroy,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/metric",
+			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/rmap-match-condition/metric",
 			.cbs = {
 				.modify = lib_route_map_entry_match_condition_metric_modify,
 				.destroy = lib_route_map_entry_match_condition_metric_destroy,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/tag",
+			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/rmap-match-condition/tag",
 			.cbs = {
 				.modify = lib_route_map_entry_match_condition_tag_modify,
 				.destroy = lib_route_map_entry_match_condition_tag_destroy,
@@ -1302,68 +1462,90 @@ const struct frr_yang_module_info frr_route_map_info = {
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/set-action/ipv4-address",
+			.xpath = "/frr-route-map:lib/route-map/entry/set-action/rmap-set-action/ipv4-address",
 			.cbs = {
 				.modify = lib_route_map_entry_set_action_ipv4_address_modify,
 				.destroy = lib_route_map_entry_set_action_ipv4_address_destroy,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/set-action/ipv6-address",
+			.xpath = "/frr-route-map:lib/route-map/entry/set-action/rmap-set-action/ipv6-address",
 			.cbs = {
 				.modify = lib_route_map_entry_set_action_ipv6_address_modify,
 				.destroy = lib_route_map_entry_set_action_ipv6_address_destroy,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/set-action/value",
+			.xpath = "/frr-route-map:lib/route-map/entry/set-action/rmap-set-action/value",
 			.cbs = {
 				.modify = lib_route_map_entry_set_action_value_modify,
 				.destroy = lib_route_map_entry_set_action_value_destroy,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/set-action/add-metric",
+			.xpath = "/frr-route-map:lib/route-map/entry/set-action/rmap-set-action/min-metric",
+			.cbs = {
+				.modify = lib_route_map_entry_set_action_min_metric_modify,
+				.destroy = lib_route_map_entry_set_action_min_metric_destroy,
+			}
+		},
+		{
+			.xpath = "/frr-route-map:lib/route-map/entry/set-action/rmap-set-action/max-metric",
+			.cbs = {
+				.modify = lib_route_map_entry_set_action_max_metric_modify,
+				.destroy = lib_route_map_entry_set_action_max_metric_destroy,
+			}
+		},
+		{
+			.xpath = "/frr-route-map:lib/route-map/entry/set-action/rmap-set-action/add-metric",
 			.cbs = {
 				.modify = lib_route_map_entry_set_action_add_metric_modify,
 				.destroy = lib_route_map_entry_set_action_add_metric_destroy,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/set-action/subtract-metric",
+			.xpath = "/frr-route-map:lib/route-map/entry/set-action/rmap-set-action/subtract-metric",
 			.cbs = {
 				.modify = lib_route_map_entry_set_action_subtract_metric_modify,
 				.destroy = lib_route_map_entry_set_action_subtract_metric_destroy,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/set-action/use-round-trip-time",
+			.xpath = "/frr-route-map:lib/route-map/entry/set-action/rmap-set-action/use-round-trip-time",
 			.cbs = {
 				.modify = lib_route_map_entry_set_action_use_round_trip_time_modify,
 				.destroy = lib_route_map_entry_set_action_use_round_trip_time_destroy,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/set-action/add-round-trip-time",
+			.xpath = "/frr-route-map:lib/route-map/entry/set-action/rmap-set-action/add-round-trip-time",
 			.cbs = {
 				.modify = lib_route_map_entry_set_action_add_round_trip_time_modify,
 				.destroy = lib_route_map_entry_set_action_add_round_trip_time_destroy,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/set-action/subtract-round-trip-time",
+			.xpath = "/frr-route-map:lib/route-map/entry/set-action/rmap-set-action/subtract-round-trip-time",
 			.cbs = {
 				.modify = lib_route_map_entry_set_action_subtract_round_trip_time_modify,
 				.destroy = lib_route_map_entry_set_action_subtract_round_trip_time_destroy,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/set-action/tag",
+			.xpath = "/frr-route-map:lib/route-map/entry/set-action/rmap-set-action/tag",
 			.cbs = {
 				.modify = lib_route_map_entry_set_action_tag_modify,
 				.destroy = lib_route_map_entry_set_action_tag_destroy,
 			}
 		},
+		{
+			.xpath = "/frr-route-map:lib/route-map/entry/set-action/rmap-set-action/policy",
+			.cbs = {
+				.modify = lib_route_map_entry_set_action_policy_modify,
+				.destroy = lib_route_map_entry_set_action_policy_destroy,
+			}
+		},
+
 		{
 			.xpath = NULL,
 		},
