@@ -1,21 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* Generic vector interface routine
  * Copyright (C) 1997 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -23,8 +8,8 @@
 #include "vector.h"
 #include "memory.h"
 
-DEFINE_MTYPE_STATIC(LIB, VECTOR, "Vector")
-DEFINE_MTYPE_STATIC(LIB, VECTOR_INDEX, "Vector index")
+DEFINE_MTYPE_STATIC(LIB, VECTOR, "Vector");
+DEFINE_MTYPE_STATIC(LIB, VECTOR_INDEX, "Vector index");
 
 /* Initialize vector : allocate memory and return vector. */
 vector vector_init(unsigned int size)
@@ -37,6 +22,7 @@ vector vector_init(unsigned int size)
 
 	v->alloced = size;
 	v->active = 0;
+	v->count = 0;
 	v->index = XCALLOC(MTYPE_VECTOR_INDEX, sizeof(void *) * size);
 	return v;
 }
@@ -54,6 +40,7 @@ vector vector_copy(vector v)
 
 	new->active = v->active;
 	new->alloced = v->alloced;
+	new->count = v->count;
 
 	size = sizeof(void *) * (v->alloced);
 	new->index = XCALLOC(MTYPE_VECTOR_INDEX, size);
@@ -84,6 +71,9 @@ int vector_empty_slot(vector v)
 {
 	unsigned int i;
 
+	if (v->active == v->count)
+		return v->active;
+
 	if (v->active == 0)
 		return 0;
 
@@ -102,6 +92,10 @@ int vector_set(vector v, void *val)
 	i = vector_empty_slot(v);
 	vector_ensure(v, i);
 
+	if (v->index[i])
+		v->count--;
+	if (val)
+		v->count++;
 	v->index[i] = val;
 
 	if (v->active <= i)
@@ -115,6 +109,10 @@ int vector_set_index(vector v, unsigned int i, void *val)
 {
 	vector_ensure(v, i);
 
+	if (v->index[i])
+		v->count--;
+	if (val)
+		v->count++;
 	v->index[i] = val;
 
 	if (v->active <= i)
@@ -144,6 +142,9 @@ void vector_unset(vector v, unsigned int i)
 	if (i >= v->alloced)
 		return;
 
+	if (v->index[i])
+		v->count--;
+
 	v->index[i] = NULL;
 
 	if (i + 1 == v->active) {
@@ -153,6 +154,30 @@ void vector_unset(vector v, unsigned int i)
 	}
 }
 
+void vector_remove(vector v, unsigned int ix)
+{
+	if (ix >= v->active)
+		return;
+
+	if (v->index[ix])
+		v->count--;
+
+	int n = (--v->active) - ix;
+
+	memmove(&v->index[ix], &v->index[ix + 1], n * sizeof(void *));
+	v->index[v->active] = NULL;
+}
+
+void vector_compact(vector v)
+{
+	for (unsigned int i = 0; i < vector_active(v); ++i) {
+		if (vector_slot(v, i) == NULL) {
+			vector_remove(v, i);
+			--i;
+		}
+	}
+}
+
 void vector_unset_value(vector v, void *val)
 {
 	size_t i;
@@ -160,6 +185,7 @@ void vector_unset_value(vector v, void *val)
 	for (i = 0; i < v->active; i++)
 		if (v->index[i] == val) {
 			v->index[i] = NULL;
+			v->count--;
 			break;
 		}
 
@@ -169,15 +195,18 @@ void vector_unset_value(vector v, void *val)
 		while (i && v->index[--i] == NULL);
 }
 
-/* Count the number of not emplty slot. */
-unsigned int vector_count(vector v)
+void vector_to_array(vector v, void ***dest, int *argc)
 {
-	unsigned int i;
-	unsigned count = 0;
+	*dest = XCALLOC(MTYPE_TMP, sizeof(void *) * v->active);
+	memcpy(*dest, v->index, sizeof(void *) * v->active);
+	*argc = v->active;
+}
 
-	for (i = 0; i < v->active; i++)
-		if (v->index[i] != NULL)
-			count++;
+vector array_to_vector(void **src, int argc)
+{
+	vector v = vector_init(VECTOR_MIN_SIZE);
 
-	return count;
+	for (int i = 0; i < argc; i++)
+		vector_set_index(v, i, src[i]);
+	return v;
 }
