@@ -1,16 +1,16 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* NHRP packet handling functions
  * Copyright (c) 2014-2015 Timo Teräs
- *
- * This file is free software: you may copy, redistribute and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 2 of the License, or
- * (at your option) any later version.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <netinet/if_ether.h>
 #include "nhrpd.h"
 #include "zbuf.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "hash.h"
 
 #include "nhrp_protocol.h"
@@ -164,7 +164,7 @@ struct nhrp_cie_header *nhrp_cie_pull(struct zbuf *zb,
 	if (!cie)
 		return NULL;
 
-	if (cie->nbma_address_len + cie->nbma_subaddress_len) {
+	if (cie->nbma_address_len + cie->nbma_subaddress_len > 0) {
 		sockunion_set(nbma, afi2family(htons(hdr->afnum)),
 			      zbuf_pulln(zb,
 					 cie->nbma_address_len
@@ -264,6 +264,7 @@ int nhrp_ext_reply(struct zbuf *zb, struct nhrp_packet_header *hdr,
 				    &ad->addr);
 		if (!cie)
 			goto err;
+		cie->mtu = htons(ad->mtu);
 		cie->holding_time = htons(ad->holdtime);
 		break;
 	default:
@@ -285,9 +286,9 @@ err:
 	return -1;
 }
 
-static int nhrp_packet_recvraw(struct thread *t)
+static void nhrp_packet_recvraw(struct event *t)
 {
-	int fd = THREAD_FD(t), ifindex;
+	int fd = EVENT_FD(t), ifindex;
 	struct zbuf *zb;
 	struct interface *ifp;
 	struct nhrp_peer *p;
@@ -295,11 +296,11 @@ static int nhrp_packet_recvraw(struct thread *t)
 	uint8_t addr[64];
 	size_t len, addrlen;
 
-	thread_add_read(master, nhrp_packet_recvraw, 0, fd, NULL);
+	event_add_read(master, nhrp_packet_recvraw, 0, fd, NULL);
 
 	zb = zbuf_alloc(1500);
 	if (!zb)
-		return 0;
+		return;
 
 	len = zbuf_size(zb);
 	addrlen = sizeof(addr);
@@ -327,15 +328,14 @@ static int nhrp_packet_recvraw(struct thread *t)
 
 	nhrp_peer_recv(p, zb);
 	nhrp_peer_unref(p);
-	return 0;
+	return;
 
 err:
 	zbuf_free(zb);
-	return 0;
 }
 
 int nhrp_packet_init(void)
 {
-	thread_add_read(master, nhrp_packet_recvraw, 0, os_socket(), NULL);
+	event_add_read(master, nhrp_packet_recvraw, 0, os_socket(), NULL);
 	return 0;
 }
