@@ -1,10 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* NHRP routing functions
  * Copyright (c) 2014-2015 Timo Teräs
- *
- * This file is free software: you may copy, redistribute and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 2 of the License, or
- * (at your option) any later version.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -18,7 +14,7 @@
 #include "log.h"
 #include "zclient.h"
 
-DEFINE_MTYPE_STATIC(NHRPD, NHRP_ROUTE, "NHRP routing entry")
+DEFINE_MTYPE_STATIC(NHRPD, NHRP_ROUTE, "NHRP routing entry");
 
 static struct zclient *zclient;
 static struct route_table *zebra_rib[AFI_MAX];
@@ -56,7 +52,7 @@ static void nhrp_route_update_put(struct route_node *rn)
 	struct route_info *ri = rn->info;
 
 	if (!ri->ifp && !ri->nhrp_ifp
-	    && sockunion_family(&ri->via) == AF_UNSPEC) {
+	    && sockunion_is_null(&ri->via)) {
 		XFREE(MTYPE_NHRP_ROUTE, rn->info);
 		route_unlock_node(rn);
 	}
@@ -70,8 +66,7 @@ static void nhrp_route_update_zebra(const struct prefix *p,
 	struct route_node *rn;
 	struct route_info *ri;
 
-	rn = nhrp_route_update_get(
-		p, (sockunion_family(nexthop) != AF_UNSPEC) || ifp);
+	rn = nhrp_route_update_get(p, !sockunion_is_null(nexthop) || ifp);
 	if (rn) {
 		ri = rn->info;
 		ri->via = *nexthop;
@@ -80,6 +75,24 @@ static void nhrp_route_update_zebra(const struct prefix *p,
 	}
 }
 
+static void nhrp_zebra_register_neigh(vrf_id_t vrf_id, afi_t afi, bool reg)
+{
+	struct stream *s;
+
+	if (!zclient || zclient->sock < 0)
+		return;
+
+	s = zclient->obuf;
+	stream_reset(s);
+
+	zclient_create_header(s, reg ? ZEBRA_NHRP_NEIGH_REGISTER :
+			      ZEBRA_NHRP_NEIGH_UNREGISTER,
+			      vrf_id);
+	stream_putw(s, afi);
+	stream_putw_at(s, 0, stream_get_endp(s));
+	zclient_send_message(zclient);
+}
+
 void nhrp_route_update_nhrp(const struct prefix *p, struct interface *ifp)
 {
 	struct route_node *rn;
@@ -95,7 +108,7 @@ void nhrp_route_update_nhrp(const struct prefix *p, struct interface *ifp)
 
 void nhrp_route_announce(int add, enum nhrp_cache_type type,
 			 const struct prefix *p, struct interface *ifp,
-			 const union sockunion *nexthop, uint32_t mtu)
+			 const union sockunion *nexthop_ref, uint32_t mtu)
 {
 	struct zapi_route api;
 	struct zapi_nexthop *api_nh;
@@ -111,9 +124,10 @@ void nhrp_route_announce(int add, enum nhrp_cache_type type,
 
 	switch (type) {
 	case NHRP_CACHE_NEGATIVE:
+		/* Fill in a blackhole nexthop */
 		zapi_route_set_blackhole(&api, BLACKHOLE_REJECT);
 		ifp = NULL;
-		nexthop = NULL;
+		nexthop_ref = NULL;
 		break;
 	case NHRP_CACHE_DYNAMIC:
 	case NHRP_CACHE_NHS:
@@ -121,7 +135,17 @@ void nhrp_route_announce(int add, enum nhrp_cache_type type,
 		/* Regular route, so these are announced
 		 * to other routing daemons */
 		break;
-	default:
+	case NHRP_CACHE_INVALID:
+	case NHRP_CACHE_INCOMPLETE:
+		/*
+		 * I cannot believe that we want to set a FIB_OVERRIDE
+		 * for invalid state or incomplete.  But this matches
+		 * the original code.  Someone will probably notice
+		 * the problem eventually
+		 */
+	case NHRP_CACHE_CACHED:
+	case NHRP_CACHE_LOCAL:
+	case NHRP_CACHE_NUM_TYPES:
 		SET_FLAG(api.flags, ZEBRA_FLAG_FIB_OVERRIDE);
 		break;
 	}
@@ -134,8 +158,14 @@ void nhrp_route_announce(int add, enum nhrp_cache_type type,
 
 	switch (api.prefix.family) {
 	case AF_INET:
-		if (nexthop) {
-			api_nh->gate.ipv4 = nexthop->sin.sin_addr;
+		if (api.prefix.prefixlen == IPV4_MAX_BITLEN &&
+		    nexthop_ref &&
+		    memcmp(&nexthop_ref->sin.sin_addr, &api.prefix.u.prefix4,
+			   sizeof(struct in_addr)) == 0) {
+			nexthop_ref = NULL;
+		}
+		if (nexthop_ref) {
+			api_nh->gate.ipv4 = nexthop_ref->sin.sin_addr;
 			api_nh->type = NEXTHOP_TYPE_IPV4;
 		}
 		if (ifp) {
@@ -147,8 +177,14 @@ void nhrp_route_announce(int add, enum nhrp_cache_type type,
 		}
 		break;
 	case AF_INET6:
-		if (nexthop) {
-			api_nh->gate.ipv6 = nexthop->sin6.sin6_addr;
+		if (api.prefix.prefixlen == IPV6_MAX_BITLEN &&
+		    nexthop_ref &&
+		    memcmp(&nexthop_ref->sin6.sin6_addr, &api.prefix.u.prefix6,
+			   sizeof(struct in6_addr)) == 0) {
+			nexthop_ref = NULL;
+		}
+		if (nexthop_ref) {
+			api_nh->gate.ipv6 = nexthop_ref->sin6.sin6_addr;
 			api_nh->type = NEXTHOP_TYPE_IPV6;
 		}
 		if (ifp) {
@@ -166,15 +202,14 @@ void nhrp_route_announce(int add, enum nhrp_cache_type type,
 	}
 
 	if (unlikely(debug_flags & NHRP_DEBUG_ROUTE)) {
-		char buf[2][PREFIX_STRLEN];
+		char buf[PREFIX_STRLEN];
 
-		prefix2str(&api.prefix, buf[0], sizeof(buf[0]));
 		zlog_debug(
-			"Zebra send: route %s %s nexthop %s metric %u"
-			" count %d dev %s",
-			add ? "add" : "del", buf[0],
-			nexthop ? inet_ntop(api.prefix.family, &api_nh->gate,
-					    buf[1], sizeof(buf[1]))
+			"Zebra send: route %s %pFX nexthop %s metric %u count %d dev %s",
+			add ? "add" : "del", &api.prefix,
+			nexthop_ref ? inet_ntop(api.prefix.family,
+						&api_nh->gate,
+						buf, sizeof(buf))
 				: "<onlink>",
 			api.metric, api.nexthop_num, ifp ? ifp->name : "none");
 	}
@@ -189,7 +224,6 @@ int nhrp_route_read(ZAPI_CALLBACK_ARGS)
 	struct zapi_nexthop *api_nh;
 	struct interface *ifp = NULL;
 	union sockunion nexthop_addr;
-	char buf[2][PREFIX_STRLEN];
 	int added;
 
 	if (zapi_route_decode(zclient->ibuf, &api) < 0)
@@ -199,6 +233,10 @@ int nhrp_route_read(ZAPI_CALLBACK_ARGS)
 	if (CHECK_FLAG(api.message, ZAPI_MESSAGE_SRCPFX))
 		return 0;
 
+	/* ignore our routes */
+	if (api.type == ZEBRA_ROUTE_NHRP)
+		return 0;
+
 	sockunion_family(&nexthop_addr) = AF_UNSPEC;
 	if (CHECK_FLAG(api.message, ZAPI_MESSAGE_NEXTHOP)) {
 		api_nh = &api.nexthops[0];
@@ -218,13 +256,11 @@ int nhrp_route_read(ZAPI_CALLBACK_ARGS)
 	}
 
 	added = (cmd == ZEBRA_REDISTRIBUTE_ROUTE_ADD);
-	debugf(NHRP_DEBUG_ROUTE, "if-route-%s: %s via %s dev %s",
-	       added ? "add" : "del",
-	       prefix2str(&api.prefix, buf[0], sizeof(buf[0])),
-	       sockunion2str(&nexthop_addr, buf[1], sizeof(buf[1])),
+	debugf(NHRP_DEBUG_ROUTE, "if-route-%s: %pFX via %pSU dev %s",
+	       added ? "add" : "del", &api.prefix, &nexthop_addr,
 	       ifp ? ifp->name : "(none)");
 
-	nhrp_route_update_zebra(&api.prefix, &nexthop_addr, ifp);
+	nhrp_route_update_zebra(&api.prefix, &nexthop_addr, added ? ifp : NULL);
 	nhrp_shortcut_prefix_change(&api.prefix, !added);
 
 	return 0;
@@ -237,7 +273,6 @@ int nhrp_route_get_nexthop(const union sockunion *addr, struct prefix *p,
 	struct route_info *ri;
 	struct prefix lookup;
 	afi_t afi = family2afi(sockunion_family(addr));
-	char buf[PREFIX_STRLEN];
 
 	sockunion2hostprefix(addr, &lookup);
 
@@ -247,8 +282,7 @@ int nhrp_route_get_nexthop(const union sockunion *addr, struct prefix *p,
 
 	ri = rn->info;
 	if (ri->nhrp_ifp) {
-		debugf(NHRP_DEBUG_ROUTE, "lookup %s: nhrp_if=%s",
-		       prefix2str(&lookup, buf, sizeof(buf)),
+		debugf(NHRP_DEBUG_ROUTE, "lookup %pFX: nhrp_if=%s", &lookup,
 		       ri->nhrp_ifp->name);
 
 		if (via)
@@ -256,9 +290,8 @@ int nhrp_route_get_nexthop(const union sockunion *addr, struct prefix *p,
 		if (ifp)
 			*ifp = ri->nhrp_ifp;
 	} else {
-		debugf(NHRP_DEBUG_ROUTE, "lookup %s: zebra route dev %s",
-		       prefix2str(&lookup, buf, sizeof(buf)),
-		       ri->ifp ? ri->ifp->name : "(none)");
+		debugf(NHRP_DEBUG_ROUTE, "lookup %pFX: zebra route dev %s",
+		       &lookup, ri->ifp ? ri->ifp->name : "(none)");
 
 		if (via)
 			*via = ri->via;
@@ -335,27 +368,163 @@ static void nhrp_zebra_connected(struct zclient *zclient)
 				ZEBRA_ROUTE_ALL, 0, VRF_DEFAULT);
 	zebra_redistribute_send(ZEBRA_REDISTRIBUTE_ADD, zclient, AFI_IP6,
 				ZEBRA_ROUTE_ALL, 0, VRF_DEFAULT);
+	nhrp_zebra_register_neigh(VRF_DEFAULT, AFI_IP, true);
+	nhrp_zebra_register_neigh(VRF_DEFAULT, AFI_IP6, true);
 }
 
+static zclient_handler *const nhrp_handlers[] = {
+	[ZEBRA_INTERFACE_ADDRESS_ADD] = nhrp_interface_address_add,
+	[ZEBRA_INTERFACE_ADDRESS_DELETE] = nhrp_interface_address_delete,
+	[ZEBRA_REDISTRIBUTE_ROUTE_ADD] = nhrp_route_read,
+	[ZEBRA_REDISTRIBUTE_ROUTE_DEL] = nhrp_route_read,
+	[ZEBRA_NHRP_NEIGH_ADDED] = nhrp_neighbor_operation,
+	[ZEBRA_NHRP_NEIGH_REMOVED] = nhrp_neighbor_operation,
+	[ZEBRA_NHRP_NEIGH_GET] = nhrp_neighbor_operation,
+	[ZEBRA_GRE_UPDATE] = nhrp_gre_update,
+};
+
 void nhrp_zebra_init(void)
 {
 	zebra_rib[AFI_IP] = route_table_init();
 	zebra_rib[AFI_IP6] = route_table_init();
 
-	zclient = zclient_new(master, &zclient_options_default);
+	zclient = zclient_new(master, &zclient_options_default, nhrp_handlers,
+			      array_size(nhrp_handlers));
 	zclient->zebra_connected = nhrp_zebra_connected;
-	zclient->interface_address_add = nhrp_interface_address_add;
-	zclient->interface_address_delete = nhrp_interface_address_delete;
-	zclient->redistribute_route_add = nhrp_route_read;
-	zclient->redistribute_route_del = nhrp_route_read;
-
 	zclient_init(zclient, ZEBRA_ROUTE_NHRP, 0, &nhrpd_privs);
 }
 
+static void nhrp_table_node_cleanup(struct route_table *table,
+				    struct route_node *node)
+{
+	if (!node->info)
+		return;
+
+	XFREE(MTYPE_NHRP_ROUTE, node->info);
+}
+
+void nhrp_send_zebra_configure_arp(struct interface *ifp, int family)
+{
+	struct stream *s;
+
+	if (!zclient || zclient->sock < 0) {
+		debugf(NHRP_DEBUG_COMMON, "%s() : zclient not ready",
+		       __func__);
+		return;
+	}
+	s = zclient->obuf;
+	stream_reset(s);
+	zclient_create_header(s, ZEBRA_CONFIGURE_ARP, ifp->vrf->vrf_id);
+	stream_putc(s, family);
+	stream_putl(s, ifp->ifindex);
+	stream_putw_at(s, 0, stream_get_endp(s));
+	zclient_send_message(zclient);
+}
+
+void nhrp_send_zebra_gre_source_set(struct interface *ifp,
+				    unsigned int link_idx,
+				    vrf_id_t link_vrf_id)
+{
+	struct stream *s;
+
+	if (!zclient || zclient->sock < 0) {
+		zlog_err("%s : zclient not ready", __func__);
+		return;
+	}
+	if (link_idx == IFINDEX_INTERNAL || link_vrf_id == VRF_UNKNOWN) {
+		/* silently ignore */
+		return;
+	}
+	s = zclient->obuf;
+	stream_reset(s);
+	zclient_create_header(s, ZEBRA_GRE_SOURCE_SET, ifp->vrf->vrf_id);
+	stream_putl(s, ifp->ifindex);
+	stream_putl(s, link_idx);
+	stream_putl(s, link_vrf_id);
+	stream_putl(s, 0); /* mtu provisioning */
+	stream_putw_at(s, 0, stream_get_endp(s));
+	zclient_send_message(zclient);
+}
+
+void nhrp_send_zebra_nbr(union sockunion *in,
+			 union sockunion *out,
+			 struct interface *ifp)
+{
+	struct stream *s;
+
+	if (!zclient || zclient->sock < 0)
+		return;
+	s = zclient->obuf;
+	stream_reset(s);
+	zclient_neigh_ip_encode(s, out ? ZEBRA_NEIGH_IP_ADD :
+				ZEBRA_NEIGH_IP_DEL, in, out,
+				ifp, out ? ZEBRA_NEIGH_STATE_REACHABLE
+				: ZEBRA_NEIGH_STATE_FAILED);
+	stream_putw_at(s, 0, stream_get_endp(s));
+	zclient_send_message(zclient);
+}
+
+int nhrp_send_zebra_gre_request(struct interface *ifp)
+{
+	return zclient_send_zebra_gre_request(zclient, ifp);
+}
+
 void nhrp_zebra_terminate(void)
 {
+	nhrp_zebra_register_neigh(VRF_DEFAULT, AFI_IP, false);
+	nhrp_zebra_register_neigh(VRF_DEFAULT, AFI_IP6, false);
 	zclient_stop(zclient);
 	zclient_free(zclient);
+
+	zebra_rib[AFI_IP]->cleanup = nhrp_table_node_cleanup;
+	zebra_rib[AFI_IP6]->cleanup = nhrp_table_node_cleanup;
 	route_table_finish(zebra_rib[AFI_IP]);
 	route_table_finish(zebra_rib[AFI_IP6]);
 }
+
+int nhrp_gre_update(ZAPI_CALLBACK_ARGS)
+{
+	struct stream *s;
+	struct nhrp_gre_info gre_info, *val;
+	struct interface *ifp;
+
+	/* result */
+	s = zclient->ibuf;
+	if (vrf_id != VRF_DEFAULT)
+		return 0;
+
+	/* read GRE information */
+	STREAM_GETL(s, gre_info.ifindex);
+	STREAM_GETL(s, gre_info.ikey);
+	STREAM_GETL(s, gre_info.okey);
+	STREAM_GETL(s, gre_info.ifindex_link);
+	STREAM_GETL(s, gre_info.vrfid_link);
+	STREAM_GETL(s, gre_info.vtep_ip.s_addr);
+	STREAM_GETL(s, gre_info.vtep_ip_remote.s_addr);
+	if (gre_info.ifindex == IFINDEX_INTERNAL)
+		val = NULL;
+	else
+		val = hash_lookup(nhrp_gre_list, &gre_info);
+	if (val) {
+		if (gre_info.vtep_ip.s_addr != val->vtep_ip.s_addr ||
+		    gre_info.vrfid_link != val->vrfid_link ||
+		    gre_info.ifindex_link != val->ifindex_link ||
+		    gre_info.ikey != val->ikey ||
+		    gre_info.okey != val->okey) {
+			/* update */
+			memcpy(val, &gre_info, sizeof(struct nhrp_gre_info));
+		}
+	} else {
+		val = nhrp_gre_info_alloc(&gre_info);
+	}
+	ifp = if_lookup_by_index(gre_info.ifindex, vrf_id);
+	debugf(NHRP_DEBUG_EVENT, "%s: gre interface %d vr %d obtained from system",
+	       ifp ? ifp->name : "<none>", gre_info.ifindex, vrf_id);
+	if (ifp)
+		nhrp_interface_update_nbma(ifp, val);
+	return 0;
+
+stream_failure:
+	zlog_err("%s(): error reading response ..", __func__);
+	return -1;
+}
