@@ -1,10 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* NHRP daemon internal structures and function prototypes
  * Copyright (c) 2014-2015 Timo Teräs
- *
- * This file is free software: you may copy, redistribute and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 2 of the License, or
- * (at your option) any later version.
  */
 
 #ifndef NHRPD_H
@@ -23,7 +19,7 @@ DECLARE_MGROUP(NHRPD);
 #define NHRP_VTY_PORT		2610
 #define NHRP_DEFAULT_CONFIG	"nhrpd.conf"
 
-extern struct thread_master *master;
+extern struct event_loop *master;
 
 enum { NHRP_OK = 0,
        NHRP_ERR_FAIL,
@@ -170,9 +166,9 @@ struct nhrp_peer {
 	struct notifier_list notifier_list;
 	struct interface *ifp;
 	struct nhrp_vc *vc;
-	struct thread *t_fallback;
+	struct event *t_fallback;
 	struct notifier_block vc_notifier, ifp_notifier;
-	struct thread *t_timer;
+	struct event *t_timer;
 };
 
 struct nhrp_packet_parser {
@@ -236,8 +232,8 @@ struct nhrp_cache {
 	struct notifier_block newpeer_notifier;
 	struct notifier_list notifier_list;
 	struct nhrp_reqid eventid;
-	struct thread *t_timeout;
-	struct thread *t_auth;
+	struct event *t_timeout;
+	struct event *t_auth;
 
 	struct {
 		enum nhrp_cache_type type;
@@ -255,7 +251,7 @@ struct nhrp_shortcut {
 	union sockunion addr;
 
 	struct nhrp_reqid reqid;
-	struct thread *t_timer;
+	struct event *t_timer;
 
 	enum nhrp_cache_type type;
 	unsigned int holding_time;
@@ -279,7 +275,7 @@ struct nhrp_nhs {
 	union sockunion proto_addr;
 	const char *nbma_fqdn; /* IP-address or FQDN */
 
-	struct thread *t_resolve;
+	struct event *t_resolve;
 	struct resolver_query dns_resolve;
 	struct nhrp_reglist_head reglist_head;
 };
@@ -297,7 +293,7 @@ DECLARE_DLIST(nhrp_mcastlist, struct nhrp_multicast, mcastlist_entry);
 
 struct nhrp_registration {
 	struct nhrp_reglist_item reglist_entry;
-	struct thread *t_register;
+	struct event *t_register;
 	struct nhrp_nhs *nhs;
 	struct nhrp_reqid reqid;
 	unsigned int timeout;
