@@ -1,29 +1,15 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 1999 Yasuhiro Ohara
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 #include <lib/version.h>
+#include <lib/keychain.h>
 #include <stdlib.h>
 
 #include "getopt.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "log.h"
 #include "command.h"
 #include "vty.h"
@@ -47,6 +33,7 @@
 #include "ospf6_lsa.h"
 #include "ospf6_interface.h"
 #include "ospf6_zebra.h"
+#include "ospf6_routemap_nb.h"
 
 /* Default configuration file name for ospf6d. */
 #define OSPF6_DEFAULT_CONFIG       "ospf6d.conf"
@@ -55,7 +42,7 @@
 #define OSPF6_VTY_PORT             2606
 
 /* ospf6d privileges */
-zebra_capabilities_t _caps_p[] = {ZCAP_NET_RAW, ZCAP_BIND};
+zebra_capabilities_t _caps_p[] = {ZCAP_NET_RAW, ZCAP_BIND, ZCAP_SYS_ADMIN};
 
 struct zebra_privs_t ospf6d_privs = {
 #if defined(FRR_USER)
@@ -68,38 +55,39 @@ struct zebra_privs_t ospf6d_privs = {
 	.vty_group = VTY_GROUP,
 #endif
 	.caps_p = _caps_p,
-	.cap_num_p = 2,
+	.cap_num_p = array_size(_caps_p),
 	.cap_num_i = 0};
 
 /* ospf6d options, we use GNU getopt library. */
 struct option longopts[] = {{0}};
 
 /* Master of threads. */
-struct thread_master *master;
+struct event_loop *master;
 
 static void __attribute__((noreturn)) ospf6_exit(int status)
 {
-	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
+	struct vrf *vrf;
 	struct interface *ifp;
+	struct ospf6 *ospf6;
+	struct listnode *node, *nnode;
 
 	frr_early_fini();
 
-	if (ospf6) {
+	bfd_protocol_integration_set_shutdown(true);
+
+	for (ALL_LIST_ELEMENTS(om6->ospf6, node, nnode, ospf6)) {
+		vrf = vrf_lookup_by_id(ospf6->vrf_id);
 		ospf6_delete(ospf6);
 		ospf6 = NULL;
+		FOR_ALL_INTERFACES (vrf, ifp)
+			if (ifp->info != NULL)
+				ospf6_interface_delete(ifp->info);
 	}
 
-	bfd_gbl_exit();
-
-	FOR_ALL_INTERFACES (vrf, ifp)
-		if (ifp->info != NULL)
-			ospf6_interface_delete(ifp->info);
-
 	ospf6_message_terminate();
 	ospf6_asbr_terminate();
 	ospf6_lsa_terminate();
 
-	ospf6_serv_close();
 	/* reverse access_list_init */
 	access_list_reset();
 
@@ -146,7 +134,7 @@ static void sigusr1(void)
 	zlog_rotate();
 }
 
-struct quagga_signal_t ospf6_signals[] = {
+struct frr_signal_t ospf6_signals[] = {
 	{
 		.signal = SIGHUP,
 		.handler = &sighup,
@@ -166,9 +154,12 @@ struct quagga_signal_t ospf6_signals[] = {
 };
 
 static const struct frr_yang_module_info *const ospf6d_yang_modules[] = {
+	&frr_filter_info,
 	&frr_interface_info,
 	&frr_route_map_info,
 	&frr_vrf_info,
+	&frr_ospf_route_map_info,
+	&frr_ospf6_route_map_info,
 };
 
 FRR_DAEMON_INFO(ospf6d, OSPF6, .vty_port = OSPF6_VTY_PORT,
@@ -179,7 +170,34 @@ FRR_DAEMON_INFO(ospf6d, OSPF6, .vty_port = OSPF6_VTY_PORT,
 		.n_signals = array_size(ospf6_signals),
 
 		.privs = &ospf6d_privs, .yang_modules = ospf6d_yang_modules,
-		.n_yang_modules = array_size(ospf6d_yang_modules), )
+		.n_yang_modules = array_size(ospf6d_yang_modules),
+);
+
+/* Max wait time for config to load before accepting hellos */
+#define OSPF6_PRE_CONFIG_MAX_WAIT_SECONDS 600
+
+static void ospf6_config_finish(struct event *t)
+{
+	zlog_err("OSPF6 configuration end timer expired after %d seconds.",
+		 OSPF6_PRE_CONFIG_MAX_WAIT_SECONDS);
+}
+
+static void ospf6_config_start(void)
+{
+	if (IS_OSPF6_DEBUG_EVENT)
+		zlog_debug("ospf6d config start received");
+	EVENT_OFF(t_ospf6_cfg);
+	event_add_timer(master, ospf6_config_finish, NULL,
+			OSPF6_PRE_CONFIG_MAX_WAIT_SECONDS, &t_ospf6_cfg);
+}
+
+static void ospf6_config_end(void)
+{
+	if (IS_OSPF6_DEBUG_EVENT)
+		zlog_debug("ospf6d config end received");
+
+	EVENT_OFF(t_ospf6_cfg);
+}
 
 /* Main routine of ospf6d. Treatment of argument and starting ospf finite
    state machine is handled here. */
@@ -202,7 +220,6 @@ int main(int argc, char *argv[], char *envp[])
 			break;
 		default:
 			frr_help_exit(1);
-			break;
 		}
 	}
 
@@ -213,17 +230,21 @@ int main(int argc, char *argv[], char *envp[])
 	}
 
 	/* OSPF6 master init. */
-	ospf6_master_init();
+	ospf6_master_init(frr_init());
 
 	/* thread master */
-	master = frr_init();
+	master = om6->master;
 
-	vrf_init(NULL, NULL, NULL, NULL, NULL);
+	keychain_init();
+	ospf6_vrf_init();
 	access_list_init();
 	prefix_list_init();
 
 	/* initialize ospf6 */
-	ospf6_init();
+	ospf6_init(master);
+
+	/* Configuration processing callback initialization. */
+	cmd_init_config_callbacks(ospf6_config_start, ospf6_config_end);
 
 	frr_config_fork();
 	frr_run(master);
