@@ -1,27 +1,12 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * OSPF Interface functions.
  * Copyright (C) 1999, 2000 Toshiaki Takada
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published
- * by the Free Software Foundation; either version 2, or (at your
- * option) any later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
-#include "thread.h"
+#include "frrevent.h"
 #include "linklist.h"
 #include "prefix.h"
 #include "if.h"
@@ -286,6 +271,10 @@ struct ospf_interface *ospf_if_new(struct ospf *ospf, struct interface *ifp,
 
 	QOBJ_REG(oi, ospf_interface);
 
+	/* If first oi, check per-intf write socket */
+	if (ospf->oi_running && ospf->intf_socket_enabled)
+		ospf_ifp_sock_init(ifp);
+
 	if (IS_DEBUG_OSPF_EVENT)
 		zlog_debug("%s: ospf interface %s vrf %s id %u created",
 			   __func__, ifp->name, ospf_get_name(ospf),
@@ -307,7 +296,7 @@ void ospf_if_cleanup(struct ospf_interface *oi)
 	/* oi->nbrs and oi->nbr_nbma should be deleted on InterfaceDown event */
 	/* delete all static neighbors attached to this interface */
 	for (ALL_LIST_ELEMENTS(oi->nbr_nbma, node, nnode, nbr_nbma)) {
-		THREAD_OFF(nbr_nbma->t_poll);
+		EVENT_OFF(nbr_nbma->t_poll);
 
 		if (nbr_nbma->nbr) {
 			nbr_nbma->nbr->nbr_nbma = NULL;
@@ -342,6 +331,8 @@ void ospf_if_cleanup(struct ospf_interface *oi)
 
 void ospf_if_free(struct ospf_interface *oi)
 {
+	struct interface *ifp = oi->ifp;
+
 	ospf_if_down(oi);
 
 	ospf_fifo_free(oi->obuf);
@@ -374,7 +365,11 @@ void ospf_if_free(struct ospf_interface *oi)
 	listnode_delete(oi->ospf->oiflist, oi);
 	listnode_delete(oi->area->oiflist, oi);
 
-	thread_cancel_event(master, oi);
+	event_cancel_event(master, oi);
+
+	/* If last oi, close per-interface socket */
+	if (ospf_oi_count(ifp) == 0)
+		ospf_ifp_sock_close(ifp);
 
 	memset(oi, 0, sizeof(*oi));
 	XFREE(MTYPE_OSPF_IF, oi);
@@ -510,7 +505,7 @@ void ospf_interface_fifo_flush(struct ospf_interface *oi)
 	if (oi->on_write_q) {
 		listnode_delete(ospf->oi_write_q, oi);
 		if (list_isempty(ospf->oi_write_q))
-			THREAD_OFF(ospf->t_write);
+			EVENT_OFF(ospf->t_write);
 		oi->on_write_q = 0;
 	}
 }
@@ -545,6 +540,7 @@ static struct ospf_if_params *ospf_new_if_params(void)
 	UNSET_IF_PARAM(oip, passive_interface);
 	UNSET_IF_PARAM(oip, v_hello);
 	UNSET_IF_PARAM(oip, fast_hello);
+	UNSET_IF_PARAM(oip, v_gr_hello_delay);
 	UNSET_IF_PARAM(oip, v_wait);
 	UNSET_IF_PARAM(oip, priority);
 	UNSET_IF_PARAM(oip, type);
@@ -552,6 +548,8 @@ static struct ospf_if_params *ospf_new_if_params(void)
 	UNSET_IF_PARAM(oip, auth_crypt);
 	UNSET_IF_PARAM(oip, auth_type);
 	UNSET_IF_PARAM(oip, if_area);
+	UNSET_IF_PARAM(oip, opaque_capable);
+	UNSET_IF_PARAM(oip, keychain_name);
 
 	oip->auth_crypt = list_new();
 
@@ -559,6 +557,8 @@ static struct ospf_if_params *ospf_new_if_params(void)
 	oip->is_v_wait_set = false;
 
 	oip->ptp_dmvpn = 0;
+	oip->p2mp_delay_reflood = OSPF_P2MP_DELAY_REFLOOD_DEFAULT;
+	oip->opaque_capable = OSPF_OPAQUE_CAPABLE_DEFAULT;
 
 	return oip;
 }
@@ -567,6 +567,7 @@ static void ospf_del_if_params(struct interface *ifp,
 			       struct ospf_if_params *oip)
 {
 	list_delete(&oip->auth_crypt);
+	XFREE(MTYPE_OSPF_IF_PARAMS, oip->keychain_name);
 	ospf_interface_disable_bfd(ifp, oip);
 	ldp_sync_info_free(&(oip->ldp_sync_info));
 	XFREE(MTYPE_OSPF_IF_PARAMS, oip);
@@ -588,19 +589,22 @@ void ospf_free_if_params(struct interface *ifp, struct in_addr addr)
 	oip = rn->info;
 	route_unlock_node(rn);
 
-	if (!OSPF_IF_PARAM_CONFIGURED(oip, output_cost_cmd)
-	    && !OSPF_IF_PARAM_CONFIGURED(oip, transmit_delay)
-	    && !OSPF_IF_PARAM_CONFIGURED(oip, retransmit_interval)
-	    && !OSPF_IF_PARAM_CONFIGURED(oip, passive_interface)
-	    && !OSPF_IF_PARAM_CONFIGURED(oip, v_hello)
-	    && !OSPF_IF_PARAM_CONFIGURED(oip, fast_hello)
-	    && !OSPF_IF_PARAM_CONFIGURED(oip, v_wait)
-	    && !OSPF_IF_PARAM_CONFIGURED(oip, priority)
-	    && !OSPF_IF_PARAM_CONFIGURED(oip, type)
-	    && !OSPF_IF_PARAM_CONFIGURED(oip, auth_simple)
-	    && !OSPF_IF_PARAM_CONFIGURED(oip, auth_type)
-	    && !OSPF_IF_PARAM_CONFIGURED(oip, if_area)
-	    && listcount(oip->auth_crypt) == 0) {
+	if (!OSPF_IF_PARAM_CONFIGURED(oip, output_cost_cmd) &&
+	    !OSPF_IF_PARAM_CONFIGURED(oip, transmit_delay) &&
+	    !OSPF_IF_PARAM_CONFIGURED(oip, retransmit_interval) &&
+	    !OSPF_IF_PARAM_CONFIGURED(oip, passive_interface) &&
+	    !OSPF_IF_PARAM_CONFIGURED(oip, v_hello) &&
+	    !OSPF_IF_PARAM_CONFIGURED(oip, fast_hello) &&
+	    !OSPF_IF_PARAM_CONFIGURED(oip, v_wait) &&
+	    !OSPF_IF_PARAM_CONFIGURED(oip, priority) &&
+	    !OSPF_IF_PARAM_CONFIGURED(oip, type) &&
+	    !OSPF_IF_PARAM_CONFIGURED(oip, auth_simple) &&
+	    !OSPF_IF_PARAM_CONFIGURED(oip, auth_type) &&
+	    !OSPF_IF_PARAM_CONFIGURED(oip, if_area) &&
+	    !OSPF_IF_PARAM_CONFIGURED(oip, opaque_capable) &&
+	    !OSPF_IF_PARAM_CONFIGURED(oip, prefix_suppression) &&
+		!OSPF_IF_PARAM_CONFIGURED(oip, keychain_name) &&
+	    listcount(oip->auth_crypt) == 0) {
 		ospf_del_if_params(ifp, oip);
 		rn->info = NULL;
 		route_unlock_node(rn);
@@ -669,6 +673,8 @@ int ospf_if_new_hook(struct interface *ifp)
 
 	ifp->info = XCALLOC(MTYPE_OSPF_IF_INFO, sizeof(struct ospf_if_info));
 
+	IF_OSPF_IF_INFO(ifp)->oii_fd = -1;
+
 	IF_OIFS(ifp) = route_table_init();
 	IF_OIFS_PARAMS(ifp) = route_table_init();
 
@@ -692,6 +698,9 @@ int ospf_if_new_hook(struct interface *ifp)
 	SET_IF_PARAM(IF_DEF_PARAMS(ifp), fast_hello);
 	IF_DEF_PARAMS(ifp)->fast_hello = OSPF_FAST_HELLO_DEFAULT;
 
+	SET_IF_PARAM(IF_DEF_PARAMS(ifp), v_gr_hello_delay);
+	IF_DEF_PARAMS(ifp)->v_gr_hello_delay = OSPF_HELLO_DELAY_DEFAULT;
+
 	SET_IF_PARAM(IF_DEF_PARAMS(ifp), v_wait);
 	IF_DEF_PARAMS(ifp)->v_wait = OSPF_ROUTER_DEAD_INTERVAL_DEFAULT;
 
@@ -701,6 +710,11 @@ int ospf_if_new_hook(struct interface *ifp)
 	SET_IF_PARAM(IF_DEF_PARAMS(ifp), auth_type);
 	IF_DEF_PARAMS(ifp)->auth_type = OSPF_AUTH_NOTSET;
 
+	SET_IF_PARAM(IF_DEF_PARAMS(ifp), opaque_capable);
+	IF_DEF_PARAMS(ifp)->opaque_capable = OSPF_OPAQUE_CAPABLE_DEFAULT;
+
+	IF_DEF_PARAMS(ifp)->prefix_suppression = OSPF_PREFIX_SUPPRESSION_DEFAULT;
+
 	rc = ospf_opaque_new_if(ifp);
 	return rc;
 }
@@ -709,6 +723,8 @@ static int ospf_if_delete_hook(struct interface *ifp)
 {
 	int rc = 0;
 	struct route_node *rn;
+	struct ospf_if_info *oii;
+
 	rc = ospf_opaque_del_if(ifp);
 
 	/*
@@ -725,6 +741,13 @@ static int ospf_if_delete_hook(struct interface *ifp)
 	route_table_finish(IF_OIFS(ifp));
 	route_table_finish(IF_OIFS_PARAMS(ifp));
 
+	/* Close per-interface socket */
+	oii = ifp->info;
+	if (oii && oii->oii_fd > 0) {
+		close(oii->oii_fd);
+		oii->oii_fd = -1;
+	}
+
 	XFREE(MTYPE_OSPF_IF_INFO, ifp->info);
 
 	return rc;
@@ -1368,8 +1391,7 @@ static int ospf_ifp_create(struct interface *ifp)
 	    (!OSPF_IF_PARAM_CONFIGURED(IF_DEF_PARAMS(ifp), type) ||
 	     if_is_loopback(ifp))) {
 		SET_IF_PARAM(IF_DEF_PARAMS(ifp), type);
-		if (!IF_DEF_PARAMS(ifp)->type_cfg)
-			IF_DEF_PARAMS(ifp)->type = ospf_default_iftype(ifp);
+		IF_DEF_PARAMS(ifp)->type = ospf_default_iftype(ifp);
 	}
 
 	ospf = ifp->vrf->info;
@@ -1396,6 +1418,17 @@ static int ospf_ifp_up(struct interface *ifp)
 	struct ospf_interface *oi;
 	struct route_node *rn;
 	struct ospf_if_info *oii = ifp->info;
+	struct ospf *ospf;
+
+	if (IS_DEBUG_OSPF(zebra, ZEBRA_INTERFACE))
+		zlog_debug("Zebra: Interface[%s] state change to up.",
+			   ifp->name);
+
+	/* Open per-intf write socket if configured */
+	ospf = ifp->vrf->info;
+
+	if (ospf && ospf->oi_running && ospf->intf_socket_enabled)
+		ospf_ifp_sock_init(ifp);
 
 	ospf_if_recalculate_output_cost(ifp);
 
@@ -1413,10 +1446,6 @@ static int ospf_ifp_up(struct interface *ifp)
 		return 0;
 	}
 
-	if (IS_DEBUG_OSPF(zebra, ZEBRA_INTERFACE))
-		zlog_debug("Zebra: Interface[%s] state change to up.",
-			   ifp->name);
-
 	for (rn = route_top(IF_OIFS(ifp)); rn; rn = route_next(rn)) {
 		if ((oi = rn->info) == NULL)
 			continue;
@@ -1445,6 +1474,9 @@ static int ospf_ifp_down(struct interface *ifp)
 		ospf_if_down(oi);
 	}
 
+	/* Close per-interface write socket if configured */
+	ospf_ifp_sock_close(ifp);
+
 	return 0;
 }
 
@@ -1499,7 +1531,7 @@ void ospf_reset_hello_timer(struct interface *ifp, struct in_addr addr,
 			ospf_hello_send(oi);
 
 			/* Restart hello timer for this interface */
-			THREAD_OFF(oi->t_hello);
+			EVENT_OFF(oi->t_hello);
 			OSPF_HELLO_TIMER_ON(oi);
 		}
 
@@ -1523,7 +1555,7 @@ void ospf_reset_hello_timer(struct interface *ifp, struct in_addr addr,
 		ospf_hello_send(oi);
 
 		/* Restart the hello timer. */
-		THREAD_OFF(oi->t_hello);
+		EVENT_OFF(oi->t_hello);
 		OSPF_HELLO_TIMER_ON(oi);
 	}
 }
