@@ -1,28 +1,13 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * OSPF version 2  Neighbor State Machine
  * From RFC2328 [OSPF Version 2]
  * Copyright (C) 1999, 2000 Toshiaki Takada
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
-#include "thread.h"
+#include "frrevent.h"
 #include "memory.h"
 #include "hash.h"
 #include "linklist.h"
@@ -49,42 +34,53 @@
 #include "ospfd/ospf_flood.h"
 #include "ospfd/ospf_abr.h"
 #include "ospfd/ospf_bfd.h"
+#include "ospfd/ospf_gr.h"
 #include "ospfd/ospf_errors.h"
 
 DEFINE_HOOK(ospf_nsm_change,
 	    (struct ospf_neighbor * on, int state, int oldstate),
-	    (on, state, oldstate))
+	    (on, state, oldstate));
 
 static void nsm_clear_adj(struct ospf_neighbor *);
 
 /* OSPF NSM Timer functions. */
-static int ospf_inactivity_timer(struct thread *thread)
+static void ospf_inactivity_timer(struct event *thread)
 {
 	struct ospf_neighbor *nbr;
 
-	nbr = THREAD_ARG(thread);
+	nbr = EVENT_ARG(thread);
 	nbr->t_inactivity = NULL;
 
 	if (IS_DEBUG_OSPF(nsm, NSM_TIMERS))
-		zlog_debug("NSM[%s:%s:%s]: Timer (Inactivity timer expire)",
-			   IF_NAME(nbr->oi), inet_ntoa(nbr->router_id),
+		zlog_debug("NSM[%s:%pI4:%s]: Timer (Inactivity timer expire)",
+			   IF_NAME(nbr->oi), &nbr->router_id,
 			   ospf_get_name(nbr->oi->ospf));
 
-	OSPF_NSM_EVENT_SCHEDULE(nbr, NSM_InactivityTimer);
-
-	return 0;
+	/* Dont trigger NSM_InactivityTimer event , if the current
+	 * router acting as HELPER for this neighbour.
+	 */
+	if (!OSPF_GR_IS_ACTIVE_HELPER(nbr))
+		OSPF_NSM_EVENT_SCHEDULE(nbr, NSM_InactivityTimer);
+	else {
+		if (IS_DEBUG_OSPF_GR)
+			zlog_debug(
+				"%s, Acting as HELPER for this neighbour, So restart the dead timer",
+				__func__);
+		OSPF_NSM_TIMER_ON(nbr->t_inactivity, ospf_inactivity_timer,
+				  nbr->v_inactivity);
+	}
 }
 
-static int ospf_db_desc_timer(struct thread *thread)
+static void ospf_db_desc_timer(struct event *thread)
 {
 	struct ospf_neighbor *nbr;
 
-	nbr = THREAD_ARG(thread);
+	nbr = EVENT_ARG(thread);
 	nbr->t_db_desc = NULL;
 
 	if (IS_DEBUG_OSPF(nsm, NSM_TIMERS))
-		zlog_debug("NSM[%s:%s:%s]: Timer (DD Retransmit timer expire)",
-			   IF_NAME(nbr->oi), inet_ntoa(nbr->src),
+		zlog_debug("NSM[%s:%pI4:%s]: Timer (DD Retransmit timer expire)",
+			   IF_NAME(nbr->oi), &nbr->src,
 			   ospf_get_name(nbr->oi->ospf));
 
 	/* resent last send DD packet. */
@@ -93,8 +89,6 @@ static int ospf_db_desc_timer(struct thread *thread)
 
 	/* DD Retransmit timer set. */
 	OSPF_NSM_TIMER_ON(nbr->t_db_desc, ospf_db_desc_timer, nbr->v_db_desc);
-
-	return 0;
 }
 
 /* Hook function called after ospf NSM event is occurred.
@@ -111,32 +105,32 @@ static void nsm_timer_set(struct ospf_neighbor *nbr)
 	switch (nbr->state) {
 	case NSM_Deleted:
 	case NSM_Down:
-		OSPF_NSM_TIMER_OFF(nbr->t_inactivity);
-		OSPF_NSM_TIMER_OFF(nbr->t_hello_reply);
+		EVENT_OFF(nbr->t_inactivity);
+		EVENT_OFF(nbr->t_hello_reply);
 	/* fallthru */
 	case NSM_Attempt:
 	case NSM_Init:
 	case NSM_TwoWay:
-		OSPF_NSM_TIMER_OFF(nbr->t_db_desc);
-		OSPF_NSM_TIMER_OFF(nbr->t_ls_upd);
-		OSPF_NSM_TIMER_OFF(nbr->t_ls_req);
+		EVENT_OFF(nbr->t_db_desc);
+		EVENT_OFF(nbr->t_ls_upd);
+		EVENT_OFF(nbr->t_ls_req);
 		break;
 	case NSM_ExStart:
 		OSPF_NSM_TIMER_ON(nbr->t_db_desc, ospf_db_desc_timer,
 				  nbr->v_db_desc);
-		OSPF_NSM_TIMER_OFF(nbr->t_ls_upd);
-		OSPF_NSM_TIMER_OFF(nbr->t_ls_req);
+		EVENT_OFF(nbr->t_ls_upd);
+		EVENT_OFF(nbr->t_ls_req);
 		break;
 	case NSM_Exchange:
 		OSPF_NSM_TIMER_ON(nbr->t_ls_upd, ospf_ls_upd_timer,
 				  nbr->v_ls_upd);
 		if (!IS_SET_DD_MS(nbr->dd_flags))
-			OSPF_NSM_TIMER_OFF(nbr->t_db_desc);
+			EVENT_OFF(nbr->t_db_desc);
 		break;
 	case NSM_Loading:
 	case NSM_Full:
 	default:
-		OSPF_NSM_TIMER_OFF(nbr->t_db_desc);
+		EVENT_OFF(nbr->t_db_desc);
 		break;
 	}
 }
@@ -144,7 +138,7 @@ static void nsm_timer_set(struct ospf_neighbor *nbr)
 /* 10.4 of RFC2328, indicate whether an adjacency is appropriate with
  * the given neighbour
  */
-static int nsm_should_adj(struct ospf_neighbor *nbr)
+int nsm_should_adj(struct ospf_neighbor *nbr)
 {
 	struct ospf_interface *oi = nbr->oi;
 
@@ -164,16 +158,16 @@ static int nsm_should_adj(struct ospf_neighbor *nbr)
 }
 
 /* OSPF NSM functions. */
-static int nsm_packet_received(struct ospf_neighbor *nbr)
+static int nsm_hello_received(struct ospf_neighbor *nbr)
 {
 	/* Start or Restart Inactivity Timer. */
-	OSPF_NSM_TIMER_OFF(nbr->t_inactivity);
+	EVENT_OFF(nbr->t_inactivity);
 
 	OSPF_NSM_TIMER_ON(nbr->t_inactivity, ospf_inactivity_timer,
 			  nbr->v_inactivity);
 
 	if (nbr->oi->type == OSPF_IFTYPE_NBMA && nbr->nbr_nbma)
-		OSPF_POLL_TIMER_OFF(nbr->nbr_nbma->t_poll);
+		EVENT_OFF(nbr->nbr_nbma->t_poll);
 
 	/* Send proactive ARP requests */
 	if (nbr->state < NSM_Exchange)
@@ -185,9 +179,9 @@ static int nsm_packet_received(struct ospf_neighbor *nbr)
 static int nsm_start(struct ospf_neighbor *nbr)
 {
 	if (nbr->nbr_nbma)
-		OSPF_POLL_TIMER_OFF(nbr->nbr_nbma->t_poll);
+		EVENT_OFF(nbr->nbr_nbma->t_poll);
 
-	OSPF_NSM_TIMER_OFF(nbr->t_inactivity);
+	EVENT_OFF(nbr->t_inactivity);
 
 	OSPF_NSM_TIMER_ON(nbr->t_inactivity, ospf_inactivity_timer,
 			  nbr->v_inactivity);
@@ -289,8 +283,6 @@ static int nsm_negotiation_done(struct ospf_neighbor *nbr)
 		ospf_db_summary_add(nbr, lsa);
 	LSDB_LOOP (SUMMARY_LSDB(area), rn, lsa)
 		ospf_db_summary_add(nbr, lsa);
-	LSDB_LOOP (ASBR_SUMMARY_LSDB(area), rn, lsa)
-		ospf_db_summary_add(nbr, lsa);
 
 	/* Process only if the neighbor is opaque capable. */
 	if (CHECK_FLAG(nbr->options, OSPF_OPTION_O)) {
@@ -305,10 +297,14 @@ static int nsm_negotiation_done(struct ospf_neighbor *nbr)
 			ospf_db_summary_add(nbr, lsa);
 	}
 
+	/* For Stub/NSSA area, we should not send Type-4 and Type-5 LSAs */
 	if (nbr->oi->type != OSPF_IFTYPE_VIRTUALLINK
-	    && area->external_routing == OSPF_AREA_DEFAULT)
+	    && area->external_routing == OSPF_AREA_DEFAULT) {
+		LSDB_LOOP (ASBR_SUMMARY_LSDB(area), rn, lsa)
+			ospf_db_summary_add(nbr, lsa);
 		LSDB_LOOP (EXTERNAL_LSDB(nbr->oi->ospf), rn, lsa)
 			ospf_db_summary_add(nbr, lsa);
+	}
 
 	if (CHECK_FLAG(nbr->options, OSPF_OPTION_O)
 	    && (nbr->oi->type != OSPF_IFTYPE_VIRTUALLINK
@@ -371,6 +367,10 @@ static void nsm_clear_adj(struct ospf_neighbor *nbr)
 
 static int nsm_kill_nbr(struct ospf_neighbor *nbr)
 {
+	struct ospf_interface *oi = nbr->oi;
+	struct ospf_neighbor *on;
+	struct route_node *rn;
+
 	/* killing nbr_self is invalid */
 	if (nbr == nbr->oi->nbr_self) {
 		assert(nbr != nbr->oi->nbr_self);
@@ -390,12 +390,41 @@ static int nsm_kill_nbr(struct ospf_neighbor *nbr)
 
 		if (IS_DEBUG_OSPF(nsm, NSM_EVENTS))
 			zlog_debug(
-				"NSM[%s:%s:%s]: Down (PollIntervalTimer scheduled)",
+				"NSM[%s:%pI4:%s]: Down (PollIntervalTimer scheduled)",
 				IF_NAME(nbr->oi),
-				inet_ntoa(nbr->address.u.prefix4),
+				&nbr->address.u.prefix4,
 				ospf_get_name(nbr->oi->ospf));
 	}
 
+	/*
+	 * Do we have any neighbors that are also operating
+	 * on this interface?
+	 */
+	for (rn = route_top(oi->nbrs); rn; rn = route_next(rn)) {
+		on = rn->info;
+
+		if (!on)
+			continue;
+
+		if (on == nbr || on == oi->nbr_self)
+			continue;
+
+		/*
+		 * on is in some state where we might be
+		 * sending packets on this interface
+		 */
+		if (on->state > NSM_Down) {
+			route_unlock_node(rn);
+			return 0;
+		}
+	}
+	/*
+	 * If we get here we know that this interface
+	 * has no neighbors in a state where we could
+	 * be sending packets.  Let's flush anything
+	 * we got.
+	 */
+	ospf_interface_fifo_flush(oi);
 	return 0;
 }
 
@@ -407,7 +436,7 @@ const struct {
 	{
 		/* DependUpon: dummy state. */
 		{NULL, NSM_DependUpon}, /* NoEvent           */
-		{NULL, NSM_DependUpon}, /* PacketReceived    */
+		{NULL, NSM_DependUpon}, /* HelloReceived     */
 		{NULL, NSM_DependUpon}, /* Start             */
 		{NULL, NSM_DependUpon}, /* 2-WayReceived     */
 		{NULL, NSM_DependUpon}, /* NegotiationDone   */
@@ -424,7 +453,7 @@ const struct {
 	{
 		/* Deleted: dummy state. */
 		{NULL, NSM_Deleted}, /* NoEvent           */
-		{NULL, NSM_Deleted}, /* PacketReceived    */
+		{NULL, NSM_Deleted}, /* HelloReceived     */
 		{NULL, NSM_Deleted}, /* Start             */
 		{NULL, NSM_Deleted}, /* 2-WayReceived     */
 		{NULL, NSM_Deleted}, /* NegotiationDone   */
@@ -441,8 +470,8 @@ const struct {
 	{
 		/* Down: */
 		{NULL, NSM_DependUpon},		 /* NoEvent           */
-		{nsm_packet_received, NSM_Init}, /* PacketReceived    */
-		{nsm_start, NSM_Attempt},	/* Start             */
+		{nsm_hello_received, NSM_Init},  /* HelloReceived     */
+		{nsm_start, NSM_Attempt},	 /* Start             */
 		{NULL, NSM_Down},		 /* 2-WayReceived     */
 		{NULL, NSM_Down},		 /* NegotiationDone   */
 		{NULL, NSM_Down},		 /* ExchangeDone      */
@@ -458,7 +487,7 @@ const struct {
 	{
 		/* Attempt: */
 		{NULL, NSM_DependUpon},		 /* NoEvent           */
-		{nsm_packet_received, NSM_Init}, /* PacketReceived    */
+		{nsm_hello_received, NSM_Init},  /* HelloReceived     */
 		{NULL, NSM_Attempt},		 /* Start             */
 		{NULL, NSM_Attempt},		 /* 2-WayReceived     */
 		{NULL, NSM_Attempt},		 /* NegotiationDone   */
@@ -475,7 +504,7 @@ const struct {
 	{
 		/* Init: */
 		{NULL, NSM_DependUpon},		       /* NoEvent           */
-		{nsm_packet_received, NSM_Init},       /* PacketReceived    */
+		{nsm_hello_received, NSM_Init},        /* HelloReceived     */
 		{NULL, NSM_Init},		       /* Start             */
 		{nsm_twoway_received, NSM_DependUpon}, /* 2-WayReceived     */
 		{NULL, NSM_Init},		       /* NegotiationDone   */
@@ -492,7 +521,7 @@ const struct {
 	{
 		/* 2-Way: */
 		{NULL, NSM_DependUpon},		   /* NoEvent           */
-		{nsm_packet_received, NSM_TwoWay}, /* HelloReceived     */
+		{nsm_hello_received, NSM_TwoWay},  /* HelloReceived     */
 		{NULL, NSM_TwoWay},		   /* Start             */
 		{NULL, NSM_TwoWay},		   /* 2-WayReceived     */
 		{NULL, NSM_TwoWay},		   /* NegotiationDone   */
@@ -509,7 +538,7 @@ const struct {
 	{
 		/* ExStart: */
 		{NULL, NSM_DependUpon},		      /* NoEvent           */
-		{nsm_packet_received, NSM_ExStart},   /* PacaketReceived   */
+		{nsm_hello_received, NSM_ExStart},    /* HelloReceived     */
 		{NULL, NSM_ExStart},		      /* Start             */
 		{NULL, NSM_ExStart},		      /* 2-WayReceived     */
 		{nsm_negotiation_done, NSM_Exchange}, /* NegotiationDone   */
@@ -526,7 +555,7 @@ const struct {
 	{
 		/* Exchange: */
 		{NULL, NSM_DependUpon},		     /* NoEvent           */
-		{nsm_packet_received, NSM_Exchange}, /* PacketReceived    */
+		{nsm_hello_received, NSM_Exchange},  /* HelloReceived     */
 		{NULL, NSM_Exchange},		     /* Start             */
 		{NULL, NSM_Exchange},		     /* 2-WayReceived     */
 		{NULL, NSM_Exchange},		     /* NegotiationDone   */
@@ -543,7 +572,7 @@ const struct {
 	{
 		/* Loading: */
 		{NULL, NSM_DependUpon},		    /* NoEvent           */
-		{nsm_packet_received, NSM_Loading}, /* PacketReceived    */
+		{nsm_hello_received, NSM_Loading},  /* HelloReceived     */
 		{NULL, NSM_Loading},		    /* Start             */
 		{NULL, NSM_Loading},		    /* 2-WayReceived     */
 		{NULL, NSM_Loading},		    /* NegotiationDone   */
@@ -560,7 +589,7 @@ const struct {
 	{
 		/* Full: */
 		{NULL, NSM_DependUpon},		 /* NoEvent           */
-		{nsm_packet_received, NSM_Full}, /* PacketReceived    */
+		{nsm_hello_received, NSM_Full},  /* HelloReceived     */
 		{NULL, NSM_Full},		 /* Start             */
 		{NULL, NSM_Full},		 /* 2-WayReceived     */
 		{NULL, NSM_Full},		 /* NegotiationDone   */
@@ -577,7 +606,7 @@ const struct {
 };
 
 static const char *const ospf_nsm_event_str[] = {
-	"NoEvent",	   "PacketReceived",  "Start",
+	"NoEvent",	   "HelloReceived",  "Start",
 	"2-WayReceived",     "NegotiationDone", "ExchangeDone",
 	"BadLSReq",	  "LoadingDone",     "AdjOK?",
 	"SeqNumberMismatch", "1-WayReceived",   "KillNbr",
@@ -589,8 +618,8 @@ static void nsm_notice_state_change(struct ospf_neighbor *nbr, int next_state,
 {
 	/* Logging change of status. */
 	if (IS_DEBUG_OSPF(nsm, NSM_STATUS))
-		zlog_debug("NSM[%s:%s:%s]: State change %s -> %s (%s)",
-			   IF_NAME(nbr->oi), inet_ntoa(nbr->router_id),
+		zlog_debug("NSM[%s:%pI4:%s]: State change %s -> %s (%s)",
+			   IF_NAME(nbr->oi), &nbr->router_id,
 			   ospf_get_name(nbr->oi->ospf),
 			   lookup_msg(ospf_nsm_state_msg, nbr->state, NULL),
 			   lookup_msg(ospf_nsm_state_msg, next_state, NULL),
@@ -600,12 +629,13 @@ static void nsm_notice_state_change(struct ospf_neighbor *nbr, int next_state,
 	if (CHECK_FLAG(nbr->oi->ospf->config, OSPF_LOG_ADJACENCY_CHANGES)
 	    && (CHECK_FLAG(nbr->oi->ospf->config, OSPF_LOG_ADJACENCY_DETAIL)
 		|| (next_state == NSM_Full) || (next_state < nbr->state)))
-		zlog_notice("AdjChg: Nbr %s(%s) on %s: %s -> %s (%s)",
-			    inet_ntoa(nbr->router_id),
-			    ospf_get_name(nbr->oi->ospf), IF_NAME(nbr->oi),
-			    lookup_msg(ospf_nsm_state_msg, nbr->state, NULL),
-			    lookup_msg(ospf_nsm_state_msg, next_state, NULL),
-			    ospf_nsm_event_str[event]);
+		zlog_notice(
+			"AdjChg: Nbr %pI4, NbrIP %pI4 (%s) on %s: %s -> %s (%s)",
+			&nbr->router_id, &nbr->src,
+			ospf_get_name(nbr->oi->ospf), IF_NAME(nbr->oi),
+			lookup_msg(ospf_nsm_state_msg, nbr->state, NULL),
+			lookup_msg(ospf_nsm_state_msg, next_state, NULL),
+			ospf_nsm_event_str[event]);
 
 	/* Advance in NSM */
 	if (next_state > nbr->state)
@@ -683,14 +713,17 @@ static void nsm_change_state(struct ospf_neighbor *nbr, int state)
 
 		if (CHECK_FLAG(oi->ospf->config, OSPF_LOG_ADJACENCY_DETAIL))
 			zlog_info(
-				"%s:[%s:%s], %s -> %s): "
-				"scheduling new router-LSA origination",
-				__func__, inet_ntoa(nbr->router_id),
+				"%s:[%pI4:%s], %s -> %s): scheduling new router-LSA origination",
+				__func__, &nbr->router_id,
 				ospf_get_name(oi->ospf),
 				lookup_msg(ospf_nsm_state_msg, old_state, NULL),
 				lookup_msg(ospf_nsm_state_msg, state, NULL));
 
-		ospf_router_lsa_update_area(oi->area);
+		/* Dont originate router LSA if the current
+		 * router is acting as a HELPER for this neighbour.
+		 */
+		if (!OSPF_GR_IS_ACTIVE_HELPER(nbr))
+			ospf_router_lsa_update_area(oi->area);
 
 		if (oi->type == OSPF_IFTYPE_VIRTUALLINK) {
 			vl_area = ospf_area_lookup_by_area_id(
@@ -700,16 +733,25 @@ static void nsm_change_state(struct ospf_neighbor *nbr, int state)
 				ospf_router_lsa_update_area(vl_area);
 		}
 
-		/* Originate network-LSA. */
-		if (oi->state == ISM_DR) {
-			if (oi->network_lsa_self && oi->full_nbrs == 0) {
-				ospf_lsa_flush_area(oi->network_lsa_self,
-						    oi->area);
-				ospf_lsa_unlock(&oi->network_lsa_self);
-				oi->network_lsa_self = NULL;
-			} else
-				ospf_network_lsa_update(oi);
+		/* Dont originate/flush network LSA if the current
+		 * router is acting as a HELPER for this neighbour.
+		 */
+		if (!OSPF_GR_IS_ACTIVE_HELPER(nbr)) {
+			/* Originate network-LSA. */
+			if (oi->state == ISM_DR) {
+				if (oi->network_lsa_self
+				    && oi->full_nbrs == 0) {
+					ospf_lsa_flush_area(
+						oi->network_lsa_self, oi->area);
+					ospf_lsa_unlock(&oi->network_lsa_self);
+					oi->network_lsa_self = NULL;
+				} else
+					ospf_network_lsa_update(oi);
+			}
 		}
+
+		if (state == NSM_Full && oi->ospf->gr_info.restart_in_progress)
+			ospf_gr_check_adjs(oi->ospf);
 	}
 
 	ospf_opaque_nsm_change(nbr, old_state);
@@ -732,11 +774,9 @@ static void nsm_change_state(struct ospf_neighbor *nbr, int state)
 			OSPF_DD_FLAG_I | OSPF_DD_FLAG_M | OSPF_DD_FLAG_MS;
 		if (CHECK_FLAG(oi->ospf->config, OSPF_LOG_ADJACENCY_DETAIL))
 			zlog_info(
-				"%s: Initializing [DD]: %s with seqnum:%x , flags:%x",
-				(oi->ospf->name) ? oi->ospf->name
-						 : VRF_DEFAULT_NAME,
-				inet_ntoa(nbr->router_id), nbr->dd_seqnum,
-				nbr->dd_flags);
+				"%s: Initializing [DD]: %pI4 with seqnum:%x , flags:%x",
+				ospf_get_name(oi->ospf), &nbr->router_id,
+				nbr->dd_seqnum, nbr->dd_flags);
 		ospf_db_desc_send(nbr);
 	}
 
@@ -744,24 +784,25 @@ static void nsm_change_state(struct ospf_neighbor *nbr, int state)
 	if (state == NSM_Down)
 		nbr->crypt_seqnum = 0;
 
-	ospf_bfd_trigger_event(nbr, old_state, state);
+	if (nbr->bfd_session)
+		ospf_bfd_trigger_event(nbr, old_state, state);
 
 	/* Preserve old status? */
 }
 
 /* Execute NSM event process. */
-int ospf_nsm_event(struct thread *thread)
+void ospf_nsm_event(struct event *thread)
 {
 	int event;
 	int next_state;
 	struct ospf_neighbor *nbr;
 
-	nbr = THREAD_ARG(thread);
-	event = THREAD_VAL(thread);
+	nbr = EVENT_ARG(thread);
+	event = EVENT_VAL(thread);
 
 	if (IS_DEBUG_OSPF(nsm, NSM_EVENTS))
-		zlog_debug("NSM[%s:%s:%s]: %s (%s)", IF_NAME(nbr->oi),
-			   inet_ntoa(nbr->router_id),
+		zlog_debug("NSM[%s:%pI4:%s]: %s (%s)", IF_NAME(nbr->oi),
+			   &nbr->router_id,
 			   ospf_get_name(nbr->oi->ospf),
 			   lookup_msg(ospf_nsm_state_msg, nbr->state, NULL),
 			   ospf_nsm_event_str[event]);
@@ -785,9 +826,8 @@ int ospf_nsm_event(struct thread *thread)
 			 */
 			flog_err(
 				EC_OSPF_FSM_INVALID_STATE,
-				"NSM[%s:%s:%s]: %s (%s): "
-				"Warning: action tried to change next_state to %s",
-				IF_NAME(nbr->oi), inet_ntoa(nbr->router_id),
+				"NSM[%s:%pI4:%s]: %s (%s): Warning: action tried to change next_state to %s",
+				IF_NAME(nbr->oi), &nbr->router_id,
 				ospf_get_name(nbr->oi->ospf),
 				lookup_msg(ospf_nsm_state_msg, nbr->state,
 					   NULL),
@@ -821,8 +861,6 @@ int ospf_nsm_event(struct thread *thread)
 	 */
 	if (nbr->state == NSM_Deleted)
 		ospf_nbr_delete(nbr);
-
-	return 0;
 }
 
 /* Check loading state. */
