@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * This is an implementation of RFC4970 Router Information
  * with support of RFC5088 PCE Capabilites announcement
@@ -5,22 +6,6 @@
  * Module name: Router Information
  * Author: Olivier Dugeon <olivier.dugeon@orange.com>
  * Copyright (C) 2012 - 2017 Orange Labs http://www.orange.com/
- *
- * This file is part of GNU Quagga.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Quagga is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -35,7 +20,7 @@
 #include "vty.h"
 #include "stream.h"
 #include "log.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "hash.h"
 #include "sockunion.h" /* for inet_aton() */
 #include "mpls.h"
@@ -66,14 +51,16 @@
 static struct ospf_router_info OspfRI;
 
 /*------------------------------------------------------------------------------*
- * Followings are initialize/terminate functions for Router Information
+ * Following are initialize/terminate functions for Router Information
  *handling.
  *------------------------------------------------------------------------------*/
 
 static void ospf_router_info_ism_change(struct ospf_interface *oi,
 					int old_status);
 static void ospf_router_info_config_write_router(struct vty *vty);
-static void ospf_router_info_show_info(struct vty *vty, struct ospf_lsa *lsa);
+static void ospf_router_info_show_info(struct vty *vty,
+				       struct json_object *json,
+				       struct ospf_lsa *lsa);
 static int ospf_router_info_lsa_originate(void *arg);
 static struct ospf_lsa *ospf_router_info_lsa_refresh(struct ospf_lsa *lsa);
 static void ospf_router_info_lsa_schedule(struct ospf_ri_area_info *ai,
@@ -88,7 +75,7 @@ int ospf_router_info_init(void)
 
 	zlog_info("RI (%s): Initialize Router Information", __func__);
 
-	memset(&OspfRI, 0, sizeof(struct ospf_router_info));
+	memset(&OspfRI, 0, sizeof(OspfRI));
 	OspfRI.enabled = false;
 	OspfRI.registered = 0;
 	OspfRI.scope = OSPF_OPAQUE_AS_LSA;
@@ -178,6 +165,14 @@ void ospf_router_info_term(void)
 
 void ospf_router_info_finish(void)
 {
+	struct listnode *node, *nnode;
+	struct ospf_ri_area_info *ai;
+
+	/* Flush Router Info LSA */
+	for (ALL_LIST_ELEMENTS(OspfRI.area_info, node, nnode, ai))
+		if (CHECK_FLAG(ai->flags, RIFLG_LSA_ENGAGED))
+			ospf_router_info_lsa_schedule(ai, FLUSH_THIS_LSA);
+
 	list_delete_all_node(OspfRI.pce_info.pce_domain);
 	list_delete_all_node(OspfRI.pce_info.pce_neighbor);
 
@@ -222,7 +217,7 @@ static struct ospf_ri_area_info *lookup_by_area(struct ospf_area *area)
 }
 
 /*------------------------------------------------------------------------*
- * Followings are control functions for ROUTER INFORMATION parameters
+ * Following are control functions for ROUTER INFORMATION parameters
  *management.
  *------------------------------------------------------------------------*/
 
@@ -286,8 +281,8 @@ static void set_pce_address(struct in_addr ipv4, struct ospf_pce_info *pce)
 	pce->pce_header.header.type = htons(RI_TLV_PCE);
 	/* Set PCE Address */
 	pce->pce_address.header.type = htons(RI_PCE_SUBTLV_ADDRESS);
-	pce->pce_address.header.length = htons(PCE_ADDRESS_LENGTH_IPV4);
-	pce->pce_address.address.type = htons(PCE_ADDRESS_TYPE_IPV4);
+	pce->pce_address.header.length = htons(PCE_ADDRESS_IPV4_SIZE);
+	pce->pce_address.address.type = htons(PCE_ADDRESS_IPV4);
 	pce->pce_address.address.value = ipv4;
 
 	return;
@@ -315,7 +310,7 @@ static void set_pce_domain(uint16_t type, uint32_t domain,
 		      sizeof(struct ri_pce_subtlv_domain));
 
 	new->header.type = htons(RI_PCE_SUBTLV_DOMAIN);
-	new->header.length = htons(PCE_ADDRESS_LENGTH_IPV4);
+	new->header.length = htons(PCE_ADDRESS_IPV4_SIZE);
 	new->type = htons(type);
 	new->value = htonl(domain);
 
@@ -361,7 +356,7 @@ static void set_pce_neighbor(uint16_t type, uint32_t domain,
 		      sizeof(struct ri_pce_subtlv_neighbor));
 
 	new->header.type = htons(RI_PCE_SUBTLV_NEIGHBOR);
-	new->header.length = htons(PCE_ADDRESS_LENGTH_IPV4);
+	new->header.length = htons(PCE_ADDRESS_IPV4_SIZE);
 	new->type = htons(type);
 	new->value = htonl(domain);
 
@@ -434,29 +429,50 @@ static void unset_sr_algorithm(uint8_t algo)
 	TLV_LEN(OspfRI.sr_info.algo) = htons(0);
 }
 
-/* Segment Routing Global Block SubTLV - section 3.2 */
-static void set_sr_sid_label_range(struct sr_srgb srgb)
+/* Set Segment Routing Global Block SubTLV - section 3.2 */
+static void set_sr_global_label_range(struct sr_block srgb)
 {
 	/* Set Header */
-	TLV_TYPE(OspfRI.sr_info.range) = htons(RI_SR_TLV_SID_LABEL_RANGE);
-	TLV_LEN(OspfRI.sr_info.range) =
-		htons(SUBTLV_SID_LABEL_SIZE + sizeof(uint32_t));
+	TLV_TYPE(OspfRI.sr_info.srgb) = htons(RI_SR_TLV_SRGB_LABEL_RANGE);
+	TLV_LEN(OspfRI.sr_info.srgb) = htons(RI_SR_TLV_LABEL_RANGE_SIZE);
 	/* Set Range Size */
-	OspfRI.sr_info.range.size = htonl(SET_RANGE_SIZE(srgb.range_size));
+	OspfRI.sr_info.srgb.size = htonl(SET_RANGE_SIZE(srgb.range_size));
 	/* Set Lower bound label SubTLV */
-	TLV_TYPE(OspfRI.sr_info.range.lower) = htons(SUBTLV_SID_LABEL);
-	TLV_LEN(OspfRI.sr_info.range.lower) = htons(SID_RANGE_LABEL_LENGTH);
-	OspfRI.sr_info.range.lower.value = htonl(SET_LABEL(srgb.lower_bound));
+	TLV_TYPE(OspfRI.sr_info.srgb.lower) = htons(SUBTLV_SID_LABEL);
+	TLV_LEN(OspfRI.sr_info.srgb.lower) = htons(SID_RANGE_LABEL_LENGTH);
+	OspfRI.sr_info.srgb.lower.value = htonl(SET_LABEL(srgb.lower_bound));
 }
 
-/* Unset this SRGB SubTLV */
-static void unset_sr_sid_label_range(void)
+/* Unset Segment Routing Global Block SubTLV */
+static void unset_sr_global_label_range(void)
 {
+	TLV_TYPE(OspfRI.sr_info.srgb) = htons(0);
+	TLV_LEN(OspfRI.sr_info.srgb) = htons(0);
+	TLV_TYPE(OspfRI.sr_info.srgb.lower) = htons(0);
+	TLV_LEN(OspfRI.sr_info.srgb.lower) = htons(0);
+}
 
-	TLV_TYPE(OspfRI.sr_info.range) = htons(0);
-	TLV_LEN(OspfRI.sr_info.range) = htons(0);
-	TLV_TYPE(OspfRI.sr_info.range.lower) = htons(0);
-	TLV_LEN(OspfRI.sr_info.range.lower) = htons(0);
+/* Set Segment Routing Local Block SubTLV - section 3.2 */
+static void set_sr_local_label_range(struct sr_block srlb)
+{
+	/* Set Header */
+	TLV_TYPE(OspfRI.sr_info.srlb) = htons(RI_SR_TLV_SRLB_LABEL_RANGE);
+	TLV_LEN(OspfRI.sr_info.srlb) = htons(RI_SR_TLV_LABEL_RANGE_SIZE);
+	/* Set Range Size */
+	OspfRI.sr_info.srlb.size = htonl(SET_RANGE_SIZE(srlb.range_size));
+	/* Set Lower bound label SubTLV */
+	TLV_TYPE(OspfRI.sr_info.srlb.lower) = htons(SUBTLV_SID_LABEL);
+	TLV_LEN(OspfRI.sr_info.srlb.lower) = htons(SID_RANGE_LABEL_LENGTH);
+	OspfRI.sr_info.srlb.lower.value = htonl(SET_LABEL(srlb.lower_bound));
+}
+
+/* Unset Segment Routing Local Block SubTLV */
+static void unset_sr_local_label_range(void)
+{
+	TLV_TYPE(OspfRI.sr_info.srlb) = htons(0);
+	TLV_LEN(OspfRI.sr_info.srlb) = htons(0);
+	TLV_TYPE(OspfRI.sr_info.srlb.lower) = htons(0);
+	TLV_LEN(OspfRI.sr_info.srlb.lower) = htons(0);
 }
 
 /* Set Maximum Stack Depth for this router */
@@ -510,9 +526,11 @@ static void initialize_params(struct ospf_router_info *ori)
 	/* Try to get available Area's context from ospf at this step.
 	 * Do it latter if not available */
 	if (OspfRI.scope == OSPF_OPAQUE_AREA_LSA) {
+		if (!list_isempty(OspfRI.area_info))
+			list_delete_all_node(OspfRI.area_info);
 		for (ALL_LIST_ELEMENTS(top->areas, node, nnode, area)) {
-			zlog_debug("RI (%s): Add area %s to Router Information",
-				__func__, inet_ntoa(area->area_id));
+			zlog_debug("RI (%s): Add area %pI4 to Router Information",
+				__func__, &area->area_id);
 			new = XCALLOC(MTYPE_OSPF_ROUTER_INFO,
 				sizeof(struct ospf_ri_area_info));
 			new->area = area;
@@ -540,20 +558,23 @@ static void initialize_params(struct ospf_router_info *ori)
 	return;
 }
 
-static int is_mandated_params_set(struct ospf_router_info ori)
+static int is_mandated_params_set(struct ospf_router_info *ori)
 {
 	int rc = 0;
 
-	if (ntohs(ori.router_cap.header.type) == 0)
+	if (ori == NULL)
 		return rc;
 
-	if ((ntohs(ori.pce_info.pce_header.header.type) == RI_TLV_PCE)
-	    && (ntohs(ori.pce_info.pce_address.header.type) == 0)
-	    && (ntohs(ori.pce_info.pce_cap_flag.header.type) == 0))
+	if (ntohs(ori->router_cap.header.type) == 0)
 		return rc;
 
-	if ((ori.sr_info.enabled) && (ntohs(TLV_TYPE(ori.sr_info.algo)) == 0)
-	    && (ntohs(TLV_TYPE(ori.sr_info.range)) == 0))
+	if ((ntohs(ori->pce_info.pce_header.header.type) == RI_TLV_PCE)
+	    && (ntohs(ori->pce_info.pce_address.header.type) == 0)
+	    && (ntohs(ori->pce_info.pce_cap_flag.header.type) == 0))
+		return rc;
+
+	if ((ori->sr_info.enabled) && (ntohs(TLV_TYPE(ori->sr_info.algo)) == 0)
+	    && (ntohs(TLV_TYPE(ori->sr_info.srgb)) == 0))
 		return rc;
 
 	rc = 1;
@@ -565,13 +586,11 @@ static int is_mandated_params_set(struct ospf_router_info ori)
  * Used by Segment Routing to set new TLVs and Sub-TLVs values
  *
  * @param enable To activate or not Segment Routing router Information flooding
- * @param size   Size of Label Range i.e. SRGB size
- * @param lower  Lower bound of the Label Range i.e. SRGB first label
- * @param msd    Maximum label Stack Depth supported by the router
+ * @param srn    Self Segment Routing node
  *
  * @return none
  */
-void ospf_router_info_update_sr(bool enable, struct sr_srgb srgb, uint8_t msd)
+void ospf_router_info_update_sr(bool enable, struct sr_node *srn)
 {
 	struct listnode *node, *nnode;
 	struct ospf_ri_area_info *ai;
@@ -595,6 +614,10 @@ void ospf_router_info_update_sr(bool enable, struct sr_srgb srgb, uint8_t msd)
 		initialize_params(&OspfRI);
 	}
 
+	/* Check that SR node is valid */
+	if (srn == NULL)
+		return;
+
 	if (IS_DEBUG_OSPF_SR)
 		zlog_debug("RI (%s): %s Routing Information for Segment Routing",
 			   __func__, enable ? "Enable" : "Disable");
@@ -602,15 +625,17 @@ void ospf_router_info_update_sr(bool enable, struct sr_srgb srgb, uint8_t msd)
 	/* Unset or Set SR parameters */
 	if (!enable) {
 		unset_sr_algorithm(SR_ALGORITHM_SPF);
-		unset_sr_sid_label_range();
+		unset_sr_global_label_range();
+		unset_sr_local_label_range();
 		unset_sr_node_msd();
 		OspfRI.sr_info.enabled = false;
 	} else {
 		// Only SR_ALGORITHM_SPF is supported
 		set_sr_algorithm(SR_ALGORITHM_SPF);
-		set_sr_sid_label_range(srgb);
-		if (msd != 0)
-			set_sr_node_msd(msd);
+		set_sr_global_label_range(srn->srgb);
+		set_sr_local_label_range(srn->srlb);
+		if (srn->msd != 0)
+			set_sr_node_msd(srn->msd);
 		else
 			unset_sr_node_msd();
 		OspfRI.sr_info.enabled = true;
@@ -628,7 +653,7 @@ void ospf_router_info_update_sr(bool enable, struct sr_srgb srgb, uint8_t msd)
 }
 
 /*------------------------------------------------------------------------*
- * Followings are callback functions against generic Opaque-LSAs handling.
+ * Following are callback functions against generic Opaque-LSAs handling.
  *------------------------------------------------------------------------*/
 static void ospf_router_info_ism_change(struct ospf_interface *oi,
 					int old_state)
@@ -653,7 +678,7 @@ static void ospf_router_info_ism_change(struct ospf_interface *oi,
 }
 
 /*------------------------------------------------------------------------*
- * Followings are OSPF protocol processing functions for ROUTER INFORMATION
+ * Following are OSPF protocol processing functions for ROUTER INFORMATION
  *------------------------------------------------------------------------*/
 
 static void build_tlv_header(struct stream *s, struct tlv_header *tlvh)
@@ -688,7 +713,9 @@ static void ospf_router_info_lsa_body_set(struct stream *s)
 		/* Build Algorithm TLV */
 		build_tlv(s, &TLV_HDR(OspfRI.sr_info.algo));
 		/* Build SRGB TLV */
-		build_tlv(s, &TLV_HDR(OspfRI.sr_info.range));
+		build_tlv(s, &TLV_HDR(OspfRI.sr_info.srgb));
+		/* Build SRLB TLV */
+		build_tlv(s, &TLV_HDR(OspfRI.sr_info.srlb));
 		/* Build MSD TLV */
 		build_tlv(s, &TLV_HDR(OspfRI.sr_info.msd));
 	}
@@ -753,8 +780,8 @@ static struct ospf_lsa *ospf_router_info_lsa_new(struct ospf_area *area)
 
 	if (IS_DEBUG_OSPF(lsa, LSA_GENERATE))
 		zlog_debug(
-			"LSA[Type%d:%s]: Create an Opaque-LSA/ROUTER INFORMATION instance",
-			lsa_type, inet_ntoa(lsa_id));
+			"LSA[Type%d:%pI4]: Create an Opaque-LSA/ROUTER INFORMATION instance",
+			lsa_type, &lsa_id);
 
 	top = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
@@ -771,13 +798,10 @@ static struct ospf_lsa *ospf_router_info_lsa_new(struct ospf_area *area)
 	/* Now, create an OSPF LSA instance. */
 	new = ospf_lsa_new_and_data(length);
 
+	/* Routing Information is only supported for default VRF */
+	new->vrf_id = VRF_DEFAULT;
 	new->area = area;
 
-	if (new->area && new->area->ospf)
-		new->vrf_id = new->area->ospf->vrf_id;
-	else
-		new->vrf_id = VRF_DEFAULT;
-
 	SET_FLAG(new->flags, OSPF_LSA_SELF);
 	memcpy(new->data, lsah, length);
 	stream_free(s);
@@ -790,7 +814,6 @@ static int ospf_router_info_lsa_originate_as(void *arg)
 	struct ospf_lsa *new;
 	struct ospf *top;
 	int rc = -1;
-	vrf_id_t vrf_id = VRF_DEFAULT;
 
 	/* Sanity Check */
 	if (OspfRI.scope == OSPF_OPAQUE_AREA_LSA) {
@@ -803,13 +826,12 @@ static int ospf_router_info_lsa_originate_as(void *arg)
 
 	/* Create new Opaque-LSA/ROUTER INFORMATION instance. */
 	new = ospf_router_info_lsa_new(NULL);
-	new->vrf_id = VRF_DEFAULT;
 	top = (struct ospf *)arg;
 
 	/* Check ospf info */
 	if (top == NULL) {
 		zlog_debug("RI (%s): ospf instance not found for vrf id %u",
-			   __func__, vrf_id);
+			   __func__, VRF_DEFAULT);
 		ospf_lsa_unlock(&new);
 		return rc;
 	}
@@ -832,8 +854,8 @@ static int ospf_router_info_lsa_originate_as(void *arg)
 
 	if (IS_DEBUG_OSPF(lsa, LSA_GENERATE)) {
 		zlog_debug(
-			"LSA[Type%d:%s]: Originate Opaque-LSA/ROUTER INFORMATION",
-			new->data->type, inet_ntoa(new->data->id));
+			"LSA[Type%d:%pI4]: Originate Opaque-LSA/ROUTER INFORMATION",
+			new->data->type, &new->data->id);
 		ospf_lsa_header_dump(new->data);
 	}
 
@@ -847,7 +869,6 @@ static int ospf_router_info_lsa_originate_area(void *arg)
 	struct ospf *top;
 	struct ospf_ri_area_info *ai = NULL;
 	int rc = -1;
-	vrf_id_t vrf_id = VRF_DEFAULT;
 
 	/* Sanity Check */
 	if (OspfRI.scope == OSPF_OPAQUE_AS_LSA) {
@@ -866,19 +887,18 @@ static int ospf_router_info_lsa_originate_area(void *arg)
 			__func__);
 		return rc;
 	}
-	if (ai->area->ospf) {
-		vrf_id = ai->area->ospf->vrf_id;
+
+	if (ai->area->ospf)
 		top = ai->area->ospf;
-	} else {
-		top = ospf_lookup_by_vrf_id(vrf_id);
-	}
+	else
+		top = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+
 	new = ospf_router_info_lsa_new(ai->area);
-	new->vrf_id = vrf_id;
 
 	/* Check ospf info */
 	if (top == NULL) {
 		zlog_debug("RI (%s): ospf instance not found for vrf id %u",
-			   __func__, vrf_id);
+			   __func__, VRF_DEFAULT);
 		ospf_lsa_unlock(&new);
 		return rc;
 	}
@@ -901,8 +921,8 @@ static int ospf_router_info_lsa_originate_area(void *arg)
 
 	if (IS_DEBUG_OSPF(lsa, LSA_GENERATE)) {
 		zlog_debug(
-			"LSA[Type%d:%s]: Originate Opaque-LSA/ROUTER INFORMATION",
-			new->data->type, inet_ntoa(new->data->id));
+			"LSA[Type%d:%pI4]: Originate Opaque-LSA/ROUTER INFORMATION",
+			new->data->type, &new->data->id);
 		ospf_lsa_header_dump(new->data);
 	}
 
@@ -951,7 +971,7 @@ static int ospf_router_info_lsa_originate(void *arg)
 	}
 
 	/* Router Information is not yet Engaged, check parameters */
-	if (!is_mandated_params_set(OspfRI))
+	if (!is_mandated_params_set(&OspfRI))
 		flog_warn(
 			EC_OSPF_LSA,
 			"RI (%s): lacks mandated ROUTER INFORMATION parameters",
@@ -1012,10 +1032,9 @@ static struct ospf_lsa *ospf_router_info_lsa_refresh(struct ospf_lsa *lsa)
 		/* Create new Opaque-LSA/ROUTER INFORMATION instance. */
 		new = ospf_router_info_lsa_new(ai->area);
 		new->data->ls_seqnum = lsa_seqnum_increment(lsa);
-		new->vrf_id = lsa->vrf_id;
 		/* Install this LSA into LSDB. */
 		/* Given "lsa" will be freed in the next function. */
-		top = ospf_lookup_by_vrf_id(lsa->vrf_id);
+		top = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 		if (ospf_lsa_install(top, NULL /*oi */, new) == NULL) {
 			flog_warn(EC_OSPF_LSA_INSTALL_FAILURE,
 				  "RI (%s): ospf_lsa_install() ?", __func__);
@@ -1035,10 +1054,9 @@ static struct ospf_lsa *ospf_router_info_lsa_refresh(struct ospf_lsa *lsa)
 		/* Create new Opaque-LSA/ROUTER INFORMATION instance. */
 		new = ospf_router_info_lsa_new(NULL);
 		new->data->ls_seqnum = lsa_seqnum_increment(lsa);
-		new->vrf_id = lsa->vrf_id;
 		/* Install this LSA into LSDB. */
 		/* Given "lsa" will be freed in the next function. */
-		top = ospf_lookup_by_vrf_id(lsa->vrf_id);
+		top = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 		if (ospf_lsa_install(top, NULL /*oi */, new) == NULL) {
 			flog_warn(EC_OSPF_LSA_INSTALL_FAILURE,
 				  "RI (%s): ospf_lsa_install() ?", __func__);
@@ -1052,8 +1070,8 @@ static struct ospf_lsa *ospf_router_info_lsa_refresh(struct ospf_lsa *lsa)
 	/* Debug logging. */
 	if (IS_DEBUG_OSPF(lsa, LSA_GENERATE)) {
 		zlog_debug(
-			"LSA[Type%d:%s]: Refresh Opaque-LSA/ROUTER INFORMATION",
-			new->data->type, inet_ntoa(new->data->id));
+			"LSA[Type%d:%pI4]: Refresh Opaque-LSA/ROUTER INFORMATION",
+			new->data->type, &new->data->id);
 		ospf_lsa_header_dump(new->data);
 	}
 
@@ -1181,13 +1199,28 @@ static int ospf_router_info_lsa_update(struct ospf_lsa *lsa)
 }
 
 /*------------------------------------------------------------------------*
- * Followings are vty session control functions.
+ * Following are vty session control functions.
  *------------------------------------------------------------------------*/
 
+#define check_tlv_size(size, msg)                                              \
+	do {                                                                   \
+		if (ntohs(tlvh->length) > size) {                              \
+			if (vty != NULL)                                       \
+				vty_out(vty, "  Wrong %s TLV size: %d(%d)\n",  \
+					msg, ntohs(tlvh->length), size);       \
+			else                                                   \
+				zlog_debug("    Wrong %s TLV size: %d(%d)",    \
+					   msg, ntohs(tlvh->length), size);    \
+			return size + TLV_HDR_SIZE;                            \
+		}                                                              \
+	} while (0)
+
 static uint16_t show_vty_router_cap(struct vty *vty, struct tlv_header *tlvh)
 {
 	struct ri_tlv_router_cap *top = (struct ri_tlv_router_cap *)tlvh;
 
+	check_tlv_size(RI_TLV_CAPABILITIES_SIZE, "Router Capabilities");
+
 	if (vty != NULL)
 		vty_out(vty, "  Router Capabilities: 0x%x\n",
 			ntohl(top->value));
@@ -1203,21 +1236,30 @@ static uint16_t show_vty_pce_subtlv_address(struct vty *vty,
 	struct ri_pce_subtlv_address *top =
 		(struct ri_pce_subtlv_address *)tlvh;
 
-	if (ntohs(top->address.type) == PCE_ADDRESS_TYPE_IPV4) {
+	if (ntohs(top->address.type) == PCE_ADDRESS_IPV4) {
+		check_tlv_size(PCE_ADDRESS_IPV4_SIZE, "PCE Address");
 		if (vty != NULL)
-			vty_out(vty, "  PCE Address: %s\n",
-				inet_ntoa(top->address.value));
+			vty_out(vty, "  PCE Address: %pI4\n",
+				&top->address.value);
 		else
-			zlog_debug("    PCE Address: %s",
-				   inet_ntoa(top->address.value));
-	} else {
+			zlog_debug("    PCE Address: %pI4",
+				   &top->address.value);
+	} else if (ntohs(top->address.type) == PCE_ADDRESS_IPV6) {
 		/* TODO: Add support to IPv6 with inet_ntop() */
+		check_tlv_size(PCE_ADDRESS_IPV6_SIZE, "PCE Address");
 		if (vty != NULL)
 			vty_out(vty, "  PCE Address: 0x%x\n",
 				ntohl(top->address.value.s_addr));
 		else
 			zlog_debug("    PCE Address: 0x%x",
 				   ntohl(top->address.value.s_addr));
+	} else {
+		if (vty != NULL)
+			vty_out(vty, "  Wrong PCE Address type: 0x%x\n",
+				ntohl(top->address.type));
+		else
+			zlog_debug("    Wrong PCE Address type: 0x%x",
+				   ntohl(top->address.type));
 	}
 
 	return TLV_SIZE(tlvh);
@@ -1229,6 +1271,8 @@ static uint16_t show_vty_pce_subtlv_path_scope(struct vty *vty,
 	struct ri_pce_subtlv_path_scope *top =
 		(struct ri_pce_subtlv_path_scope *)tlvh;
 
+	check_tlv_size(RI_PCE_SUBTLV_PATH_SCOPE_SIZE, "PCE Path Scope");
+
 	if (vty != NULL)
 		vty_out(vty, "  PCE Path Scope: 0x%x\n", ntohl(top->value));
 	else
@@ -1243,19 +1287,29 @@ static uint16_t show_vty_pce_subtlv_domain(struct vty *vty,
 	struct ri_pce_subtlv_domain *top = (struct ri_pce_subtlv_domain *)tlvh;
 	struct in_addr tmp;
 
+	check_tlv_size(RI_PCE_SUBTLV_DOMAIN_SIZE, "PCE Domain");
+
 	if (ntohs(top->type) == PCE_DOMAIN_TYPE_AREA) {
 		tmp.s_addr = top->value;
 		if (vty != NULL)
-			vty_out(vty, "  PCE domain Area: %s\n", inet_ntoa(tmp));
+			vty_out(vty, "  PCE Domain Area: %pI4\n", &tmp);
 		else
-			zlog_debug("    PCE domain Area: %s", inet_ntoa(tmp));
-	} else {
+			zlog_debug("    PCE Domain Area: %pI4", &tmp);
+	} else if (ntohs(top->type) == PCE_DOMAIN_TYPE_AS) {
 		if (vty != NULL)
-			vty_out(vty, "  PCE domain AS: %d\n",
+			vty_out(vty, "  PCE Domain AS: %d\n",
 				ntohl(top->value));
 		else
-			zlog_debug("    PCE domain AS: %d", ntohl(top->value));
+			zlog_debug("    PCE Domain AS: %d", ntohl(top->value));
+	} else {
+		if (vty != NULL)
+			vty_out(vty, "  Wrong PCE Domain type: %d\n",
+				ntohl(top->type));
+		else
+			zlog_debug("    Wrong PCE Domain type: %d",
+				   ntohl(top->type));
 	}
+
 	return TLV_SIZE(tlvh);
 }
 
@@ -1267,21 +1321,30 @@ static uint16_t show_vty_pce_subtlv_neighbor(struct vty *vty,
 		(struct ri_pce_subtlv_neighbor *)tlvh;
 	struct in_addr tmp;
 
+	check_tlv_size(RI_PCE_SUBTLV_NEIGHBOR_SIZE, "PCE Neighbor");
+
 	if (ntohs(top->type) == PCE_DOMAIN_TYPE_AREA) {
 		tmp.s_addr = top->value;
 		if (vty != NULL)
-			vty_out(vty, "  PCE neighbor Area: %s\n",
-				inet_ntoa(tmp));
+			vty_out(vty, "  PCE Neighbor Area: %pI4\n", &tmp);
 		else
-			zlog_debug("    PCE neighbor Area: %s", inet_ntoa(tmp));
-	} else {
+			zlog_debug("    PCE Neighbor Area: %pI4", &tmp);
+	} else if (ntohs(top->type) == PCE_DOMAIN_TYPE_AS) {
 		if (vty != NULL)
-			vty_out(vty, "  PCE neighbor AS: %d\n",
+			vty_out(vty, "  PCE Neighbor AS: %d\n",
 				ntohl(top->value));
 		else
-			zlog_debug("    PCE neighbor AS: %d",
+			zlog_debug("    PCE Neighbor AS: %d",
 				   ntohl(top->value));
+	} else {
+		if (vty != NULL)
+			vty_out(vty, "  Wrong PCE Neighbor type: %d\n",
+				ntohl(top->type));
+		else
+			zlog_debug("    Wrong PCE Neighbor type: %d",
+				   ntohl(top->type));
 	}
+
 	return TLV_SIZE(tlvh);
 }
 
@@ -1291,6 +1354,8 @@ static uint16_t show_vty_pce_subtlv_cap_flag(struct vty *vty,
 	struct ri_pce_subtlv_cap_flag *top =
 		(struct ri_pce_subtlv_cap_flag *)tlvh;
 
+	check_tlv_size(RI_PCE_SUBTLV_CAP_FLAG_SIZE, "PCE Capabilities");
+
 	if (vty != NULL)
 		vty_out(vty, "  PCE Capabilities Flag: 0x%x\n",
 			ntohl(top->value));
@@ -1301,8 +1366,21 @@ static uint16_t show_vty_pce_subtlv_cap_flag(struct vty *vty,
 	return TLV_SIZE(tlvh);
 }
 
-static uint16_t show_vty_unknown_tlv(struct vty *vty, struct tlv_header *tlvh)
+static uint16_t show_vty_unknown_tlv(struct vty *vty, struct tlv_header *tlvh,
+				     size_t buf_size)
 {
+	if (TLV_SIZE(tlvh) > buf_size) {
+		if (vty != NULL)
+			vty_out(vty,
+				"    TLV size %d exceeds buffer size. Abort!",
+				TLV_SIZE(tlvh));
+		else
+			zlog_debug(
+				"    TLV size %d exceeds buffer size. Abort!",
+				TLV_SIZE(tlvh));
+		return buf_size;
+	}
+
 	if (vty != NULL)
 		vty_out(vty, "  Unknown TLV: [type(0x%x), length(0x%x)]\n",
 			ntohs(tlvh->type), ntohs(tlvh->length));
@@ -1314,12 +1392,21 @@ static uint16_t show_vty_unknown_tlv(struct vty *vty, struct tlv_header *tlvh)
 }
 
 static uint16_t show_vty_pce_info(struct vty *vty, struct tlv_header *ri,
-				  uint32_t total)
+				  size_t buf_size)
 {
 	struct tlv_header *tlvh;
+	uint16_t length = ntohs(ri->length);
 	uint16_t sum = 0;
 
-	for (tlvh = ri; sum < total; tlvh = TLV_HDR_NEXT(tlvh)) {
+	/* Verify that TLV length is valid against remaining buffer size */
+	if (length > buf_size) {
+		vty_out(vty,
+			"  PCE Info TLV size %d exceeds buffer size. Abort!\n",
+			length);
+		return buf_size;
+	}
+
+	for (tlvh = ri; sum < length; tlvh = TLV_HDR_NEXT(tlvh)) {
 		switch (ntohs(tlvh->type)) {
 		case RI_PCE_SUBTLV_ADDRESS:
 			sum += show_vty_pce_subtlv_address(vty, tlvh);
@@ -1337,7 +1424,7 @@ static uint16_t show_vty_pce_info(struct vty *vty, struct tlv_header *ri,
 			sum += show_vty_pce_subtlv_cap_flag(vty, tlvh);
 			break;
 		default:
-			sum += show_vty_unknown_tlv(vty, tlvh);
+			sum += show_vty_unknown_tlv(vty, tlvh, length - sum);
 			break;
 		}
 	}
@@ -1351,6 +1438,8 @@ static uint16_t show_vty_sr_algorithm(struct vty *vty, struct tlv_header *tlvh)
 		(struct ri_sr_tlv_sr_algorithm *)tlvh;
 	int i;
 
+	check_tlv_size(ALGORITHM_COUNT, "Segment Routing Algorithm");
+
 	if (vty != NULL) {
 		vty_out(vty, "  Segment Routing Algorithm TLV:\n");
 		for (i = 0; i < ntohs(algo->header.length); i++) {
@@ -1369,9 +1458,7 @@ static uint16_t show_vty_sr_algorithm(struct vty *vty, struct tlv_header *tlvh)
 				break;
 			}
 		}
-	}
-
-	else {
+	} else {
 		zlog_debug("  Segment Routing Algorithm TLV:");
 		for (i = 0; i < ntohs(algo->header.length); i++)
 			switch (algo->value[i]) {
@@ -1397,18 +1484,24 @@ static uint16_t show_vty_sr_range(struct vty *vty, struct tlv_header *tlvh)
 	struct ri_sr_tlv_sid_label_range *range =
 		(struct ri_sr_tlv_sid_label_range *)tlvh;
 
+	check_tlv_size(RI_SR_TLV_LABEL_RANGE_SIZE, "SR Label Range");
+
 	if (vty != NULL) {
 		vty_out(vty,
-			"  Segment Routing Range TLV:\n"
+			"  Segment Routing %s Range TLV:\n"
 			"    Range Size = %d\n"
 			"    SID Label = %d\n\n",
+			ntohs(range->header.type) == RI_SR_TLV_SRGB_LABEL_RANGE
+				? "Global"
+				: "Local",
 			GET_RANGE_SIZE(ntohl(range->size)),
 			GET_LABEL(ntohl(range->lower.value)));
 	} else {
 		zlog_debug(
-			"  Segment Routing Range TLV:\n"
-			"    Range Size = %d\n"
-			"    SID Label = %d\n\n",
+			"  Segment Routing %s Range TLV:  Range Size = %d  SID Label = %d",
+			ntohs(range->header.type) == RI_SR_TLV_SRGB_LABEL_RANGE
+				? "Global"
+				: "Local",
 			GET_RANGE_SIZE(ntohl(range->size)),
 			GET_LABEL(ntohl(range->lower.value)));
 	}
@@ -1421,6 +1514,8 @@ static uint16_t show_vty_sr_msd(struct vty *vty, struct tlv_header *tlvh)
 {
 	struct ri_sr_tlv_node_msd *msd = (struct ri_sr_tlv_node_msd *)tlvh;
 
+	check_tlv_size(RI_SR_TLV_NODE_MSD_SIZE, "Node Maximum Stack Depth");
+
 	if (vty != NULL) {
 		vty_out(vty,
 			"  Segment Routing MSD TLV:\n"
@@ -1428,24 +1523,28 @@ static uint16_t show_vty_sr_msd(struct vty *vty, struct tlv_header *tlvh)
 			msd->value);
 	} else {
 		zlog_debug(
-			"  Segment Routing MSD TLV:\n"
-			"    Node Maximum Stack Depth = %d\n",
+			"  Segment Routing MSD TLV:  Node Maximum Stack Depth = %d",
 			msd->value);
 	}
 
 	return TLV_SIZE(tlvh);
 }
 
-static void ospf_router_info_show_info(struct vty *vty, struct ospf_lsa *lsa)
+static void ospf_router_info_show_info(struct vty *vty,
+				       struct json_object *json,
+				       struct ospf_lsa *lsa)
 {
 	struct lsa_header *lsah = lsa->data;
 	struct tlv_header *tlvh;
 	uint16_t length = 0, sum = 0;
 
-	/* Initialize TLV browsing */
-	length = ntohs(lsah->length) - OSPF_LSA_HEADER_SIZE;
+	if (json)
+		return;
 
-	for (tlvh = TLV_HDR_TOP(lsah); sum < length;
+	/* Initialize TLV browsing */
+	length = lsa->size - OSPF_LSA_HEADER_SIZE;
+
+	for (tlvh = TLV_HDR_TOP(lsah); sum < length && tlvh;
 	     tlvh = TLV_HDR_NEXT(tlvh)) {
 		switch (ntohs(tlvh->type)) {
 		case RI_TLV_CAPABILITIES:
@@ -1459,7 +1558,8 @@ static void ospf_router_info_show_info(struct vty *vty, struct ospf_lsa *lsa)
 		case RI_SR_TLV_SR_ALGORITHM:
 			sum += show_vty_sr_algorithm(vty, tlvh);
 			break;
-		case RI_SR_TLV_SID_LABEL_RANGE:
+		case RI_SR_TLV_SRGB_LABEL_RANGE:
+		case RI_SR_TLV_SRLB_LABEL_RANGE:
 			sum += show_vty_sr_range(vty, tlvh);
 			break;
 		case RI_SR_TLV_NODE_MSD:
@@ -1467,7 +1567,7 @@ static void ospf_router_info_show_info(struct vty *vty, struct ospf_lsa *lsa)
 			break;
 
 		default:
-			sum += show_vty_unknown_tlv(vty, tlvh);
+			sum += show_vty_unknown_tlv(vty, tlvh, length);
 			break;
 		}
 	}
@@ -1494,8 +1594,8 @@ static void ospf_router_info_config_write_router(struct vty *vty)
 	if (OspfRI.pce_info.enabled) {
 
 		if (pce->pce_address.header.type != 0)
-			vty_out(vty, "  pce address %s\n",
-				inet_ntoa(pce->pce_address.address.value));
+			vty_out(vty, "  pce address %pI4\n",
+				&pce->pce_address.address.value);
 
 		if (pce->pce_cap_flag.header.type != 0)
 			vty_out(vty, "  pce flag 0x%x\n",
@@ -1505,8 +1605,8 @@ static void ospf_router_info_config_write_router(struct vty *vty)
 			if (domain->header.type != 0) {
 				if (domain->type == PCE_DOMAIN_TYPE_AREA) {
 					tmp.s_addr = domain->value;
-					vty_out(vty, "  pce domain area %s\n",
-						inet_ntoa(tmp));
+					vty_out(vty, "  pce domain area %pI4\n",
+						&tmp);
 				} else {
 					vty_out(vty, "  pce domain as %d\n",
 						ntohl(domain->value));
@@ -1518,8 +1618,9 @@ static void ospf_router_info_config_write_router(struct vty *vty)
 			if (neighbor->header.type != 0) {
 				if (neighbor->type == PCE_DOMAIN_TYPE_AREA) {
 					tmp.s_addr = neighbor->value;
-					vty_out(vty, "  pce neighbor area %s\n",
-						inet_ntoa(tmp));
+					vty_out(vty,
+						"  pce neighbor area %pI4\n",
+						&tmp);
 				} else {
 					vty_out(vty, "  pce neighbor as %d\n",
 						ntohl(neighbor->value));
@@ -1535,7 +1636,7 @@ static void ospf_router_info_config_write_router(struct vty *vty)
 }
 
 /*------------------------------------------------------------------------*
- * Followings are vty command functions.
+ * Following are vty command functions.
  *------------------------------------------------------------------------*/
 /* Simple wrapper schedule RI LSA action in function of the scope */
 static void ospf_router_info_schedule(enum lsa_opcode opcode)
@@ -1566,10 +1667,18 @@ DEFUN (router_info,
 {
 	int idx_mode = 1;
 	uint8_t scope;
+	VTY_DECLVAR_INSTANCE_CONTEXT(ospf, ospf);
 
 	if (OspfRI.enabled)
 		return CMD_SUCCESS;
 
+	/* Check that the OSPF is using default VRF */
+	if (ospf->vrf_id != VRF_DEFAULT) {
+		vty_out(vty,
+			"Router Information is only supported in default VRF\n");
+		return CMD_WARNING_CONFIG_FAILED;
+	}
+
 	/* Check and get Area value if present */
 	if (strncmp(argv[idx_mode]->arg, "as", 2) == 0)
 		scope = OSPF_OPAQUE_AS_LSA;
