@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Zebra connect library for PBR
  * Copyright (C) 2018 Cumulus Networks, Inc.
  *               Donald Sharp
- *
- * FRR is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * FRR is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #ifndef __PBR_ZEBRA_H__
 #define __PBR_ZEBRA_H__
@@ -24,7 +11,7 @@ struct pbr_interface {
 	char mapname[100];
 };
 
-extern struct thread_master *master;
+extern struct event_loop *master;
 
 extern void pbr_zebra_init(void);
 
@@ -35,7 +22,7 @@ extern void route_delete(struct pbr_nexthop_group_cache *pnhgc,
 
 extern void pbr_send_rnh(struct nexthop *nhop, bool reg);
 
-extern void pbr_send_pbr_map(struct pbr_map_sequence *pbrms,
+extern bool pbr_send_pbr_map(struct pbr_map_sequence *pbrms,
 			     struct pbr_map_interface *pmi, bool install,
 			     bool changed);
 
@@ -46,4 +33,7 @@ extern int pbr_ifp_up(struct interface *ifp);
 extern int pbr_ifp_down(struct interface *ifp);
 extern int pbr_ifp_destroy(struct interface *ifp);
 
+/* Free the ifp->info pointer */
+extern void pbr_if_del(struct interface *ifp);
+
 #endif
