@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * PIM for IPv6 FRR
  * Copyright (C) 2022  Vmware, Inc.
  *		       Mobashshera Rasool <mrasool@vmware.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -257,7 +244,7 @@ DEFPY (interface_no_ipv6_pim,
 
 DEFPY (interface_ipv6_pim_drprio,
        interface_ipv6_pim_drprio_cmd,
-       "ipv6 pim drpriority (0-4294967295)",
+       "ipv6 pim drpriority (1-4294967295)",
        IPV6_STR
        PIM_STR
        "Set the Designated Router Election Priority\n"
@@ -268,7 +255,7 @@ DEFPY (interface_ipv6_pim_drprio,
 
 DEFPY (interface_no_ipv6_pim_drprio,
        interface_no_ipv6_pim_drprio_cmd,
-       "no ipv6 pim drpriority [(0-4294967295)]",
+       "no ipv6 pim drpriority [(1-4294967295)]",
        NO_STR
        IPV6_STR
        PIM_STR
@@ -553,6 +540,11 @@ DEFPY (interface_ipv6_mld_join,
 {
 	char xpath[XPATH_MAXLEN];
 
+	if (!IN6_IS_ADDR_MULTICAST(&group)) {
+		vty_out(vty, "Invalid Multicast Address\n");
+		return CMD_WARNING_CONFIG_FAILED;
+	}
+
 	if (source_str) {
 		if (IPV6_ADDR_SAME(&source, &in6addr_any)) {
 			vty_out(vty, "Bad source address %s\n", source_str);
@@ -751,7 +743,7 @@ DEFPY (interface_ipv6_mld_query_max_response_time,
        IPV6_STR
        IFACE_MLD_STR
        IFACE_MLD_QUERY_MAX_RESPONSE_TIME_STR
-       "Query response value in milliseconds\n")
+       "Query response value in deci-seconds\n")
 {
 	return gm_process_query_max_response_time_cmd(vty, qmrt_str);
 }
