@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * PIMv6 MLD querier
  * Copyright (C) 2021-2022  David Lamparter for NetDEF, Inc.
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /*
@@ -32,7 +19,7 @@
 #include "lib/jhash.h"
 #include "lib/prefix.h"
 #include "lib/checksum.h"
-#include "lib/thread.h"
+#include "lib/frrevent.h"
 #include "termtable.h"
 
 #include "pimd/pim6_mld.h"
@@ -58,7 +45,7 @@ DEFINE_MTYPE_STATIC(PIMD, GM_SG, "MLD (S,G)");
 DEFINE_MTYPE_STATIC(PIMD, GM_GRP_PENDING, "MLD group query state");
 DEFINE_MTYPE_STATIC(PIMD, GM_GSQ_PENDING, "MLD group/source query aggregate");
 
-static void gm_t_query(struct thread *t);
+static void gm_t_query(struct event *t);
 static void gm_trigger_specific(struct gm_sg *sg);
 static void gm_sg_timer_start(struct gm_if *gm_ifp, struct gm_sg *sg,
 			      struct timeval expire_wait);
@@ -340,7 +327,7 @@ static void gm_expiry_calc(struct gm_query_timers *timers)
 static void gm_sg_free(struct gm_sg *sg)
 {
 	/* t_sg_expiry is handled before this is reached */
-	THREAD_OFF(sg->t_sg_query);
+	EVENT_OFF(sg->t_sg_query);
 	gm_packet_sg_subs_fini(sg->subs_negative);
 	gm_packet_sg_subs_fini(sg->subs_positive);
 	XFREE(MTYPE_GM_SG, sg);
@@ -357,7 +344,8 @@ static const char *const gm_states[] = {
 };
 /* clang-format on */
 
-CPP_NOTICE("TODO: S,G entries in EXCLUDE (i.e. prune) unsupported");
+/* TODO: S,G entries in EXCLUDE (i.e. prune) unsupported" */
+
 /* tib_sg_gm_prune() below is an "un-join", it doesn't prune S,G when *,G is
  * joined.  Whether we actually want/need to support this is a separate
  * question - it is almost never used.  In fact this is exactly what RFC5790
@@ -367,6 +355,7 @@ CPP_NOTICE("TODO: S,G entries in EXCLUDE (i.e. prune) unsupported");
 static void gm_sg_update(struct gm_sg *sg, bool has_expired)
 {
 	struct gm_if *gm_ifp = sg->iface;
+	struct pim_interface *pim_ifp = gm_ifp->ifp->info;
 	enum gm_sg_state prev, desired;
 	bool new_join;
 	struct gm_sg *grp = NULL;
@@ -415,10 +404,13 @@ static void gm_sg_update(struct gm_sg *sg, bool has_expired)
 			gm_expiry_calc(&timers);
 			gm_sg_timer_start(gm_ifp, sg, timers.expire_wait);
 
-			THREAD_OFF(sg->t_sg_query);
-			sg->n_query = gm_ifp->cur_lmqc;
+			EVENT_OFF(sg->t_sg_query);
 			sg->query_sbit = false;
-			gm_trigger_specific(sg);
+			/* Trigger the specific queries only for querier. */
+			if (IPV6_ADDR_SAME(&gm_ifp->querier, &pim_ifp->ll_lowest)) {
+				sg->n_query = gm_ifp->cur_lmqc;
+				gm_trigger_specific(sg);
+			}
 		}
 	}
 	prev = sg->state;
@@ -449,6 +441,13 @@ static void gm_sg_update(struct gm_sg *sg, bool has_expired)
 	}
 
 	if (desired == GM_SG_NOINFO) {
+		/* multiple paths can lead to the last state going away;
+		 * t_sg_expire can still be running if we're arriving from
+		 * another path.
+		 */
+		if (has_expired)
+			EVENT_OFF(sg->t_sg_expire);
+
 		assertf((!sg->t_sg_expire &&
 			 !gm_packet_sg_subs_count(sg->subs_positive) &&
 			 !gm_packet_sg_subs_count(sg->subs_negative)),
@@ -586,7 +585,7 @@ static void gm_sg_expiry_cancel(struct gm_sg *sg)
 {
 	if (sg->t_sg_expire && PIM_DEBUG_GM_TRACE)
 		zlog_debug(log_sg(sg, "alive, cancelling expiry timer"));
-	THREAD_OFF(sg->t_sg_expire);
+	EVENT_OFF(sg->t_sg_expire);
 	sg->query_sbit = true;
 }
 
@@ -600,7 +599,7 @@ static void gm_sg_expiry_cancel(struct gm_sg *sg)
  * everything else is thrown into pkt for creation of state in pass 2
  */
 static void gm_handle_v2_pass1(struct gm_packet_state *pkt,
-			       struct mld_v2_rec_hdr *rechdr)
+			       struct mld_v2_rec_hdr *rechdr, size_t n_src)
 {
 	/* NB: pkt->subscriber can be NULL here if the subscriber was not
 	 * previously seen!
@@ -609,7 +608,6 @@ static void gm_handle_v2_pass1(struct gm_packet_state *pkt,
 	struct gm_sg *grp;
 	struct gm_packet_sg *old_grp = NULL;
 	struct gm_packet_sg *item;
-	size_t n_src = ntohs(rechdr->n_src);
 	size_t j;
 	bool is_excl = false;
 
@@ -652,7 +650,7 @@ static void gm_handle_v2_pass1(struct gm_packet_state *pkt,
 			 */
 			gm_packet_sg_drop(old_grp);
 			gm_sg_update(grp, false);
-			CPP_NOTICE("need S,G PRUNE => NO_INFO transition here");
+/* TODO "need S,G PRUNE => NO_INFO transition here" */
 		}
 		break;
 
@@ -800,7 +798,8 @@ static void gm_handle_v2_pass2_excl(struct gm_packet_state *pkt, size_t offs)
 	gm_sg_update(sg_grp, false);
 }
 
-CPP_NOTICE("TODO: QRV/QQIC are not copied from queries to local state");
+/* TODO: QRV/QQIC are not copied from queries to local state" */
+
 /* on receiving a query, we need to update our robustness/query interval to
  * match, so we correctly process group/source specific queries after last
  * member leaves
@@ -822,13 +821,24 @@ static void gm_handle_v2_report(struct gm_if *gm_ifp,
 		return;
 	}
 
-	/* errors after this may at least partially process the packet */
-	gm_ifp->stats.rx_new_report++;
-
 	hdr = (struct mld_v2_report_hdr *)data;
 	data += sizeof(*hdr);
 	len -= sizeof(*hdr);
 
+	n_records = ntohs(hdr->n_records);
+	if (n_records > len / sizeof(struct mld_v2_rec_hdr)) {
+		/* note this is only an upper bound, records with source lists
+		 * are larger.  This is mostly here to make coverity happy.
+		 */
+		zlog_warn(log_pkt_src(
+			"malformed MLDv2 report (infeasible record count)"));
+		gm_ifp->stats.rx_drop_malformed++;
+		return;
+	}
+
+	/* errors after this may at least partially process the packet */
+	gm_ifp->stats.rx_new_report++;
+
 	/* can't have more *,G and S,G items than there is space for ipv6
 	 * addresses, so just use this to allocate temporary buffer
 	 */
@@ -839,8 +849,6 @@ static void gm_handle_v2_report(struct gm_if *gm_ifp,
 	pkt->iface = gm_ifp;
 	pkt->subscriber = gm_subscriber_findref(gm_ifp, pkt_src->sin6_addr);
 
-	n_records = ntohs(hdr->n_records);
-
 	/* validate & remove state in v2_pass1() */
 	for (i = 0; i < n_records; i++) {
 		struct mld_v2_rec_hdr *rechdr;
@@ -878,7 +886,7 @@ static void gm_handle_v2_report(struct gm_if *gm_ifp,
 		data += record_size;
 		len -= record_size;
 
-		gm_handle_v2_pass1(pkt, rechdr);
+		gm_handle_v2_pass1(pkt, rechdr, n_src);
 	}
 
 	if (!pkt->n_active) {
@@ -947,7 +955,8 @@ static void gm_handle_v1_report(struct gm_if *gm_ifp,
 
 	item = gm_packet_sg_setup(pkt, grp, true, false);
 	item->n_exclude = 0;
-	CPP_NOTICE("set v1-seen timer on grp here");
+
+/* TODO "set v1-seen timer on grp here" */
 
 	/* } */
 
@@ -1010,7 +1019,9 @@ static void gm_handle_v1_leave(struct gm_if *gm_ifp,
 		if (old_grp) {
 			gm_packet_sg_drop(old_grp);
 			gm_sg_update(grp, false);
-			CPP_NOTICE("need S,G PRUNE => NO_INFO transition here");
+
+/* TODO "need S,G PRUNE => NO_INFO transition here" */
+
 		}
 	}
 
@@ -1033,9 +1044,9 @@ static void gm_handle_v1_leave(struct gm_if *gm_ifp,
  * its own path too and won't hit this.  This is really only triggered when a
  * host straight up disappears.
  */
-static void gm_t_expire(struct thread *t)
+static void gm_t_expire(struct event *t)
 {
-	struct gm_if *gm_ifp = THREAD_ARG(t);
+	struct gm_if *gm_ifp = EVENT_ARG(t);
 	struct gm_packet_state *pkt;
 
 	zlog_info(log_ifp("general expiry timer"));
@@ -1052,8 +1063,8 @@ static void gm_t_expire(struct thread *t)
 					log_ifp("next general expiry in %" PRId64 "ms"),
 					remain_ms / 1000);
 
-			thread_add_timer_tv(router->master, gm_t_expire, gm_ifp,
-					    &remain, &gm_ifp->t_expire);
+			event_add_timer_tv(router->master, gm_t_expire, gm_ifp,
+					   &remain, &gm_ifp->t_expire);
 			return;
 		}
 
@@ -1107,7 +1118,7 @@ static void gm_handle_q_general(struct gm_if *gm_ifp,
 
 		gm_ifp->n_pending--;
 		if (!gm_ifp->n_pending)
-			THREAD_OFF(gm_ifp->t_expire);
+			EVENT_OFF(gm_ifp->t_expire);
 	}
 
 	/* people might be messing with their configs or something */
@@ -1123,16 +1134,16 @@ static void gm_handle_q_general(struct gm_if *gm_ifp,
 			zlog_debug(
 				log_ifp("starting general timer @ 0: %pTVMu"),
 				&pend->expiry);
-		thread_add_timer_tv(router->master, gm_t_expire, gm_ifp,
-				    &timers->expire_wait, &gm_ifp->t_expire);
+		event_add_timer_tv(router->master, gm_t_expire, gm_ifp,
+				   &timers->expire_wait, &gm_ifp->t_expire);
 	} else if (PIM_DEBUG_GM_TRACE)
 		zlog_debug(log_ifp("appending general timer @ %u: %pTVMu"),
 			   gm_ifp->n_pending, &pend->expiry);
 }
 
-static void gm_t_sg_expire(struct thread *t)
+static void gm_t_sg_expire(struct event *t)
 {
-	struct gm_sg *sg = THREAD_ARG(t);
+	struct gm_sg *sg = EVENT_ARG(t);
 	struct gm_if *gm_ifp = sg->iface;
 	struct gm_packet_sg *item;
 
@@ -1212,15 +1223,15 @@ static void gm_sg_timer_start(struct gm_if *gm_ifp, struct gm_sg *sg,
 	if (sg->t_sg_expire) {
 		struct timeval remain;
 
-		remain = thread_timer_remain(sg->t_sg_expire);
+		remain = event_timer_remain(sg->t_sg_expire);
 		if (timercmp(&remain, &expire_wait, <=))
 			return;
 
-		THREAD_OFF(sg->t_sg_expire);
+		EVENT_OFF(sg->t_sg_expire);
 	}
 
-	thread_add_timer_tv(router->master, gm_t_sg_expire, sg, &expire_wait,
-			    &sg->t_sg_expire);
+	event_add_timer_tv(router->master, gm_t_sg_expire, sg, &expire_wait,
+			   &sg->t_sg_expire);
 }
 
 static void gm_handle_q_groupsrc(struct gm_if *gm_ifp,
@@ -1232,11 +1243,12 @@ static void gm_handle_q_groupsrc(struct gm_if *gm_ifp,
 
 	for (i = 0; i < n_src; i++) {
 		sg = gm_sg_find(gm_ifp, grp, srcs[i]);
+		GM_UPDATE_SG_STATE(sg);
 		gm_sg_timer_start(gm_ifp, sg, timers->expire_wait);
 	}
 }
 
-static void gm_t_grp_expire(struct thread *t)
+static void gm_t_grp_expire(struct event *t)
 {
 	/* if we're here, that means when we received the group-specific query
 	 * there was one or more active S,G for this group.  For *,G the timer
@@ -1244,7 +1256,7 @@ static void gm_t_grp_expire(struct thread *t)
 	 * receive a report, so that work is left to gm_t_sg_expire and we
 	 * shouldn't worry about it here.
 	 */
-	struct gm_grp_pending *pend = THREAD_ARG(t);
+	struct gm_grp_pending *pend = EVENT_ARG(t);
 	struct gm_if *gm_ifp = pend->iface;
 	struct gm_sg *sg, *sg_start, sg_ref = {};
 
@@ -1273,7 +1285,7 @@ static void gm_t_grp_expire(struct thread *t)
 		 * parallel.  But if we received nothing for the *,G query,
 		 * the S,G query is kinda irrelevant.
 		 */
-		THREAD_OFF(sg->t_sg_expire);
+		EVENT_OFF(sg->t_sg_expire);
 
 		frr_each_safe (gm_packet_sg_subs, sg->subs_positive, item)
 			/* this will also drop the EXCLUDE S,G lists */
@@ -1306,6 +1318,7 @@ static void gm_handle_q_group(struct gm_if *gm_ifp,
 		if (PIM_DEBUG_GM_TRACE)
 			zlog_debug(log_ifp("*,%pPAs expiry timer starting"),
 				   &grp);
+		GM_UPDATE_SG_STATE(sg);
 		gm_sg_timer_start(gm_ifp, sg, timers->expire_wait);
 
 		sg = gm_sgs_next(gm_ifp->sgs, sg);
@@ -1320,11 +1333,11 @@ static void gm_handle_q_group(struct gm_if *gm_ifp,
 	if (pend) {
 		struct timeval remain;
 
-		remain = thread_timer_remain(pend->t_expire);
+		remain = event_timer_remain(pend->t_expire);
 		if (timercmp(&remain, &timers->expire_wait, <=))
 			return;
 
-		THREAD_OFF(pend->t_expire);
+		EVENT_OFF(pend->t_expire);
 	} else {
 		pend = XCALLOC(MTYPE_GM_GRP_PENDING, sizeof(*pend));
 		pend->grp = grp;
@@ -1333,8 +1346,8 @@ static void gm_handle_q_group(struct gm_if *gm_ifp,
 	}
 
 	monotime(&pend->query);
-	thread_add_timer_tv(router->master, gm_t_grp_expire, pend,
-			    &timers->expire_wait, &pend->t_expire);
+	event_add_timer_tv(router->master, gm_t_grp_expire, pend,
+			   &timers->expire_wait, &pend->t_expire);
 
 	if (PIM_DEBUG_GM_TRACE)
 		zlog_debug(log_ifp("*,%pPAs S,G timer started: %pTHD"), &grp,
@@ -1345,7 +1358,7 @@ static void gm_bump_querier(struct gm_if *gm_ifp)
 {
 	struct pim_interface *pim_ifp = gm_ifp->ifp->info;
 
-	THREAD_OFF(gm_ifp->t_query);
+	EVENT_OFF(gm_ifp->t_query);
 
 	if (pim_addr_is_any(pim_ifp->ll_lowest))
 		return;
@@ -1354,12 +1367,12 @@ static void gm_bump_querier(struct gm_if *gm_ifp)
 
 	gm_ifp->n_startup = gm_ifp->cur_qrv;
 
-	thread_execute(router->master, gm_t_query, gm_ifp, 0);
+	event_execute(router->master, gm_t_query, gm_ifp, 0, NULL);
 }
 
-static void gm_t_other_querier(struct thread *t)
+static void gm_t_other_querier(struct event *t)
 {
-	struct gm_if *gm_ifp = THREAD_ARG(t);
+	struct gm_if *gm_ifp = EVENT_ARG(t);
 	struct pim_interface *pim_ifp = gm_ifp->ifp->info;
 
 	zlog_info(log_ifp("other querier timer expired"));
@@ -1367,7 +1380,7 @@ static void gm_t_other_querier(struct thread *t)
 	gm_ifp->querier = pim_ifp->ll_lowest;
 	gm_ifp->n_startup = gm_ifp->cur_qrv;
 
-	thread_execute(router->master, gm_t_query, gm_ifp, 0);
+	event_execute(router->master, gm_t_query, gm_ifp, 0, NULL);
 }
 
 static void gm_handle_query(struct gm_if *gm_ifp,
@@ -1470,13 +1483,12 @@ static void gm_handle_query(struct gm_if *gm_ifp,
 	if (IPV6_ADDR_CMP(&pkt_src->sin6_addr, &pim_ifp->ll_lowest) < 0) {
 		unsigned int other_ms;
 
-		THREAD_OFF(gm_ifp->t_query);
-		THREAD_OFF(gm_ifp->t_other_querier);
+		EVENT_OFF(gm_ifp->t_query);
+		EVENT_OFF(gm_ifp->t_other_querier);
 
 		other_ms = timers.qrv * timers.qqic_ms + timers.max_resp_ms / 2;
-		thread_add_timer_msec(router->master, gm_t_other_querier,
-				      gm_ifp, other_ms,
-				      &gm_ifp->t_other_querier);
+		event_add_timer_msec(router->master, gm_t_other_querier, gm_ifp,
+				     other_ms, &gm_ifp->t_other_querier);
 	}
 
 	if (len == sizeof(struct mld_v1_pkt)) {
@@ -1503,6 +1515,15 @@ static void gm_handle_query(struct gm_if *gm_ifp,
 		gm_handle_q_group(gm_ifp, &timers, hdr->grp);
 		gm_ifp->stats.rx_query_new_group++;
 	} else {
+		/* this is checked above:
+		 * if (len >= sizeof(struct mld_v2_query_hdr)) {
+		 *   size_t src_space = ntohs(hdr->n_src) * sizeof(pim_addr);
+		 *   if (len < sizeof(struct mld_v2_query_hdr) + src_space) {
+		 */
+		assume(ntohs(hdr->n_src) <=
+		       (len - sizeof(struct mld_v2_query_hdr)) /
+			       sizeof(pim_addr));
+
 		gm_handle_q_groupsrc(gm_ifp, &timers, hdr->grp, hdr->srcs,
 				     ntohs(hdr->n_src));
 		gm_ifp->stats.rx_query_new_groupsrc++;
@@ -1590,9 +1611,9 @@ static bool ip6_check_hopopts_ra(uint8_t *hopopts, size_t hopopt_len,
 	return false;
 }
 
-static void gm_t_recv(struct thread *t)
+static void gm_t_recv(struct event *t)
 {
-	struct pim_instance *pim = THREAD_ARG(t);
+	struct pim_instance *pim = EVENT_ARG(t);
 	union {
 		char buf[CMSG_SPACE(sizeof(struct in6_pktinfo)) +
 			 CMSG_SPACE(256) /* hop options */ +
@@ -1611,8 +1632,8 @@ static void gm_t_recv(struct thread *t)
 	ssize_t nread;
 	size_t pktlen;
 
-	thread_add_read(router->master, gm_t_recv, pim, pim->gm_socket,
-			&pim->t_gm_recv);
+	event_add_read(router->master, gm_t_recv, pim, pim->gm_socket,
+		       &pim->t_gm_recv);
 
 	iov->iov_base = rxbuf;
 	iov->iov_len = sizeof(rxbuf);
@@ -1862,9 +1883,9 @@ static void gm_send_query(struct gm_if *gm_ifp, pim_addr grp,
 	}
 }
 
-static void gm_t_query(struct thread *t)
+static void gm_t_query(struct event *t)
 {
-	struct gm_if *gm_ifp = THREAD_ARG(t);
+	struct gm_if *gm_ifp = EVENT_ARG(t);
 	unsigned int timer_ms = gm_ifp->cur_query_intv;
 
 	if (gm_ifp->n_startup) {
@@ -1872,15 +1893,15 @@ static void gm_t_query(struct thread *t)
 		gm_ifp->n_startup--;
 	}
 
-	thread_add_timer_msec(router->master, gm_t_query, gm_ifp, timer_ms,
-			      &gm_ifp->t_query);
+	event_add_timer_msec(router->master, gm_t_query, gm_ifp, timer_ms,
+			     &gm_ifp->t_query);
 
 	gm_send_query(gm_ifp, PIMADDR_ANY, NULL, 0, false);
 }
 
-static void gm_t_sg_query(struct thread *t)
+static void gm_t_sg_query(struct event *t)
 {
-	struct gm_sg *sg = THREAD_ARG(t);
+	struct gm_sg *sg = EVENT_ARG(t);
 
 	gm_trigger_specific(sg);
 }
@@ -1899,9 +1920,9 @@ static void gm_send_specific(struct gm_gsq_pending *pend_gsq)
 	XFREE(MTYPE_GM_GSQ_PENDING, pend_gsq);
 }
 
-static void gm_t_gsq_pend(struct thread *t)
+static void gm_t_gsq_pend(struct event *t)
 {
-	struct gm_gsq_pending *pend_gsq = THREAD_ARG(t);
+	struct gm_gsq_pending *pend_gsq = EVENT_ARG(t);
 
 	gm_send_specific(pend_gsq);
 }
@@ -1909,17 +1930,28 @@ static void gm_t_gsq_pend(struct thread *t)
 static void gm_trigger_specific(struct gm_sg *sg)
 {
 	struct gm_if *gm_ifp = sg->iface;
-	struct pim_interface *pim_ifp = gm_ifp->ifp->info;
 	struct gm_gsq_pending *pend_gsq, ref = {};
 
 	sg->n_query--;
 	if (sg->n_query)
-		thread_add_timer_msec(router->master, gm_t_sg_query, sg,
-				      gm_ifp->cur_query_intv_trig,
-				      &sg->t_sg_query);
+		event_add_timer_msec(router->master, gm_t_sg_query, sg,
+				     gm_ifp->cur_query_intv_trig,
+				     &sg->t_sg_query);
+
+	/* As per RFC 2271, s6 p14:
+	 * E.g. a router that starts as a Querier, receives a
+	 * Done message for a group and then receives a Query from a router with
+	 * a lower address (causing a transition to the Non-Querier state)
+	 * continues to send multicast-address-specific queries for the group in
+	 * question until it either receives a Report or its timer expires, at
+	 * which time it starts performing the actions of a Non-Querier for this
+	 * group.
+	 */
+	 /* Therefore here we do not need to check if this router is querier or
+	  * not. This is called only for querier, hence it will work even if the
+	  * router transitions from querier to non-querier.
+	  */
 
-	if (!IPV6_ADDR_SAME(&gm_ifp->querier, &pim_ifp->ll_lowest))
-		return;
 	if (gm_ifp->pim->gm_socket == -1)
 		return;
 
@@ -1942,9 +1974,8 @@ static void gm_trigger_specific(struct gm_sg *sg)
 		pend_gsq->iface = gm_ifp;
 		gm_gsq_pends_add(gm_ifp->gsq_pends, pend_gsq);
 
-		thread_add_timer_tv(router->master, gm_t_gsq_pend, pend_gsq,
-				    &gm_ifp->cfg_timing_fuzz,
-				    &pend_gsq->t_send);
+		event_add_timer_tv(router->master, gm_t_gsq_pend, pend_gsq,
+				   &gm_ifp->cfg_timing_fuzz, &pend_gsq->t_send);
 	}
 
 	assert(pend_gsq->n_src < array_size(pend_gsq->srcs));
@@ -1953,7 +1984,7 @@ static void gm_trigger_specific(struct gm_sg *sg)
 	pend_gsq->n_src++;
 
 	if (pend_gsq->n_src == array_size(pend_gsq->srcs)) {
-		THREAD_OFF(pend_gsq->t_send);
+		EVENT_OFF(pend_gsq->t_send);
 		gm_send_specific(pend_gsq);
 		pend_gsq = NULL;
 	}
@@ -2050,8 +2081,8 @@ static void gm_vrf_socket_incref(struct pim_instance *pim)
 				vrf->name);
 	}
 
-	thread_add_read(router->master, gm_t_recv, pim, pim->gm_socket,
-			&pim->t_gm_recv);
+	event_add_read(router->master, gm_t_recv, pim, pim->gm_socket,
+		       &pim->t_gm_recv);
 }
 
 static void gm_vrf_socket_decref(struct pim_instance *pim)
@@ -2059,7 +2090,7 @@ static void gm_vrf_socket_decref(struct pim_instance *pim)
 	if (--pim->gm_socket_if_count)
 		return;
 
-	THREAD_OFF(pim->t_gm_recv);
+	EVENT_OFF(pim->t_gm_recv);
 	close(pim->gm_socket);
 	pim->gm_socket = -1;
 }
@@ -2132,17 +2163,17 @@ void gm_group_delete(struct gm_if *gm_ifp)
 		gm_packet_drop(pkt, false);
 
 	while ((pend_grp = gm_grp_pends_pop(gm_ifp->grp_pends))) {
-		THREAD_OFF(pend_grp->t_expire);
+		EVENT_OFF(pend_grp->t_expire);
 		XFREE(MTYPE_GM_GRP_PENDING, pend_grp);
 	}
 
 	while ((pend_gsq = gm_gsq_pends_pop(gm_ifp->gsq_pends))) {
-		THREAD_OFF(pend_gsq->t_send);
+		EVENT_OFF(pend_gsq->t_send);
 		XFREE(MTYPE_GM_GSQ_PENDING, pend_gsq);
 	}
 
 	while ((sg = gm_sgs_pop(gm_ifp->sgs))) {
-		THREAD_OFF(sg->t_sg_expire);
+		EVENT_OFF(sg->t_sg_expire);
 		assertf(!gm_packet_sg_subs_count(sg->subs_negative), "%pSG",
 			&sg->sgaddr);
 		assertf(!gm_packet_sg_subs_count(sg->subs_positive), "%pSG",
@@ -2170,9 +2201,9 @@ void gm_ifp_teardown(struct interface *ifp)
 	if (PIM_DEBUG_GM_EVENTS)
 		zlog_debug(log_ifp("MLD stop"));
 
-	THREAD_OFF(gm_ifp->t_query);
-	THREAD_OFF(gm_ifp->t_other_querier);
-	THREAD_OFF(gm_ifp->t_expire);
+	EVENT_OFF(gm_ifp->t_query);
+	EVENT_OFF(gm_ifp->t_other_querier);
+	EVENT_OFF(gm_ifp->t_expire);
 
 	frr_with_privs (&pimd_privs) {
 		struct ipv6_mreq mreq;
@@ -2215,7 +2246,7 @@ static void gm_update_ll(struct interface *ifp)
 	gm_ifp->cur_ll_lowest = pim_ifp->ll_lowest;
 	if (was_querier)
 		gm_ifp->querier = pim_ifp->ll_lowest;
-	THREAD_OFF(gm_ifp->t_query);
+	EVENT_OFF(gm_ifp->t_query);
 
 	if (pim_addr_is_any(gm_ifp->cur_ll_lowest)) {
 		if (was_querier)
@@ -2236,7 +2267,7 @@ static void gm_update_ll(struct interface *ifp)
 		return;
 
 	gm_ifp->n_startup = gm_ifp->cur_qrv;
-	thread_execute(router->master, gm_t_query, gm_ifp, 0);
+	event_execute(router->master, gm_t_query, gm_ifp, 0, NULL);
 }
 
 void gm_ifp_update(struct interface *ifp)
@@ -2262,6 +2293,7 @@ void gm_ifp_update(struct interface *ifp)
 	if (!pim_ifp->mld) {
 		changed = true;
 		gm_start(ifp);
+		assume(pim_ifp->mld != NULL);
 	}
 
 	gm_ifp = pim_ifp->mld;
@@ -2397,6 +2429,8 @@ static void gm_show_if_one(struct vty *vty, struct interface *ifp,
 	struct gm_if *gm_ifp = pim_ifp->mld;
 	bool querier;
 
+	assume(js_if || tt);
+
 	querier = IPV6_ADDR_SAME(&gm_ifp->querier, &pim_ifp->ll_lowest);
 
 	if (js_if) {
@@ -2479,6 +2513,19 @@ static void gm_show_if_vrf(struct vty *vty, struct vrf *vrf, const char *ifname,
 
 		if (js) {
 			js_if = json_object_new_object();
+			/*
+			 * If we have js as true and detail as false
+			 * and if Coverity thinks that js_if is NULL
+			 * because of a failed call to new then
+			 * when we call gm_show_if_one below
+			 * the tt can be deref'ed and as such
+			 * FRR will crash.  But since we know
+			 * that json_object_new_object never fails
+			 * then let's tell Coverity that this assumption
+			 * is true.  I'm not worried about fast path
+			 * here at all.
+			 */
+			assert(js_if);
 			json_object_object_add(js_vrf, ifp->name, js_if);
 		}
 
