@@ -1,26 +1,13 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * PIM for Quagga
  * Copyright (C) 2008  Everton da Silva Marques
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
 #include "linklist.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "memory.h"
 #include "if.h"
 #include "vrf.h"
@@ -148,7 +135,7 @@ void pim_ifchannel_delete(struct pim_ifchannel *ch)
 		 * being inherited.  So let's figure out what
 		 * needs to be done here
 		 */
-		if (!pim_addr_is_any(ch->sg.src) &&
+		if (!pim_addr_is_any(ch->sg.src) && ch->parent &&
 		    pim_upstream_evaluate_join_desired_interface(
 			    ch->upstream, ch, ch->parent))
 			pim_channel_add_oif(ch->upstream->channel_oil,
@@ -206,9 +193,9 @@ void pim_ifchannel_delete(struct pim_ifchannel *ch)
 
 	ch->upstream = NULL;
 
-	THREAD_OFF(ch->t_ifjoin_expiry_timer);
-	THREAD_OFF(ch->t_ifjoin_prune_pending_timer);
-	THREAD_OFF(ch->t_ifassert_timer);
+	EVENT_OFF(ch->t_ifjoin_expiry_timer);
+	EVENT_OFF(ch->t_ifjoin_prune_pending_timer);
+	EVENT_OFF(ch->t_ifassert_timer);
 
 	if (ch->parent) {
 		listnode_delete(ch->parent->sources, ch);
@@ -355,13 +342,6 @@ void pim_ifchannel_ifjoin_switch(const char *caller, struct pim_ifchannel *ch,
 				   ch->sg_str, ch->interface->name);
 		}
 
-		/* pim_upstream_update_join_desired looks at up->channel_oil,
-		 * but that's updated from pim_forward_stop().  Need this here
-		 * so we correctly determine join_desired right below.
-		 */
-		if (new_state == PIM_IFJOIN_NOINFO)
-			pim_forward_stop(ch);
-
 		/*
 		  Record uptime of state transition to/from NOINFO
 		*/
@@ -431,7 +411,7 @@ const char *pim_ifchannel_ifassert_name(enum pim_ifassert_state ifassert_state)
 */
 void reset_ifassert_state(struct pim_ifchannel *ch)
 {
-	THREAD_OFF(ch->t_ifassert_timer);
+	EVENT_OFF(ch->t_ifassert_timer);
 
 	pim_ifassert_winner_set(ch, PIM_IFASSERT_NOINFO, PIMADDR_ANY,
 				router->infinite_assert_metric);
@@ -639,6 +619,7 @@ struct pim_ifchannel *pim_ifchannel_add(struct interface *ifp, pim_sgaddr *sg,
 static void ifjoin_to_noinfo(struct pim_ifchannel *ch)
 {
 	pim_ifchannel_ifjoin_switch(__func__, ch, PIM_IFJOIN_NOINFO);
+	pim_forward_stop(ch);
 
 	PIM_UPSTREAM_FLAG_UNSET_SRC_PIM(ch->upstream->flags);
 
@@ -647,11 +628,11 @@ static void ifjoin_to_noinfo(struct pim_ifchannel *ch)
 	delete_on_noinfo(ch);
 }
 
-static void on_ifjoin_expiry_timer(struct thread *t)
+static void on_ifjoin_expiry_timer(struct event *t)
 {
 	struct pim_ifchannel *ch;
 
-	ch = THREAD_ARG(t);
+	ch = EVENT_ARG(t);
 
 	if (PIM_DEBUG_PIM_TRACE)
 		zlog_debug("%s: ifchannel %s expiry timer", __func__,
@@ -661,14 +642,14 @@ static void on_ifjoin_expiry_timer(struct thread *t)
 	/* ch may have been deleted */
 }
 
-static void on_ifjoin_prune_pending_timer(struct thread *t)
+static void on_ifjoin_prune_pending_timer(struct event *t)
 {
 	struct pim_ifchannel *ch;
 	int send_prune_echo; /* boolean */
 	struct interface *ifp;
 	struct pim_interface *pim_ifp;
 
-	ch = THREAD_ARG(t);
+	ch = EVENT_ARG(t);
 
 	if (PIM_DEBUG_PIM_TRACE)
 		zlog_debug("%s: IFCHANNEL%pSG %s Prune Pending Timer Popped",
@@ -915,7 +896,7 @@ void pim_ifchannel_join_add(struct interface *ifp, pim_addr neigh_addr,
 		  previously received join message with holdtime=0xFFFF.
 		 */
 		if (ch->t_ifjoin_expiry_timer) {
-			unsigned long remain = thread_timer_remain_second(
+			unsigned long remain = event_timer_remain_second(
 				ch->t_ifjoin_expiry_timer);
 			if (remain > holdtime) {
 				/*
@@ -939,13 +920,13 @@ void pim_ifchannel_join_add(struct interface *ifp, pim_addr neigh_addr,
 				return;
 			}
 		}
-		THREAD_OFF(ch->t_ifjoin_expiry_timer);
+		EVENT_OFF(ch->t_ifjoin_expiry_timer);
 		break;
 	case PIM_IFJOIN_PRUNE:
 		if (source_flags & PIM_ENCODE_RPT_BIT) {
 			pim_ifchannel_ifjoin_switch(__func__, ch,
 						    PIM_IFJOIN_NOINFO);
-			THREAD_OFF(ch->t_ifjoin_expiry_timer);
+			EVENT_OFF(ch->t_ifjoin_expiry_timer);
 			delete_on_noinfo(ch);
 			return;
 		} else
@@ -962,7 +943,7 @@ void pim_ifchannel_join_add(struct interface *ifp, pim_addr neigh_addr,
 		 * maximum of its current value and the HoldTime from the
 		 * triggering Join/Prune message.
 		 */
-		THREAD_OFF(ch->t_ifjoin_prune_pending_timer);
+		EVENT_OFF(ch->t_ifjoin_prune_pending_timer);
 
 		/* Check if SGRpt join Received */
 		if ((source_flags & PIM_ENCODE_RPT_BIT) &&
@@ -974,7 +955,7 @@ void pim_ifchannel_join_add(struct interface *ifp, pim_addr neigh_addr,
 			 * I transitions to the NoInfo state.The ET and PPT are
 			 * cancelled.
 			 */
-			THREAD_OFF(ch->t_ifjoin_expiry_timer);
+			EVENT_OFF(ch->t_ifjoin_expiry_timer);
 			pim_ifchannel_ifjoin_switch(__func__, ch,
 						    PIM_IFJOIN_NOINFO);
 			return;
@@ -983,13 +964,13 @@ void pim_ifchannel_join_add(struct interface *ifp, pim_addr neigh_addr,
 		pim_ifchannel_ifjoin_handler(ch, pim_ifp);
 
 		if (ch->t_ifjoin_expiry_timer) {
-			unsigned long remain = thread_timer_remain_second(
+			unsigned long remain = event_timer_remain_second(
 				ch->t_ifjoin_expiry_timer);
 
 			if (remain > holdtime)
 				return;
 		}
-		THREAD_OFF(ch->t_ifjoin_expiry_timer);
+		EVENT_OFF(ch->t_ifjoin_expiry_timer);
 
 		break;
 	case PIM_IFJOIN_PRUNE_TMP:
@@ -999,8 +980,8 @@ void pim_ifchannel_join_add(struct interface *ifp, pim_addr neigh_addr,
 	}
 
 	if (holdtime != 0xFFFF) {
-		thread_add_timer(router->master, on_ifjoin_expiry_timer, ch,
-				 holdtime, &ch->t_ifjoin_expiry_timer);
+		event_add_timer(router->master, on_ifjoin_expiry_timer, ch,
+				holdtime, &ch->t_ifjoin_expiry_timer);
 	}
 }
 
@@ -1049,15 +1030,15 @@ void pim_ifchannel_prune(struct interface *ifp, pim_addr upstream,
 			   be taken not to use "ch" afterwards since it would be
 			   deleted. */
 
-			THREAD_OFF(ch->t_ifjoin_prune_pending_timer);
-			THREAD_OFF(ch->t_ifjoin_expiry_timer);
-			thread_add_timer_msec(
-				router->master, on_ifjoin_prune_pending_timer,
-				ch, jp_override_interval_msec,
-				&ch->t_ifjoin_prune_pending_timer);
-			thread_add_timer(router->master, on_ifjoin_expiry_timer,
-					 ch, holdtime,
-					 &ch->t_ifjoin_expiry_timer);
+			EVENT_OFF(ch->t_ifjoin_prune_pending_timer);
+			EVENT_OFF(ch->t_ifjoin_expiry_timer);
+			event_add_timer_msec(router->master,
+					     on_ifjoin_prune_pending_timer, ch,
+					     jp_override_interval_msec,
+					     &ch->t_ifjoin_prune_pending_timer);
+			event_add_timer(router->master, on_ifjoin_expiry_timer,
+					ch, holdtime,
+					&ch->t_ifjoin_expiry_timer);
 			pim_upstream_update_join_desired(pim_ifp->pim,
 							 ch->upstream);
 		}
@@ -1087,15 +1068,15 @@ void pim_ifchannel_prune(struct interface *ifp, pim_addr upstream,
 		/* If we called ifjoin_prune() directly instead, care should
 		   be taken not to use "ch" afterwards since it would be
 		   deleted. */
-		THREAD_OFF(ch->t_ifjoin_prune_pending_timer);
-		thread_add_timer_msec(router->master,
-				      on_ifjoin_prune_pending_timer, ch,
-				      jp_override_interval_msec,
-				      &ch->t_ifjoin_prune_pending_timer);
+		EVENT_OFF(ch->t_ifjoin_prune_pending_timer);
+		event_add_timer_msec(router->master,
+				     on_ifjoin_prune_pending_timer, ch,
+				     jp_override_interval_msec,
+				     &ch->t_ifjoin_prune_pending_timer);
 		break;
 	case PIM_IFJOIN_PRUNE:
 		if (source_flags & PIM_ENCODE_RPT_BIT) {
-			THREAD_OFF(ch->t_ifjoin_prune_pending_timer);
+			EVENT_OFF(ch->t_ifjoin_prune_pending_timer);
 			/*
 			 * While in Prune State, Receive SGRpt Prune.
 			 * RFC 7761 Sec 4.5.3:
@@ -1106,35 +1087,35 @@ void pim_ifchannel_prune(struct interface *ifp, pim_addr upstream,
 			 * Join/Prune message.
 			 */
 			if (ch->t_ifjoin_expiry_timer) {
-				unsigned long rem = thread_timer_remain_second(
+				unsigned long rem = event_timer_remain_second(
 					ch->t_ifjoin_expiry_timer);
 
 				if (rem > holdtime)
 					return;
-				THREAD_OFF(ch->t_ifjoin_expiry_timer);
+				EVENT_OFF(ch->t_ifjoin_expiry_timer);
 			}
 
-			thread_add_timer(router->master, on_ifjoin_expiry_timer,
-					 ch, holdtime,
-					 &ch->t_ifjoin_expiry_timer);
+			event_add_timer(router->master, on_ifjoin_expiry_timer,
+					ch, holdtime,
+					&ch->t_ifjoin_expiry_timer);
 		}
 		break;
 	case PIM_IFJOIN_PRUNE_TMP:
 		if (source_flags & PIM_ENCODE_RPT_BIT) {
 			ch->ifjoin_state = PIM_IFJOIN_PRUNE;
-			THREAD_OFF(ch->t_ifjoin_expiry_timer);
-			thread_add_timer(router->master, on_ifjoin_expiry_timer,
-					 ch, holdtime,
-					 &ch->t_ifjoin_expiry_timer);
+			EVENT_OFF(ch->t_ifjoin_expiry_timer);
+			event_add_timer(router->master, on_ifjoin_expiry_timer,
+					ch, holdtime,
+					&ch->t_ifjoin_expiry_timer);
 		}
 		break;
 	case PIM_IFJOIN_PRUNE_PENDING_TMP:
 		if (source_flags & PIM_ENCODE_RPT_BIT) {
 			ch->ifjoin_state = PIM_IFJOIN_PRUNE_PENDING;
-			THREAD_OFF(ch->t_ifjoin_expiry_timer);
-			thread_add_timer(router->master, on_ifjoin_expiry_timer,
-					 ch, holdtime,
-					 &ch->t_ifjoin_expiry_timer);
+			EVENT_OFF(ch->t_ifjoin_expiry_timer);
+			event_add_timer(router->master, on_ifjoin_expiry_timer,
+					ch, holdtime,
+					&ch->t_ifjoin_expiry_timer);
 		}
 		break;
 	}
@@ -1485,8 +1466,8 @@ void pim_ifchannel_set_star_g_join_state(struct pim_ifchannel *ch, int eom,
 				break;
 
 			if (child->ifjoin_state == PIM_IFJOIN_PRUNE_PENDING_TMP)
-				THREAD_OFF(child->t_ifjoin_prune_pending_timer);
-			THREAD_OFF(child->t_ifjoin_expiry_timer);
+				EVENT_OFF(child->t_ifjoin_prune_pending_timer);
+			EVENT_OFF(child->t_ifjoin_expiry_timer);
 
 			PIM_IF_FLAG_UNSET_S_G_RPT(child->flags);
 			child->ifjoin_state = PIM_IFJOIN_NOINFO;
