@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * This is an implementation of PIM MLAG Functionality
  *
@@ -6,20 +7,6 @@
  * Author: sathesh Kumar karra <sathk@cumulusnetworks.com>
  *
  * Copyright (C) 2019 Cumulus Networks http://www.cumulusnetworks.com
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include <zebra.h>
 
@@ -41,8 +28,7 @@ extern struct zclient *zclient;
 				"%s: add Dual-active Interface to %s "         \
 				"to oil:%s",                                   \
 				__func__, ch->interface->name, ch->sg_str);    \
-		pim_channel_add_oif(ch_oil, ch->interface,                     \
-				    PIM_OIF_FLAG_PROTO_IGMP, __func__);        \
+		pim_channel_update_oif_mute(ch_oil, ch->interface->info);      \
 	} while (0)
 
 #define PIM_MLAG_DEL_OIF_TO_OIL(ch, ch_oil)                                    \
@@ -52,8 +38,7 @@ extern struct zclient *zclient;
 				"%s: del Dual-active Interface to %s "         \
 				"to oil:%s",                                   \
 				__func__, ch->interface->name, ch->sg_str);    \
-		pim_channel_del_oif(ch_oil, ch->interface,                     \
-				    PIM_OIF_FLAG_PROTO_IGMP, __func__);        \
+		pim_channel_update_oif_mute(ch_oil, ch->interface->info);      \
 	} while (0)
 
 
@@ -91,14 +76,36 @@ static void pim_mlag_inherit_mlag_flags(struct pim_upstream *up, bool is_df)
 {
 	struct listnode *listnode;
 	struct pim_upstream *child;
+	struct listnode *chnode;
+	struct listnode *chnextnode;
+	struct pim_ifchannel *ch;
+	struct pim_interface *pim_ifp = NULL;
+	struct channel_oil *ch_oil = NULL;
 
-	for (ALL_LIST_ELEMENTS_RO(up->sources, listnode, child)) {
-		PIM_UPSTREAM_FLAG_SET_MLAG_PEER(child->flags);
-		if (is_df)
-			PIM_UPSTREAM_FLAG_UNSET_MLAG_NON_DF(child->flags);
-		else
-			PIM_UPSTREAM_FLAG_SET_MLAG_NON_DF(child->flags);
-		pim_mlag_calculate_df_for_ifchannels(child, is_df);
+	if (PIM_DEBUG_MLAG)
+		zlog_debug("%s: Updating DF for uptream:%s children", __func__,
+			   up->sg_str);
+
+
+	for (ALL_LIST_ELEMENTS(up->ifchannels, chnode, chnextnode, ch)) {
+		pim_ifp = (ch->interface) ? ch->interface->info : NULL;
+		if (!pim_ifp || !PIM_I_am_DualActive(pim_ifp))
+			continue;
+
+		for (ALL_LIST_ELEMENTS_RO(up->sources, listnode, child)) {
+			if (PIM_DEBUG_MLAG)
+				zlog_debug("%s: Updating DF for child:%s",
+					   __func__, child->sg_str);
+			ch_oil = (child) ? child->channel_oil : NULL;
+
+			if (!ch_oil)
+				continue;
+
+			if (is_df)
+				PIM_MLAG_ADD_OIF_TO_OIL(ch, ch_oil);
+			else
+				PIM_MLAG_DEL_OIF_TO_OIL(ch, ch_oil);
+		}
 	}
 }
 
@@ -134,8 +141,8 @@ bool pim_mlag_up_df_role_update(struct pim_instance *pim,
 	 * Interface configuration
 	 */
 	if (PIM_UPSTREAM_FLAG_TEST_MLAG_INTERFACE(up->flags)) {
-		pim_mlag_calculate_df_for_ifchannels(up, is_df);
 		pim_mlag_inherit_mlag_flags(up, is_df);
+		pim_mlag_calculate_df_for_ifchannels(up, is_df);
 	}
 
 	/* If the DF role has changed check if ipmr-lo needs to be
@@ -154,8 +161,8 @@ bool pim_mlag_up_df_role_update(struct pim_instance *pim,
 	/* If DF role changed on a (*,G) termination mroute update the
 	 * associated DF role on the inherited (S,G) entries
 	 */
-	if ((up->sg.src.s_addr == INADDR_ANY) &&
-			PIM_UPSTREAM_FLAG_TEST_MLAG_VXLAN(up->flags))
+	if (pim_addr_is_any(up->sg.src) &&
+	    PIM_UPSTREAM_FLAG_TEST_MLAG_VXLAN(up->flags))
 		pim_vxlan_inherit_mlag_flags(pim, up, true /* inherit */);
 
 	return true;
@@ -170,8 +177,7 @@ static bool pim_mlag_up_df_role_elect(struct pim_instance *pim,
 	uint32_t local_cost;
 	bool rv;
 
-	if (!pim_up_mlag_is_local(up)
-	    && !PIM_UPSTREAM_FLAG_TEST_MLAG_INTERFACE(up->flags))
+	if (!pim_up_mlag_is_local(up))
 		return false;
 
 	/* We are yet to rx a status update from the local MLAG daemon so
@@ -234,19 +240,16 @@ static void pim_mlag_up_peer_add(struct mlag_mroute_add *msg)
 	struct pim_upstream *up;
 	struct pim_instance *pim;
 	int flags = 0;
-	struct prefix_sg sg;
+	pim_sgaddr sg;
 	struct vrf *vrf;
-	char sg_str[PIM_SG_LEN];
 
-	memset(&sg, 0, sizeof(struct prefix_sg));
+	memset(&sg, 0, sizeof(sg));
 	sg.src.s_addr = htonl(msg->source_ip);
 	sg.grp.s_addr = htonl(msg->group_ip);
-	if (PIM_DEBUG_MLAG)
-		pim_str_sg_set(&sg, sg_str);
 
 	if (PIM_DEBUG_MLAG)
-		zlog_debug("peer MLAG mroute add %s:%s cost %d",
-			msg->vrf_name, sg_str, msg->cost_to_rp);
+		zlog_debug("peer MLAG mroute add %s:%pSG cost %d",
+			   msg->vrf_name, &sg, msg->cost_to_rp);
 
 	/* XXX - this is not correct. we MUST cache updates to avoid losing
 	 * an entry because of race conditions with the peer switch.
@@ -254,8 +257,9 @@ static void pim_mlag_up_peer_add(struct mlag_mroute_add *msg)
 	vrf = vrf_lookup_by_name(msg->vrf_name);
 	if  (!vrf) {
 		if (PIM_DEBUG_MLAG)
-			zlog_debug("peer MLAG mroute add failed %s:%s; no vrf",
-					msg->vrf_name, sg_str);
+			zlog_debug(
+				"peer MLAG mroute add failed %s:%pSG; no vrf",
+				msg->vrf_name, &sg);
 		return;
 	}
 	pim = vrf->info;
@@ -275,8 +279,9 @@ static void pim_mlag_up_peer_add(struct mlag_mroute_add *msg)
 
 		if (!up) {
 			if (PIM_DEBUG_MLAG)
-				zlog_debug("peer MLAG mroute add failed %s:%s",
-						vrf->name, sg_str);
+				zlog_debug(
+					"peer MLAG mroute add failed %s:%pSG",
+					vrf->name, &sg);
 			return;
 		}
 	}
@@ -308,25 +313,22 @@ static void pim_mlag_up_peer_del(struct mlag_mroute_del *msg)
 {
 	struct pim_upstream *up;
 	struct pim_instance *pim;
-	struct prefix_sg sg;
+	pim_sgaddr sg;
 	struct vrf *vrf;
-	char sg_str[PIM_SG_LEN];
 
-	memset(&sg, 0, sizeof(struct prefix_sg));
+	memset(&sg, 0, sizeof(sg));
 	sg.src.s_addr = htonl(msg->source_ip);
 	sg.grp.s_addr = htonl(msg->group_ip);
-	if (PIM_DEBUG_MLAG)
-		pim_str_sg_set(&sg, sg_str);
 
 	if (PIM_DEBUG_MLAG)
-		zlog_debug("peer MLAG mroute del %s:%s", msg->vrf_name,
-				sg_str);
+		zlog_debug("peer MLAG mroute del %s:%pSG", msg->vrf_name, &sg);
 
 	vrf = vrf_lookup_by_name(msg->vrf_name);
 	if  (!vrf) {
 		if (PIM_DEBUG_MLAG)
-			zlog_debug("peer MLAG mroute del skipped %s:%s; no vrf",
-					msg->vrf_name, sg_str);
+			zlog_debug(
+				"peer MLAG mroute del skipped %s:%pSG; no vrf",
+				msg->vrf_name, &sg);
 		return;
 	}
 	pim = vrf->info;
@@ -334,8 +336,9 @@ static void pim_mlag_up_peer_del(struct mlag_mroute_del *msg)
 	up = pim_upstream_find(pim, &sg);
 	if  (!up) {
 		if (PIM_DEBUG_MLAG)
-			zlog_debug("peer MLAG mroute del skipped %s:%s; no up",
-					vrf->name, sg_str);
+			zlog_debug(
+				"peer MLAG mroute del skipped %s:%pSG; no up",
+				vrf->name, &sg);
 		return;
 	}
 
@@ -501,8 +504,7 @@ static void pim_mlag_up_local_replay(void)
 	RB_FOREACH(vrf, vrf_name_head, &vrfs_by_name) {
 		pim = vrf->info;
 		frr_each (rb_pim_upstream, &pim->upstream_head, up) {
-			if (pim_up_mlag_is_local(up)
-			    || PIM_UPSTREAM_FLAG_TEST_MLAG_INTERFACE(up->flags))
+			if (pim_up_mlag_is_local(up))
 				pim_mlag_up_local_add_send(pim, up);
 		}
 	}
@@ -523,9 +525,7 @@ static void pim_mlag_up_local_reeval(bool mlagd_send, const char *reason_code)
 	RB_FOREACH(vrf, vrf_name_head, &vrfs_by_name) {
 		pim = vrf->info;
 		frr_each (rb_pim_upstream, &pim->upstream_head, up) {
-			if (!pim_up_mlag_is_local(up)
-			    && !PIM_UPSTREAM_FLAG_TEST_MLAG_INTERFACE(
-				    up->flags))
+			if (!pim_up_mlag_is_local(up))
 				continue;
 			/* if role changes re-send to peer */
 			if (pim_mlag_up_df_role_elect(pim, up) &&
@@ -721,14 +721,19 @@ static void pim_mlag_process_vxlan_update(struct mlag_vxlan *msg)
 static void pim_mlag_process_mroute_add(struct mlag_mroute_add msg)
 {
 	if (PIM_DEBUG_MLAG) {
+		pim_sgaddr sg;
+
+		sg.grp.s_addr = ntohl(msg.group_ip);
+		sg.src.s_addr = ntohl(msg.source_ip);
+
 		zlog_debug(
-				"%s: msg dump: vrf_name: %s, s.ip: 0x%x, g.ip: 0x%x cost: %u",
-				__func__, msg.vrf_name, msg.source_ip,
-				msg.group_ip, msg.cost_to_rp);
+			"%s: msg dump: vrf_name: %s, s.ip: 0x%x, g.ip: 0x%x (%pSG) cost: %u",
+			__func__, msg.vrf_name, msg.source_ip, msg.group_ip,
+			&sg, msg.cost_to_rp);
 		zlog_debug(
-				"owner_id: %d, DR: %d, Dual active: %d, vrf_id: 0x%x intf_name: %s",
-				msg.owner_id, msg.am_i_dr, msg.am_i_dual_active,
-				msg.vrf_id, msg.intf_name);
+			"(%pSG)owner_id: %d, DR: %d, Dual active: %d, vrf_id: 0x%x intf_name: %s",
+			&sg, msg.owner_id, msg.am_i_dr, msg.am_i_dual_active,
+			msg.vrf_id, msg.intf_name);
 	}
 
 	if (!(router->mlag_flags & PIM_MLAGF_LOCAL_CONN_UP)) {
@@ -746,12 +751,16 @@ static void pim_mlag_process_mroute_add(struct mlag_mroute_add msg)
 static void pim_mlag_process_mroute_del(struct mlag_mroute_del msg)
 {
 	if (PIM_DEBUG_MLAG) {
+		pim_sgaddr sg;
+
+		sg.grp.s_addr = ntohl(msg.group_ip);
+		sg.src.s_addr = ntohl(msg.source_ip);
 		zlog_debug(
-				"%s: msg dump: vrf_name: %s, s.ip: 0x%x, g.ip: 0x%x ",
-				__func__, msg.vrf_name, msg.source_ip,
-				msg.group_ip);
-		zlog_debug("owner_id: %d, vrf_id: 0x%x intf_name: %s",
-				msg.owner_id, msg.vrf_id, msg.intf_name);
+			"%s: msg dump: vrf_name: %s, s.ip: 0x%x, g.ip: 0x%x(%pSG)",
+			__func__, msg.vrf_name, msg.source_ip, msg.group_ip,
+			&sg);
+		zlog_debug("(%pSG)owner_id: %d, vrf_id: 0x%x intf_name: %s",
+			   &sg, msg.owner_id, msg.vrf_id, msg.intf_name);
 	}
 
 	if (!(router->mlag_flags & PIM_MLAGF_LOCAL_CONN_UP)) {
@@ -766,8 +775,10 @@ static void pim_mlag_process_mroute_del(struct mlag_mroute_del msg)
 	pim_mlag_up_peer_del(&msg);
 }
 
-int pim_zebra_mlag_handle_msg(struct stream *s, int len)
+int pim_zebra_mlag_handle_msg(int cmd, struct zclient *zclient,
+			      uint16_t zapi_length, vrf_id_t vrf_id)
 {
+	struct stream *s = zclient->ibuf;
 	struct mlag_msg mlag_msg;
 	char buf[80];
 	int rc = 0;
@@ -847,7 +858,11 @@ int pim_zebra_mlag_handle_msg(struct stream *s, int len)
 			pim_mlag_process_mroute_del(msg);
 		}
 	} break;
-	default:
+	case MLAG_MSG_NONE:
+	case MLAG_REGISTER:
+	case MLAG_DEREGISTER:
+	case MLAG_DUMP:
+	case MLAG_PIM_CFG_DUMP:
 		break;
 	}
 	return 0;
@@ -855,7 +870,7 @@ int pim_zebra_mlag_handle_msg(struct stream *s, int len)
 
 /****************End of PIM Mesasge processing handler********************/
 
-int pim_zebra_mlag_process_up(void)
+int pim_zebra_mlag_process_up(ZAPI_CALLBACK_ARGS)
 {
 	if (PIM_DEBUG_MLAG)
 		zlog_debug("%s: Received Process-Up from Mlag", __func__);
@@ -883,7 +898,7 @@ static void pim_mlag_param_reset(void)
 	router->peerlink_rif[0] = '\0';
 }
 
-int pim_zebra_mlag_process_down(void)
+int pim_zebra_mlag_process_down(ZAPI_CALLBACK_ARGS)
 {
 	if (PIM_DEBUG_MLAG)
 		zlog_debug("%s: Received Process-Down from Mlag", __func__);
@@ -906,12 +921,12 @@ int pim_zebra_mlag_process_down(void)
 	return 0;
 }
 
-static int pim_mlag_register_handler(struct thread *thread)
+static void pim_mlag_register_handler(struct event *thread)
 {
 	uint32_t bit_mask = 0;
 
 	if (!zclient)
-		return -1;
+		return;
 
 	SET_FLAG(bit_mask, (1 << MLAG_STATUS_UPDATE));
 	SET_FLAG(bit_mask, (1 << MLAG_MROUTE_ADD));
@@ -928,7 +943,6 @@ static int pim_mlag_register_handler(struct thread *thread)
 			   __func__, bit_mask);
 
 	zclient_send_mlag_register(zclient, bit_mask);
-	return 0;
 }
 
 void pim_mlag_register(void)
@@ -938,21 +952,20 @@ void pim_mlag_register(void)
 
 	router->mlag_process_register = true;
 
-	thread_add_event(router->master, pim_mlag_register_handler, NULL, 0,
-			 NULL);
+	event_add_event(router->master, pim_mlag_register_handler, NULL, 0,
+			NULL);
 }
 
-static int pim_mlag_deregister_handler(struct thread *thread)
+static void pim_mlag_deregister_handler(struct event *thread)
 {
 	if (!zclient)
-		return -1;
+		return;
 
 	if (PIM_DEBUG_MLAG)
 		zlog_debug("%s: Posting Client De-Register to MLAG from PIM",
 			   __func__);
 	router->connected_to_mlag = false;
 	zclient_send_mlag_deregister(zclient);
-	return 0;
 }
 
 void pim_mlag_deregister(void)
@@ -967,8 +980,8 @@ void pim_mlag_deregister(void)
 
 	router->mlag_process_register = false;
 
-	thread_add_event(router->master, pim_mlag_deregister_handler, NULL, 0,
-			 NULL);
+	event_add_event(router->master, pim_mlag_deregister_handler, NULL, 0,
+			NULL);
 }
 
 void pim_if_configure_mlag_dualactive(struct pim_interface *pim_ifp)
@@ -976,6 +989,10 @@ void pim_if_configure_mlag_dualactive(struct pim_interface *pim_ifp)
 	if (!pim_ifp || !pim_ifp->pim || pim_ifp->activeactive == true)
 		return;
 
+	if (PIM_DEBUG_MLAG)
+		zlog_debug("%s: Configuring active-active on Interface: %s",
+			   __func__, "NULL");
+
 	pim_ifp->activeactive = true;
 	if (pim_ifp->pim)
 		pim_ifp->pim->inst_mlag_intf_cnt++;
@@ -989,7 +1006,7 @@ void pim_if_configure_mlag_dualactive(struct pim_interface *pim_ifp)
 
 	if (router->pim_mlag_intf_cnt == 1) {
 		/*
-		 * atleast one Interface is configured for MLAG, send register
+		 * at least one Interface is configured for MLAG, send register
 		 * to Zebra for receiving MLAG Updates
 		 */
 		pim_mlag_register();
@@ -1001,6 +1018,10 @@ void pim_if_unconfigure_mlag_dualactive(struct pim_interface *pim_ifp)
 	if (!pim_ifp || !pim_ifp->pim || pim_ifp->activeactive == false)
 		return;
 
+	if (PIM_DEBUG_MLAG)
+		zlog_debug("%s: UnConfiguring active-active on Interface: %s",
+			   __func__, "NULL");
+
 	pim_ifp->activeactive = false;
 	pim_ifp->pim->inst_mlag_intf_cnt--;
 
@@ -1049,6 +1070,14 @@ void pim_instance_mlag_terminate(struct pim_instance *pim)
 	pim->inst_mlag_intf_cnt = 0;
 }
 
+void pim_mlag_terminate(void)
+{
+	stream_free(router->mlag_stream);
+	router->mlag_stream = NULL;
+	stream_fifo_free(router->mlag_fifo);
+	router->mlag_fifo = NULL;
+}
+
 void pim_mlag_init(void)
 {
 	pim_mlag_param_reset();
