@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 2020 VmWare
  *                    Sarita Patra
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -58,20 +45,6 @@ MACRO_REQUIRE_SEMICOLON()
 #define yang_dnode_get_pimaddr yang_dnode_get_ipv4
 #endif /* PIM_IPV != 6 */
 
-static void pim_if_membership_clear(struct interface *ifp)
-{
-	struct pim_interface *pim_ifp;
-
-	pim_ifp = ifp->info;
-	assert(pim_ifp);
-
-	if (pim_ifp->pim_enable && pim_ifp->gm_enable) {
-		return;
-	}
-
-	pim_ifchannel_membership_clear(ifp);
-}
-
 /*
  * When PIM is disabled on interface, IGMPv3 local membership
  * information is not injected into PIM interface state.
@@ -174,32 +147,6 @@ static int pim_cmd_interface_add(struct interface *ifp)
 	return 1;
 }
 
-static int pim_cmd_interface_delete(struct interface *ifp)
-{
-	struct pim_interface *pim_ifp = ifp->info;
-
-	if (!pim_ifp)
-		return 1;
-
-	pim_ifp->pim_enable = false;
-
-	pim_if_membership_clear(ifp);
-
-	/*
-	 * pim_sock_delete() removes all neighbors from
-	 * pim_ifp->pim_neighbor_list.
-	 */
-	pim_sock_delete(ifp, "pim unconfigured on interface");
-	pim_upstream_nh_if_update(pim_ifp->pim, ifp);
-
-	if (!pim_ifp->gm_enable) {
-		pim_if_addr_del_all(ifp);
-		pim_if_delete(ifp);
-	}
-
-	return 1;
-}
-
 static int interface_pim_use_src_cmd_worker(struct interface *ifp,
 		pim_addr source_addr, char *errmsg, size_t errmsg_len)
 {
@@ -291,7 +238,7 @@ static int pim_rp_cmd_worker(struct pim_instance *pim, pim_addr rp_addr,
 	if (result == PIM_RP_NO_PATH) {
 		snprintfrr(errmsg, errmsg_len,
 			   "No Path to RP address specified: %pPA", &rp_addr);
-		return NB_ERR_INCONSISTENCY;
+		return NB_OK;
 	}
 
 	if (result == PIM_GROUP_OVERLAP) {
@@ -975,7 +922,8 @@ int routing_control_plane_protocols_control_plane_protocol_pim_address_family_ss
 	case NB_EV_APPLY:
 		vrf = nb_running_get_entry(args->dnode, NULL, true);
 		pim = vrf->info;
-		yang_dnode_get_pimaddr(&source_addr, args->dnode, NULL);
+		yang_dnode_get_pimaddr(&source_addr, args->dnode,
+				       "./source-addr");
 		result = pim_ssmpingd_start(pim, source_addr);
 		if (result) {
 			snprintf(
@@ -1005,7 +953,8 @@ int routing_control_plane_protocols_control_plane_protocol_pim_address_family_ss
 	case NB_EV_APPLY:
 		vrf = nb_running_get_entry(args->dnode, NULL, true);
 		pim = vrf->info;
-		yang_dnode_get_pimaddr(&source_addr, args->dnode, NULL);
+		yang_dnode_get_pimaddr(&source_addr, args->dnode,
+				       "./source-addr");
 		result = pim_ssmpingd_stop(pim, source_addr);
 		if (result) {
 			snprintf(
@@ -1584,12 +1533,7 @@ int lib_interface_pim_address_family_destroy(struct nb_cb_destroy_args *args)
 		if (!pim_ifp)
 			return NB_OK;
 
-		if (!pim_cmd_interface_delete(ifp)) {
-			snprintf(args->errmsg, args->errmsg_len,
-				 "Unable to delete interface information %s",
-				 ifp->name);
-			return NB_ERR_INCONSISTENCY;
-		}
+		pim_pim_interface_delete(ifp);
 	}
 
 	return NB_OK;
@@ -1637,11 +1581,7 @@ int lib_interface_pim_address_family_pim_enable_modify(struct nb_cb_modify_args
 			if (!pim_ifp)
 				return NB_ERR_INCONSISTENCY;
 
-			if (!pim_cmd_interface_delete(ifp)) {
-				snprintf(args->errmsg, args->errmsg_len,
-					 "Unable to delete interface information");
-				return NB_ERR_INCONSISTENCY;
-			}
+			pim_pim_interface_delete(ifp);
 		}
 		break;
 	}
@@ -1693,7 +1633,7 @@ int lib_interface_pim_address_family_hello_interval_modify(
 		ifp = nb_running_get_entry(args->dnode, NULL, true);
 		pim_ifp = ifp->info;
 		pim_ifp->pim_hello_period =
-			yang_dnode_get_uint8(args->dnode, NULL);
+			yang_dnode_get_uint16(args->dnode, NULL);
 		pim_ifp->pim_default_holdtime = -1;
 		break;
 	}
@@ -2576,7 +2516,6 @@ int lib_interface_gmp_address_family_create(struct nb_cb_create_args *args)
 int lib_interface_gmp_address_family_destroy(struct nb_cb_destroy_args *args)
 {
 	struct interface *ifp;
-	struct pim_interface *pim_ifp;
 
 	switch (args->event) {
 	case NB_EV_VALIDATE:
@@ -2585,19 +2524,7 @@ int lib_interface_gmp_address_family_destroy(struct nb_cb_destroy_args *args)
 		break;
 	case NB_EV_APPLY:
 		ifp = nb_running_get_entry(args->dnode, NULL, true);
-		pim_ifp = ifp->info;
-
-		if (!pim_ifp)
-			return NB_OK;
-
-		pim_ifp->gm_enable = false;
-
-		pim_if_membership_clear(ifp);
-
-		pim_if_addr_del_all_igmp(ifp);
-
-		if (!pim_ifp->pim_enable)
-			pim_if_delete(ifp);
+		pim_gm_interface_delete(ifp);
 	}
 
 	return NB_OK;
@@ -2611,7 +2538,6 @@ int lib_interface_gmp_address_family_enable_modify(
 {
 	struct interface *ifp;
 	bool gm_enable;
-	struct pim_interface *pim_ifp;
 	int mcast_if_count;
 	const char *ifp_name;
 	const struct lyd_node *if_dnode;
@@ -2641,25 +2567,8 @@ int lib_interface_gmp_address_family_enable_modify(
 		if (gm_enable)
 			return pim_cmd_gm_start(ifp);
 
-		else {
-			pim_ifp = ifp->info;
-
-			if (!pim_ifp)
-				return NB_ERR_INCONSISTENCY;
-
-			pim_ifp->gm_enable = false;
-
-			pim_if_membership_clear(ifp);
-
-#if PIM_IPV == 4
-			pim_if_addr_del_all_igmp(ifp);
-#else
-			gm_ifp_teardown(ifp);
-#endif
-
-			if (!pim_ifp->pim_enable)
-				pim_if_delete(ifp);
-		}
+		else
+			pim_gm_interface_delete(ifp);
 	}
 	return NB_OK;
 }
@@ -2911,10 +2820,9 @@ int lib_interface_gmp_address_family_robustness_variable_modify(
 int lib_interface_gmp_address_family_static_group_create(
 	struct nb_cb_create_args *args)
 {
-#if PIM_IPV == 4
 	struct interface *ifp;
-	struct ipaddr source_addr;
-	struct ipaddr group_addr;
+	pim_addr source_addr;
+	pim_addr group_addr;
 	int result;
 	const char *ifp_name;
 	const struct lyd_node *if_dnode;
@@ -2930,33 +2838,40 @@ int lib_interface_gmp_address_family_static_group_create(
 			return NB_ERR_VALIDATION;
 		}
 
-		yang_dnode_get_ip(&group_addr, args->dnode, "./group-addr");
-		if (pim_is_group_224_0_0_0_24(group_addr.ip._v4_addr)) {
+		yang_dnode_get_pimaddr(&group_addr, args->dnode,
+				       "./group-addr");
+#if PIM_IPV == 4
+		if (pim_is_group_224_0_0_0_24(group_addr)) {
 			snprintf(
 				args->errmsg, args->errmsg_len,
 				"Groups within 224.0.0.0/24 are reserved and cannot be joined");
 			return NB_ERR_VALIDATION;
 		}
+#else
+		if (ipv6_mcast_reserved(&group_addr)) {
+			snprintf(
+				args->errmsg, args->errmsg_len,
+				"Groups within ffx2::/16 are reserved and cannot be joined");
+			return NB_ERR_VALIDATION;
+		}
+#endif
 		break;
 	case NB_EV_PREPARE:
 	case NB_EV_ABORT:
 		break;
 	case NB_EV_APPLY:
 		ifp = nb_running_get_entry(args->dnode, NULL, true);
-		yang_dnode_get_ip(&source_addr, args->dnode, "./source-addr");
-		yang_dnode_get_ip(&group_addr, args->dnode, "./group-addr");
-
-		result = pim_if_igmp_join_add(ifp, group_addr.ip._v4_addr,
-				source_addr.ip._v4_addr);
+		yang_dnode_get_pimaddr(&source_addr, args->dnode,
+				       "./source-addr");
+		yang_dnode_get_pimaddr(&group_addr, args->dnode,
+				       "./group-addr");
+		result = pim_if_gm_join_add(ifp, group_addr, source_addr);
 		if (result) {
 			snprintf(args->errmsg, args->errmsg_len,
-				 "Failure joining IGMP group");
+				 "Failure joining " GM " group");
 			return NB_ERR_INCONSISTENCY;
 		}
 	}
-#else
-	/* TBD Depends on MLD data structure changes */
-#endif /* PIM_IPV == 4 */
 	return NB_OK;
 }
 
@@ -2964,8 +2879,8 @@ int lib_interface_gmp_address_family_static_group_destroy(
 	struct nb_cb_destroy_args *args)
 {
 	struct interface *ifp;
-	struct ipaddr source_addr;
-	struct ipaddr group_addr;
+	pim_addr source_addr;
+	pim_addr group_addr;
 	int result;
 
 	switch (args->event) {
@@ -2975,22 +2890,17 @@ int lib_interface_gmp_address_family_static_group_destroy(
 		break;
 	case NB_EV_APPLY:
 		ifp = nb_running_get_entry(args->dnode, NULL, true);
-		yang_dnode_get_ip(&source_addr, args->dnode, "./source-addr");
-		yang_dnode_get_ip(&group_addr, args->dnode, "./group-addr");
-
-		result = pim_if_igmp_join_del(ifp, group_addr.ip._v4_addr,
-				source_addr.ip._v4_addr);
+		yang_dnode_get_pimaddr(&source_addr, args->dnode,
+				       "./source-addr");
+		yang_dnode_get_pimaddr(&group_addr, args->dnode,
+				       "./group-addr");
+		result = pim_if_gm_join_del(ifp, group_addr, source_addr);
 
 		if (result) {
-			char src_str[INET_ADDRSTRLEN];
-			char grp_str[INET_ADDRSTRLEN];
-
-			ipaddr2str(&source_addr, src_str, sizeof(src_str));
-			ipaddr2str(&group_addr, grp_str, sizeof(grp_str));
-
 			snprintf(args->errmsg, args->errmsg_len,
-				 "%% Failure leaving IGMP group %s %s on interface %s: %d",
-				 src_str, grp_str, ifp->name, result);
+				 "%% Failure leaving " GM
+				 " group %pPAs %pPAs on interface %s: %d",
+				 &source_addr, &group_addr, ifp->name, result);
 
 			return NB_ERR_INCONSISTENCY;
 		}
