@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * PIM for Quagga
  * Copyright (C) 2008  Everton da Silva Marques
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -22,7 +9,7 @@
 #include "log.h"
 #include "zclient.h"
 #include "memory.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "linklist.h"
 #include "vty.h"
 #include "plist.h"
@@ -179,10 +166,10 @@ static void upstream_channel_oil_detach(struct pim_upstream *up)
 
 static void pim_upstream_timers_stop(struct pim_upstream *up)
 {
-	THREAD_OFF(up->t_ka_timer);
-	THREAD_OFF(up->t_rs_timer);
-	THREAD_OFF(up->t_msdp_reg_timer);
-	THREAD_OFF(up->t_join_timer);
+	EVENT_OFF(up->t_ka_timer);
+	EVENT_OFF(up->t_rs_timer);
+	EVENT_OFF(up->t_msdp_reg_timer);
+	EVENT_OFF(up->t_join_timer);
 }
 
 struct pim_upstream *pim_upstream_del(struct pim_instance *pim,
@@ -194,7 +181,7 @@ struct pim_upstream *pim_upstream_del(struct pim_instance *pim,
 
 	if (PIM_DEBUG_PIM_TRACE)
 		zlog_debug(
-			"%s(%s): Delete %s[%s] ref count: %d , flags: %d c_oil ref count %d (Pre decrement)",
+			"%s(%s): Delete %s[%s] ref count: %d, flags: %d c_oil ref count %d (Pre decrement)",
 			__func__, name, up->sg_str, pim->vrf->name,
 			up->ref_count, up->flags,
 			up->channel_oil->oil_ref_count);
@@ -302,11 +289,11 @@ void pim_upstream_send_join(struct pim_upstream *up)
 	pim_jp_agg_single_upstream_send(&up->rpf, up, 1 /* join */);
 }
 
-static void on_join_timer(struct thread *t)
+static void on_join_timer(struct event *t)
 {
 	struct pim_upstream *up;
 
-	up = THREAD_ARG(t);
+	up = EVENT_ARG(t);
 
 	if (!up->rpf.source_nexthop.interface) {
 		if (PIM_DEBUG_PIM_TRACE)
@@ -336,7 +323,7 @@ static void join_timer_stop(struct pim_upstream *up)
 {
 	struct pim_neighbor *nbr = NULL;
 
-	THREAD_OFF(up->t_join_timer);
+	EVENT_OFF(up->t_join_timer);
 
 	if (up->rpf.source_nexthop.interface)
 		nbr = pim_neighbor_find(up->rpf.source_nexthop.interface,
@@ -366,9 +353,9 @@ void join_timer_start(struct pim_upstream *up)
 	if (nbr)
 		pim_jp_agg_add_group(nbr->upstream_jp_agg, up, 1, nbr);
 	else {
-		THREAD_OFF(up->t_join_timer);
-		thread_add_timer(router->master, on_join_timer, up,
-				 router->t_periodic, &up->t_join_timer);
+		EVENT_OFF(up->t_join_timer);
+		event_add_timer(router->master, on_join_timer, up,
+				router->t_periodic, &up->t_join_timer);
 	}
 	pim_jp_agg_upstream_verification(up, true);
 }
@@ -383,7 +370,7 @@ void join_timer_start(struct pim_upstream *up)
 void pim_upstream_join_timer_restart(struct pim_upstream *up,
 				     struct pim_rpf *old)
 {
-	// THREAD_OFF(up->t_join_timer);
+	// EVENT_OFF(up->t_join_timer);
 	join_timer_start(up);
 }
 
@@ -395,9 +382,9 @@ static void pim_upstream_join_timer_restart_msec(struct pim_upstream *up,
 			   __func__, interval_msec, up->sg_str);
 	}
 
-	THREAD_OFF(up->t_join_timer);
-	thread_add_timer_msec(router->master, on_join_timer, up, interval_msec,
-			      &up->t_join_timer);
+	EVENT_OFF(up->t_join_timer);
+	event_add_timer_msec(router->master, on_join_timer, up, interval_msec,
+			     &up->t_join_timer);
 }
 
 void pim_update_suppress_timers(uint32_t suppress_time)
@@ -675,10 +662,9 @@ void pim_upstream_update_use_rpt(struct pim_upstream *up,
 	new_use_rpt = !!PIM_UPSTREAM_FLAG_TEST_USE_RPT(up->flags);
 	if (old_use_rpt != new_use_rpt) {
 		if (PIM_DEBUG_PIM_EVENTS)
-			zlog_debug("%s switched from %s to %s",
-					up->sg_str,
-					old_use_rpt?"RPT":"SPT",
-					new_use_rpt?"RPT":"SPT");
+			zlog_debug("%s switched from %s to %s", up->sg_str,
+				   old_use_rpt ? "RPT" : "SPT",
+				   new_use_rpt ? "RPT" : "SPT");
 		if (update_mroute)
 			pim_upstream_mroute_add(up->channel_oil, __func__);
 	}
@@ -917,14 +903,21 @@ static struct pim_upstream *pim_upstream_new(struct pim_instance *pim,
 				false /*update_mroute*/);
 		pim_upstream_mroute_iif_update(up->channel_oil, __func__);
 
-		if (PIM_UPSTREAM_FLAG_TEST_SRC_NOCACHE(up->flags))
+		if (PIM_UPSTREAM_FLAG_TEST_SRC_NOCACHE(up->flags)) {
+			/*
+			 * Set the right RPF so that future changes will
+			 * be right
+			 */
+			rpf_result = pim_rpf_update(pim, up, NULL, __func__);
 			pim_upstream_keep_alive_timer_start(
 				up, pim->keep_alive_time);
+		}
 	} else if (!pim_addr_is_any(up->upstream_addr)) {
 		pim_upstream_update_use_rpt(up,
 				false /*update_mroute*/);
 		rpf_result = pim_rpf_update(pim, up, NULL, __func__);
 		if (rpf_result == PIM_RPF_FAILURE) {
+			up->channel_oil->oil_inherited_rescan = 1;
 			if (PIM_DEBUG_PIM_TRACE)
 				zlog_debug(
 					"%s: Attempting to create upstream(%s), Unable to RPF for source",
@@ -1374,7 +1367,7 @@ static void pim_upstream_fhr_kat_expiry(struct pim_instance *pim,
 			   up->sg_str);
 
 	/* stop reg-stop timer */
-	THREAD_OFF(up->t_rs_timer);
+	EVENT_OFF(up->t_rs_timer);
 	/* remove regiface from the OIL if it is there*/
 	pim_channel_del_oif(up->channel_oil, pim->regiface,
 			    PIM_OIF_FLAG_PROTO_PIM, __func__);
@@ -1474,11 +1467,11 @@ struct pim_upstream *pim_upstream_keep_alive_timer_proc(
 
 	return up;
 }
-static void pim_upstream_keep_alive_timer(struct thread *t)
+static void pim_upstream_keep_alive_timer(struct event *t)
 {
 	struct pim_upstream *up;
 
-	up = THREAD_ARG(t);
+	up = EVENT_ARG(t);
 
 	/* pull the stats and re-check */
 	if (pim_upstream_sg_running_proc(up))
@@ -1495,9 +1488,9 @@ void pim_upstream_keep_alive_timer_start(struct pim_upstream *up, uint32_t time)
 			zlog_debug("kat start on %s with no stream reference",
 				   up->sg_str);
 	}
-	THREAD_OFF(up->t_ka_timer);
-	thread_add_timer(router->master, pim_upstream_keep_alive_timer, up,
-			 time, &up->t_ka_timer);
+	EVENT_OFF(up->t_ka_timer);
+	event_add_timer(router->master, pim_upstream_keep_alive_timer, up, time,
+			&up->t_ka_timer);
 
 	/* any time keepalive is started against a SG we will have to
 	 * re-evaluate our active source database */
@@ -1507,9 +1500,9 @@ void pim_upstream_keep_alive_timer_start(struct pim_upstream *up, uint32_t time)
 }
 
 /* MSDP on RP needs to know if a source is registerable to this RP */
-static void pim_upstream_msdp_reg_timer(struct thread *t)
+static void pim_upstream_msdp_reg_timer(struct event *t)
 {
-	struct pim_upstream *up = THREAD_ARG(t);
+	struct pim_upstream *up = EVENT_ARG(t);
 	struct pim_instance *pim = up->channel_oil->pim;
 
 	/* source is no longer active - pull the SA from MSDP's cache */
@@ -1518,9 +1511,9 @@ static void pim_upstream_msdp_reg_timer(struct thread *t)
 
 void pim_upstream_msdp_reg_timer_start(struct pim_upstream *up)
 {
-	THREAD_OFF(up->t_msdp_reg_timer);
-	thread_add_timer(router->master, pim_upstream_msdp_reg_timer, up,
-			 PIM_MSDP_REG_RXED_PERIOD, &up->t_msdp_reg_timer);
+	EVENT_OFF(up->t_msdp_reg_timer);
+	event_add_timer(router->master, pim_upstream_msdp_reg_timer, up,
+			PIM_MSDP_REG_RXED_PERIOD, &up->t_msdp_reg_timer);
 
 	pim_msdp_sa_local_update(up);
 }
@@ -1693,14 +1686,12 @@ const char *pim_reg_state2str(enum pim_reg_state reg_state, char *state_str,
 	return state_str;
 }
 
-static void pim_upstream_start_register_stop_timer(struct pim_upstream *up);
-
-static void pim_upstream_register_stop_timer(struct thread *t)
+static void pim_upstream_register_stop_timer(struct event *t)
 {
 	struct pim_interface *pim_ifp;
 	struct pim_instance *pim;
 	struct pim_upstream *up;
-	up = THREAD_ARG(t);
+	up = EVENT_ARG(t);
 	pim = up->channel_oil->pim;
 
 	if (PIM_DEBUG_PIM_TRACE) {
@@ -1728,6 +1719,7 @@ static void pim_upstream_register_stop_timer(struct thread *t)
 				zlog_debug("%s: up %s RPF is not present",
 					   __func__, up->sg_str);
 			up->reg_state = PIM_REG_NOINFO;
+			PIM_UPSTREAM_FLAG_UNSET_FHR(up->flags);
 			return;
 		}
 
@@ -1741,7 +1733,7 @@ static void pim_upstream_register_stop_timer(struct thread *t)
 			return;
 		}
 		up->reg_state = PIM_REG_JOIN_PENDING;
-		pim_upstream_start_register_stop_timer(up);
+		pim_upstream_start_register_stop_timer(up, 1);
 
 		if (((up->channel_oil->cc.lastused / 100)
 		     > pim->keep_alive_time)
@@ -1759,58 +1751,31 @@ static void pim_upstream_register_stop_timer(struct thread *t)
 	}
 }
 
-static void pim_upstream_start_register_stop_timer(struct pim_upstream *up)
+void pim_upstream_start_register_stop_timer(struct pim_upstream *up,
+					    int null_register)
 {
 	uint32_t time;
 
-	THREAD_OFF(up->t_rs_timer);
+	EVENT_OFF(up->t_rs_timer);
 
-	time = router->register_probe_time;
+	if (!null_register) {
+		uint32_t lower = (0.5 * router->register_suppress_time);
+		uint32_t upper = (1.5 * router->register_suppress_time);
+		time = lower + (frr_weak_random() % (upper - lower + 1));
+		/* Make sure we don't wrap around */
+		if (time >= router->register_probe_time)
+			time -= router->register_probe_time;
+		else
+			time = 0;
+	} else
+		time = router->register_probe_time;
 
 	if (PIM_DEBUG_PIM_TRACE) {
 		zlog_debug(
 			"%s: (S,G)=%s Starting upstream register stop timer %d",
 			__func__, up->sg_str, time);
 	}
-	thread_add_timer(router->master, pim_upstream_register_stop_timer, up,
-			 time, &up->t_rs_timer);
-}
-
-static void pim_upstream_register_probe_timer(struct thread *t)
-{
-	struct pim_upstream *up = THREAD_ARG(t);
-
-	if (!up->rpf.source_nexthop.interface ||
-	    !up->rpf.source_nexthop.interface->info) {
-		if (PIM_DEBUG_PIM_REG)
-			zlog_debug("cannot send Null register for %pSG, no path to RP",
-				   &up->sg);
-	} else
-		pim_null_register_send(up);
-
-	pim_upstream_start_register_stop_timer(up);
-}
-
-void pim_upstream_start_register_probe_timer(struct pim_upstream *up)
-{
-	uint32_t time;
-
-	THREAD_OFF(up->t_rs_timer);
-
-	uint32_t lower = (0.5 * router->register_suppress_time);
-	uint32_t upper = (1.5 * router->register_suppress_time);
-	time = lower + (frr_weak_random() % (upper - lower + 1));
-	/* Make sure we don't wrap around */
-	if (time >= router->register_probe_time)
-		time -= router->register_probe_time;
-	else
-		time = 0;
-
-	if (PIM_DEBUG_PIM_TRACE)
-		zlog_debug("%s: (S,G)=%s Starting upstream register stop null probe timer %d",
-			   __func__, up->sg_str, time);
-
-	thread_add_timer(router->master, pim_upstream_register_probe_timer, up,
+	event_add_timer(router->master, pim_upstream_register_stop_timer, up,
 			time, &up->t_rs_timer);
 }
 
@@ -1978,40 +1943,6 @@ void pim_upstream_terminate(struct pim_instance *pim)
 		wheel_delete(pim->upstream_sg_wheel);
 	pim->upstream_sg_wheel = NULL;
 }
-bool pim_sg_is_reevaluate_oil_req(struct pim_instance *pim,
-				  struct pim_upstream *up)
-{
-	struct pim_interface *pim_ifp = NULL;
-
-	/*
-	 * Attempt to retrieve the PIM interface information if the RPF
-	 * interface is present
-	 */
-	if (up->rpf.source_nexthop.interface) {
-		pim_ifp = up->rpf.source_nexthop.interface->info;
-	} else {
-		if (PIM_DEBUG_PIM_TRACE) {
-			zlog_debug("%s: up %s RPF is not present", __func__,
-				   up->sg_str);
-		}
-	}
-
-	/*
-	 * Determine if a reevaluation of the outgoing interface list (OIL) is
-	 * required. This may be necessary in scenarios such as MSDP where the
-	 * RP role for a group changes from secondary to primary. In such cases,
-	 * SGRpt may receive a prune, resulting in an S,G entry with a NULL OIL.
-	 * The S,G upstream should then inherit the OIL from *,G, which is
-	 * particularly important for VXLAN setups.
-	 */
-	if (up->channel_oil->oil_inherited_rescan ||
-	    (pim_ifp && I_am_RP(pim_ifp->pim, up->sg.grp)) ||
-	    pim_upstream_empty_inherited_olist(up)) {
-		return true;
-	}
-
-	return false;
-}
 
 bool pim_upstream_equal(const void *arg1, const void *arg2)
 {
@@ -2040,6 +1971,7 @@ static bool pim_upstream_kat_start_ok(struct pim_upstream *up)
 	struct channel_oil *c_oil = up->channel_oil;
 	struct interface *ifp = up->rpf.source_nexthop.interface;
 	struct pim_interface *pim_ifp;
+	struct pim_instance *pim = up->channel_oil->pim;
 
 	/* "iif == RPF_interface(S)" check is not easy to do as the info
 	 * we get from the kernel/ASIC is really a "lookup/key hit".
@@ -2050,7 +1982,7 @@ static bool pim_upstream_kat_start_ok(struct pim_upstream *up)
 		return false;
 
 	pim_ifp = ifp->info;
-	if (pim_ifp->mroute_vif_index != *oil_parent(c_oil))
+	if (pim_ifp->mroute_vif_index != *oil_incoming_vif(c_oil))
 		return false;
 
 	if (pim_if_connected_to_source(up->rpf.source_nexthop.interface,
@@ -2059,8 +1991,9 @@ static bool pim_upstream_kat_start_ok(struct pim_upstream *up)
 	}
 
 	if ((up->join_state == PIM_UPSTREAM_JOINED)
-			&& !pim_upstream_empty_inherited_olist(up)) {
-		return true;
+	    && !pim_upstream_empty_inherited_olist(up)) {
+		if (I_am_RP(pim, up->sg.grp))
+			return true;
 	}
 
 	return false;
@@ -2132,7 +2065,7 @@ static void pim_upstream_sg_running(void *arg)
 	// No packet can have arrived here if this is the case
 	if (!up->channel_oil->installed) {
 		if (PIM_DEBUG_TRACE)
-			zlog_debug("%s: %s%s is not installed in mroute",
+			zlog_debug("%s: %s[%s] is not installed in mroute",
 				   __func__, up->sg_str, pim->vrf->name);
 		return;
 	}
@@ -2145,7 +2078,7 @@ static void pim_upstream_sg_running(void *arg)
 	 * only doing this at this point in time
 	 * to get us up and working for the moment
 	 */
-	if (pim_sg_is_reevaluate_oil_req(pim, up)) {
+	if (up->channel_oil->oil_inherited_rescan) {
 		if (PIM_DEBUG_TRACE)
 			zlog_debug(
 				"%s: Handling unscanned inherited_olist for %s[%s]",
