@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * PIM for Quagga
  * Copyright (C) 2008  Everton da Silva Marques
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -64,37 +51,6 @@ static int pim_router_id_update_zebra(ZAPI_CALLBACK_ARGS)
 	return 0;
 }
 
-static int pim_zebra_interface_vrf_update(ZAPI_CALLBACK_ARGS)
-{
-	struct interface *ifp;
-	vrf_id_t new_vrf_id;
-	struct pim_instance *pim;
-	struct pim_interface *pim_ifp;
-
-	ifp = zebra_interface_vrf_update_read(zclient->ibuf, vrf_id,
-					      &new_vrf_id);
-	if (!ifp)
-		return 0;
-
-	if (PIM_DEBUG_ZEBRA)
-		zlog_debug("%s: %s updating from %u to %u", __func__, ifp->name,
-			   vrf_id, new_vrf_id);
-
-	pim = pim_get_pim_instance(new_vrf_id);
-
-	if_update_to_new_vrf(ifp, new_vrf_id);
-
-	pim_ifp = ifp->info;
-	if (!pim_ifp)
-		return 0;
-
-	pim_ifp->pim->mcast_if_count--;
-	pim_ifp->pim = pim;
-	pim_ifp->pim->mcast_if_count++;
-
-	return 0;
-}
-
 #ifdef PIM_DEBUG_IFADDR_DUMP
 static void dump_if_address(struct interface *ifp)
 {
@@ -410,9 +366,9 @@ void pim_scan_oil(struct pim_instance *pim)
 		pim_upstream_mroute_iif_update(c_oil, __func__);
 }
 
-static void on_rpf_cache_refresh(struct thread *t)
+static void on_rpf_cache_refresh(struct event *t)
 {
-	struct pim_instance *pim = THREAD_ARG(t);
+	struct pim_instance *pim = EVENT_ARG(t);
 
 	/* update kernel multicast forwarding cache (MFC) */
 	pim_scan_oil(pim);
@@ -442,9 +398,9 @@ void sched_rpf_cache_refresh(struct pim_instance *pim)
 			   router->rpf_cache_refresh_delay_msec);
 	}
 
-	thread_add_timer_msec(router->master, on_rpf_cache_refresh, pim,
-			      router->rpf_cache_refresh_delay_msec,
-			      &pim->rpf_cache_refresher);
+	event_add_timer_msec(router->master, on_rpf_cache_refresh, pim,
+			     router->rpf_cache_refresh_delay_msec,
+			     &pim->rpf_cache_refresher);
 }
 
 static void pim_zebra_connected(struct zclient *zclient)
@@ -474,7 +430,6 @@ static zclient_handler *const pim_handlers[] = {
 
 	[ZEBRA_NEXTHOP_UPDATE] = pim_parse_nexthop_update,
 	[ZEBRA_ROUTER_ID_UPDATE] = pim_router_id_update_zebra,
-	[ZEBRA_INTERFACE_VRF_UPDATE] = pim_zebra_interface_vrf_update,
 
 #if PIM_IPV == 4
 	[ZEBRA_VXLAN_SG_ADD] = pim_zebra_vxlan_sg_proc,
