@@ -1,27 +1,14 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * STATICd - main code
  * Copyright (C) 2018 Cumulus Networks, Inc.
  *               Donald Sharp
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include <zebra.h>
 
 #include <lib/version.h>
 #include "getopt.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "command.h"
 #include "log.h"
 #include "memory.h"
@@ -40,6 +27,8 @@
 #include "static_debug.h"
 #include "static_nb.h"
 
+#include "mgmt_be_client.h"
+
 char backup_config_file[256];
 
 bool mpls_enabled;
@@ -62,14 +51,16 @@ struct zebra_privs_t static_privs = {
 struct option longopts[] = { { 0 } };
 
 /* Master of threads. */
-struct thread_master *master;
+struct event_loop *master;
+
+struct mgmt_be_client *mgmt_be_client;
 
 static struct frr_daemon_info staticd_di;
+
 /* SIGHUP handler. */
 static void sighup(void)
 {
-	zlog_info("SIGHUP received");
-	vty_read_config(NULL, staticd_di.config_file, config_default);
+	zlog_info("SIGHUP received and ignored");
 }
 
 /* SIGINT / SIGTERM handler. */
@@ -80,6 +71,8 @@ static void sigint(void)
 	/* Disable BFD events to avoid wasting processing. */
 	bfd_protocol_integration_set_shutdown(true);
 
+	mgmt_be_client_destroy(mgmt_be_client);
+
 	static_vrf_terminate();
 
 	static_zebra_stop();
@@ -114,7 +107,6 @@ struct frr_signal_t static_signals[] = {
 };
 
 static const struct frr_yang_module_info *const staticd_yang_modules[] = {
-	&frr_filter_info,
 	&frr_interface_info,
 	&frr_vrf_info,
 	&frr_routing_info,
@@ -123,6 +115,10 @@ static const struct frr_yang_module_info *const staticd_yang_modules[] = {
 
 #define STATIC_VTY_PORT 2616
 
+/*
+ * NOTE: .flags == FRR_NO_SPLIT_CONFIG to avoid reading split config, mgmtd will
+ * do this for us now
+ */
 FRR_DAEMON_INFO(staticd, STATIC, .vty_port = STATIC_VTY_PORT,
 
 		.proghelp = "Implementation of STATIC.",
@@ -132,7 +128,8 @@ FRR_DAEMON_INFO(staticd, STATIC, .vty_port = STATIC_VTY_PORT,
 
 		.privs = &static_privs, .yang_modules = staticd_yang_modules,
 		.n_yang_modules = array_size(staticd_yang_modules),
-);
+
+		.flags = FRR_NO_SPLIT_CONFIG);
 
 int main(int argc, char **argv, char **envp)
 {
@@ -163,14 +160,20 @@ int main(int argc, char **argv, char **envp)
 	static_zebra_init();
 	static_vty_init();
 
+	/* Initialize MGMT backend functionalities */
+	mgmt_be_client = mgmt_be_client_create("staticd", NULL, 0, master);
+
 	hook_register(routing_conf_event,
 		      routing_control_plane_protocols_name_validate);
 
 	routing_control_plane_protocols_register_vrf_dependency();
 
-	snprintf(backup_config_file, sizeof(backup_config_file),
-		 "%s/zebra.conf", frr_sysconfdir);
-	staticd_di.backup_config_file = backup_config_file;
+	/*
+	 * We set FRR_NO_SPLIT_CONFIG flag to avoid reading our config, but we
+	 * still need to write one if vtysh tells us to. Setting the host
+	 * config filename does this.
+	 */
+	host_config_set(config_default);
 
 	frr_config_fork();
 	frr_run(master);
