@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * STATICd - vty code
  * Copyright (C) 2018 Cumulus Networks, Inc.
  *               Donald Sharp
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include <zebra.h>
 
@@ -26,6 +13,7 @@
 #include "nexthop.h"
 #include "table.h"
 #include "srcdest_table.h"
+#include "mgmt_be_client.h"
 #include "mpls.h"
 #include "northbound.h"
 #include "libfrr.h"
@@ -59,6 +47,7 @@ struct static_route_args {
 	const char *source;
 	const char *gateway;
 	const char *interface_name;
+	const char *segs;
 	const char *flag;
 	const char *tag;
 	const char *distance;
@@ -70,6 +59,8 @@ struct static_route_args {
 	bool bfd_multi_hop;
 	const char *bfd_source;
 	const char *bfd_profile;
+
+	const char *input;
 };
 
 static int static_route_nb_run(struct vty *vty, struct static_route_args *args)
@@ -83,12 +74,16 @@ static int static_route_nb_run(struct vty *vty, struct static_route_args *args)
 	char xpath_nexthop[XPATH_MAXLEN];
 	char xpath_mpls[XPATH_MAXLEN];
 	char xpath_label[XPATH_MAXLEN];
+	char xpath_segs[XPATH_MAXLEN];
+	char xpath_seg[XPATH_MAXLEN];
 	char ab_xpath[XPATH_MAXLEN];
 	char buf_prefix[PREFIX_STRLEN];
 	char buf_src_prefix[PREFIX_STRLEN] = {};
 	char buf_nh_type[PREFIX_STRLEN] = {};
 	char buf_tag[PREFIX_STRLEN];
 	uint8_t label_stack_id = 0;
+	uint8_t segs_stack_id = 0;
+	char *orig_label = NULL, *orig_seg = NULL;
 	const char *buf_gate_str;
 	uint8_t distance = ZEBRA_STATIC_DISTANCE_DEFAULT;
 	route_tag_t tag = 0;
@@ -135,7 +130,9 @@ static int static_route_nb_run(struct vty *vty, struct static_route_args *args)
 		if (args->source)
 			assert(!!str2prefix(args->source, &src));
 		break;
-	default:
+	case AFI_L2VPN:
+	case AFI_UNSPEC:
+	case AFI_MAX:
 		break;
 	}
 
@@ -155,9 +152,20 @@ static int static_route_nb_run(struct vty *vty, struct static_route_args *args)
 	else
 		buf_gate_str = "";
 
-	if (args->gateway == NULL && args->interface_name == NULL)
+	if (args->gateway == NULL && args->interface_name == NULL) {
 		type = STATIC_BLACKHOLE;
-	else if (args->gateway && args->interface_name) {
+		/* If this is blackhole/reject flagged route, then
+		 * specify interface_name with the value of what was really
+		 * entered.
+		 * interface_name will be validated later in NB functions
+		 * to check if we don't create blackhole/reject routes that
+		 * match the real interface names.
+		 * E.g.: `ip route 10.0.0.1/32 bla` will create a blackhole
+		 * route despite the real interface named `bla` exists.
+		 */
+		if (args->input)
+			args->interface_name = args->input;
+	} else if (args->gateway && args->interface_name) {
 		if (args->afi == AFI_IP)
 			type = STATIC_IPV4_GATEWAY_IFNAME;
 		else
@@ -296,10 +304,10 @@ static int static_route_nb_run(struct vty *vty, struct static_route_args *args)
 				nb_cli_enqueue_change(vty, ab_xpath,
 						      NB_OP_MODIFY, "false");
 		}
-		if (type == STATIC_IPV4_GATEWAY
-		    || type == STATIC_IPV6_GATEWAY
-		    || type == STATIC_IPV4_GATEWAY_IFNAME
-		    || type == STATIC_IPV6_GATEWAY_IFNAME) {
+		if (type == STATIC_IPV4_GATEWAY ||
+		    type == STATIC_IPV6_GATEWAY ||
+		    type == STATIC_IPV4_GATEWAY_IFNAME ||
+		    type == STATIC_IPV6_GATEWAY_IFNAME) {
 			strlcpy(ab_xpath, xpath_nexthop, sizeof(ab_xpath));
 			strlcat(ab_xpath, FRR_STATIC_ROUTE_NH_COLOR_XPATH,
 				sizeof(ab_xpath));
@@ -321,7 +329,7 @@ static int static_route_nb_run(struct vty *vty, struct static_route_args *args)
 			nb_cli_enqueue_change(vty, xpath_mpls, NB_OP_DESTROY,
 					      NULL);
 
-			ostr = XSTRDUP(MTYPE_TMP, args->label);
+			orig_label = ostr = XSTRDUP(MTYPE_TMP, args->label);
 			while ((nump = strsep(&ostr, "/")) != NULL) {
 				snprintf(ab_xpath, sizeof(ab_xpath),
 					 FRR_STATIC_ROUTE_NHLB_KEY_XPATH,
@@ -334,7 +342,6 @@ static int static_route_nb_run(struct vty *vty, struct static_route_args *args)
 						      NB_OP_MODIFY, nump);
 				label_stack_id++;
 			}
-			XFREE(MTYPE_TMP, ostr);
 		} else {
 			strlcpy(xpath_mpls, xpath_nexthop, sizeof(xpath_mpls));
 			strlcat(xpath_mpls, FRR_STATIC_ROUTE_NH_LABEL_XPATH,
@@ -342,7 +349,38 @@ static int static_route_nb_run(struct vty *vty, struct static_route_args *args)
 			nb_cli_enqueue_change(vty, xpath_mpls, NB_OP_DESTROY,
 					      NULL);
 		}
+		if (args->segs) {
+			/* copy of seg string (start) */
+			char *ostr;
+			/* pointer to next segment */
+			char *nump;
 
+			strlcpy(xpath_segs, xpath_nexthop, sizeof(xpath_segs));
+			strlcat(xpath_segs, FRR_STATIC_ROUTE_NH_SRV6_SEGS_XPATH,
+				sizeof(xpath_segs));
+
+			nb_cli_enqueue_change(vty, xpath_segs, NB_OP_DESTROY,
+					      NULL);
+
+			orig_seg = ostr = XSTRDUP(MTYPE_TMP, args->segs);
+			while ((nump = strsep(&ostr, "/")) != NULL) {
+				snprintf(ab_xpath, sizeof(ab_xpath),
+					 FRR_STATIC_ROUTE_NH_SRV6_KEY_SEG_XPATH,
+					 segs_stack_id);
+				strlcpy(xpath_seg, xpath_segs,
+					sizeof(xpath_seg));
+				strlcat(xpath_seg, ab_xpath, sizeof(xpath_seg));
+				nb_cli_enqueue_change(vty, xpath_seg,
+						      NB_OP_MODIFY, nump);
+				segs_stack_id++;
+			}
+		} else {
+			strlcpy(xpath_segs, xpath_nexthop, sizeof(xpath_segs));
+			strlcat(xpath_segs, FRR_STATIC_ROUTE_NH_SRV6_SEGS_XPATH,
+				sizeof(xpath_segs));
+			nb_cli_enqueue_change(vty, xpath_segs, NB_OP_DESTROY,
+					      NULL);
+		}
 		if (args->bfd) {
 			char xpath_bfd[XPATH_MAXLEN];
 
@@ -378,26 +416,57 @@ static int static_route_nb_run(struct vty *vty, struct static_route_args *args)
 		}
 
 		ret = nb_cli_apply_changes(vty, "%s", xpath_prefix);
+
+		if (orig_label)
+			XFREE(MTYPE_TMP, orig_label);
+		if (orig_seg)
+			XFREE(MTYPE_TMP, orig_seg);
 	} else {
-		if (args->source)
-			snprintf(ab_xpath, sizeof(ab_xpath),
-				 FRR_DEL_S_ROUTE_SRC_NH_KEY_NO_DISTANCE_XPATH,
-				 "frr-staticd:staticd", "staticd", args->vrf,
-				 buf_prefix,
-				 yang_afi_safi_value2identity(args->afi,
-							      args->safi),
-				 buf_src_prefix, table_id, buf_nh_type,
-				 args->nexthop_vrf, buf_gate_str,
-				 args->interface_name);
-		else
-			snprintf(ab_xpath, sizeof(ab_xpath),
-				 FRR_DEL_S_ROUTE_NH_KEY_NO_DISTANCE_XPATH,
-				 "frr-staticd:staticd", "staticd", args->vrf,
-				 buf_prefix,
-				 yang_afi_safi_value2identity(args->afi,
-							      args->safi),
-				 table_id, buf_nh_type, args->nexthop_vrf,
-				 buf_gate_str, args->interface_name);
+		if (args->source) {
+			if (args->distance)
+				snprintf(ab_xpath, sizeof(ab_xpath),
+					 FRR_DEL_S_ROUTE_SRC_NH_KEY_XPATH,
+					 "frr-staticd:staticd", "staticd",
+					 args->vrf, buf_prefix,
+					 yang_afi_safi_value2identity(
+						 args->afi, args->safi),
+					 buf_src_prefix, table_id, distance,
+					 buf_nh_type, args->nexthop_vrf,
+					 buf_gate_str, args->interface_name);
+			else
+				snprintf(
+					ab_xpath, sizeof(ab_xpath),
+					FRR_DEL_S_ROUTE_SRC_NH_KEY_NO_DISTANCE_XPATH,
+					"frr-staticd:staticd", "staticd",
+					args->vrf, buf_prefix,
+					yang_afi_safi_value2identity(
+						args->afi, args->safi),
+					buf_src_prefix, table_id, buf_nh_type,
+					args->nexthop_vrf, buf_gate_str,
+					args->interface_name);
+		} else {
+			if (args->distance)
+				snprintf(ab_xpath, sizeof(ab_xpath),
+					 FRR_DEL_S_ROUTE_NH_KEY_XPATH,
+					 "frr-staticd:staticd", "staticd",
+					 args->vrf, buf_prefix,
+					 yang_afi_safi_value2identity(
+						 args->afi, args->safi),
+					 table_id, distance, buf_nh_type,
+					 args->nexthop_vrf, buf_gate_str,
+					 args->interface_name);
+			else
+				snprintf(
+					ab_xpath, sizeof(ab_xpath),
+					FRR_DEL_S_ROUTE_NH_KEY_NO_DISTANCE_XPATH,
+					"frr-staticd:staticd", "staticd",
+					args->vrf, buf_prefix,
+					yang_afi_safi_value2identity(
+						args->afi, args->safi),
+					table_id, buf_nh_type,
+					args->nexthop_vrf, buf_gate_str,
+					args->interface_name);
+		}
 
 		dnode = yang_dnode_get(vty->candidate_config->dnode, ab_xpath);
 		if (!dnode) {
@@ -483,6 +552,8 @@ DEFPY_YANG(ip_route_blackhole,
       "Table to configure\n"
       "The table number to configure\n")
 {
+	int idx_flag = 0;
+
 	struct static_route_args args = {
 		.delete = !!no,
 		.afi = AFI_IP,
@@ -497,6 +568,9 @@ DEFPY_YANG(ip_route_blackhole,
 		.vrf = vrf,
 	};
 
+	if (flag && argv_find(argv, argc, flag, &idx_flag))
+		args.input = argv[idx_flag]->arg;
+
 	return static_route_nb_run(vty, &args);
 }
 
@@ -525,6 +599,8 @@ DEFPY_YANG(ip_route_blackhole_vrf,
       "Table to configure\n"
       "The table number to configure\n")
 {
+	int idx_flag = 0;
+
 	struct static_route_args args = {
 		.delete = !!no,
 		.afi = AFI_IP,
@@ -546,6 +622,9 @@ DEFPY_YANG(ip_route_blackhole_vrf,
 	 */
 	assert(args.prefix);
 
+	if (flag && argv_find(argv, argc, flag, &idx_flag))
+		args.input = argv[idx_flag]->arg;
+
 	return static_route_nb_run(vty, &args);
 }
 
@@ -836,6 +915,8 @@ DEFPY_YANG(ipv6_route_blackhole,
       "Table to configure\n"
       "The table number to configure\n")
 {
+	int idx_flag = 0;
+
 	struct static_route_args args = {
 		.delete = !!no,
 		.afi = AFI_IP6,
@@ -850,6 +931,9 @@ DEFPY_YANG(ipv6_route_blackhole,
 		.vrf = vrf,
 	};
 
+	if (flag && argv_find(argv, argc, flag, &idx_flag))
+		args.input = argv[idx_flag]->arg;
+
 	return static_route_nb_run(vty, &args);
 }
 
@@ -878,6 +962,8 @@ DEFPY_YANG(ipv6_route_blackhole_vrf,
       "Table to configure\n"
       "The table number to configure\n")
 {
+	int idx_flag = 0;
+
 	struct static_route_args args = {
 		.delete = !!no,
 		.afi = AFI_IP6,
@@ -899,12 +985,14 @@ DEFPY_YANG(ipv6_route_blackhole_vrf,
 	 */
 	assert(args.prefix);
 
+	if (flag && argv_find(argv, argc, flag, &idx_flag))
+		args.input = argv[idx_flag]->arg;
+
 	return static_route_nb_run(vty, &args);
 }
 
-DEFPY_YANG(ipv6_route_address_interface,
-      ipv6_route_address_interface_cmd,
-      "[no] ipv6 route X:X::X:X/M$prefix [from X:X::X:X/M] \
+DEFPY_YANG(ipv6_route_address_interface, ipv6_route_address_interface_cmd,
+	   "[no] ipv6 route X:X::X:X/M$prefix [from X:X::X:X/M] \
           X:X::X:X$gate                                    \
           <INTERFACE|Null0>$ifname                         \
           [{                                               \
@@ -917,33 +1005,28 @@ DEFPY_YANG(ipv6_route_address_interface,
 	    |onlink$onlink                                 \
 	    |color (1-4294967295)                          \
 	    |bfd$bfd [{multi-hop$bfd_multi_hop|source X:X::X:X$bfd_source|profile BFDPROF$bfd_profile}] \
+		|segments WORD 								   \
           }]",
-      NO_STR
-      IPV6_STR
-      "Establish static routes\n"
-      "IPv6 destination prefix (e.g. 3ffe:506::/32)\n"
-      "IPv6 source-dest route\n"
-      "IPv6 source prefix\n"
-      "IPv6 gateway address\n"
-      "IPv6 gateway interface name\n"
-      "Null interface\n"
-      "Set tag for this route\n"
-      "Tag value\n"
-      "Distance value for this prefix\n"
-      VRF_CMD_HELP_STR
-      MPLS_LABEL_HELPSTR
-      "Table to configure\n"
-      "The table number to configure\n"
-      VRF_CMD_HELP_STR
-      "Treat the nexthop as directly attached to the interface\n"
-      "SR-TE color\n"
-      "The SR-TE color to configure\n"
-      BFD_INTEGRATION_STR
-      BFD_INTEGRATION_MULTI_HOP_STR
-      BFD_INTEGRATION_SOURCE_STR
-      BFD_INTEGRATION_SOURCEV4_STR
-      BFD_PROFILE_STR
-      BFD_PROFILE_NAME_STR)
+	   NO_STR IPV6_STR
+	   "Establish static routes\n"
+	   "IPv6 destination prefix (e.g. 3ffe:506::/32)\n"
+	   "IPv6 source-dest route\n"
+	   "IPv6 source prefix\n"
+	   "IPv6 gateway address\n"
+	   "IPv6 gateway interface name\n"
+	   "Null interface\n"
+	   "Set tag for this route\n"
+	   "Tag value\n"
+	   "Distance value for this prefix\n" VRF_CMD_HELP_STR MPLS_LABEL_HELPSTR
+	   "Table to configure\n"
+	   "The table number to configure\n" VRF_CMD_HELP_STR
+	   "Treat the nexthop as directly attached to the interface\n"
+	   "SR-TE color\n"
+	   "The SR-TE color to configure\n" BFD_INTEGRATION_STR
+		   BFD_INTEGRATION_MULTI_HOP_STR BFD_INTEGRATION_SOURCE_STR
+			   BFD_INTEGRATION_SOURCEV4_STR BFD_PROFILE_STR
+				   BFD_PROFILE_NAME_STR "Value of segs\n"
+	   "Segs (SIDs)\n")
 {
 	struct static_route_args args = {
 		.delete = !!no,
@@ -965,14 +1048,15 @@ DEFPY_YANG(ipv6_route_address_interface,
 		.bfd_multi_hop = !!bfd_multi_hop,
 		.bfd_source = bfd_source_str,
 		.bfd_profile = bfd_profile,
+		.segs = segments,
 	};
 
 	return static_route_nb_run(vty, &args);
 }
 
 DEFPY_YANG(ipv6_route_address_interface_vrf,
-      ipv6_route_address_interface_vrf_cmd,
-      "[no] ipv6 route X:X::X:X/M$prefix [from X:X::X:X/M] \
+	   ipv6_route_address_interface_vrf_cmd,
+	   "[no] ipv6 route X:X::X:X/M$prefix [from X:X::X:X/M] \
           X:X::X:X$gate                                    \
           <INTERFACE|Null0>$ifname                         \
           [{                                               \
@@ -984,32 +1068,28 @@ DEFPY_YANG(ipv6_route_address_interface_vrf,
 	    |onlink$onlink                                 \
 	    |color (1-4294967295)                          \
 	    |bfd$bfd [{multi-hop$bfd_multi_hop|source X:X::X:X$bfd_source|profile BFDPROF$bfd_profile}] \
+		|segments WORD 								   \
           }]",
-      NO_STR
-      IPV6_STR
-      "Establish static routes\n"
-      "IPv6 destination prefix (e.g. 3ffe:506::/32)\n"
-      "IPv6 source-dest route\n"
-      "IPv6 source prefix\n"
-      "IPv6 gateway address\n"
-      "IPv6 gateway interface name\n"
-      "Null interface\n"
-      "Set tag for this route\n"
-      "Tag value\n"
-      "Distance value for this prefix\n"
-      MPLS_LABEL_HELPSTR
-      "Table to configure\n"
-      "The table number to configure\n"
-      VRF_CMD_HELP_STR
-      "Treat the nexthop as directly attached to the interface\n"
-      "SR-TE color\n"
-      "The SR-TE color to configure\n"
-      BFD_INTEGRATION_STR
-      BFD_INTEGRATION_MULTI_HOP_STR
-      BFD_INTEGRATION_SOURCE_STR
-      BFD_INTEGRATION_SOURCEV4_STR
-      BFD_PROFILE_STR
-      BFD_PROFILE_NAME_STR)
+	   NO_STR IPV6_STR
+	   "Establish static routes\n"
+	   "IPv6 destination prefix (e.g. 3ffe:506::/32)\n"
+	   "IPv6 source-dest route\n"
+	   "IPv6 source prefix\n"
+	   "IPv6 gateway address\n"
+	   "IPv6 gateway interface name\n"
+	   "Null interface\n"
+	   "Set tag for this route\n"
+	   "Tag value\n"
+	   "Distance value for this prefix\n" MPLS_LABEL_HELPSTR
+	   "Table to configure\n"
+	   "The table number to configure\n" VRF_CMD_HELP_STR
+	   "Treat the nexthop as directly attached to the interface\n"
+	   "SR-TE color\n"
+	   "The SR-TE color to configure\n" BFD_INTEGRATION_STR
+		   BFD_INTEGRATION_MULTI_HOP_STR BFD_INTEGRATION_SOURCE_STR
+			   BFD_INTEGRATION_SOURCEV4_STR BFD_PROFILE_STR
+				   BFD_PROFILE_NAME_STR "Value of segs\n"
+	   "Segs (SIDs)\n")
 {
 	struct static_route_args args = {
 		.delete = !!no,
@@ -1031,14 +1111,14 @@ DEFPY_YANG(ipv6_route_address_interface_vrf,
 		.bfd_multi_hop = !!bfd_multi_hop,
 		.bfd_source = bfd_source_str,
 		.bfd_profile = bfd_profile,
+		.segs = segments,
 	};
 
 	return static_route_nb_run(vty, &args);
 }
 
-DEFPY_YANG(ipv6_route,
-      ipv6_route_cmd,
-      "[no] ipv6 route X:X::X:X/M$prefix [from X:X::X:X/M] \
+DEFPY_YANG(ipv6_route, ipv6_route_cmd,
+	   "[no] ipv6 route X:X::X:X/M$prefix [from X:X::X:X/M] \
           <X:X::X:X$gate|<INTERFACE|Null0>$ifname>         \
           [{                                               \
             tag (1-4294967295)                             \
@@ -1049,32 +1129,26 @@ DEFPY_YANG(ipv6_route,
             |nexthop-vrf NAME                              \
             |color (1-4294967295)                          \
 	    |bfd$bfd [{multi-hop$bfd_multi_hop|source X:X::X:X$bfd_source|profile BFDPROF$bfd_profile}] \
+			|segments WORD 								   \
           }]",
-      NO_STR
-      IPV6_STR
-      "Establish static routes\n"
-      "IPv6 destination prefix (e.g. 3ffe:506::/32)\n"
-      "IPv6 source-dest route\n"
-      "IPv6 source prefix\n"
-      "IPv6 gateway address\n"
-      "IPv6 gateway interface name\n"
-      "Null interface\n"
-      "Set tag for this route\n"
-      "Tag value\n"
-      "Distance value for this prefix\n"
-      VRF_CMD_HELP_STR
-      MPLS_LABEL_HELPSTR
-      "Table to configure\n"
-      "The table number to configure\n"
-      VRF_CMD_HELP_STR
-      "SR-TE color\n"
-      "The SR-TE color to configure\n"
-      BFD_INTEGRATION_STR
-      BFD_INTEGRATION_MULTI_HOP_STR
-      BFD_INTEGRATION_SOURCE_STR
-      BFD_INTEGRATION_SOURCEV4_STR
-      BFD_PROFILE_STR
-      BFD_PROFILE_NAME_STR)
+	   NO_STR IPV6_STR
+	   "Establish static routes\n"
+	   "IPv6 destination prefix (e.g. 3ffe:506::/32)\n"
+	   "IPv6 source-dest route\n"
+	   "IPv6 source prefix\n"
+	   "IPv6 gateway address\n"
+	   "IPv6 gateway interface name\n"
+	   "Null interface\n"
+	   "Set tag for this route\n"
+	   "Tag value\n"
+	   "Distance value for this prefix\n" VRF_CMD_HELP_STR MPLS_LABEL_HELPSTR
+	   "Table to configure\n"
+	   "The table number to configure\n" VRF_CMD_HELP_STR "SR-TE color\n"
+	   "The SR-TE color to configure\n" BFD_INTEGRATION_STR
+		   BFD_INTEGRATION_MULTI_HOP_STR BFD_INTEGRATION_SOURCE_STR
+			   BFD_INTEGRATION_SOURCEV4_STR BFD_PROFILE_STR
+				   BFD_PROFILE_NAME_STR "Value of segs\n"
+	   "Segs (SIDs)\n")
 {
 	struct static_route_args args = {
 		.delete = !!no,
@@ -1095,14 +1169,15 @@ DEFPY_YANG(ipv6_route,
 		.bfd_multi_hop = !!bfd_multi_hop,
 		.bfd_source = bfd_source_str,
 		.bfd_profile = bfd_profile,
+		.segs = segments,
+
 	};
 
 	return static_route_nb_run(vty, &args);
 }
 
-DEFPY_YANG(ipv6_route_vrf,
-      ipv6_route_vrf_cmd,
-      "[no] ipv6 route X:X::X:X/M$prefix [from X:X::X:X/M] \
+DEFPY_YANG(ipv6_route_vrf, ipv6_route_vrf_cmd,
+	   "[no] ipv6 route X:X::X:X/M$prefix [from X:X::X:X/M] \
           <X:X::X:X$gate|<INTERFACE|Null0>$ifname>                 \
           [{                                               \
             tag (1-4294967295)                             \
@@ -1112,31 +1187,26 @@ DEFPY_YANG(ipv6_route_vrf,
             |nexthop-vrf NAME                              \
 	    |color (1-4294967295)                          \
 	    |bfd$bfd [{multi-hop$bfd_multi_hop|source X:X::X:X$bfd_source|profile BFDPROF$bfd_profile}] \
+		|segments WORD 								   \
           }]",
-      NO_STR
-      IPV6_STR
-      "Establish static routes\n"
-      "IPv6 destination prefix (e.g. 3ffe:506::/32)\n"
-      "IPv6 source-dest route\n"
-      "IPv6 source prefix\n"
-      "IPv6 gateway address\n"
-      "IPv6 gateway interface name\n"
-      "Null interface\n"
-      "Set tag for this route\n"
-      "Tag value\n"
-      "Distance value for this prefix\n"
-      MPLS_LABEL_HELPSTR
-      "Table to configure\n"
-      "The table number to configure\n"
-      VRF_CMD_HELP_STR
-      "SR-TE color\n"
-      "The SR-TE color to configure\n"
-      BFD_INTEGRATION_STR
-      BFD_INTEGRATION_MULTI_HOP_STR
-      BFD_INTEGRATION_SOURCE_STR
-      BFD_INTEGRATION_SOURCEV4_STR
-      BFD_PROFILE_STR
-      BFD_PROFILE_NAME_STR)
+	   NO_STR IPV6_STR
+	   "Establish static routes\n"
+	   "IPv6 destination prefix (e.g. 3ffe:506::/32)\n"
+	   "IPv6 source-dest route\n"
+	   "IPv6 source prefix\n"
+	   "IPv6 gateway address\n"
+	   "IPv6 gateway interface name\n"
+	   "Null interface\n"
+	   "Set tag for this route\n"
+	   "Tag value\n"
+	   "Distance value for this prefix\n" MPLS_LABEL_HELPSTR
+	   "Table to configure\n"
+	   "The table number to configure\n" VRF_CMD_HELP_STR "SR-TE color\n"
+	   "The SR-TE color to configure\n" BFD_INTEGRATION_STR
+		   BFD_INTEGRATION_MULTI_HOP_STR BFD_INTEGRATION_SOURCE_STR
+			   BFD_INTEGRATION_SOURCEV4_STR BFD_PROFILE_STR
+				   BFD_PROFILE_NAME_STR "Value of segs\n"
+	   "Segs (SIDs)\n")
 {
 	struct static_route_args args = {
 		.delete = !!no,
@@ -1157,6 +1227,7 @@ DEFPY_YANG(ipv6_route_vrf,
 		.bfd_multi_hop = !!bfd_multi_hop,
 		.bfd_source = bfd_source_str,
 		.bfd_profile = bfd_profile,
+		.segs = segments,
 	};
 
 	return static_route_nb_run(vty, &args);
@@ -1203,6 +1274,39 @@ static int mpls_label_iter_cb(const struct lyd_node *dnode, void *arg)
 	return YANG_ITER_CONTINUE;
 }
 
+struct srv6_seg_iter {
+	struct vty *vty;
+	bool first;
+};
+
+static int srv6_seg_iter_cb(const struct lyd_node *dnode, void *arg)
+{
+	struct srv6_seg_iter *iter = arg;
+	char buffer[INET6_ADDRSTRLEN];
+	struct in6_addr cli_seg;
+
+	if (yang_dnode_exists(dnode, "./seg")) {
+		if (iter->first) {
+			yang_dnode_get_ipv6(&cli_seg, dnode, "./seg");
+			if (inet_ntop(AF_INET6, &cli_seg, buffer,
+				      INET6_ADDRSTRLEN) == NULL) {
+				return 1;
+			}
+			vty_out(iter->vty, " segments %s", buffer);
+		} else {
+			yang_dnode_get_ipv6(&cli_seg, dnode, "./seg");
+			if (inet_ntop(AF_INET6, &cli_seg, buffer,
+				      INET6_ADDRSTRLEN) == NULL) {
+				return 1;
+			}
+			vty_out(iter->vty, "/%s", buffer);
+		}
+		iter->first = false;
+	}
+
+	return YANG_ITER_CONTINUE;
+}
+
 static void nexthop_cli_show(struct vty *vty, const struct lyd_node *route,
 			     const struct lyd_node *src,
 			     const struct lyd_node *path,
@@ -1217,6 +1321,7 @@ static void nexthop_cli_show(struct vty *vty, const struct lyd_node *route,
 	uint32_t tag;
 	uint8_t distance;
 	struct mpls_label_iter iter;
+	struct srv6_seg_iter seg_iter;
 	const char *nexthop_vrf;
 	uint32_t table_id;
 	bool onlink;
@@ -1293,6 +1398,11 @@ static void nexthop_cli_show(struct vty *vty, const struct lyd_node *route,
 	yang_dnode_iterate(mpls_label_iter_cb, &iter, nexthop,
 			   "./mpls-label-stack/entry");
 
+	seg_iter.vty = vty;
+	seg_iter.first = true;
+	yang_dnode_iterate(srv6_seg_iter_cb, &seg_iter, nexthop,
+			   "./srv6-segs-stack/entry");
+
 	nexthop_vrf = yang_dnode_get_string(nexthop, "./vrf");
 	if (strcmp(vrf, nexthop_vrf))
 		vty_out(vty, " nexthop-vrf %s", nexthop_vrf);
@@ -1358,6 +1468,7 @@ int static_nexthop_cli_cmp(const struct lyd_node *dnode1,
 {
 	enum static_nh_type nh_type1, nh_type2;
 	struct prefix prefix1, prefix2;
+	const char *vrf1, *vrf2;
 	int ret = 0;
 
 	nh_type1 = yang_dnode_get_enum(dnode1, "./nh-type");
@@ -1397,8 +1508,14 @@ int static_nexthop_cli_cmp(const struct lyd_node *dnode1,
 	if (ret)
 		return ret;
 
-	return if_cmp_name_func(yang_dnode_get_string(dnode1, "./vrf"),
-				yang_dnode_get_string(dnode2, "./vrf"));
+	vrf1 = yang_dnode_get_string(dnode1, "./vrf");
+	if (strmatch(vrf1, "default"))
+		vrf1 = "";
+	vrf2 = yang_dnode_get_string(dnode2, "./vrf");
+	if (strmatch(vrf2, "default"))
+		vrf2 = "";
+
+	return if_cmp_name_func(vrf1, vrf2);
 }
 
 int static_route_list_cli_cmp(const struct lyd_node *dnode1,
@@ -1456,6 +1573,8 @@ int static_path_list_cli_cmp(const struct lyd_node *dnode1,
 	return (int)distance1 - (int)distance2;
 }
 
+#ifndef INCLUDE_MGMTD_CMDDEFS_ONLY
+
 DEFPY_YANG(debug_staticd, debug_staticd_cmd,
 	   "[no] debug static [{events$events|route$route|bfd$bfd}]",
 	   NO_STR DEBUG_STR STATICD_STR
@@ -1507,9 +1626,17 @@ static struct cmd_node debug_node = {
 	.config_write = static_config_write_debug,
 };
 
+#endif /* ifndef INCLUDE_MGMTD_CMDDEFS_ONLY */
+
 void static_vty_init(void)
 {
+#ifndef INCLUDE_MGMTD_CMDDEFS_ONLY
 	install_node(&debug_node);
+	install_element(ENABLE_NODE, &debug_staticd_cmd);
+	install_element(CONFIG_NODE, &debug_staticd_cmd);
+	install_element(ENABLE_NODE, &show_debugging_static_cmd);
+	install_element(ENABLE_NODE, &staticd_show_bfd_routes_cmd);
+#endif /* ifndef INCLUDE_MGMTD_CMDDEFS_ONLY */
 
 	install_element(CONFIG_NODE, &ip_mroute_dist_cmd);
 
@@ -1527,9 +1654,5 @@ void static_vty_init(void)
 	install_element(CONFIG_NODE, &ipv6_route_cmd);
 	install_element(VRF_NODE, &ipv6_route_vrf_cmd);
 
-	install_element(ENABLE_NODE, &show_debugging_static_cmd);
-	install_element(ENABLE_NODE, &debug_staticd_cmd);
-	install_element(CONFIG_NODE, &debug_staticd_cmd);
-
-	install_element(ENABLE_NODE, &staticd_show_bfd_routes_cmd);
+	mgmt_be_client_lib_vty_init();
 }
