@@ -1,26 +1,12 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
- * This file is part of Quagga.
- *
- * Quagga is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * Quagga is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
 #include <lib/version.h>
 #include "getopt.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "vty.h"
 #include "command.h"
 #include "memory.h"
@@ -28,7 +14,7 @@
 
 extern void test_init(void);
 
-struct thread_master *master;
+struct event_loop *master;
 
 struct option longopts[] = {{"daemon", no_argument, NULL, 'd'},
 			    {"config_file", required_argument, NULL, 'f'},
@@ -47,18 +33,17 @@ DEFUN (daemon_exit,
 }
 
 static int timer_count;
-static int test_timer(struct thread *thread)
+static void test_timer(struct event *thread)
 {
-	int *count = THREAD_ARG(thread);
+	int *count = EVENT_ARG(thread);
 
 	printf("run %d of timer\n", (*count)++);
-	thread_add_timer(master, test_timer, count, 5, NULL);
-	return 0;
+	event_add_timer(master, test_timer, count, 5, NULL);
 }
 
 static void test_timer_init(void)
 {
-	thread_add_timer(master, test_timer, &timer_count, 10, NULL);
+	event_add_timer(master, test_timer, &timer_count, 10, NULL);
 }
 
 static void test_vty_init(void)
@@ -97,7 +82,7 @@ int main(int argc, char **argv)
 	int vty_port = 4000;
 	int daemon_mode = 0;
 	char *progname;
-	struct thread thread;
+	struct event thread;
 	char *config_file = NULL;
 
 	/* Set umask before anything for security */
@@ -107,7 +92,7 @@ int main(int argc, char **argv)
 	progname = ((p = strrchr(argv[0], '/')) ? ++p : argv[0]);
 
 	/* master init. */
-	master = thread_master_create(NULL);
+	master = event_master_create(NULL);
 
 	while (1) {
 		int opt;
@@ -155,8 +140,7 @@ int main(int argc, char **argv)
 	cmd_init(1);
 	vty_init(master, false);
 	lib_cmd_init();
-	yang_init(true);
-	nb_init(master, NULL, 0);
+	nb_init(master, NULL, 0, false);
 
 	/* OSPF vty inits. */
 	test_vty_init();
@@ -168,7 +152,7 @@ int main(int argc, char **argv)
 	}
 
 	/* Create VTY socket */
-	vty_serv_sock(vty_addr, vty_port, "/tmp/.heavy.sock");
+	vty_serv_start(vty_addr, vty_port, "/tmp/.heavy.sock");
 
 	/* Configuration file read*/
 	if (!config_file)
@@ -180,8 +164,8 @@ int main(int argc, char **argv)
 	test_init();
 
 	/* Fetch next active thread. */
-	while (thread_fetch(master, &thread))
-		thread_call(&thread);
+	while (event_fetch(master, &thread))
+		event_call(&thread);
 
 	/* Not reached. */
 	exit(0);
