@@ -1,19 +1,5 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
- * This file is part of Quagga.
- *
- * Quagga is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * Quagga is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -22,7 +8,7 @@
 #include "getopt.h"
 #include "privs.h"
 #include "memory.h"
-#include "memory_vty.h"
+#include "lib_vty.h"
 
 zebra_capabilities_t _caps_p[] = {
 	ZCAP_NET_RAW, ZCAP_BIND, ZCAP_NET_ADMIN, ZCAP_DAC_OVERRIDE,
@@ -37,7 +23,7 @@ struct zebra_privs_t test_privs = {
 	.vty_group = VTY_GROUP,
 #endif
 	.caps_p = _caps_p,
-	.cap_num_p = sizeof(_caps_p) / sizeof(_caps_p[0]),
+	.cap_num_p = array_size(_caps_p),
 	.cap_num_i = 0};
 
 struct option longopts[] = {{"help", no_argument, NULL, 'h'},
@@ -64,7 +50,7 @@ Report bugs to %s\n",
 	exit(status);
 }
 
-struct thread_master *master;
+struct event_loop *master;
 /* main routine. */
 int main(int argc, char **argv)
 {
@@ -105,7 +91,7 @@ int main(int argc, char **argv)
 	}
 
 	/* Library inits. */
-	memory_init();
+	lib_cmd_init();
 	zprivs_preinit(&test_privs);
 	zprivs_init(&test_privs);
 
@@ -113,10 +99,9 @@ int main(int argc, char **argv)
 	((test_privs.current_state() == ZPRIVS_RAISED) ? "Raised" : "Lowered")
 
 	printf("%s\n", PRIV_STATE());
-	test_privs.change(ZPRIVS_RAISE);
-
-	printf("%s\n", PRIV_STATE());
-	test_privs.change(ZPRIVS_LOWER);
+	frr_with_privs(&test_privs) {
+		printf("%s\n", PRIV_STATE());
+	}
 
 	printf("%s\n", PRIV_STATE());
 	zprivs_get_ids(&ids);
@@ -126,10 +111,9 @@ int main(int argc, char **argv)
 
 	/* but these should continue to work... */
 	printf("%s\n", PRIV_STATE());
-	test_privs.change(ZPRIVS_RAISE);
-
-	printf("%s\n", PRIV_STATE());
-	test_privs.change(ZPRIVS_LOWER);
+	frr_with_privs(&test_privs) {
+		printf("%s\n", PRIV_STATE());
+	}
 
 	printf("%s\n", PRIV_STATE());
 	zprivs_get_ids(&ids);
