@@ -1,22 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Interface looking up by ioctl ().
  * Copyright (C) 1997, 98 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -29,13 +14,15 @@
 #include "ioctl.h"
 #include "connected.h"
 #include "memory.h"
-#include "zebra_memory.h"
 #include "log.h"
 #include "vrf.h"
 #include "vty.h"
+#include "lib_errors.h"
 
 #include "zebra/interface.h"
 #include "zebra/rib.h"
+#include "zebra/rt.h"
+#include "zebra/zebra_errors.h"
 
 #include <ifaddrs.h>
 
@@ -55,8 +42,9 @@ static int interface_list_ioctl(void)
 	/* Normally SIOCGIFCONF works with AF_INET socket. */
 	sock = socket(AF_INET, SOCK_DGRAM, 0);
 	if (sock < 0) {
-		zlog_warn("Can't make AF_INET socket stream: %s",
-			  safe_strerror(errno));
+		flog_err_sys(EC_LIB_SOCKET,
+			     "Can't make AF_INET socket stream: %s",
+			     safe_strerror(errno));
 		return -1;
 	}
 
@@ -84,7 +72,8 @@ static int interface_list_ioctl(void)
 		ret = ioctl(sock, SIOCGIFCONF, &ifconf);
 
 		if (ret < 0) {
-			zlog_warn("SIOCGIFCONF: %s", safe_strerror(errno));
+			flog_err_sys(EC_LIB_SYSTEM_CALL, "SIOCGIFCONF: %s",
+				     safe_strerror(errno));
 			goto end;
 		}
 		/* Repeatedly get info til buffer fails to grow. */
@@ -105,7 +94,8 @@ static int interface_list_ioctl(void)
 		unsigned int size;
 
 		ifreq = (struct ifreq *)((caddr_t)ifconf.ifc_req + n);
-		ifp = if_get_by_name(ifreq->ifr_name, VRF_DEFAULT, 0);
+		ifp = if_get_by_name(ifreq->ifr_name, VRF_DEFAULT,
+				     VRF_DEFAULT_NAME);
 		if_add_update(ifp);
 		size = ifreq->ifr_addr.sa_len;
 		if (size < sizeof(ifreq->ifr_addr))
@@ -115,7 +105,8 @@ static int interface_list_ioctl(void)
 	}
 #else
 	for (n = 0; n < ifconf.ifc_len; n += sizeof(struct ifreq)) {
-		ifp = if_get_by_name(ifreq->ifr_name, VRF_DEFAULT, 0);
+		ifp = if_get_by_name(ifreq->ifr_name, VRF_DEFAULT,
+				     VRF_DEFAULT_NAME);
 		if_add_update(ifp);
 		ifreq++;
 	}
@@ -142,11 +133,11 @@ static int if_get_hwaddr(struct interface *ifp)
 	struct ifreq ifreq;
 	int i;
 
-	strncpy(ifreq.ifr_name, ifp->name, IFNAMSIZ);
+	strlcpy(ifreq.ifr_name, ifp->name, sizeof(ifreq.ifr_name));
 	ifreq.ifr_addr.sa_family = AF_INET;
 
 	/* Fetch Hardware address if available. */
-	ret = vrf_if_ioctl(SIOCGIFHWADDR, (caddr_t)&ifreq, ifp->vrf_id);
+	ret = vrf_if_ioctl(SIOCGIFHWADDR, (caddr_t)&ifreq, ifp->vrf->vrf_id);
 	if (ret < 0)
 		ifp->hw_addr_len = 0;
 	else {
@@ -175,13 +166,15 @@ static int if_getaddrs(void)
 
 	ret = getifaddrs(&ifap);
 	if (ret != 0) {
-		zlog_err("getifaddrs(): %s", safe_strerror(errno));
+		flog_err_sys(EC_LIB_SYSTEM_CALL, "getifaddrs(): %s",
+			     safe_strerror(errno));
 		return -1;
 	}
 
 	for (ifapfree = ifap; ifap; ifap = ifap->ifa_next) {
 		if (ifap->ifa_addr == NULL) {
-			zlog_err(
+			flog_err(
+				EC_LIB_INTERFACE,
 				"%s: nonsensical ifaddr with NULL ifa_addr, ifname %s",
 				__func__,
 				(ifap->ifa_name ? ifap->ifa_name : "(null)"));
@@ -190,7 +183,8 @@ static int if_getaddrs(void)
 
 		ifp = if_lookup_by_name(ifap->ifa_name, VRF_DEFAULT);
 		if (ifp == NULL) {
-			zlog_err("if_getaddrs(): Can't lookup interface %s\n",
+			flog_err(EC_LIB_INTERFACE,
+				 "%s: Can't lookup interface %s", __func__,
 				 ifap->ifa_name);
 			continue;
 		}
@@ -228,7 +222,8 @@ static int if_getaddrs(void)
 			}
 
 			connected_add_ipv4(ifp, flags, &addr->sin_addr,
-					   prefixlen, dest_pnt, NULL);
+					   prefixlen, dest_pnt, NULL,
+					   METRIC_MAX);
 		}
 		if (ifap->ifa_addr->sa_family == AF_INET6) {
 			struct sockaddr_in6 *addr;
@@ -250,7 +245,7 @@ static int if_getaddrs(void)
 #endif
 
 			connected_add_ipv6(ifp, flags, &addr->sin6_addr, NULL,
-					   prefixlen, NULL);
+					   prefixlen, NULL, METRIC_MAX);
 		}
 	}
 
@@ -280,7 +275,7 @@ static void interface_info_ioctl()
 void interface_list(struct zebra_ns *zns)
 {
 
-	zlog_info("interface_list: NS %u", zns->ns_id);
+	zlog_info("%s: NS %u", __func__, zns->ns_id);
 
 /* Linux can do both proc & ioctl, ioctl is the only way to get
    interface aliases in 2.2 series kernels. */
