@@ -1,22 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Common ioctl functions.
  * Copyright (C) 1997, 98 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -27,13 +12,14 @@
 #include "ioctl.h"
 #include "log.h"
 #include "privs.h"
+#include "lib_errors.h"
 
 #include "vty.h"
 #include "zebra/rib.h"
 #include "zebra/rt.h"
 #include "zebra/interface.h"
-
-#ifndef SUNOS_5
+#include "zebra/zebra_errors.h"
+#include "zebra/debug.h"
 
 #ifdef HAVE_BSD_LINK_DETECT
 #include <net/if_media.h>
@@ -47,6 +33,7 @@ void ifreq_set_name(struct ifreq *ifreq, struct interface *ifp)
 	strlcpy(ifreq->ifr_name, ifp->name, sizeof(ifreq->ifr_name));
 }
 
+#ifndef HAVE_NETLINK
 /* call ioctl system call */
 int if_ioctl(unsigned long request, caddr_t buffer)
 {
@@ -54,22 +41,16 @@ int if_ioctl(unsigned long request, caddr_t buffer)
 	int ret;
 	int err = 0;
 
-	if (zserv_privs.change(ZPRIVS_RAISE))
-		zlog_err("Can't raise privileges");
-	sock = socket(AF_INET, SOCK_DGRAM, 0);
-	if (sock < 0) {
-		int save_errno = errno;
-
-		if (zserv_privs.change(ZPRIVS_LOWER))
-			zlog_err("Can't lower privileges");
-		zlog_err("Cannot create UDP socket: %s",
-			 safe_strerror(save_errno));
-		exit(1);
+	frr_with_privs(&zserv_privs) {
+		sock = socket(AF_INET, SOCK_DGRAM, 0);
+		if (sock < 0) {
+			zlog_err("Cannot create UDP socket: %s",
+				 safe_strerror(errno));
+			exit(1);
+		}
+		if ((ret = ioctl(sock, request, buffer)) < 0)
+			err = errno;
 	}
-	if ((ret = ioctl(sock, request, buffer)) < 0)
-		err = errno;
-	if (zserv_privs.change(ZPRIVS_LOWER))
-		zlog_err("Can't lower privileges");
 	close(sock);
 
 	if (ret < 0) {
@@ -78,6 +59,7 @@ int if_ioctl(unsigned long request, caddr_t buffer)
 	}
 	return 0;
 }
+#endif
 
 /* call ioctl system call */
 int vrf_if_ioctl(unsigned long request, caddr_t buffer, vrf_id_t vrf_id)
@@ -86,23 +68,17 @@ int vrf_if_ioctl(unsigned long request, caddr_t buffer, vrf_id_t vrf_id)
 	int ret;
 	int err = 0;
 
-	if (zserv_privs.change(ZPRIVS_RAISE))
-		zlog_err("Can't raise privileges");
-	sock = vrf_socket(AF_INET, SOCK_DGRAM, 0, vrf_id, NULL);
-	if (sock < 0) {
-		int save_errno = errno;
-
-		if (zserv_privs.change(ZPRIVS_LOWER))
-			zlog_err("Can't lower privileges");
-		zlog_err("Cannot create UDP socket: %s",
-			 safe_strerror(save_errno));
-		exit(1);
+	frr_with_privs(&zserv_privs) {
+		sock = vrf_socket(AF_INET, SOCK_DGRAM, 0, vrf_id, NULL);
+		if (sock < 0) {
+			zlog_err("Cannot create UDP socket: %s",
+				 safe_strerror(errno));
+			exit(1);
+		}
+		ret = vrf_ioctl(vrf_id, sock, request, buffer);
+		if (ret < 0)
+			err = errno;
 	}
-	ret = vrf_ioctl(vrf_id, sock, request, buffer);
-	if (ret < 0)
-		err = errno;
-	if (zserv_privs.change(ZPRIVS_LOWER))
-		zlog_err("Can't lower privileges");
 	close(sock);
 
 	if (ret < 0) {
@@ -119,23 +95,17 @@ static int if_ioctl_ipv6(unsigned long request, caddr_t buffer)
 	int ret;
 	int err = 0;
 
-	if (zserv_privs.change(ZPRIVS_RAISE))
-		zlog_err("Can't raise privileges");
-	sock = socket(AF_INET6, SOCK_DGRAM, 0);
-	if (sock < 0) {
-		int save_errno = errno;
+	frr_with_privs(&zserv_privs) {
+		sock = socket(AF_INET6, SOCK_DGRAM, 0);
+		if (sock < 0) {
+			zlog_err("Cannot create IPv6 datagram socket: %s",
+				 safe_strerror(errno));
+			exit(1);
+		}
 
-		if (zserv_privs.change(ZPRIVS_LOWER))
-			zlog_err("Can't lower privileges");
-		zlog_err("Cannot create IPv6 datagram socket: %s",
-			 safe_strerror(save_errno));
-		exit(1);
+		if ((ret = ioctl(sock, request, buffer)) < 0)
+			err = errno;
 	}
-
-	if ((ret = ioctl(sock, request, buffer)) < 0)
-		err = errno;
-	if (zserv_privs.change(ZPRIVS_LOWER))
-		zlog_err("Can't lower privileges");
 	close(sock);
 
 	if (ret < 0) {
@@ -144,7 +114,6 @@ static int if_ioctl_ipv6(unsigned long request, caddr_t buffer)
 	}
 	return 0;
 }
-#endif /* ! HAVE_NETLINK */
 
 /*
  * get interface metric
@@ -153,11 +122,11 @@ static int if_ioctl_ipv6(unsigned long request, caddr_t buffer)
 void if_get_metric(struct interface *ifp)
 {
 #ifdef SIOCGIFMETRIC
-	struct ifreq ifreq;
+	struct ifreq ifreq = {};
 
 	ifreq_set_name(&ifreq, ifp);
 
-	if (vrf_if_ioctl(SIOCGIFMETRIC, (caddr_t)&ifreq, ifp->vrf_id) < 0)
+	if (vrf_if_ioctl(SIOCGIFMETRIC, (caddr_t)&ifreq, ifp->vrf->vrf_id) < 0)
 		return;
 	ifp->metric = ifreq.ifr_metric;
 	if (ifp->metric == 0)
@@ -170,68 +139,92 @@ void if_get_metric(struct interface *ifp)
 /* get interface MTU */
 void if_get_mtu(struct interface *ifp)
 {
-	struct ifreq ifreq;
+	struct ifreq ifreq = {};
 
 	ifreq_set_name(&ifreq, ifp);
 
 #if defined(SIOCGIFMTU)
-	if (vrf_if_ioctl(SIOCGIFMTU, (caddr_t)&ifreq, ifp->vrf_id) < 0) {
-		zlog_info("Can't lookup mtu by ioctl(SIOCGIFMTU)");
+	if (vrf_if_ioctl(SIOCGIFMTU, (caddr_t)&ifreq, ifp->vrf->vrf_id) < 0) {
+		zlog_info("Can't lookup mtu by ioctl(SIOCGIFMTU) for %s(%u)",
+			  ifp->name, ifp->vrf->vrf_id);
 		ifp->mtu6 = ifp->mtu = -1;
 		return;
 	}
 
-#ifdef SUNOS_5
-	ifp->mtu6 = ifp->mtu = ifreq.ifr_metric;
-#else
 	ifp->mtu6 = ifp->mtu = ifreq.ifr_mtu;
-#endif /* SUNOS_5 */
 
 	/* propogate */
 	zebra_interface_up_update(ifp);
 
 #else
-	zlog_info("Can't lookup mtu on this system");
+	zlog_info("Can't lookup mtu on this system for %s(%u)", ifp->name,
+		  ifp->vrf->vrf_id);
 	ifp->mtu6 = ifp->mtu = -1;
 #endif
 }
+#endif /* ! HAVE_NETLINK */
 
-#ifdef HAVE_NETLINK
-/* Interface address setting via netlink interface. */
-int if_set_prefix(struct interface *ifp, struct connected *ifc)
+/*
+ * Handler for interface address programming via the zebra dplane,
+ * for non-netlink platforms. This handler dispatches to per-platform
+ * helpers, based on the operation requested.
+ */
+#ifndef HAVE_NETLINK
+
+/* Prototypes: these are placed in this block so that they're only seen
+ * on non-netlink platforms.
+ */
+static int if_set_prefix_ctx(const struct zebra_dplane_ctx *ctx);
+static int if_unset_prefix_ctx(const struct zebra_dplane_ctx *ctx);
+static int if_set_prefix6_ctx(const struct zebra_dplane_ctx *ctx);
+static int if_unset_prefix6_ctx(const struct zebra_dplane_ctx *ctx);
+
+enum zebra_dplane_result kernel_address_update_ctx(
+	struct zebra_dplane_ctx *ctx)
 {
-	return kernel_address_add_ipv4(ifp, ifc);
+	int ret = -1;
+	const struct prefix *p;
+
+	p = dplane_ctx_get_intf_addr(ctx);
+
+	if (dplane_ctx_get_op(ctx) == DPLANE_OP_ADDR_INSTALL) {
+		if (p->family == AF_INET)
+			ret = if_set_prefix_ctx(ctx);
+		else
+			ret = if_set_prefix6_ctx(ctx);
+	} else if (dplane_ctx_get_op(ctx) == DPLANE_OP_ADDR_UNINSTALL) {
+		if (p->family == AF_INET)
+			ret = if_unset_prefix_ctx(ctx);
+		else
+			ret = if_unset_prefix6_ctx(ctx);
+	} else {
+		if (IS_ZEBRA_DEBUG_DPLANE)
+			zlog_debug("Invalid op in interface-addr install");
+	}
+
+	return (ret == 0 ?
+		ZEBRA_DPLANE_REQUEST_SUCCESS : ZEBRA_DPLANE_REQUEST_FAILURE);
 }
 
-/* Interface address is removed using netlink interface. */
-int if_unset_prefix(struct interface *ifp, struct connected *ifc)
-{
-	return kernel_address_delete_ipv4(ifp, ifc);
-}
-#else /* ! HAVE_NETLINK */
 #ifdef HAVE_STRUCT_IFALIASREQ
-/* Set up interface's IP address, netmask (and broadcas? ).  *BSD may
-   has ifaliasreq structure.  */
-int if_set_prefix(struct interface *ifp, struct connected *ifc)
+
+/*
+ * Helper for interface-addr install, non-netlink
+ */
+static int if_set_prefix_ctx(const struct zebra_dplane_ctx *ctx)
 {
 	int ret;
 	struct ifaliasreq addreq;
 	struct sockaddr_in addr, mask, peer;
 	struct prefix_ipv4 *p;
 
-	/* don't configure PtP addresses on broadcast ifs or reverse */
-	if (!(ifp->flags & IFF_POINTOPOINT) != !CONNECTED_PEER(ifc)) {
-		errno = EINVAL;
-		return -1;
-	}
+	p = (struct prefix_ipv4 *)dplane_ctx_get_intf_addr(ctx);
 
-	p = (struct prefix_ipv4 *)ifc->address;
-	rib_lookup_and_pushup(p, ifp->vrf_id);
+	memset(&addreq, 0, sizeof(addreq));
+	strlcpy((char *)&addreq.ifra_name, dplane_ctx_get_ifname(ctx),
+		sizeof(addreq.ifra_name));
 
-	memset(&addreq, 0, sizeof addreq);
-	strncpy((char *)&addreq.ifra_name, ifp->name, sizeof addreq.ifra_name);
-
-	memset(&addr, 0, sizeof(struct sockaddr_in));
+	memset(&addr, 0, sizeof(addr));
 	addr.sin_addr = p->prefix;
 	addr.sin_family = p->family;
 #ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
@@ -239,9 +232,9 @@ int if_set_prefix(struct interface *ifp, struct connected *ifc)
 #endif
 	memcpy(&addreq.ifra_addr, &addr, sizeof(struct sockaddr_in));
 
-	if (CONNECTED_PEER(ifc)) {
-		p = (struct prefix_ipv4 *)ifc->destination;
-		memset(&mask, 0, sizeof(struct sockaddr_in));
+	if (dplane_ctx_intf_is_connected(ctx)) {
+		p = (struct prefix_ipv4 *)dplane_ctx_get_intf_dest(ctx);
+		memset(&mask, 0, sizeof(mask));
 		peer.sin_addr = p->prefix;
 		peer.sin_family = p->family;
 #ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
@@ -251,7 +244,7 @@ int if_set_prefix(struct interface *ifp, struct connected *ifc)
 		       sizeof(struct sockaddr_in));
 	}
 
-	memset(&mask, 0, sizeof(struct sockaddr_in));
+	memset(&mask, 0, sizeof(mask));
 	masklen2ip(p->prefixlen, &mask.sin_addr);
 	mask.sin_family = p->family;
 #ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
@@ -263,29 +256,26 @@ int if_set_prefix(struct interface *ifp, struct connected *ifc)
 	if (ret < 0)
 		return ret;
 	return 0;
+
 }
 
-/* Set up interface's IP address, netmask (and broadcas? ).  *BSD may
-   has ifaliasreq structure.  */
-int if_unset_prefix(struct interface *ifp, struct connected *ifc)
+/*
+ * Helper for interface-addr un-install, non-netlink
+ */
+static int if_unset_prefix_ctx(const struct zebra_dplane_ctx *ctx)
 {
 	int ret;
 	struct ifaliasreq addreq;
 	struct sockaddr_in addr, mask, peer;
 	struct prefix_ipv4 *p;
 
-	/* this would probably wreak havoc */
-	if (!(ifp->flags & IFF_POINTOPOINT) != !CONNECTED_PEER(ifc)) {
-		errno = EINVAL;
-		return -1;
-	}
+	p = (struct prefix_ipv4 *)dplane_ctx_get_intf_addr(ctx);
 
-	p = (struct prefix_ipv4 *)ifc->address;
+	memset(&addreq, 0, sizeof(addreq));
+	strlcpy((char *)&addreq.ifra_name, dplane_ctx_get_ifname(ctx),
+		sizeof(addreq.ifra_name));
 
-	memset(&addreq, 0, sizeof addreq);
-	strncpy((char *)&addreq.ifra_name, ifp->name, sizeof addreq.ifra_name);
-
-	memset(&addr, 0, sizeof(struct sockaddr_in));
+	memset(&addr, 0, sizeof(addr));
 	addr.sin_addr = p->prefix;
 	addr.sin_family = p->family;
 #ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
@@ -293,9 +283,9 @@ int if_unset_prefix(struct interface *ifp, struct connected *ifc)
 #endif
 	memcpy(&addreq.ifra_addr, &addr, sizeof(struct sockaddr_in));
 
-	if (CONNECTED_PEER(ifc)) {
-		p = (struct prefix_ipv4 *)ifc->destination;
-		memset(&mask, 0, sizeof(struct sockaddr_in));
+	if (dplane_ctx_intf_is_connected(ctx)) {
+		p = (struct prefix_ipv4 *)dplane_ctx_get_intf_dest(ctx);
+		memset(&mask, 0, sizeof(mask));
 		peer.sin_addr = p->prefix;
 		peer.sin_family = p->family;
 #ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
@@ -305,7 +295,7 @@ int if_unset_prefix(struct interface *ifp, struct connected *ifc)
 		       sizeof(struct sockaddr_in));
 	}
 
-	memset(&mask, 0, sizeof(struct sockaddr_in));
+	memset(&mask, 0, sizeof(mask));
 	masklen2ip(p->prefixlen, &mask.sin_addr);
 	mask.sin_family = p->family;
 #ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
@@ -321,7 +311,7 @@ int if_unset_prefix(struct interface *ifp, struct connected *ifc)
 #else
 /* Set up interface's address, netmask (and broadcas? ).  Linux or
    Solaris uses ifname:number semantics to set IP address aliases. */
-int if_set_prefix(struct interface *ifp, struct connected *ifc)
+int if_set_prefix_ctx(const struct zebra_dplane_ctx *ctx)
 {
 	int ret;
 	struct ifreq ifreq;
@@ -331,11 +321,12 @@ int if_set_prefix(struct interface *ifp, struct connected *ifc)
 	struct prefix_ipv4 ifaddr;
 	struct prefix_ipv4 *p;
 
-	p = (struct prefix_ipv4 *)ifc->address;
+	p = (struct prefix_ipv4 *)dplane_ctx_get_intf_addr(ctx);
 
 	ifaddr = *p;
 
-	ifreq_set_name(&ifreq, ifp);
+	strlcpy(ifreq.ifr_name, dplane_ctx_get_ifname(ctx),
+		sizeof(ifreq.ifr_name));
 
 	addr.sin_addr = p->prefix;
 	addr.sin_family = p->family;
@@ -347,7 +338,7 @@ int if_set_prefix(struct interface *ifp, struct connected *ifc)
 	/* We need mask for make broadcast addr. */
 	masklen2ip(p->prefixlen, &mask.sin_addr);
 
-	if (if_is_broadcast(ifp)) {
+	if (dplane_ctx_intf_is_broadcast(ctx)) {
 		apply_mask_ipv4(&ifaddr);
 		addr.sin_addr = ifaddr.prefix;
 
@@ -363,11 +354,7 @@ int if_set_prefix(struct interface *ifp, struct connected *ifc)
 	}
 
 	mask.sin_family = p->family;
-#ifdef SUNOS_5
-	memcpy(&mask, &ifreq.ifr_addr, sizeof(mask));
-#else
-	memcpy(&ifreq.ifr_netmask, &mask, sizeof(struct sockaddr_in));
-#endif /* SUNOS5 */
+	memcpy(&ifreq.ifr_addr, &mask, sizeof(struct sockaddr_in));
 	ret = if_ioctl(SIOCSIFNETMASK, (caddr_t)&ifreq);
 	if (ret < 0)
 		return ret;
@@ -377,18 +364,19 @@ int if_set_prefix(struct interface *ifp, struct connected *ifc)
 
 /* Set up interface's address, netmask (and broadcas? ).  Linux or
    Solaris uses ifname:number semantics to set IP address aliases. */
-int if_unset_prefix(struct interface *ifp, struct connected *ifc)
+int if_unset_prefix_ctx(const struct zebra_dplane_ctx *ctx)
 {
 	int ret;
 	struct ifreq ifreq;
 	struct sockaddr_in addr;
 	struct prefix_ipv4 *p;
 
-	p = (struct prefix_ipv4 *)ifc->address;
+	p = (struct prefix_ipv4 *)dplane_ctx_get_intf_addr(ctx);
 
-	ifreq_set_name(&ifreq, ifp);
+	strlcpy(ifreq.ifr_name, dplane_ctx_get_ifname(ctx),
+		sizeof(ifreq.ifr_name));
 
-	memset(&addr, 0, sizeof(struct sockaddr_in));
+	memset(&addr, 0, sizeof(addr));
 	addr.sin_family = p->family;
 	memcpy(&ifreq.ifr_addr, &addr, sizeof(struct sockaddr_in));
 	ret = if_ioctl(SIOCSIFADDR, (caddr_t)&ifreq);
@@ -404,46 +392,96 @@ int if_unset_prefix(struct interface *ifp, struct connected *ifc)
 void if_get_flags(struct interface *ifp)
 {
 	int ret;
-	struct ifreq ifreq;
-#ifdef HAVE_BSD_LINK_DETECT
-	struct ifmediareq ifmr;
-#endif /* HAVE_BSD_LINK_DETECT */
+	struct ifreq ifreqflags = {};
+	struct ifreq ifreqdata = {};
 
-	ifreq_set_name(&ifreq, ifp);
+	ifreq_set_name(&ifreqflags, ifp);
+	ifreq_set_name(&ifreqdata, ifp);
 
-	ret = vrf_if_ioctl(SIOCGIFFLAGS, (caddr_t)&ifreq, ifp->vrf_id);
+	ret = vrf_if_ioctl(SIOCGIFFLAGS, (caddr_t)&ifreqflags,
+			   ifp->vrf->vrf_id);
 	if (ret < 0) {
-		zlog_err("vrf_if_ioctl(SIOCGIFFLAGS) failed: %s",
-			 safe_strerror(errno));
+		flog_err_sys(EC_LIB_SYSTEM_CALL,
+			     "vrf_if_ioctl(SIOCGIFFLAGS %s) failed: %s",
+			     ifp->name, safe_strerror(errno));
 		return;
 	}
-#ifdef HAVE_BSD_LINK_DETECT /* Detect BSD link-state at start-up */
 
-	/* Per-default, IFF_RUNNING is held high, unless link-detect says
-	 * otherwise - we abuse IFF_RUNNING inside zebra as a link-state flag,
-	 * following practice on Linux and Solaris kernels
+	if (!CHECK_FLAG(ifp->status, ZEBRA_INTERFACE_LINKDETECTION))
+		goto out;
+
+	/* Per-default, IFF_RUNNING is held high, unless link-detect
+	 * says otherwise - we abuse IFF_RUNNING inside zebra as a
+	 * link-state flag, following practice on Linux and Solaris
+	 * kernels
 	 */
-	SET_FLAG(ifreq.ifr_flags, IFF_RUNNING);
 
-	if (CHECK_FLAG(ifp->status, ZEBRA_INTERFACE_LINKDETECTION)) {
-		(void)memset(&ifmr, 0, sizeof(ifmr));
-		strncpy(ifmr.ifm_name, ifp->name, IFNAMSIZ);
+#ifdef SIOCGIFDATA
+	/*
+	 * BSD gets link state from ifi_link_link in struct if_data.
+	 * All BSD's have this in getifaddrs(3) ifa_data for AF_LINK
+	 * addresses. We can also access it via SIOCGIFDATA.
+	 */
 
-		/* Seems not all interfaces implement this ioctl */
-		if (if_ioctl(SIOCGIFMEDIA, (caddr_t)&ifmr) < 0)
-			zlog_err("if_ioctl(SIOCGIFMEDIA) failed: %s",
-				 safe_strerror(errno));
-		else if (ifmr.ifm_status & IFM_AVALID) /* Link state is valid */
-		{
-			if (ifmr.ifm_status & IFM_ACTIVE)
-				SET_FLAG(ifreq.ifr_flags, IFF_RUNNING);
-			else
-				UNSET_FLAG(ifreq.ifr_flags, IFF_RUNNING);
-		}
+#ifdef __NetBSD__
+	struct ifdatareq ifdr = {.ifdr_data.ifi_link_state = 0};
+	struct if_data *ifdata = &ifdr.ifdr_data;
+
+	strlcpy(ifdr.ifdr_name, ifp->name, sizeof(ifdr.ifdr_name));
+	ret = vrf_if_ioctl(SIOCGIFDATA, (caddr_t)&ifdr, ifp->vrf->vrf_id);
+#else
+	struct if_data ifd = {.ifi_link_state = 0};
+	struct if_data *ifdata = &ifd;
+
+	ifreqdata.ifr_data = (caddr_t)ifdata;
+	ret = vrf_if_ioctl(SIOCGIFDATA, (caddr_t)&ifreqdata, ifp->vrf->vrf_id);
+#endif
+
+	if (ret == -1)
+		/* Very unlikely. Did the interface disappear? */
+		flog_err_sys(EC_LIB_SYSTEM_CALL,
+			     "if_ioctl(SIOCGIFDATA %s) failed: %s", ifp->name,
+			     safe_strerror(errno));
+	else {
+		if (ifdata->ifi_link_state >= LINK_STATE_UP)
+			SET_FLAG(ifreqflags.ifr_flags, IFF_RUNNING);
+		else if (ifdata->ifi_link_state == LINK_STATE_UNKNOWN)
+			/* BSD traditionally treats UNKNOWN as UP */
+			SET_FLAG(ifreqflags.ifr_flags, IFF_RUNNING);
+		else
+			UNSET_FLAG(ifreqflags.ifr_flags, IFF_RUNNING);
+	}
+
+#elif defined(HAVE_BSD_LINK_DETECT)
+	/*
+	 * This is only needed for FreeBSD older than FreeBSD-13.
+	 * Valid and active media generally means the link state is
+	 * up, but this is not always the case.
+	 * For example, some BSD's with a net80211 interface in MONITOR
+	 * mode will treat the media as valid and active but the
+	 * link state is down - because we cannot send anything.
+	 * Also, virtual interfaces such as PPP, VLAN, etc generally
+	 * don't support media at all, so the ioctl will just fail.
+	 */
+	struct ifmediareq ifmr = {.ifm_status = 0};
+
+	strlcpy(ifmr.ifm_name, ifp->name, sizeof(ifmr.ifm_name));
+
+	if (if_ioctl(SIOCGIFMEDIA, (caddr_t)&ifmr) == -1) {
+		if (errno != EINVAL)
+			flog_err_sys(EC_LIB_SYSTEM_CALL,
+				     "if_ioctl(SIOCGIFMEDIA %s) failed: %s",
+				     ifp->name, safe_strerror(errno));
+	} else if (ifmr.ifm_status & IFM_AVALID) { /* media state is valid */
+		if (ifmr.ifm_status & IFM_ACTIVE)  /* media is active */
+			SET_FLAG(ifreqflags.ifr_flags, IFF_RUNNING);
+		else
+			UNSET_FLAG(ifreqflags.ifr_flags, IFF_RUNNING);
 	}
 #endif /* HAVE_BSD_LINK_DETECT */
 
-	if_flags_update(ifp, (ifreq.ifr_flags & 0x0000ffff));
+out:
+	if_flags_update(ifp, (ifreqflags.ifr_flags & 0x0000ffff));
 }
 
 /* Set interface flags */
@@ -452,16 +490,17 @@ int if_set_flags(struct interface *ifp, uint64_t flags)
 	int ret;
 	struct ifreq ifreq;
 
-	memset(&ifreq, 0, sizeof(struct ifreq));
+	memset(&ifreq, 0, sizeof(ifreq));
 	ifreq_set_name(&ifreq, ifp);
 
 	ifreq.ifr_flags = ifp->flags;
 	ifreq.ifr_flags |= flags;
 
-	ret = vrf_if_ioctl(SIOCSIFFLAGS, (caddr_t)&ifreq, ifp->vrf_id);
+	ret = vrf_if_ioctl(SIOCSIFFLAGS, (caddr_t)&ifreq, ifp->vrf->vrf_id);
 
 	if (ret < 0) {
-		zlog_info("can't set interface flags");
+		zlog_info("can't set interface %s(%u) flags %" PRIu64,
+			  ifp->name, ifp->vrf->vrf_id, flags);
 		return ret;
 	}
 	return 0;
@@ -473,50 +512,33 @@ int if_unset_flags(struct interface *ifp, uint64_t flags)
 	int ret;
 	struct ifreq ifreq;
 
-	memset(&ifreq, 0, sizeof(struct ifreq));
+	memset(&ifreq, 0, sizeof(ifreq));
 	ifreq_set_name(&ifreq, ifp);
 
 	ifreq.ifr_flags = ifp->flags;
 	ifreq.ifr_flags &= ~flags;
 
-	ret = vrf_if_ioctl(SIOCSIFFLAGS, (caddr_t)&ifreq, ifp->vrf_id);
+	ret = vrf_if_ioctl(SIOCSIFFLAGS, (caddr_t)&ifreq, ifp->vrf->vrf_id);
 
 	if (ret < 0) {
-		zlog_info("can't unset interface flags");
+		zlog_warn("can't unset interface %s(%u) flags %" PRIu64,
+			  ifp->name, ifp->vrf->vrf_id, flags);
 		return ret;
 	}
 	return 0;
 }
 
-#ifdef LINUX_IPV6
-#ifndef _LINUX_IN6_H
-/* linux/include/net/ipv6.h */
-struct in6_ifreq {
-	struct in6_addr ifr6_addr;
-	uint32_t ifr6_prefixlen;
-	int ifr6_ifindex;
-};
-#endif /* _LINUX_IN6_H */
-/* Interface's address add/delete functions. */
-int if_prefix_add_ipv6(struct interface *ifp, struct connected *ifc)
-{
-#ifdef HAVE_NETLINK
-	return kernel_address_add_ipv6(ifp, ifc);
-#endif /* HAVE_NETLINK */
-}
+#ifndef LINUX_IPV6 /* Netlink has its own code */
 
-int if_prefix_delete_ipv6(struct interface *ifp, struct connected *ifc)
-{
-#ifdef HAVE_NETLINK
-	return kernel_address_delete_ipv6(ifp, ifc);
-#endif /* HAVE_NETLINK */
-}
-#else /* LINUX_IPV6 */
 #ifdef HAVE_STRUCT_IN6_ALIASREQ
 #ifndef ND6_INFINITE_LIFETIME
 #define ND6_INFINITE_LIFETIME 0xffffffffL
 #endif /* ND6_INFINITE_LIFETIME */
-int if_prefix_add_ipv6(struct interface *ifp, struct connected *ifc)
+
+/*
+ * Helper for interface-addr install, non-netlink
+ */
+static int if_set_prefix6_ctx(const struct zebra_dplane_ctx *ctx)
 {
 	int ret;
 	struct in6_aliasreq addreq;
@@ -524,12 +546,13 @@ int if_prefix_add_ipv6(struct interface *ifp, struct connected *ifc)
 	struct sockaddr_in6 mask;
 	struct prefix_ipv6 *p;
 
-	p = (struct prefix_ipv6 *)ifc->address;
+	p = (struct prefix_ipv6 *)dplane_ctx_get_intf_addr(ctx);
 
-	memset(&addreq, 0, sizeof addreq);
-	strncpy((char *)&addreq.ifra_name, ifp->name, sizeof addreq.ifra_name);
+	memset(&addreq, 0, sizeof(addreq));
+	strlcpy((char *)&addreq.ifra_name,
+		dplane_ctx_get_ifname(ctx), sizeof(addreq.ifra_name));
 
-	memset(&addr, 0, sizeof(struct sockaddr_in6));
+	memset(&addr, 0, sizeof(addr));
 	addr.sin6_addr = p->prefix;
 	addr.sin6_family = p->family;
 #ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
@@ -537,7 +560,7 @@ int if_prefix_add_ipv6(struct interface *ifp, struct connected *ifc)
 #endif
 	memcpy(&addreq.ifra_addr, &addr, sizeof(struct sockaddr_in6));
 
-	memset(&mask, 0, sizeof(struct sockaddr_in6));
+	memset(&mask, 0, sizeof(mask));
 	masklen2ip6(p->prefixlen, &mask.sin6_addr);
 	mask.sin6_family = p->family;
 #ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
@@ -559,7 +582,10 @@ int if_prefix_add_ipv6(struct interface *ifp, struct connected *ifc)
 	return 0;
 }
 
-int if_prefix_delete_ipv6(struct interface *ifp, struct connected *ifc)
+/*
+ * Helper for interface-addr un-install, non-netlink
+ */
+static int if_unset_prefix6_ctx(const struct zebra_dplane_ctx *ctx)
 {
 	int ret;
 	struct in6_aliasreq addreq;
@@ -567,12 +593,13 @@ int if_prefix_delete_ipv6(struct interface *ifp, struct connected *ifc)
 	struct sockaddr_in6 mask;
 	struct prefix_ipv6 *p;
 
-	p = (struct prefix_ipv6 *)ifc->address;
+	p = (struct prefix_ipv6 *)dplane_ctx_get_intf_addr(ctx);
 
-	memset(&addreq, 0, sizeof addreq);
-	strncpy((char *)&addreq.ifra_name, ifp->name, sizeof addreq.ifra_name);
+	memset(&addreq, 0, sizeof(addreq));
+	strlcpy((char *)&addreq.ifra_name,
+		dplane_ctx_get_ifname(ctx), sizeof(addreq.ifra_name));
 
-	memset(&addr, 0, sizeof(struct sockaddr_in6));
+	memset(&addr, 0, sizeof(addr));
 	addr.sin6_addr = p->prefix;
 	addr.sin6_family = p->family;
 #ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
@@ -580,7 +607,7 @@ int if_prefix_delete_ipv6(struct interface *ifp, struct connected *ifc)
 #endif
 	memcpy(&addreq.ifra_addr, &addr, sizeof(struct sockaddr_in6));
 
-	memset(&mask, 0, sizeof(struct sockaddr_in6));
+	memset(&mask, 0, sizeof(mask));
 	masklen2ip6(p->prefixlen, &mask.sin6_addr);
 	mask.sin6_family = p->family;
 #ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
@@ -599,17 +626,18 @@ int if_prefix_delete_ipv6(struct interface *ifp, struct connected *ifc)
 	return 0;
 }
 #else
-int if_prefix_add_ipv6(struct interface *ifp, struct connected *ifc)
+/* The old, pre-dataplane code here just returned, so we're retaining that
+ * choice.
+ */
+static int if_set_prefix6_ctx(const struct zebra_dplane_ctx *ctx)
 {
 	return 0;
 }
 
-int if_prefix_delete_ipv6(struct interface *ifp, struct connected *ifc)
+static int if_unset_prefix6_ctx(const struct zebra_dplane_ctx *ctx)
 {
 	return 0;
 }
 #endif /* HAVE_STRUCT_IN6_ALIASREQ */
 
 #endif /* LINUX_IPV6 */
-
-#endif /* !SUNOS_5 */
