@@ -1,21 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* zebra daemon main routine.
  * Copyright (C) 1997, 98 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -23,7 +8,7 @@
 #include <lib/version.h>
 #include "getopt.h"
 #include "command.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "filter.h"
 #include "memory.h"
 #include "prefix.h"
@@ -33,6 +18,7 @@
 #include "sigevent.h"
 #include "vrf.h"
 #include "libfrr.h"
+#include "affinitymap.h"
 #include "routemap.h"
 #include "routing_nb.h"
 
@@ -66,7 +52,7 @@
 pid_t pid;
 
 /* Pacify zclient.o in libfrr, which expects this variable. */
-struct thread_master *master;
+struct event_loop *master;
 
 /* Route retain mode flag. */
 int retain_mode = 0;
@@ -83,24 +69,30 @@ uint32_t rcvbufsize = RCVBUFSIZE_MIN;
 uint32_t rcvbufsize = 128 * 1024;
 #endif
 
+uint32_t rt_table_main_id = RT_TABLE_MAIN;
+
 #define OPTION_V6_RR_SEMANTICS 2000
 #define OPTION_ASIC_OFFLOAD    2001
+#define OPTION_V6_WITH_V4_NEXTHOP 2002
 
 /* Command line options. */
 const struct option longopts[] = {
-	{"batch", no_argument, NULL, 'b'},
-	{"allow_delete", no_argument, NULL, 'a'},
-	{"socket", required_argument, NULL, 'z'},
-	{"ecmp", required_argument, NULL, 'e'},
-	{"retain", no_argument, NULL, 'r'},
-	{"graceful_restart", required_argument, NULL, 'K'},
-	{"asic-offload", optional_argument, NULL, OPTION_ASIC_OFFLOAD},
+	{ "batch", no_argument, NULL, 'b' },
+	{ "allow_delete", no_argument, NULL, 'a' },
+	{ "socket", required_argument, NULL, 'z' },
+	{ "ecmp", required_argument, NULL, 'e' },
+	{ "retain", no_argument, NULL, 'r' },
+	{ "graceful_restart", required_argument, NULL, 'K' },
+	{ "asic-offload", optional_argument, NULL, OPTION_ASIC_OFFLOAD },
+	{ "v6-with-v4-nexthops", no_argument, NULL, OPTION_V6_WITH_V4_NEXTHOP },
 #ifdef HAVE_NETLINK
-	{"vrfwnetns", no_argument, NULL, 'n'},
-	{"nl-bufsize", required_argument, NULL, 's'},
-	{"v6-rr-semantics", no_argument, NULL, OPTION_V6_RR_SEMANTICS},
+	{ "vrfwnetns", no_argument, NULL, 'n' },
+	{ "nl-bufsize", required_argument, NULL, 's' },
+	{ "v6-rr-semantics", no_argument, NULL, OPTION_V6_RR_SEMANTICS },
 #endif /* HAVE_NETLINK */
-	{0}};
+	{"routing-table", optional_argument, NULL, 'R'},
+	{ 0 }
+};
 
 zebra_capabilities_t _caps_p[] = {ZCAP_NET_ADMIN, ZCAP_SYS_ADMIN,
 				  ZCAP_NET_RAW,
@@ -189,11 +181,6 @@ static void sigint(void)
 	if (zrouter.lsp_process_q)
 		work_queue_free_and_null(&zrouter.lsp_process_q);
 
-	vrf_terminate();
-
-	ns_walk_func(zebra_ns_early_shutdown, NULL, NULL);
-	zebra_ns_notify_close();
-
 	access_list_reset();
 	prefix_list_reset();
 	/*
@@ -219,13 +206,23 @@ static void sigint(void)
  * Final shutdown step for the zebra main thread. This is run after all
  * async update processing has completed.
  */
-void zebra_finalize(struct thread *dummy)
+void zebra_finalize(struct event *dummy)
 {
 	zlog_info("Zebra final shutdown");
 
-	/* Stop dplane thread and finish any cleanup */
+	vrf_terminate();
+
+	/*
+	 * Stop dplane thread and finish any cleanup
+	 * This is before the zebra_ns_early_shutdown call
+	 * because sockets that the dplane depends on are closed
+	 * in those functions
+	 */
 	zebra_dplane_shutdown();
 
+	ns_walk_func(zebra_ns_early_shutdown, NULL, NULL);
+	zebra_ns_notify_close();
+
 	/* Final shutdown of ns resources */
 	ns_walk_func(zebra_ns_final_shutdown, NULL, NULL);
 
@@ -261,6 +258,7 @@ struct frr_signal_t zebra_signals[] = {
 	},
 };
 
+/* clang-format off */
 static const struct frr_yang_module_info *const zebra_yang_modules[] = {
 	&frr_filter_info,
 	&frr_interface_info,
@@ -268,8 +266,10 @@ static const struct frr_yang_module_info *const zebra_yang_modules[] = {
 	&frr_zebra_info,
 	&frr_vrf_info,
 	&frr_routing_info,
+	&frr_affinity_map_info,
 	&frr_zebra_route_map_info,
 };
+/* clang-format on */
 
 FRR_DAEMON_INFO(
 	zebra, ZEBRA, .vty_port = ZEBRA_VTY_PORT, .flags = FRR_NO_ZCLIENT,
@@ -293,6 +293,7 @@ int main(int argc, char **argv)
 	struct sockaddr_storage dummy;
 	socklen_t dummylen;
 	bool asic_offload = false;
+	bool v6_with_v4_nexthop = false;
 	bool notify_on_ack = true;
 
 	graceful_restart = 0;
@@ -300,27 +301,28 @@ int main(int argc, char **argv)
 
 	frr_preinit(&zebra_di, argc, argv);
 
-	frr_opt_add(
-		"baz:e:rK:s:"
+	frr_opt_add("baz:e:rK:s:R:"
 #ifdef HAVE_NETLINK
-		"n"
+		    "n"
 #endif
-		,
-		longopts,
-		"  -b, --batch              Runs in batch mode\n"
-		"  -a, --allow_delete       Allow other processes to delete zebra routes\n"
-		"  -z, --socket             Set path of zebra socket\n"
-		"  -e, --ecmp               Specify ECMP to use.\n"
-		"  -r, --retain             When program terminates, retain added route by zebra.\n"
-		"  -K, --graceful_restart   Graceful restart at the kernel level, timer in seconds for expiration\n"
-		"  -A, --asic-offload       FRR is interacting with an asic underneath the linux kernel\n"
+		    ,
+		    longopts,
+		    "  -b, --batch               Runs in batch mode\n"
+		    "  -a, --allow_delete        Allow other processes to delete zebra routes\n"
+		    "  -z, --socket              Set path of zebra socket\n"
+		    "  -e, --ecmp                Specify ECMP to use.\n"
+		    "  -r, --retain              When program terminates, retain added route by zebra.\n"
+		    "  -K, --graceful_restart    Graceful restart at the kernel level, timer in seconds for expiration\n"
+		    "  -A, --asic-offload        FRR is interacting with an asic underneath the linux kernel\n"
+		    "      --v6-with-v4-nexthops Underlying dataplane supports v6 routes with v4 nexthops"
 #ifdef HAVE_NETLINK
-		"  -s, --nl-bufsize         Set netlink receive buffer size\n"
-		"  -n, --vrfwnetns          Use NetNS as VRF backend\n"
-		"      --v6-rr-semantics    Use v6 RR semantics\n"
+		    "  -s, --nl-bufsize          Set netlink receive buffer size\n"
+		    "  -n, --vrfwnetns           Use NetNS as VRF backend\n"
+		    "      --v6-rr-semantics     Use v6 RR semantics\n"
 #else
-		"  -s,                      Set kernel socket receive buffer size\n"
+		    "  -s,                       Set kernel socket receive buffer size\n"
 #endif /* HAVE_NETLINK */
+		    "  -R, --routing-table       Set kernel routing table\n"
 	);
 
 	while (1) {
@@ -375,6 +377,9 @@ int main(int argc, char **argv)
 					"Rcvbufsize is smaller than recommended value: %d\n",
 					RCVBUFSIZE_MIN);
 			break;
+		case 'R':
+			rt_table_main_id = atoi(optarg);
+			break;
 #ifdef HAVE_NETLINK
 		case 'n':
 			vrf_configure_backend(VRF_BACKEND_NETNS);
@@ -389,6 +394,9 @@ int main(int argc, char **argv)
 				notify_on_ack = true;
 			asic_offload = true;
 			break;
+		case OPTION_V6_WITH_V4_NEXTHOP:
+			v6_with_v4_nexthop = true;
+			break;
 #endif /* HAVE_NETLINK */
 		default:
 			frr_help_exit(1);
@@ -398,7 +406,7 @@ int main(int argc, char **argv)
 	zrouter.master = frr_init();
 
 	/* Zebra related initialize. */
-	zebra_router_init(asic_offload, notify_on_ack);
+	zebra_router_init(asic_offload, notify_on_ack, v6_with_v4_nexthop);
 	zserv_init();
 	rib_init();
 	zebra_if_init();
@@ -448,8 +456,8 @@ int main(int argc, char **argv)
 	* we have to have route_read() called before.
 	*/
 	zrouter.startup_time = monotime(NULL);
-	thread_add_timer(zrouter.master, rib_sweep_route, NULL,
-			 graceful_restart, &zrouter.sweeper);
+	event_add_timer(zrouter.master, rib_sweep_route, NULL, graceful_restart,
+			&zrouter.sweeper);
 
 	/* Needed for BSD routing socket. */
 	pid = getpid();
