@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Zebra Traffic Control (TC) interaction with the kernel using netlink.
  *
  * Copyright (C) 2022 Shichu Yang
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -173,7 +160,7 @@ static ssize_t netlink_qdisc_msg_encode(int cmd, struct zebra_dplane_ctx *ctx,
 		struct nlmsghdr n;
 		struct tcmsg t;
 		char buf[0];
-	} *req = (void *)data;
+	} *req = data;
 
 	if (datalen < sizeof(*req))
 		return 0;
@@ -249,7 +236,7 @@ static ssize_t netlink_tclass_msg_encode(int cmd, struct zebra_dplane_ctx *ctx,
 		struct nlmsghdr n;
 		struct tcmsg t;
 		char buf[0];
-	} *req = (void *)data;
+	} *req = data;
 
 	if (datalen < sizeof(*req))
 		return 0;
@@ -499,7 +486,7 @@ static ssize_t netlink_tfilter_msg_encode(int cmd, struct zebra_dplane_ctx *ctx,
 		struct nlmsghdr n;
 		struct tcmsg t;
 		char buf[0];
-	} *req = (void *)data;
+	} *req = data;
 
 	if (datalen < sizeof(*req))
 		return 0;
@@ -749,7 +736,8 @@ int netlink_qdisc_change(struct nlmsghdr *h, ns_id_t ns_id, int startup)
 	case TC_QDISC_NOQUEUE:
 		/* "noqueue" is the default qdisc */
 		break;
-	default:
+	case TC_QDISC_HTB:
+	case TC_QDISC_UNSPEC:
 		break;
 	}
 
