@@ -1,23 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* Zebra Mlag Code.
  * Copyright (C) 2018 Cumulus Networks, Inc.
  *                    Donald Sharp
- *
- * This file is part of FRR.
- *
- * FRR is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * FRR is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with FRR; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
  */
 #include "zebra.h"
 
@@ -29,29 +13,31 @@
 #include "zebra/zebra_mlag.h"
 #include "zebra/zebra_mlag_vty.h"
 #include "zebra/zebra_router.h"
-#include "zebra/zebra_memory.h"
 #include "zebra/zapi_msg.h"
 #include "zebra/debug.h"
 
+#ifdef HAVE_PROTOBUF_VERSION_3
+#include "mlag/mlag.pb-c.h"
+#endif
+
 DEFINE_HOOK(zebra_mlag_private_write_data,
-	    (uint8_t *data, uint32_t len), (data, len))
-DEFINE_HOOK(zebra_mlag_private_monitor_state, (), ())
-DEFINE_HOOK(zebra_mlag_private_open_channel, (), ())
-DEFINE_HOOK(zebra_mlag_private_close_channel, (), ())
-DEFINE_HOOK(zebra_mlag_private_cleanup_data, (), ())
+	    (uint8_t *data, uint32_t len), (data, len));
+DEFINE_HOOK(zebra_mlag_private_monitor_state, (), ());
+DEFINE_HOOK(zebra_mlag_private_open_channel, (), ());
+DEFINE_HOOK(zebra_mlag_private_close_channel, (), ());
+DEFINE_HOOK(zebra_mlag_private_cleanup_data, (), ());
 
 #define ZEBRA_MLAG_METADATA_LEN 4
 #define ZEBRA_MLAG_MSG_BCAST 0xFFFFFFFF
 
 uint8_t mlag_wr_buffer[ZEBRA_MLAG_BUF_LIMIT];
 uint8_t mlag_rd_buffer[ZEBRA_MLAG_BUF_LIMIT];
-uint32_t mlag_rd_buf_offset;
 
 static bool test_mlag_in_progress;
 
 static int zebra_mlag_signal_write_thread(void);
-static int zebra_mlag_terminate_pthread(struct thread *event);
-static int zebra_mlag_post_data_from_main_thread(struct thread *thread);
+static void zebra_mlag_terminate_pthread(struct event *event);
+static void zebra_mlag_post_data_from_main_thread(struct event *thread);
 static void zebra_mlag_publish_process_state(struct zserv *client,
 					     zebra_message_types_t msg_type);
 
@@ -107,10 +93,13 @@ void zebra_mlag_send_deregister(void)
 void zebra_mlag_process_mlag_data(uint8_t *data, uint32_t len)
 {
 	struct stream *s = NULL;
-	struct stream *s1 = NULL;
 	int msg_type = 0;
 
-	s = stream_new(ZEBRA_MAX_PACKET_SIZ);
+	s = stream_new(ZEBRA_MLAG_BUF_LIMIT);
+	/*
+	 * Place holder we need the message type first
+	 */
+	stream_putl(s, msg_type);
 	msg_type = zebra_mlag_protobuf_decode_message(s, data, len);
 
 	if (msg_type <= 0) {
@@ -124,12 +113,9 @@ void zebra_mlag_process_mlag_data(uint8_t *data, uint32_t len)
 	/*
 	 * additional four bytes are for message type
 	 */
-	s1 = stream_new(stream_get_endp(s) + ZEBRA_MLAG_METADATA_LEN);
-	stream_putl(s1, msg_type);
-	stream_put(s1, s->data, stream_get_endp(s));
-	thread_add_event(zrouter.master, zebra_mlag_post_data_from_main_thread,
-			 s1, 0, NULL);
-	stream_free(s);
+	stream_putl_at(s, 0, msg_type);
+	event_add_event(zrouter.master, zebra_mlag_post_data_from_main_thread,
+			s, 0, NULL);
 }
 
 /**********************End of MLAG Interaction********************************/
@@ -146,7 +132,7 @@ void zebra_mlag_process_mlag_data(uint8_t *data, uint32_t len)
  * This thread reads the clients data from the Global queue and encodes with
  * protobuf and pass on to the MLAG socket.
  */
-static int zebra_mlag_client_msg_handler(struct thread *event)
+static void zebra_mlag_client_msg_handler(struct event *event)
 {
 	struct stream *s;
 	uint32_t wr_count = 0;
@@ -187,9 +173,9 @@ static int zebra_mlag_client_msg_handler(struct thread *event)
 			 * main thread.
 			 */
 			if (msg_type == MLAG_DEREGISTER) {
-				thread_add_event(zrouter.master,
-						 zebra_mlag_terminate_pthread,
-						 NULL, 0, NULL);
+				event_add_event(zrouter.master,
+						zebra_mlag_terminate_pthread,
+						NULL, 0, NULL);
 			}
 		}
 
@@ -207,7 +193,6 @@ static int zebra_mlag_client_msg_handler(struct thread *event)
 	 */
 	if (wr_count >= ZEBRA_MLAG_POST_LIMIT)
 		zebra_mlag_signal_write_thread();
-	return 0;
 }
 
 /*
@@ -252,9 +237,9 @@ static int zebra_mlag_signal_write_thread(void)
 	 * during Zebra Init/after MLAG thread is destroyed.
 	 * so it is safe to use without any locking
 	 */
-	thread_add_event(zrouter.mlag_info.th_master,
-			 zebra_mlag_client_msg_handler, NULL, 0,
-			 &zrouter.mlag_info.t_write);
+	event_add_event(zrouter.mlag_info.th_master,
+			zebra_mlag_client_msg_handler, NULL, 0,
+			&zrouter.mlag_info.t_write);
 	return 0;
 }
 
@@ -294,8 +279,8 @@ static void zebra_mlag_publish_process_state(struct zserv *client,
 	s = stream_new(ZEBRA_HEADER_SIZE + ZEBRA_MLAG_METADATA_LEN);
 	stream_putl(s, ZEBRA_MLAG_MSG_BCAST);
 	zclient_create_header(s, msg_type, VRF_DEFAULT);
-	thread_add_event(zrouter.master, zebra_mlag_post_data_from_main_thread,
-			 s, 0, NULL);
+	event_add_event(zrouter.master, zebra_mlag_post_data_from_main_thread,
+			s, 0, NULL);
 }
 
 /**************************End of Multi-entrant Apis**************************/
@@ -307,9 +292,9 @@ static void zebra_mlag_publish_process_state(struct zserv *client,
  * main thread, because for that access was needed for clients list.
  * so instead of forcing the locks, messages will be posted from main thread.
  */
-static int zebra_mlag_post_data_from_main_thread(struct thread *thread)
+static void zebra_mlag_post_data_from_main_thread(struct event *thread)
 {
-	struct stream *s = THREAD_ARG(thread);
+	struct stream *s = EVENT_ARG(thread);
 	struct stream *zebra_s = NULL;
 	struct listnode *node;
 	struct zserv *client;
@@ -317,7 +302,7 @@ static int zebra_mlag_post_data_from_main_thread(struct thread *thread)
 	uint32_t msg_len = 0;
 
 	if (!s)
-		return -1;
+		return;
 
 	STREAM_GETL(s, msg_type);
 	if (IS_ZEBRA_DEBUG_MLAG)
@@ -353,13 +338,10 @@ static int zebra_mlag_post_data_from_main_thread(struct thread *thread)
 		}
 	}
 
-	stream_free(s);
-	return 0;
 stream_failure:
 	stream_free(s);
 	if (zebra_s)
 		stream_free(zebra_s);
-	return 0;
 }
 
 /*
@@ -392,7 +374,7 @@ static void zebra_mlag_spawn_pthread(void)
  * all clients are un-registered for MLAG Updates, terminate the
  * MLAG write thread
  */
-static int zebra_mlag_terminate_pthread(struct thread *event)
+static void zebra_mlag_terminate_pthread(struct event *event)
 {
 	if (IS_ZEBRA_DEBUG_MLAG)
 		zlog_debug("Zebra MLAG write thread terminate called");
@@ -401,7 +383,7 @@ static int zebra_mlag_terminate_pthread(struct thread *event)
 		if (IS_ZEBRA_DEBUG_MLAG)
 			zlog_debug(
 				"Zebra MLAG: still some clients are interested");
-		return 0;
+		return;
 	}
 
 	frr_pthread_stop(zrouter.mlag_info.zebra_pth_mlag, NULL);
@@ -417,7 +399,6 @@ static int zebra_mlag_terminate_pthread(struct thread *event)
 	 * Send Notification to clean private data
 	 */
 	hook_call(zebra_mlag_private_cleanup_data);
-	return 0;
 }
 
 /*
@@ -656,7 +637,7 @@ void zebra_mlag_terminate(void)
 
 #ifdef HAVE_PROTOBUF_VERSION_3
 
-DEFINE_MTYPE_STATIC(ZEBRA, MLAG_PBUF, "ZEBRA MLAG PROTOBUF")
+DEFINE_MTYPE_STATIC(ZEBRA, MLAG_PBUF, "ZEBRA MLAG PROTOBUF");
 
 int zebra_mlag_protobuf_encode_client_data(struct stream *s, uint32_t *msg_type)
 {
@@ -889,7 +870,14 @@ int zebra_mlag_protobuf_encode_client_data(struct stream *s, uint32_t *msg_type)
 		if (cleanup)
 			return -1;
 	} break;
-	default:
+	case MLAG_REGISTER:
+	case MLAG_DEREGISTER:
+	case MLAG_STATUS_UPDATE:
+	case MLAG_DUMP:
+	case MLAG_PIM_CFG_DUMP:
+	case MLAG_VXLAN_UPDATE:
+	case MLAG_PEER_FRR_STATUS:
+	case MLAG_MSG_NONE:
 		break;
 	}
 
@@ -1079,7 +1067,7 @@ int zebra_mlag_protobuf_decode_message(struct stream *s, uint8_t *data,
 		case ZEBRA_MLAG__HEADER__MESSAGE_TYPE__ZEBRA_MLAG_MROUTE_ADD_BULK: {
 			ZebraMlagMrouteAddBulk *Bulk_msg = NULL;
 			ZebraMlagMrouteAdd *msg = NULL;
-			size_t i;
+			size_t i, length_spot;
 
 			Bulk_msg = zebra_mlag_mroute_add_bulk__unpack(
 				NULL, hdr->data.len, hdr->data.data);
@@ -1091,10 +1079,17 @@ int zebra_mlag_protobuf_decode_message(struct stream *s, uint8_t *data,
 			stream_putw(s, (Bulk_msg->n_mroute_add
 					* sizeof(struct mlag_mroute_add)));
 			/* No. of msgs in Batch */
-			stream_putw(s, Bulk_msg->n_mroute_add);
+			length_spot = stream_putw(s, Bulk_msg->n_mroute_add);
 
 			/* Actual Data */
 			for (i = 0; i < Bulk_msg->n_mroute_add; i++) {
+				if (STREAM_SIZE(s)
+				    < VRF_NAMSIZ + 22 + INTERFACE_NAMSIZ) {
+					zlog_warn(
+						"We have received more messages than we can parse at this point in time: %zu",
+						Bulk_msg->n_mroute_add);
+					break;
+				}
 
 				msg = Bulk_msg->mroute_add[i];
 
@@ -1114,13 +1109,16 @@ int zebra_mlag_protobuf_decode_message(struct stream *s, uint8_t *data,
 				else
 					stream_put(s, NULL, INTERFACE_NAMSIZ);
 			}
+
+			stream_putw_at(s, length_spot, i + 1);
+
 			zebra_mlag_mroute_add_bulk__free_unpacked(Bulk_msg,
 								  NULL);
 		} break;
 		case ZEBRA_MLAG__HEADER__MESSAGE_TYPE__ZEBRA_MLAG_MROUTE_DEL_BULK: {
 			ZebraMlagMrouteDelBulk *Bulk_msg = NULL;
 			ZebraMlagMrouteDel *msg = NULL;
-			size_t i;
+			size_t i, length_spot;
 
 			Bulk_msg = zebra_mlag_mroute_del_bulk__unpack(
 				NULL, hdr->data.len, hdr->data.data);
@@ -1132,10 +1130,16 @@ int zebra_mlag_protobuf_decode_message(struct stream *s, uint8_t *data,
 			stream_putw(s, (Bulk_msg->n_mroute_del
 					* sizeof(struct mlag_mroute_del)));
 			/* No. of msgs in Batch */
-			stream_putw(s, Bulk_msg->n_mroute_del);
+			length_spot = stream_putw(s, Bulk_msg->n_mroute_del);
 
 			/* Actual Data */
 			for (i = 0; i < Bulk_msg->n_mroute_del; i++) {
+				if (STREAM_SIZE(s)
+				    < VRF_NAMSIZ + 16 + INTERFACE_NAMSIZ) {
+					zlog_warn(
+						"We have received more messages than we can parse at this time");
+					break;
+				}
 
 				msg = Bulk_msg->mroute_del[i];
 
@@ -1152,6 +1156,9 @@ int zebra_mlag_protobuf_decode_message(struct stream *s, uint8_t *data,
 				else
 					stream_put(s, NULL, INTERFACE_NAMSIZ);
 			}
+
+			stream_putw_at(s, length_spot, i + 1);
+
 			zebra_mlag_mroute_del_bulk__free_unpacked(Bulk_msg,
 								  NULL);
 		} break;
