@@ -1,23 +1,11 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 2020  Cumulus Networks, Inc.
  * Chirag Shah
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
+#include "interface.h"
 #include "northbound.h"
 #include "libfrr.h"
 #include "zebra_nb.h"
@@ -91,27 +79,6 @@ const struct frr_yang_module_info frr_zebra_info = {
 				.modify = zebra_dplane_queue_limit_modify,
 			}
 		},
-		{
-			.xpath = "/frr-zebra:zebra/vrf-vni-mapping",
-			.cbs = {
-				.create = zebra_vrf_vni_mapping_create,
-				.destroy = zebra_vrf_vni_mapping_destroy,
-			}
-		},
-		{
-			.xpath = "/frr-zebra:zebra/vrf-vni-mapping/vni-id",
-			.cbs = {
-				.modify = zebra_vrf_vni_mapping_vni_id_modify,
-				.destroy = zebra_vrf_vni_mapping_vni_id_destroy,
-			}
-		},
-		{
-			.xpath = "/frr-zebra:zebra/vrf-vni-mapping/prefix-only",
-			.cbs = {
-				.create = zebra_vrf_vni_mapping_prefix_only_create,
-				.destroy = zebra_vrf_vni_mapping_prefix_only_destroy,
-			}
-		},
 		{
 			.xpath = "/frr-zebra:zebra/debugs/debug-events",
 			.cbs = {
@@ -371,6 +338,48 @@ const struct frr_yang_module_info frr_zebra_info = {
 				.destroy = lib_interface_zebra_bandwidth_destroy,
 			}
 		},
+		{
+			.xpath = "/frr-interface:lib/interface/frr-zebra:zebra/mpls",
+			.cbs = {
+				.modify = lib_interface_zebra_mpls_modify,
+				.destroy = lib_interface_zebra_mpls_destroy,
+			}
+		},
+		{
+			.xpath = "/frr-interface:lib/interface/frr-zebra:zebra/bandwidth",
+			.cbs = {
+				.modify = lib_interface_zebra_bandwidth_modify,
+				.destroy = lib_interface_zebra_bandwidth_destroy,
+			}
+		},
+		{
+			.xpath = "/frr-interface:lib/interface/frr-zebra:zebra/link-params/legacy-admin-group",
+			.cbs = {
+				.modify = lib_interface_zebra_legacy_admin_group_modify,
+				.destroy = lib_interface_zebra_legacy_admin_group_destroy,
+				.cli_show = cli_show_legacy_admin_group,
+			},
+		},
+		{
+			.xpath = "/frr-interface:lib/interface/frr-zebra:zebra/link-params/affinities",
+			.cbs = {
+				.cli_show = cli_show_affinity,
+			},
+		},
+		{
+			.xpath = "/frr-interface:lib/interface/frr-zebra:zebra/link-params/affinities/affinity",
+			.cbs = {
+				.create = lib_interface_zebra_affinity_create,
+				.destroy = lib_interface_zebra_affinity_destroy,
+			},
+		},
+		{
+			.xpath = "/frr-interface:lib/interface/frr-zebra:zebra/link-params/affinity-mode",
+			.cbs = {
+				.modify = lib_interface_zebra_affinity_mode_modify,
+				.cli_show = cli_show_affinity_mode,
+			},
+		},
 		{
 			.xpath = "/frr-interface:lib/interface/frr-zebra:zebra/state/up-count",
 			.cbs = {
@@ -420,269 +429,270 @@ const struct frr_yang_module_info frr_zebra_info = {
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib",
 			.cbs = {
-				.create = lib_vrf_ribs_rib_create,
-				.destroy = lib_vrf_ribs_rib_destroy,
-				.get_next = lib_vrf_ribs_rib_get_next,
-				.get_keys = lib_vrf_ribs_rib_get_keys,
-				.lookup_entry = lib_vrf_ribs_rib_lookup_entry,
+				.get_next = lib_vrf_zebra_ribs_rib_get_next,
+				.get_keys = lib_vrf_zebra_ribs_rib_get_keys,
+				.lookup_entry = lib_vrf_zebra_ribs_rib_lookup_entry,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/afi-safi-name",
 			.cbs = {
-				.get_next = lib_vrf_ribs_rib_route_get_next,
-				.get_keys = lib_vrf_ribs_rib_route_get_keys,
-				.lookup_entry = lib_vrf_ribs_rib_route_lookup_entry,
+				.get_elem = lib_vrf_zebra_ribs_rib_afi_safi_name_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/prefix",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/table-id",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_prefix_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_table_id_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route",
 			.cbs = {
-				.get_next = lib_vrf_ribs_rib_route_route_entry_get_next,
-				.get_keys = lib_vrf_ribs_rib_route_route_entry_get_keys,
-				.lookup_entry = lib_vrf_ribs_rib_route_route_entry_lookup_entry,
+				.get_next = lib_vrf_zebra_ribs_rib_route_get_next,
+				.get_keys = lib_vrf_zebra_ribs_rib_route_get_keys,
+				.lookup_entry = lib_vrf_zebra_ribs_rib_route_lookup_entry,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/protocol",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/prefix",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_protocol_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_prefix_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/instance",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_instance_get_elem,
+				.get_next = lib_vrf_zebra_ribs_rib_route_route_entry_get_next,
+				.get_keys = lib_vrf_zebra_ribs_rib_route_route_entry_get_keys,
+				.lookup_entry = lib_vrf_zebra_ribs_rib_route_route_entry_lookup_entry,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/distance",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/protocol",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_distance_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_protocol_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/metric",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/instance",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_metric_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_instance_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/tag",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/distance",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_tag_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_distance_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/selected",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/metric",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_selected_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_metric_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/installed",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/tag",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_installed_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_tag_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/failed",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/selected",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_failed_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_selected_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/queued",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/installed",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_queued_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_installed_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/internal-flags",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/failed",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_internal_flags_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_failed_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/internal-status",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/queued",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_internal_status_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_queued_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/uptime",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/internal-flags",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_uptime_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_internal_flags_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/internal-status",
 			.cbs = {
-				.get_next = lib_vrf_ribs_rib_route_route_entry_nexthop_group_get_next,
-				.get_keys = lib_vrf_ribs_rib_route_route_entry_nexthop_group_get_keys,
-				.lookup_entry = lib_vrf_ribs_rib_route_route_entry_nexthop_group_lookup_entry,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_internal_status_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/name",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/uptime",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_name_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_uptime_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/id",
 			.cbs = {
-				.get_next = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_get_next,
-				.get_keys = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_get_keys,
-				.lookup_entry = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_lookup_entry,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_id_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/nh-type",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_nh_type_get_elem,
+				.get_next = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_get_next,
+				.get_keys = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_get_keys,
+				.lookup_entry = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_lookup_entry,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/vrf",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/nh-type",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_vrf_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_nh_type_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/gateway",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/vrf",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_gateway_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_vrf_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/interface",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/gateway",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_interface_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_gateway_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/bh-type",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/interface",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_bh_type_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_interface_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/onlink",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/bh-type",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_onlink_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_bh_type_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/mpls-label-stack/entry",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/onlink",
 			.cbs = {
-				.get_next = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_mpls_label_stack_entry_get_next,
-				.get_keys = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_mpls_label_stack_entry_get_keys,
-				.lookup_entry = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_mpls_label_stack_entry_lookup_entry,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_onlink_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/mpls-label-stack/entry/id",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/srte-color",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_mpls_label_stack_entry_id_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_color_get_elem,
+			}
+		},
+
+		{
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/srv6-segs-stack/entry",
+			.cbs = {
+				.get_next = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_srv6_segs_stack_entry_get_next,
+				.get_keys = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_srv6_segs_stack_entry_get_keys,
+				.lookup_entry = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_srv6_segs_stack_entry_lookup_entry,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/mpls-label-stack/entry/label",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/srv6-segs-stack/entry/id",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_mpls_label_stack_entry_label_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_srv6_segs_stack_entry_id_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/mpls-label-stack/entry/ttl",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/srv6-segs-stack/entry/seg",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_mpls_label_stack_entry_ttl_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_srv6_segs_stack_entry_seg_get_elem,
+			}
+		},
+
+		{
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/mpls-label-stack/entry",
+			.cbs = {
+				.get_next = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_mpls_label_stack_entry_get_next,
+				.get_keys = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_mpls_label_stack_entry_get_keys,
+				.lookup_entry = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_mpls_label_stack_entry_lookup_entry,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/mpls-label-stack/entry/traffic-class",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/mpls-label-stack/entry/id",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_mpls_label_stack_entry_traffic_class_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_mpls_label_stack_entry_id_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/duplicate",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/mpls-label-stack/entry/label",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_duplicate_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_mpls_label_stack_entry_label_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/recursive",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/mpls-label-stack/entry/ttl",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_recursive_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_mpls_label_stack_entry_ttl_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/active",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/mpls-label-stack/entry/traffic-class",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_active_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_mpls_label_stack_entry_traffic_class_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/fib",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/duplicate",
 			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_fib_get_elem,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_duplicate_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/frr-zebra:ipv4-prefix-length",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/recursive",
 			.cbs = {
-				.modify = lib_route_map_entry_match_condition_ipv4_prefix_length_modify,
-				.destroy = lib_route_map_entry_match_condition_ipv4_prefix_length_destroy,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_recursive_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/frr-zebra:ipv6-prefix-length",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/active",
 			.cbs = {
-				.modify = lib_route_map_entry_match_condition_ipv6_prefix_length_modify,
-				.destroy = lib_route_map_entry_match_condition_ipv6_prefix_length_destroy,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_active_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/frr-zebra:source-protocol",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/fib",
 			.cbs = {
-				.modify = lib_route_map_entry_match_condition_source_protocol_modify,
-				.destroy = lib_route_map_entry_match_condition_source_protocol_destroy,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_fib_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/match-condition/frr-zebra:source-instance",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/ribs/rib/route/route-entry/nexthop-group/nexthop/weight",
 			.cbs = {
-				.modify = lib_route_map_entry_match_condition_source_instance_modify,
-				.destroy = lib_route_map_entry_match_condition_source_instance_destroy,
+				.get_elem = lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_weight_get_elem,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/set-action/frr-zebra:source-v4",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/l3vni-id",
 			.cbs = {
-				.modify = lib_route_map_entry_set_action_source_v4_modify,
-				.destroy = lib_route_map_entry_set_action_source_v4_destroy,
+				.modify = lib_vrf_zebra_l3vni_id_modify,
+				.destroy = lib_vrf_zebra_l3vni_id_destroy,
 			}
 		},
 		{
-			.xpath = "/frr-route-map:lib/route-map/entry/set-action/frr-zebra:source-v6",
+			.xpath = "/frr-vrf:lib/vrf/frr-zebra:zebra/prefix-only",
 			.cbs = {
-				.modify = lib_route_map_entry_set_action_source_v6_modify,
-				.destroy = lib_route_map_entry_set_action_source_v6_destroy,
-			}
-		},
-		{
-			.xpath = "/frr-vrf:lib/vrf/frr-zebra:ribs/rib/route/route-entry/nexthop-group/frr-nexthops/nexthop/weight",
-			.cbs = {
-				.get_elem = lib_vrf_ribs_rib_route_route_entry_nexthop_group_frr_nexthops_nexthop_weight_get_elem,
+				.modify = lib_vrf_zebra_prefix_only_modify,
 			}
 		},
 		{
