@@ -1,26 +1,16 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 2019  Cumulus Networks, Inc.
  * Chirag Shah
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
+#include "lib/admin_group.h"
+#include "lib/affinitymap.h"
 #include "lib/log.h"
 #include "lib/northbound.h"
+#include "lib/printfrr.h"
 #include "libfrr.h"
 #include "lib/command.h"
 #include "lib/routemap.h"
@@ -29,6 +19,10 @@
 #include "zebra_nb.h"
 #include "zebra/interface.h"
 #include "zebra/connected.h"
+#include "zebra/zebra_router.h"
+#include "zebra/debug.h"
+#include "zebra/zebra_vxlan_private.h"
+#include "zebra/zebra_vxlan.h"
 
 /*
  * XPath: /frr-zebra:zebra/mcast-rpf-lookup
@@ -270,99 +264,6 @@ int zebra_dplane_queue_limit_modify(struct nb_cb_modify_args *args)
 	return NB_OK;
 }
 
-/*
- * XPath: /frr-zebra:zebra/vrf-vni-mapping
- */
-int zebra_vrf_vni_mapping_create(struct nb_cb_create_args *args)
-{
-	switch (args->event) {
-	case NB_EV_VALIDATE:
-	case NB_EV_PREPARE:
-	case NB_EV_ABORT:
-	case NB_EV_APPLY:
-		/* TODO: implement me. */
-		break;
-	}
-
-	return NB_OK;
-}
-
-int zebra_vrf_vni_mapping_destroy(struct nb_cb_destroy_args *args)
-{
-	switch (args->event) {
-	case NB_EV_VALIDATE:
-	case NB_EV_PREPARE:
-	case NB_EV_ABORT:
-	case NB_EV_APPLY:
-		/* TODO: implement me. */
-		break;
-	}
-
-	return NB_OK;
-}
-
-/*
- * XPath: /frr-zebra:zebra/vrf-vni-mapping/vni-id
- */
-int zebra_vrf_vni_mapping_vni_id_modify(struct nb_cb_modify_args *args)
-{
-	switch (args->event) {
-	case NB_EV_VALIDATE:
-	case NB_EV_PREPARE:
-	case NB_EV_ABORT:
-	case NB_EV_APPLY:
-		/* TODO: implement me. */
-		break;
-	}
-
-	return NB_OK;
-}
-
-int zebra_vrf_vni_mapping_vni_id_destroy(struct nb_cb_destroy_args *args)
-{
-	switch (args->event) {
-	case NB_EV_VALIDATE:
-	case NB_EV_PREPARE:
-	case NB_EV_ABORT:
-	case NB_EV_APPLY:
-		/* TODO: implement me. */
-		break;
-	}
-
-	return NB_OK;
-}
-
-/*
- * XPath: /frr-zebra:zebra/vrf-vni-mapping/prefix-only
- */
-int zebra_vrf_vni_mapping_prefix_only_create(struct nb_cb_create_args *args)
-{
-	switch (args->event) {
-	case NB_EV_VALIDATE:
-	case NB_EV_PREPARE:
-	case NB_EV_ABORT:
-	case NB_EV_APPLY:
-		/* TODO: implement me. */
-		break;
-	}
-
-	return NB_OK;
-}
-
-int zebra_vrf_vni_mapping_prefix_only_destroy(struct nb_cb_destroy_args *args)
-{
-	switch (args->event) {
-	case NB_EV_VALIDATE:
-	case NB_EV_PREPARE:
-	case NB_EV_ABORT:
-	case NB_EV_APPLY:
-		/* TODO: implement me. */
-		break;
-	}
-
-	return NB_OK;
-}
-
 /*
  * XPath: /frr-zebra:zebra/debugs/debug-events
  */
@@ -928,9 +829,7 @@ int lib_interface_zebra_ip_addrs_create(struct nb_cb_create_args *args)
 {
 	struct interface *ifp;
 	struct prefix prefix;
-	char buf[PREFIX_STRLEN] = {0};
 
-	ifp = nb_running_get_entry(args->dnode, NULL, true);
 	// addr_family = yang_dnode_get_enum(dnode, "./address-family");
 	yang_dnode_get_prefix(&prefix, args->dnode, "./ip-prefix");
 	apply_mask(&prefix);
@@ -939,13 +838,13 @@ int lib_interface_zebra_ip_addrs_create(struct nb_cb_create_args *args)
 	case NB_EV_VALIDATE:
 		if (prefix.family == AF_INET
 		    && ipv4_martian(&prefix.u.prefix4)) {
-			zlog_debug("invalid address %s",
-				   prefix2str(&prefix, buf, sizeof(buf)));
+			snprintfrr(args->errmsg, args->errmsg_len,
+				   "invalid address %pFX", &prefix);
 			return NB_ERR_VALIDATION;
 		} else if (prefix.family == AF_INET6
 			   && ipv6_martian(&prefix.u.prefix6)) {
-			zlog_debug("invalid address %s",
-				   prefix2str(&prefix, buf, sizeof(buf)));
+			snprintfrr(args->errmsg, args->errmsg_len,
+				   "invalid address %pFX", &prefix);
 			return NB_ERR_VALIDATION;
 		}
 		break;
@@ -953,6 +852,7 @@ int lib_interface_zebra_ip_addrs_create(struct nb_cb_create_args *args)
 	case NB_EV_ABORT:
 		break;
 	case NB_EV_APPLY:
+		ifp = nb_running_get_entry(args->dnode, NULL, true);
 		if (prefix.family == AF_INET)
 			if_ip_address_install(ifp, &prefix, NULL, NULL);
 		else if (prefix.family == AF_INET6)
@@ -970,26 +870,31 @@ int lib_interface_zebra_ip_addrs_destroy(struct nb_cb_destroy_args *args)
 	struct prefix prefix;
 	struct connected *ifc;
 
-	ifp = nb_running_get_entry(args->dnode, NULL, true);
 	yang_dnode_get_prefix(&prefix, args->dnode, "./ip-prefix");
 	apply_mask(&prefix);
 
 	switch (args->event) {
 	case NB_EV_VALIDATE:
+		ifp = nb_running_get_entry(args->dnode, NULL, false);
+		if (!ifp)
+			return NB_OK;
+
 		if (prefix.family == AF_INET) {
 			/* Check current interface address. */
 			ifc = connected_check_ptp(ifp, &prefix, NULL);
 			if (!ifc) {
-				zlog_debug("interface %s Can't find address\n",
-					   ifp->name);
+				snprintf(args->errmsg, args->errmsg_len,
+					 "interface %s Can't find address\n",
+					 ifp->name);
 				return NB_ERR_VALIDATION;
 			}
 		} else if (prefix.family == AF_INET6) {
 			/* Check current interface address. */
 			ifc = connected_check(ifp, &prefix);
 			if (!ifc) {
-				zlog_debug("interface can't find address %s",
-					   ifp->name);
+				snprintf(args->errmsg, args->errmsg_len,
+					 "interface can't find address %s",
+					 ifp->name);
 				return NB_ERR_VALIDATION;
 			}
 		} else
@@ -997,7 +902,8 @@ int lib_interface_zebra_ip_addrs_destroy(struct nb_cb_destroy_args *args)
 
 		/* This is not configured address. */
 		if (!CHECK_FLAG(ifc->conf, ZEBRA_IFC_CONFIGURED)) {
-			zlog_debug("interface %s not configured", ifp->name);
+			snprintf(args->errmsg, args->errmsg_len,
+				 "interface %s not configured", ifp->name);
 			return NB_ERR_VALIDATION;
 		}
 
@@ -1013,6 +919,7 @@ int lib_interface_zebra_ip_addrs_destroy(struct nb_cb_destroy_args *args)
 	case NB_EV_ABORT:
 		break;
 	case NB_EV_APPLY:
+		ifp = nb_running_get_entry(args->dnode, NULL, true);
 		if_ip_address_uinstall(ifp, &prefix);
 		break;
 	}
@@ -1154,6 +1061,9 @@ int lib_interface_zebra_link_detect_destroy(struct nb_cb_destroy_args *args)
  */
 int lib_interface_zebra_shutdown_modify(struct nb_cb_modify_args *args)
 {
+	if (args->event != NB_EV_APPLY)
+		return NB_OK;
+
 	struct interface *ifp;
 
 	ifp = nb_running_get_entry(args->dnode, NULL, true);
@@ -1165,6 +1075,9 @@ int lib_interface_zebra_shutdown_modify(struct nb_cb_modify_args *args)
 
 int lib_interface_zebra_shutdown_destroy(struct nb_cb_destroy_args *args)
 {
+	if (args->event != NB_EV_APPLY)
+		return NB_OK;
+
 	struct interface *ifp;
 
 	ifp = nb_running_get_entry(args->dnode, NULL, true);
@@ -1174,6 +1087,50 @@ int lib_interface_zebra_shutdown_destroy(struct nb_cb_destroy_args *args)
 	return NB_OK;
 }
 
+/*
+ * XPath: /frr-interface:lib/interface/frr-zebra:zebra/mpls
+ */
+int lib_interface_zebra_mpls_modify(struct nb_cb_modify_args *args)
+{
+	struct interface *ifp;
+	bool mpls;
+	struct zebra_if *zif;
+
+	if (args->event != NB_EV_APPLY)
+		return NB_OK;
+
+	ifp = nb_running_get_entry(args->dnode, NULL, true);
+	zif = ifp->info;
+	mpls = yang_dnode_get_bool(args->dnode, NULL);
+
+	if (mpls)
+		zif->mpls_config = IF_ZEBRA_DATA_ON;
+	else
+		zif->mpls_config = IF_ZEBRA_DATA_OFF;
+
+	dplane_intf_mpls_modify_state(ifp, mpls);
+
+	return NB_OK;
+}
+
+int lib_interface_zebra_mpls_destroy(struct nb_cb_destroy_args *args)
+{
+	struct interface *ifp;
+	struct zebra_if *zif;
+
+	if (args->event != NB_EV_APPLY)
+		return NB_OK;
+
+	ifp = nb_running_get_entry(args->dnode, NULL, true);
+	zif = ifp->info;
+
+	zif->mpls_config = IF_ZEBRA_DATA_UNSPEC;
+
+	/* keep the state as it is */
+
+	return NB_OK;
+}
+
 /*
  * XPath: /frr-interface:lib/interface/frr-zebra:zebra/bandwidth
  */
@@ -1216,9 +1173,401 @@ int lib_interface_zebra_bandwidth_destroy(struct nb_cb_destroy_args *args)
 }
 
 /*
- * XPath: /frr-vrf:lib/vrf/frr-zebra:ribs/rib
+ * XPath:
+ * /frr-interface:lib/interface/frr-zebra:zebra/link-params/legacy-admin-group
  */
-int lib_vrf_ribs_rib_create(struct nb_cb_create_args *args)
+int lib_interface_zebra_legacy_admin_group_modify(
+	struct nb_cb_modify_args *args)
+{
+	struct interface *ifp;
+	struct if_link_params *iflp;
+	uint32_t admin_group_value;
+
+	ifp = nb_running_get_entry(args->dnode, NULL, true);
+	admin_group_value = yang_dnode_get_uint32(args->dnode, ".");
+
+	if (!ifp)
+		return NB_ERR_RESOURCE;
+
+	iflp = if_link_params_get(ifp);
+
+	switch (args->event) {
+	case NB_EV_VALIDATE:
+	case NB_EV_PREPARE:
+	case NB_EV_ABORT:
+		break;
+	case NB_EV_APPLY:
+		if (!iflp)
+			iflp = if_link_params_enable(ifp);
+
+		iflp->admin_grp = admin_group_value;
+		SET_PARAM(iflp, LP_ADM_GRP);
+
+		admin_group_clear(&iflp->ext_admin_grp);
+		UNSET_PARAM(iflp, LP_EXTEND_ADM_GRP);
+
+		if (if_is_operative(ifp))
+			zebra_interface_parameters_update(ifp);
+		break;
+	}
+	return NB_OK;
+}
+
+int lib_interface_zebra_legacy_admin_group_destroy(
+	struct nb_cb_destroy_args *args)
+{
+	struct interface *ifp;
+	struct if_link_params *iflp;
+
+	ifp = nb_running_get_entry(args->dnode, NULL, true);
+
+	if (!ifp)
+		return NB_ERR_RESOURCE;
+
+	iflp = if_link_params_get(ifp);
+
+	switch (args->event) {
+	case NB_EV_VALIDATE:
+	case NB_EV_PREPARE:
+	case NB_EV_ABORT:
+		break;
+	case NB_EV_APPLY:
+		if (!iflp)
+			iflp = if_link_params_enable(ifp);
+
+		iflp->admin_grp = 0;
+		UNSET_PARAM(iflp, LP_ADM_GRP);
+
+		if (if_is_operative(ifp))
+			zebra_interface_parameters_update(ifp);
+		break;
+	}
+	return NB_OK;
+}
+
+/*
+ * XPath:
+ * /frr-interface:lib/interface/frr-zebra:zebra/link-params/affinities/affinity
+ */
+int lib_interface_zebra_affinity_create(struct nb_cb_create_args *args)
+{
+	struct interface *ifp;
+	const char *affname;
+	struct if_link_params *iflp;
+	struct affinity_map *affmap;
+	enum affinity_mode affinity_mode;
+
+
+	ifp = nb_running_get_entry(args->dnode, NULL, true);
+	affname = yang_dnode_get_string(args->dnode, ".");
+	affinity_mode = yang_dnode_get_enum(args->dnode, "../../affinity-mode");
+
+	if (!ifp)
+		return NB_ERR_RESOURCE;
+
+	affmap = affinity_map_get(affname);
+	iflp = if_link_params_get(ifp);
+
+	switch (args->event) {
+	case NB_EV_VALIDATE:
+		if (!affmap) {
+			snprintf(args->errmsg, args->errmsg_len,
+				 "affinity-map %s not found.", affname);
+			return NB_ERR_VALIDATION;
+		}
+		if (affinity_mode == AFFINITY_MODE_STANDARD &&
+		    affmap->bit_position > 31) {
+			snprintf(
+				args->errmsg, args->errmsg_len,
+				"affinity %s bit-position %d is not compatible with affinity-mode standard (bit-position > 31).",
+				affname, affmap->bit_position);
+			return NB_ERR_VALIDATION;
+		}
+		break;
+	case NB_EV_PREPARE:
+	case NB_EV_ABORT:
+		break;
+	case NB_EV_APPLY:
+		if (!iflp)
+			iflp = if_link_params_enable(ifp);
+
+		if (affmap->bit_position < 32 &&
+		    (affinity_mode == AFFINITY_MODE_STANDARD ||
+		     affinity_mode == AFFINITY_MODE_BOTH)) {
+			iflp->admin_grp |= 1 << affmap->bit_position;
+			SET_PARAM(iflp, LP_ADM_GRP);
+		}
+		if (affinity_mode == AFFINITY_MODE_EXTENDED ||
+		    affinity_mode == AFFINITY_MODE_BOTH) {
+			admin_group_set(&iflp->ext_admin_grp,
+					affmap->bit_position);
+			SET_PARAM(iflp, LP_EXTEND_ADM_GRP);
+		}
+
+		if (if_is_operative(ifp))
+			zebra_interface_parameters_update(ifp);
+		break;
+	}
+	return NB_OK;
+}
+
+int lib_interface_zebra_affinity_destroy(struct nb_cb_destroy_args *args)
+{
+	struct interface *ifp;
+	const char *affname;
+	struct if_link_params *iflp;
+	struct affinity_map *affmap;
+	enum affinity_mode affinity_mode;
+
+	ifp = nb_running_get_entry(args->dnode, NULL, true);
+	affname = yang_dnode_get_string(args->dnode, ".");
+	affinity_mode = yang_dnode_get_enum(args->dnode, "../../affinity-mode");
+
+	if (!ifp)
+		return NB_ERR_RESOURCE;
+
+	affmap = affinity_map_get(affname);
+	iflp = if_link_params_get(ifp);
+
+	switch (args->event) {
+	case NB_EV_VALIDATE:
+		if (!affmap) {
+			snprintf(args->errmsg, args->errmsg_len,
+				 "affinity-map %s not found.", affname);
+			return NB_ERR_VALIDATION;
+		}
+		break;
+	case NB_EV_PREPARE:
+	case NB_EV_ABORT:
+		break;
+	case NB_EV_APPLY:
+		if (!iflp)
+			return NB_OK;
+		if (affmap->bit_position < 32 &&
+		    (affinity_mode == AFFINITY_MODE_STANDARD ||
+		     affinity_mode == AFFINITY_MODE_BOTH)) {
+			iflp->admin_grp &= ~(1 << affmap->bit_position);
+			if (iflp->admin_grp == 0)
+				UNSET_PARAM(iflp, LP_ADM_GRP);
+		}
+		if (affinity_mode == AFFINITY_MODE_EXTENDED ||
+		    affinity_mode == AFFINITY_MODE_BOTH) {
+			admin_group_unset(&iflp->ext_admin_grp,
+					  affmap->bit_position);
+			if (admin_group_zero(&iflp->ext_admin_grp))
+				UNSET_PARAM(iflp, LP_EXTEND_ADM_GRP);
+		}
+
+		if (if_is_operative(ifp))
+			zebra_interface_parameters_update(ifp);
+		break;
+	}
+	return NB_OK;
+}
+
+/*
+ * XPath:
+ * /frr-interface:lib/interface/frr-zebra:zebra/link-params/affinity-mode
+ */
+int lib_interface_zebra_affinity_mode_modify(struct nb_cb_modify_args *args)
+{
+	struct interface *ifp;
+	struct if_link_params *iflp;
+	enum affinity_mode affinity_mode;
+
+
+	ifp = nb_running_get_entry(args->dnode, NULL, true);
+	affinity_mode = yang_dnode_get_enum(args->dnode, ".");
+
+	if (!ifp)
+		return NB_ERR_RESOURCE;
+
+	iflp = if_link_params_get(ifp);
+
+	switch (args->event) {
+	case NB_EV_VALIDATE:
+		if (affinity_mode == AFFINITY_MODE_STANDARD &&
+		    admin_group_nb_words(&iflp->ext_admin_grp) > 1) {
+			snprintf(
+				args->errmsg, args->errmsg_len,
+				"affinity-mode standard cannot be set when a bit-position > 31 is set.");
+			return NB_ERR_VALIDATION;
+		}
+		break;
+	case NB_EV_PREPARE:
+	case NB_EV_ABORT:
+		break;
+	case NB_EV_APPLY:
+		if (!iflp)
+			iflp = if_link_params_enable(ifp);
+		if (affinity_mode == AFFINITY_MODE_STANDARD) {
+			if (!IS_PARAM_SET(iflp, LP_ADM_GRP) &&
+			    IS_PARAM_SET(iflp, LP_EXTEND_ADM_GRP)) {
+				iflp->admin_grp = admin_group_get_offset(
+					&iflp->ext_admin_grp, 0);
+				SET_PARAM(iflp, LP_ADM_GRP);
+			}
+			admin_group_clear(&iflp->ext_admin_grp);
+			UNSET_PARAM(iflp, LP_EXTEND_ADM_GRP);
+		}
+		if (affinity_mode == AFFINITY_MODE_EXTENDED) {
+			if (!IS_PARAM_SET(iflp, LP_EXTEND_ADM_GRP) &&
+			    IS_PARAM_SET(iflp, LP_ADM_GRP)) {
+				admin_group_bulk_set(&iflp->ext_admin_grp,
+						     iflp->admin_grp, 0);
+				SET_PARAM(iflp, LP_EXTEND_ADM_GRP);
+			}
+			iflp->admin_grp = 0;
+			UNSET_PARAM(iflp, LP_ADM_GRP);
+		}
+		if (affinity_mode == AFFINITY_MODE_BOTH) {
+			if (!IS_PARAM_SET(iflp, LP_EXTEND_ADM_GRP) &&
+			    IS_PARAM_SET(iflp, LP_ADM_GRP)) {
+				admin_group_bulk_set(&iflp->ext_admin_grp,
+						     iflp->admin_grp, 0);
+				SET_PARAM(iflp, LP_EXTEND_ADM_GRP);
+			} else if (!IS_PARAM_SET(iflp, LP_ADM_GRP) &&
+				   IS_PARAM_SET(iflp, LP_EXTEND_ADM_GRP)) {
+				iflp->admin_grp = admin_group_get_offset(
+					&iflp->ext_admin_grp, 0);
+				SET_PARAM(iflp, LP_ADM_GRP);
+			}
+		}
+
+		if (if_is_operative(ifp))
+			zebra_interface_parameters_update(ifp);
+		break;
+	}
+	return NB_OK;
+}
+
+/*
+ * XPath: /frr-vrf:lib/vrf/frr-zebra:zebra/l3vni-id
+ */
+int lib_vrf_zebra_l3vni_id_modify(struct nb_cb_modify_args *args)
+{
+	struct vrf *vrf;
+	struct zebra_vrf *zvrf;
+	vni_t vni = 0;
+	struct zebra_l3vni *zl3vni = NULL;
+	char err[ERR_STR_SZ];
+	bool pfx_only = false;
+	const struct lyd_node *pn_dnode;
+	const char *vrfname;
+
+	switch (args->event) {
+	case NB_EV_PREPARE:
+	case NB_EV_ABORT:
+		return NB_OK;
+	case NB_EV_VALIDATE:
+		vni = yang_dnode_get_uint32(args->dnode, NULL);
+		/* Get vrf info from parent node, reject configuration
+		 * if zebra vrf already mapped to different vni id.
+		 */
+		pn_dnode = yang_dnode_get_parent(args->dnode, "vrf");
+		vrfname = yang_dnode_get_string(pn_dnode, "./name");
+		zvrf = zebra_vrf_lookup_by_name(vrfname);
+		if (!zvrf) {
+			snprintf(args->errmsg, args->errmsg_len,
+				 "zebra vrf info not found for vrf:%s.",
+				 vrfname);
+			return NB_ERR_VALIDATION;
+		}
+		if (zvrf->l3vni && zvrf->l3vni != vni) {
+			snprintf(
+				args->errmsg, args->errmsg_len,
+				"vni %u cannot be configured as vni %u is already configured under the vrf",
+				vni, zvrf->l3vni);
+			return NB_ERR_VALIDATION;
+		}
+
+		/* Check if this VNI is already present in the system */
+		zl3vni = zl3vni_lookup(vni);
+		if (zl3vni) {
+			snprintf(args->errmsg, args->errmsg_len,
+				 "VNI %u is already configured as L3-VNI", vni);
+			return NB_ERR_VALIDATION;
+		}
+
+		break;
+	case NB_EV_APPLY:
+
+		vrf = nb_running_get_entry(args->dnode, NULL, true);
+		zvrf = zebra_vrf_lookup_by_name(vrf->name);
+		vni = yang_dnode_get_uint32(args->dnode, NULL);
+		/* Note: This covers lib_vrf_zebra_prefix_only_modify() config
+		 * along with l3vni config
+		 */
+		pfx_only = yang_dnode_get_bool(args->dnode, "../prefix-only");
+
+		if (zebra_vxlan_process_vrf_vni_cmd(zvrf, vni, err, ERR_STR_SZ,
+						    pfx_only ? 1 : 0, 1)
+		    != 0) {
+			if (IS_ZEBRA_DEBUG_VXLAN)
+				snprintf(
+					args->errmsg, args->errmsg_len,
+					"vrf vni %u mapping failed with error: %s",
+					vni, err);
+			return NB_ERR;
+		}
+
+		break;
+	}
+
+	return NB_OK;
+}
+
+int lib_vrf_zebra_l3vni_id_destroy(struct nb_cb_destroy_args *args)
+{
+	struct vrf *vrf;
+	struct zebra_vrf *zvrf;
+	vni_t vni = 0;
+	char err[ERR_STR_SZ];
+	uint8_t filter = 0;
+
+	switch (args->event) {
+	case NB_EV_PREPARE:
+	case NB_EV_ABORT:
+	case NB_EV_VALIDATE:
+		return NB_OK;
+	case NB_EV_APPLY:
+		vrf = nb_running_get_entry(args->dnode, NULL, true);
+		zvrf = zebra_vrf_lookup_by_name(vrf->name);
+		vni = yang_dnode_get_uint32(args->dnode, NULL);
+
+		if (!zl3vni_lookup(vni))
+			return NB_OK;
+
+		if (zvrf->l3vni != vni) {
+			snprintf(args->errmsg, args->errmsg_len,
+				 "vrf %s has different vni %u mapped",
+				 vrf->name, zvrf->l3vni);
+			return NB_ERR;
+		}
+
+		if (is_l3vni_for_prefix_routes_only(zvrf->l3vni))
+			filter = 1;
+
+		if (zebra_vxlan_process_vrf_vni_cmd(zvrf, vni, err, ERR_STR_SZ,
+						    filter, 0)
+		    != 0) {
+			if (IS_ZEBRA_DEBUG_VXLAN)
+				zlog_debug(
+					"vrf vni %u unmapping failed with error: %s",
+					vni, err);
+			return NB_ERR;
+		}
+
+		break;
+	}
+
+	return NB_OK;
+}
+
+/*
+ * XPath: /frr-vrf:lib/vrf/frr-zebra:zebra/prefix-only
+ */
+int lib_vrf_zebra_prefix_only_modify(struct nb_cb_modify_args *args)
 {
 	switch (args->event) {
 	case NB_EV_VALIDATE:
@@ -1231,329 +1580,3 @@ int lib_vrf_ribs_rib_create(struct nb_cb_create_args *args)
 
 	return NB_OK;
 }
-
-int lib_vrf_ribs_rib_destroy(struct nb_cb_destroy_args *args)
-{
-	switch (args->event) {
-	case NB_EV_VALIDATE:
-	case NB_EV_PREPARE:
-	case NB_EV_ABORT:
-	case NB_EV_APPLY:
-		/* TODO: implement me. */
-		break;
-	}
-
-	return NB_OK;
-}
-
-
-/*
- * XPath:
- * /frr-route-map:lib/route-map/entry/match-condition/frr-zebra:ipv4-prefix-length
- */
-int lib_route_map_entry_match_condition_ipv4_prefix_length_modify(
-	struct nb_cb_modify_args *args)
-{
-	struct routemap_hook_context *rhc;
-	const char *length;
-	int condition, rv;
-
-	if (args->event != NB_EV_APPLY)
-		return NB_OK;
-
-	/* Add configuration. */
-	rhc = nb_running_get_entry(args->dnode, NULL, true);
-	length = yang_dnode_get_string(args->dnode, NULL);
-	condition =
-		yang_dnode_get_enum(args->dnode, "../frr-route-map:condition");
-
-	/* Set destroy information. */
-	switch (condition) {
-	case 100: /* ipv4-prefix-length */
-		rhc->rhc_rule = "ip address prefix-len";
-		break;
-
-	case 102: /* ipv4-next-hop-prefix-length */
-		rhc->rhc_rule = "ip next-hop prefix-len";
-		break;
-	}
-	rhc->rhc_mhook = generic_match_delete;
-	rhc->rhc_event = RMAP_EVENT_MATCH_DELETED;
-
-	rv = generic_match_add(NULL, rhc->rhc_rmi, rhc->rhc_rule, length,
-			       RMAP_EVENT_MATCH_ADDED);
-	if (rv != CMD_SUCCESS) {
-		rhc->rhc_mhook = NULL;
-		return NB_ERR_INCONSISTENCY;
-	}
-
-	return NB_OK;
-}
-
-int lib_route_map_entry_match_condition_ipv4_prefix_length_destroy(
-	struct nb_cb_destroy_args *args)
-{
-	return lib_route_map_entry_match_destroy(args);
-}
-
-/*
- * XPath:
- * /frr-route-map:lib/route-map/entry/match-condition/frr-zebra:ipv6-prefix-length
- */
-int lib_route_map_entry_match_condition_ipv6_prefix_length_modify(
-	struct nb_cb_modify_args *args)
-{
-	struct routemap_hook_context *rhc;
-	const char *length;
-	int rv;
-
-	if (args->event != NB_EV_APPLY)
-		return NB_OK;
-
-	/* Add configuration. */
-	rhc = nb_running_get_entry(args->dnode, NULL, true);
-	length = yang_dnode_get_string(args->dnode, NULL);
-
-	/* Set destroy information. */
-	rhc->rhc_mhook = generic_match_delete;
-	rhc->rhc_rule = "ipv6 address prefix-len";
-	rhc->rhc_event = RMAP_EVENT_MATCH_DELETED;
-
-	rv = generic_match_add(NULL, rhc->rhc_rmi, "ipv6 address prefix-len",
-			       length, RMAP_EVENT_MATCH_ADDED);
-	if (rv != CMD_SUCCESS) {
-		rhc->rhc_mhook = NULL;
-		return NB_ERR_INCONSISTENCY;
-	}
-
-	return NB_OK;
-}
-
-int lib_route_map_entry_match_condition_ipv6_prefix_length_destroy(
-	struct nb_cb_destroy_args *args)
-{
-	return lib_route_map_entry_match_destroy(args);
-}
-
-/*
- * XPath:
- * /frr-route-map:lib/route-map/entry/match-condition/frr-zebra:source-protocol
- */
-int lib_route_map_entry_match_condition_source_protocol_modify(
-	struct nb_cb_modify_args *args)
-{
-	struct routemap_hook_context *rhc;
-	const char *type;
-	int rv;
-
-	switch (args->event) {
-	case NB_EV_VALIDATE:
-		type = yang_dnode_get_string(args->dnode, NULL);
-		if (proto_name2num(type) == -1) {
-			zlog_warn("%s: invalid protocol: %s", __func__, type);
-			return NB_ERR_VALIDATION;
-		}
-		return NB_OK;
-	case NB_EV_PREPARE:
-	case NB_EV_ABORT:
-		return NB_OK;
-	case NB_EV_APPLY:
-		/* NOTHING */
-		break;
-	}
-
-	/* Add configuration. */
-	rhc = nb_running_get_entry(args->dnode, NULL, true);
-	type = yang_dnode_get_string(args->dnode, NULL);
-
-	/* Set destroy information. */
-	rhc->rhc_mhook = generic_match_delete;
-	rhc->rhc_rule = "source-protocol";
-	rhc->rhc_event = RMAP_EVENT_MATCH_DELETED;
-
-	rv = generic_match_add(NULL, rhc->rhc_rmi, "source-protocol", type,
-			       RMAP_EVENT_MATCH_ADDED);
-	if (rv != CMD_SUCCESS) {
-		rhc->rhc_mhook = NULL;
-		return NB_ERR_INCONSISTENCY;
-	}
-
-	return NB_OK;
-}
-
-int lib_route_map_entry_match_condition_source_protocol_destroy(
-	struct nb_cb_destroy_args *args)
-{
-	return lib_route_map_entry_match_destroy(args);
-}
-
-/*
- * XPath:
- * /frr-route-map:lib/route-map/entry/match-condition/frr-zebra:source-instance
- */
-int lib_route_map_entry_match_condition_source_instance_modify(
-	struct nb_cb_modify_args *args)
-{
-	struct routemap_hook_context *rhc;
-	const char *type;
-	int rv;
-
-	if (args->event != NB_EV_APPLY)
-		return NB_OK;
-
-	/* Add configuration. */
-	rhc = nb_running_get_entry(args->dnode, NULL, true);
-	type = yang_dnode_get_string(args->dnode, NULL);
-
-	/* Set destroy information. */
-	rhc->rhc_mhook = generic_match_delete;
-	rhc->rhc_rule = "source-instance";
-	rhc->rhc_event = RMAP_EVENT_MATCH_DELETED;
-
-	rv = generic_match_add(NULL, rhc->rhc_rmi, "source-instance", type,
-			       RMAP_EVENT_MATCH_ADDED);
-	if (rv != CMD_SUCCESS) {
-		rhc->rhc_mhook = NULL;
-		return NB_ERR_INCONSISTENCY;
-	}
-
-	return NB_OK;
-}
-
-int lib_route_map_entry_match_condition_source_instance_destroy(
-	struct nb_cb_destroy_args *args)
-{
-	return lib_route_map_entry_match_destroy(args);
-}
-
-/*
- * XPath: /frr-route-map:lib/route-map/entry/set-action/frr-zebra:source-v4
- */
-int lib_route_map_entry_set_action_source_v4_modify(
-	struct nb_cb_modify_args *args)
-{
-	struct routemap_hook_context *rhc;
-	struct interface *pif = NULL;
-	const char *source;
-	struct vrf *vrf;
-	struct prefix p;
-	int rv;
-
-	switch (args->event) {
-	case NB_EV_VALIDATE:
-		memset(&p, 0, sizeof(p));
-		yang_dnode_get_ipv4p(&p, args->dnode, NULL);
-		if (zebra_check_addr(&p) == 0) {
-			zlog_warn("%s: invalid IPv4 address: %s", __func__,
-				  yang_dnode_get_string(args->dnode, NULL));
-			return NB_ERR_VALIDATION;
-		}
-
-		RB_FOREACH (vrf, vrf_id_head, &vrfs_by_id) {
-			pif = if_lookup_exact_address(&p.u.prefix4, AF_INET,
-						      vrf->vrf_id);
-			if (pif != NULL)
-				break;
-		}
-		if (pif == NULL) {
-			zlog_warn("%s: is not a local adddress: %s", __func__,
-				  yang_dnode_get_string(args->dnode, NULL));
-			return NB_ERR_VALIDATION;
-		}
-		return NB_OK;
-	case NB_EV_PREPARE:
-	case NB_EV_ABORT:
-		return NB_OK;
-	case NB_EV_APPLY:
-		/* NOTHING */
-		break;
-	}
-
-	/* Add configuration. */
-	rhc = nb_running_get_entry(args->dnode, NULL, true);
-	source = yang_dnode_get_string(args->dnode, NULL);
-
-	/* Set destroy information. */
-	rhc->rhc_shook = generic_set_delete;
-	rhc->rhc_rule = "src";
-
-	rv = generic_set_add(NULL, rhc->rhc_rmi, "src", source);
-	if (rv != CMD_SUCCESS) {
-		rhc->rhc_shook = NULL;
-		return NB_ERR_INCONSISTENCY;
-	}
-
-	return NB_OK;
-}
-
-int lib_route_map_entry_set_action_source_v4_destroy(
-	struct nb_cb_destroy_args *args)
-{
-	return lib_route_map_entry_set_destroy(args);
-}
-
-/*
- * XPath: /frr-route-map:lib/route-map/entry/set-action/frr-zebra:source-v6
- */
-int lib_route_map_entry_set_action_source_v6_modify(
-	struct nb_cb_modify_args *args)
-{
-	struct routemap_hook_context *rhc;
-	struct interface *pif = NULL;
-	const char *source;
-	struct vrf *vrf;
-	struct prefix p;
-	int rv;
-
-	switch (args->event) {
-	case NB_EV_VALIDATE:
-		memset(&p, 0, sizeof(p));
-		yang_dnode_get_ipv6p(&p, args->dnode, NULL);
-		if (zebra_check_addr(&p) == 0) {
-			zlog_warn("%s: invalid IPv6 address: %s", __func__,
-				  yang_dnode_get_string(args->dnode, NULL));
-			return NB_ERR_VALIDATION;
-		}
-
-		RB_FOREACH (vrf, vrf_id_head, &vrfs_by_id) {
-			pif = if_lookup_exact_address(&p.u.prefix6, AF_INET6,
-						      vrf->vrf_id);
-			if (pif != NULL)
-				break;
-		}
-		if (pif == NULL) {
-			zlog_warn("%s: is not a local adddress: %s", __func__,
-				  yang_dnode_get_string(args->dnode, NULL));
-			return NB_ERR_VALIDATION;
-		}
-		return NB_OK;
-	case NB_EV_PREPARE:
-	case NB_EV_ABORT:
-		return NB_OK;
-	case NB_EV_APPLY:
-		/* NOTHING */
-		break;
-	}
-
-	/* Add configuration. */
-	rhc = nb_running_get_entry(args->dnode, NULL, true);
-	source = yang_dnode_get_string(args->dnode, NULL);
-
-	/* Set destroy information. */
-	rhc->rhc_shook = generic_set_delete;
-	rhc->rhc_rule = "src";
-
-	rv = generic_set_add(NULL, rhc->rhc_rmi, "src", source);
-	if (rv != CMD_SUCCESS) {
-		rhc->rhc_shook = NULL;
-		return NB_ERR_INCONSISTENCY;
-	}
-
-	return NB_OK;
-}
-
-int lib_route_map_entry_set_action_source_v6_destroy(
-	struct nb_cb_destroy_args *args)
-{
-	return lib_route_map_entry_set_destroy(args);
-}
