@@ -1,21 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* Kernel routing table updates using netlink over GNU/Linux system.
  * Copyright (C) 1997, 98, 99 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -31,7 +16,7 @@
 #include "ptm_lib.h"
 #include "rib.h"
 #include "stream.h"
-#include "version.h"
+#include "lib/version.h"
 #include "vrf.h"
 #include "vty.h"
 #include "lib_errors.h"
@@ -100,7 +85,7 @@ static ptm_lib_handle_t *ptm_hdl;
 struct zebra_ptm_cb ptm_cb;
 
 static int zebra_ptm_socket_init(void);
-int zebra_ptm_sock_read(struct thread *);
+void zebra_ptm_sock_read(struct event *thread);
 static void zebra_ptm_install_commands(void);
 static int zebra_ptm_handle_msg_cb(void *arg, void *in_ctxt);
 void zebra_bfd_peer_replay_req(void);
@@ -114,7 +99,7 @@ void zebra_ptm_init(void)
 {
 	char buf[64];
 
-	memset(&ptm_cb, 0, sizeof(struct zebra_ptm_cb));
+	memset(&ptm_cb, 0, sizeof(ptm_cb));
 
 	ptm_cb.out_data = calloc(1, ZEBRA_PTM_SEND_MAX_SOCKBUF);
 	if (!ptm_cb.out_data) {
@@ -156,13 +141,10 @@ void zebra_ptm_finish(void)
 	if (ptm_cb.in_data)
 		free(ptm_cb.in_data);
 
-	/* Release threads. */
-	if (ptm_cb.t_read)
-		thread_cancel(ptm_cb.t_read);
-	if (ptm_cb.t_write)
-		thread_cancel(ptm_cb.t_write);
-	if (ptm_cb.t_timer)
-		thread_cancel(ptm_cb.t_timer);
+	/* Cancel events. */
+	EVENT_OFF(ptm_cb.t_read);
+	EVENT_OFF(ptm_cb.t_write);
+	EVENT_OFF(ptm_cb.t_timer);
 
 	if (ptm_cb.wb)
 		buffer_free(ptm_cb.wb);
@@ -171,12 +153,12 @@ void zebra_ptm_finish(void)
 		close(ptm_cb.ptm_sock);
 }
 
-static int zebra_ptm_flush_messages(struct thread *thread)
+static void zebra_ptm_flush_messages(struct event *thread)
 {
 	ptm_cb.t_write = NULL;
 
 	if (ptm_cb.ptm_sock == -1)
-		return -1;
+		return;
 
 	errno = 0;
 
@@ -188,19 +170,17 @@ static int zebra_ptm_flush_messages(struct thread *thread)
 		ptm_cb.ptm_sock = -1;
 		zebra_ptm_reset_status(0);
 		ptm_cb.t_timer = NULL;
-		thread_add_timer(zrouter.master, zebra_ptm_connect, NULL,
-				 ptm_cb.reconnect_time, &ptm_cb.t_timer);
-		return -1;
+		event_add_timer(zrouter.master, zebra_ptm_connect, NULL,
+				ptm_cb.reconnect_time, &ptm_cb.t_timer);
+		return;
 	case BUFFER_PENDING:
 		ptm_cb.t_write = NULL;
-		thread_add_write(zrouter.master, zebra_ptm_flush_messages, NULL,
-				 ptm_cb.ptm_sock, &ptm_cb.t_write);
+		event_add_write(zrouter.master, zebra_ptm_flush_messages, NULL,
+				ptm_cb.ptm_sock, &ptm_cb.t_write);
 		break;
 	case BUFFER_EMPTY:
 		break;
 	}
-
-	return 0;
 }
 
 static int zebra_ptm_send_message(char *data, int size)
@@ -214,22 +194,22 @@ static int zebra_ptm_send_message(char *data, int size)
 		ptm_cb.ptm_sock = -1;
 		zebra_ptm_reset_status(0);
 		ptm_cb.t_timer = NULL;
-		thread_add_timer(zrouter.master, zebra_ptm_connect, NULL,
-				 ptm_cb.reconnect_time, &ptm_cb.t_timer);
+		event_add_timer(zrouter.master, zebra_ptm_connect, NULL,
+				ptm_cb.reconnect_time, &ptm_cb.t_timer);
 		return -1;
 	case BUFFER_EMPTY:
-		THREAD_OFF(ptm_cb.t_write);
+		EVENT_OFF(ptm_cb.t_write);
 		break;
 	case BUFFER_PENDING:
-		thread_add_write(zrouter.master, zebra_ptm_flush_messages, NULL,
-				 ptm_cb.ptm_sock, &ptm_cb.t_write);
+		event_add_write(zrouter.master, zebra_ptm_flush_messages, NULL,
+				ptm_cb.ptm_sock, &ptm_cb.t_write);
 		break;
 	}
 
 	return 0;
 }
 
-int zebra_ptm_connect(struct thread *t)
+void zebra_ptm_connect(struct event *t)
 {
 	int init = 0;
 
@@ -241,8 +221,8 @@ int zebra_ptm_connect(struct thread *t)
 	if (ptm_cb.ptm_sock != -1) {
 		if (init) {
 			ptm_cb.t_read = NULL;
-			thread_add_read(zrouter.master, zebra_ptm_sock_read,
-					NULL, ptm_cb.ptm_sock, &ptm_cb.t_read);
+			event_add_read(zrouter.master, zebra_ptm_sock_read,
+				       NULL, ptm_cb.ptm_sock, &ptm_cb.t_read);
 			zebra_bfd_peer_replay_req();
 		}
 		zebra_ptm_send_status_req();
@@ -253,13 +233,11 @@ int zebra_ptm_connect(struct thread *t)
 			ptm_cb.reconnect_time = ZEBRA_PTM_RECONNECT_TIME_MAX;
 
 		ptm_cb.t_timer = NULL;
-		thread_add_timer(zrouter.master, zebra_ptm_connect, NULL,
-				 ptm_cb.reconnect_time, &ptm_cb.t_timer);
+		event_add_timer(zrouter.master, zebra_ptm_connect, NULL,
+				ptm_cb.reconnect_time, &ptm_cb.t_timer);
 	} else if (ptm_cb.reconnect_time >= ZEBRA_PTM_RECONNECT_TIME_MAX) {
 		ptm_cb.reconnect_time = ZEBRA_PTM_RECONNECT_TIME_INITIAL;
 	}
-
-	return (errno);
 }
 
 DEFUN (zebra_ptm_enable,
@@ -357,7 +335,7 @@ DEFUN (no_zebra_ptm_enable_if,
 			if (IS_ZEBRA_DEBUG_EVENT)
 				zlog_debug("%s: Bringing up interface %s",
 					   __func__, ifp->name);
-			if_up(ifp);
+			if_up(ifp, true);
 		}
 	}
 
@@ -396,7 +374,7 @@ static int zebra_ptm_socket_init(void)
 	}
 
 	/* Make server socket. */
-	memset(&addr, 0, sizeof(struct sockaddr_un));
+	memset(&addr, 0, sizeof(addr));
 	addr.sun_family = AF_UNIX;
 	memcpy(&addr.sun_path, ZEBRA_PTM_SOCK_NAME,
 	       sizeof(ZEBRA_PTM_SOCK_NAME));
@@ -434,8 +412,7 @@ static void if_bfd_session_update(struct interface *ifp, struct prefix *dp,
 
 		if (ifp) {
 			zlog_debug(
-				"MESSAGE: ZEBRA_INTERFACE_BFD_DEST_UPDATE %s/%d on %s"
-				" %s event",
+				"MESSAGE: ZEBRA_INTERFACE_BFD_DEST_UPDATE %s/%d on %s %s event",
 				inet_ntop(dp->family, &dp->u.prefix, buf[0],
 					  INET6_ADDRSTRLEN),
 				dp->prefixlen, ifp->name,
@@ -444,8 +421,7 @@ static void if_bfd_session_update(struct interface *ifp, struct prefix *dp,
 			struct vrf *vrf = vrf_lookup_by_id(vrf_id);
 
 			zlog_debug(
-				"MESSAGE: ZEBRA_INTERFACE_BFD_DEST_UPDATE %s/%d "
-				"with src %s/%d and vrf %s(%u) %s event",
+				"MESSAGE: ZEBRA_INTERFACE_BFD_DEST_UPDATE %s/%d with src %s/%d and vrf %s(%u) %s event",
 				inet_ntop(dp->family, &dp->u.prefix, buf[0],
 					  INET6_ADDRSTRLEN),
 				dp->prefixlen,
@@ -502,8 +478,7 @@ static int zebra_ptm_handle_bfd_msg(void *arg, void *in_ctxt,
 
 	if (IS_ZEBRA_DEBUG_EVENT)
 		zlog_debug(
-			"%s: Recv Port [%s] bfd status [%s] vrf [%s]"
-			" peer [%s] local [%s]",
+			"%s: Recv Port [%s] bfd status [%s] vrf [%s] peer [%s] local [%s]",
 			__func__, ifp ? ifp->name : "N/A", bfdst_str, vrf_str,
 			dest_str, src_str);
 
@@ -513,7 +488,7 @@ static int zebra_ptm_handle_bfd_msg(void *arg, void *in_ctxt,
 		return -1;
 	}
 
-	memset(&src_prefix, 0, sizeof(struct prefix));
+	memset(&src_prefix, 0, sizeof(src_prefix));
 	if (strcmp(ZEBRA_PTM_INVALID_SRC_IP, src_str)) {
 		if (str2prefix(src_str, &src_prefix) == 0) {
 			flog_err(EC_ZEBRA_PREFIX_PARSE_ERROR,
@@ -524,9 +499,15 @@ static int zebra_ptm_handle_bfd_msg(void *arg, void *in_ctxt,
 	}
 
 	if (!strcmp(ZEBRA_PTM_INVALID_VRF, vrf_str) && ifp) {
-		vrf_id = ifp->vrf_id;
+		vrf_id = ifp->vrf->vrf_id;
 	} else {
-		vrf_id = vrf_name_to_id(vrf_str);
+		struct vrf *pVrf;
+
+		pVrf = vrf_lookup_by_name(vrf_str);
+		if (pVrf)
+			vrf_id = pVrf->vrf_id;
+		else
+			vrf_id = VRF_DEFAULT;
 	}
 
 	if (!strcmp(bfdst_str, ZEBRA_PTM_BFDSTATUS_DOWN_STR)) {
@@ -557,7 +538,7 @@ static int zebra_ptm_handle_cbl_msg(void *arg, void *in_ctxt,
 		ifp->ptm_status = ZEBRA_PTM_STATUS_UP;
 		if (ifp->ptm_enable && if_is_no_ptm_operative(ifp)
 		    && send_linkup)
-			if_up(ifp);
+			if_up(ifp, true);
 	} else if (!strcmp(cbl_str, ZEBRA_PTM_FAIL_STR)
 		   && (ifp->ptm_status != ZEBRA_PTM_STATUS_DOWN)) {
 		ifp->ptm_status = ZEBRA_PTM_STATUS_DOWN;
@@ -609,7 +590,17 @@ static int zebra_ptm_handle_msg_cb(void *arg, void *in_ctxt)
 	}
 
 	if (strcmp(ZEBRA_PTM_INVALID_PORT_NAME, port_str)) {
-		ifp = if_lookup_by_name_all_vrf(port_str);
+		struct vrf *vrf;
+		int count = 0;
+
+		RB_FOREACH (vrf, vrf_id_head, &vrfs_by_id) {
+			ifp = if_lookup_by_name_vrf(port_str, vrf);
+			if (ifp) {
+				count++;
+				if (!vrf_is_backend_netns())
+					break;
+			}
+		}
 
 		if (!ifp) {
 			flog_warn(EC_ZEBRA_UNKNOWN_INTERFACE,
@@ -617,6 +608,12 @@ static int zebra_ptm_handle_msg_cb(void *arg, void *in_ctxt)
 				  __func__, port_str);
 			return -1;
 		}
+		if (count > 1) {
+			flog_warn(EC_ZEBRA_UNKNOWN_INTERFACE,
+				  "%s: multiple interface with name %s",
+				  __func__, port_str);
+			return -1;
+		}
 	}
 
 	ptm_lib_find_key_in_msg(in_ctxt, ZEBRA_PTM_CBL_STR, cbl_str);
@@ -633,16 +630,16 @@ static int zebra_ptm_handle_msg_cb(void *arg, void *in_ctxt)
 	}
 }
 
-int zebra_ptm_sock_read(struct thread *thread)
+void zebra_ptm_sock_read(struct event *thread)
 {
 	int sock;
 	int rc;
 
 	errno = 0;
-	sock = THREAD_FD(thread);
+	sock = EVENT_FD(thread);
 
 	if (sock == -1)
-		return -1;
+		return;
 
 	/* PTM communicates in CSV format */
 	do {
@@ -660,17 +657,14 @@ int zebra_ptm_sock_read(struct thread *thread)
 		ptm_cb.ptm_sock = -1;
 		zebra_ptm_reset_status(0);
 		ptm_cb.t_timer = NULL;
-		thread_add_timer(zrouter.master, zebra_ptm_connect, NULL,
-				 ptm_cb.reconnect_time,
-				 &ptm_cb.t_timer);
-		return -1;
+		event_add_timer(zrouter.master, zebra_ptm_connect, NULL,
+				ptm_cb.reconnect_time, &ptm_cb.t_timer);
+		return;
 	}
 
 	ptm_cb.t_read = NULL;
-	thread_add_read(zrouter.master, zebra_ptm_sock_read, NULL,
-			ptm_cb.ptm_sock, &ptm_cb.t_read);
-
-	return 0;
+	event_add_read(zrouter.master, zebra_ptm_sock_read, NULL,
+		       ptm_cb.ptm_sock, &ptm_cb.t_read);
 }
 
 /* BFD peer/dst register/update */
@@ -704,8 +698,8 @@ void zebra_ptm_bfd_dst_register(ZAPI_HANDLER_ARGS)
 
 	if (ptm_cb.ptm_sock == -1) {
 		ptm_cb.t_timer = NULL;
-		thread_add_timer(zrouter.master, zebra_ptm_connect, NULL,
-				 ptm_cb.reconnect_time, &ptm_cb.t_timer);
+		event_add_timer(zrouter.master, zebra_ptm_connect, NULL,
+				ptm_cb.reconnect_time, &ptm_cb.t_timer);
 		return;
 	}
 
@@ -863,8 +857,8 @@ void zebra_ptm_bfd_dst_deregister(ZAPI_HANDLER_ARGS)
 
 	if (ptm_cb.ptm_sock == -1) {
 		ptm_cb.t_timer = NULL;
-		thread_add_timer(zrouter.master, zebra_ptm_connect, NULL,
-				 ptm_cb.reconnect_time, &ptm_cb.t_timer);
+		event_add_timer(zrouter.master, zebra_ptm_connect, NULL,
+				ptm_cb.reconnect_time, &ptm_cb.t_timer);
 		return;
 	}
 
@@ -991,8 +985,8 @@ void zebra_ptm_bfd_client_register(ZAPI_HANDLER_ARGS)
 
 	if (ptm_cb.ptm_sock == -1) {
 		ptm_cb.t_timer = NULL;
-		thread_add_timer(zrouter.master, zebra_ptm_connect, NULL,
-				 ptm_cb.reconnect_time, &ptm_cb.t_timer);
+		event_add_timer(zrouter.master, zebra_ptm_connect, NULL,
+				ptm_cb.reconnect_time, &ptm_cb.t_timer);
 		return;
 	}
 
@@ -1050,8 +1044,8 @@ int zebra_ptm_bfd_client_deregister(struct zserv *client)
 
 	if (ptm_cb.ptm_sock == -1) {
 		ptm_cb.t_timer = NULL;
-		thread_add_timer(zrouter.master, zebra_ptm_connect, NULL,
-				 ptm_cb.reconnect_time, &ptm_cb.t_timer);
+		event_add_timer(zrouter.master, zebra_ptm_connect, NULL,
+				ptm_cb.reconnect_time, &ptm_cb.t_timer);
 		return 0;
 	}
 
@@ -1098,14 +1092,20 @@ static const char *zebra_ptm_get_status_str(int status)
 	}
 }
 
-void zebra_ptm_show_status(struct vty *vty, struct interface *ifp)
+void zebra_ptm_show_status(struct vty *vty, json_object *json,
+			   struct interface *ifp)
 {
-	vty_out(vty, "  PTM status: ");
-	if (ifp->ptm_enable) {
-		vty_out(vty, "%s\n", zebra_ptm_get_status_str(ifp->ptm_status));
-	} else {
-		vty_out(vty, "disabled\n");
-	}
+	const char *status;
+
+	if (ifp->ptm_enable)
+		status = zebra_ptm_get_status_str(ifp->ptm_status);
+	else
+		status = "disabled";
+
+	if (json)
+		json_object_string_add(json, "ptmStatus", status);
+	else
+		vty_out(vty, "  PTM status: %s\n", status);
 }
 
 void zebra_ptm_send_status_req(void)
@@ -1147,7 +1147,7 @@ void zebra_ptm_reset_status(int ptm_disable)
 						zlog_debug(
 							"%s: Bringing up interface %s",
 							__func__, ifp->name);
-					if_up(ifp);
+					if_up(ifp, true);
 				}
 			}
 		}
@@ -1173,8 +1173,6 @@ void zebra_ptm_if_write(struct vty *vty, struct zebra_if *zebra_ifp)
 
 #else /* HAVE_BFDD */
 
-#include "zebra/zebra_memory.h"
-
 /*
  * Data structures.
  */
@@ -1187,7 +1185,7 @@ struct ptm_process {
 TAILQ_HEAD(ppqueue, ptm_process) ppqueue;
 
 DEFINE_MTYPE_STATIC(ZEBRA, ZEBRA_PTM_BFD_PROCESS,
-		    "PTM BFD process registration table.");
+		    "PTM BFD process reg table");
 
 /*
  * Prototypes.
@@ -1280,10 +1278,6 @@ static void zebra_ptm_send_bfdd(struct stream *msg)
 
 	/* Create copy for replication. */
 	msgc = stream_dup(msg);
-	if (msgc == NULL) {
-		zlog_debug("%s: not enough memory", __func__);
-		return;
-	}
 
 	/* Send message to all running BFDd daemons. */
 	for (ALL_LIST_ELEMENTS_RO(zrouter.client_list, node, client)) {
@@ -1294,10 +1288,6 @@ static void zebra_ptm_send_bfdd(struct stream *msg)
 
 		/* Allocate more messages. */
 		msg = stream_dup(msgc);
-		if (msg == NULL) {
-			zlog_debug("%s: not enough memory", __func__);
-			return;
-		}
 	}
 
 	stream_free(msgc);
@@ -1312,10 +1302,6 @@ static void zebra_ptm_send_clients(struct stream *msg)
 
 	/* Create copy for replication. */
 	msgc = stream_dup(msg);
-	if (msgc == NULL) {
-		zlog_debug("%s: not enough memory", __func__);
-		return;
-	}
 
 	/* Send message to all running client daemons. */
 	for (ALL_LIST_ELEMENTS_RO(zrouter.client_list, node, client)) {
@@ -1326,10 +1312,6 @@ static void zebra_ptm_send_clients(struct stream *msg)
 
 		/* Allocate more messages. */
 		msg = stream_dup(msgc);
-		if (msg == NULL) {
-			zlog_debug("%s: not enough memory", __func__);
-			return;
-		}
 	}
 
 	stream_free(msgc);
@@ -1360,7 +1342,7 @@ static int _zebra_ptm_bfd_client_deregister(struct zserv *zs)
 	}
 
 	/*
-	 * The message type will be BFD_DEST_REPLY so we can use only
+	 * The message type will be ZEBRA_BFD_DEST_REPLAY so we can use only
 	 * one callback at the `bfdd` side, however the real command
 	 * number will be included right after the zebra header.
 	 */
@@ -1545,6 +1527,7 @@ int zebra_ptm_get_enable_state(void)
 }
 
 void zebra_ptm_show_status(struct vty *vty __attribute__((__unused__)),
+			   json_object *json __attribute__((__unused__)),
 			   struct interface *ifp __attribute__((__unused__)))
 {
 	/* NOTHING */
