@@ -1,29 +1,17 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* Zebra PW code
  * Copyright (C) 2016 Volta Networks, Inc.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
- * MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
 #include "log.h"
 #include "memory.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "command.h"
 #include "vrf.h"
+#include "lib/json.h"
+#include "printfrr.h"
 
 #include "zebra/debug.h"
 #include "zebra/rib.h"
@@ -33,20 +21,20 @@
 #include "zebra/zebra_vrf.h"
 #include "zebra/zebra_pw.h"
 
-DEFINE_MTYPE_STATIC(LIB, PW, "Pseudowire")
+DEFINE_MTYPE_STATIC(LIB, PW, "Pseudowire");
 
-DEFINE_QOBJ_TYPE(zebra_pw)
+DEFINE_QOBJ_TYPE(zebra_pw);
 
-DEFINE_HOOK(pw_install, (struct zebra_pw * pw), (pw))
-DEFINE_HOOK(pw_uninstall, (struct zebra_pw * pw), (pw))
+DEFINE_HOOK(pw_install, (struct zebra_pw * pw), (pw));
+DEFINE_HOOK(pw_uninstall, (struct zebra_pw * pw), (pw));
 
 #define MPLS_NO_LABEL MPLS_INVALID_LABEL
 
 static int zebra_pw_enabled(struct zebra_pw *);
 static void zebra_pw_install(struct zebra_pw *);
 static void zebra_pw_uninstall(struct zebra_pw *);
-static int zebra_pw_install_retry(struct thread *);
-static int zebra_pw_check_reachability(struct zebra_pw *);
+static void zebra_pw_install_retry(struct event *thread);
+static int zebra_pw_check_reachability(const struct zebra_pw *);
 static void zebra_pw_update_status(struct zebra_pw *, int);
 
 static inline int zebra_pw_compare(const struct zebra_pw *a,
@@ -72,7 +60,7 @@ struct zebra_pw *zebra_pw_add(struct zebra_vrf *zvrf, const char *ifname,
 	pw->protocol = protocol;
 	pw->vrf_id = zvrf_id(zvrf);
 	pw->client = client;
-	pw->status = PW_STATUS_DOWN;
+	pw->status = PW_NOT_FORWARDING;
 	pw->local_label = MPLS_NO_LABEL;
 	pw->remote_label = MPLS_NO_LABEL;
 	pw->flags = F_PSEUDOWIRE_CWORD;
@@ -96,16 +84,18 @@ void zebra_pw_del(struct zebra_vrf *zvrf, struct zebra_pw *pw)
 	zebra_deregister_rnh_pseudowire(pw->vrf_id, pw);
 
 	/* uninstall */
-	if (pw->status == PW_STATUS_UP) {
+	if (pw->status == PW_FORWARDING) {
 		hook_call(pw_uninstall, pw);
 		dplane_pw_uninstall(pw);
-	} else if (pw->install_retry_timer)
-		THREAD_TIMER_OFF(pw->install_retry_timer);
+	}
+
+	EVENT_OFF(pw->install_retry_timer);
 
 	/* unlink and release memory */
 	RB_REMOVE(zebra_pw_head, &zvrf->pseudowires, pw);
 	if (pw->protocol == ZEBRA_ROUTE_STATIC)
 		RB_REMOVE(zebra_static_pw_head, &zvrf->static_pseudowires, pw);
+
 	XFREE(MTYPE_PW, pw);
 }
 
@@ -123,9 +113,12 @@ void zebra_pw_change(struct zebra_pw *pw, ifindex_t ifindex, int type, int af,
 	pw->flags = flags;
 	pw->data = *data;
 
-	if (zebra_pw_enabled(pw))
-		zebra_register_rnh_pseudowire(pw->vrf_id, pw);
-	else {
+	if (zebra_pw_enabled(pw)) {
+		bool nht_exists;
+		zebra_register_rnh_pseudowire(pw->vrf_id, pw, &nht_exists);
+		if (nht_exists)
+			zebra_pw_update(pw);
+	} else {
 		if (pw->protocol == ZEBRA_ROUTE_STATIC)
 			zebra_deregister_rnh_pseudowire(pw->vrf_id, pw);
 		zebra_pw_uninstall(pw);
@@ -154,6 +147,7 @@ void zebra_pw_update(struct zebra_pw *pw)
 {
 	if (zebra_pw_check_reachability(pw) < 0) {
 		zebra_pw_uninstall(pw);
+		zebra_pw_install_failure(pw, PW_NOT_FORWARDING);
 		/* wait for NHT and try again later */
 	} else {
 		/*
@@ -173,17 +167,17 @@ static void zebra_pw_install(struct zebra_pw *pw)
 
 	hook_call(pw_install, pw);
 	if (dplane_pw_install(pw) == ZEBRA_DPLANE_REQUEST_FAILURE) {
-		zebra_pw_install_failure(pw);
+		zebra_pw_install_failure(pw, PW_NOT_FORWARDING);
 		return;
 	}
 
-	if (pw->status == PW_STATUS_DOWN)
-		zebra_pw_update_status(pw, PW_STATUS_UP);
+	if (pw->status != PW_FORWARDING)
+		zebra_pw_update_status(pw, PW_FORWARDING);
 }
 
 static void zebra_pw_uninstall(struct zebra_pw *pw)
 {
-	if (pw->status == PW_STATUS_DOWN)
+	if (pw->status != PW_FORWARDING)
 		return;
 
 	if (IS_ZEBRA_DEBUG_PW)
@@ -196,7 +190,7 @@ static void zebra_pw_uninstall(struct zebra_pw *pw)
 	dplane_pw_uninstall(pw);
 
 	if (zebra_pw_enabled(pw))
-		zebra_pw_update_status(pw, PW_STATUS_DOWN);
+		zebra_pw_update_status(pw, PW_NOT_FORWARDING);
 }
 
 /*
@@ -205,30 +199,26 @@ static void zebra_pw_uninstall(struct zebra_pw *pw)
  * to retry the installation later. This function can be called by an external
  * agent that performs the pseudowire installation in an asynchronous way.
  */
-void zebra_pw_install_failure(struct zebra_pw *pw)
+void zebra_pw_install_failure(struct zebra_pw *pw, int pwstatus)
 {
 	if (IS_ZEBRA_DEBUG_PW)
 		zlog_debug(
-			"%u: failed installing pseudowire %s, "
-			"scheduling retry in %u seconds",
+			"%u: failed installing pseudowire %s, scheduling retry in %u seconds",
 			pw->vrf_id, pw->ifname, PW_INSTALL_RETRY_INTERVAL);
 
 	/* schedule to retry later */
-	THREAD_TIMER_OFF(pw->install_retry_timer);
-	thread_add_timer(zrouter.master, zebra_pw_install_retry, pw,
-			 PW_INSTALL_RETRY_INTERVAL, &pw->install_retry_timer);
+	EVENT_OFF(pw->install_retry_timer);
+	event_add_timer(zrouter.master, zebra_pw_install_retry, pw,
+			PW_INSTALL_RETRY_INTERVAL, &pw->install_retry_timer);
 
-	zebra_pw_update_status(pw, PW_STATUS_DOWN);
+	zebra_pw_update_status(pw, pwstatus);
 }
 
-static int zebra_pw_install_retry(struct thread *thread)
+static void zebra_pw_install_retry(struct event *thread)
 {
-	struct zebra_pw *pw = THREAD_ARG(thread);
+	struct zebra_pw *pw = EVENT_ARG(thread);
 
-	pw->install_retry_timer = NULL;
 	zebra_pw_install(pw);
-
-	return 0;
 }
 
 static void zebra_pw_update_status(struct zebra_pw *pw, int status)
@@ -238,14 +228,79 @@ static void zebra_pw_update_status(struct zebra_pw *pw, int status)
 		zsend_pw_update(pw->client, pw);
 }
 
-static int zebra_pw_check_reachability(struct zebra_pw *pw)
+static int zebra_pw_check_reachability_strict(const struct zebra_pw *pw,
+					      struct route_entry *re)
 {
-	struct route_entry *re;
-	struct nexthop *nexthop;
+	const struct nexthop *nexthop;
+	const struct nexthop_group *nhg;
+	bool found_p = false;
+	bool fail_p = false;
 
 	/* TODO: consider GRE/L2TPv3 tunnels in addition to MPLS LSPs */
 
-	/* find route to the remote end of the pseudowire */
+	/* All active nexthops must be labelled; look at
+	 * primary and backup fib lists, in case there's been
+	 * a backup nexthop activation.
+	 */
+	nhg = rib_get_fib_nhg(re);
+	if (nhg && nhg->nexthop) {
+		for (ALL_NEXTHOPS_PTR(nhg, nexthop)) {
+			if (CHECK_FLAG(nexthop->flags, NEXTHOP_FLAG_RECURSIVE))
+				continue;
+
+			if (CHECK_FLAG(nexthop->flags, NEXTHOP_FLAG_ACTIVE)) {
+				if (nexthop->nh_label != NULL)
+					found_p = true;
+				else {
+					fail_p = true;
+					break;
+				}
+			}
+		}
+	}
+
+	if (fail_p)
+		goto done;
+
+	nhg = rib_get_fib_backup_nhg(re);
+	if (nhg && nhg->nexthop) {
+		for (ALL_NEXTHOPS_PTR(nhg, nexthop)) {
+			if (CHECK_FLAG(nexthop->flags, NEXTHOP_FLAG_RECURSIVE))
+				continue;
+
+			if (CHECK_FLAG(nexthop->flags, NEXTHOP_FLAG_ACTIVE)) {
+				if (nexthop->nh_label != NULL)
+					found_p = true;
+				else {
+					fail_p = true;
+					break;
+				}
+			}
+		}
+	}
+
+done:
+
+	if (fail_p || !found_p) {
+		if (IS_ZEBRA_DEBUG_PW)
+			zlog_debug("%s: unlabeled route for %s",
+				   __func__, pw->ifname);
+		return -1;
+	}
+
+	return 0;
+}
+
+static int zebra_pw_check_reachability(const struct zebra_pw *pw)
+{
+	struct route_entry *re;
+	const struct nexthop *nexthop;
+	const struct nexthop_group *nhg;
+	bool found_p = false;
+
+	/* TODO: consider GRE/L2TPv3 tunnels in addition to MPLS LSPs */
+
+	/* Find route to the remote end of the pseudowire */
 	re = rib_match(family2afi(pw->af), SAFI_UNICAST, pw->vrf_id,
 		       &pw->nexthop, NULL);
 	if (!re) {
@@ -255,19 +310,52 @@ static int zebra_pw_check_reachability(struct zebra_pw *pw)
 		return -1;
 	}
 
-	/*
-	 * Need to ensure that there's a label binding for all nexthops.
-	 * Otherwise, ECMP for this route could render the pseudowire unusable.
+	/* Stricter checking for some OSes (OBSD, e.g.) */
+	if (mpls_pw_reach_strict)
+		return zebra_pw_check_reachability_strict(pw, re);
+
+	/* There must be at least one installed labelled nexthop;
+	 * look at primary and backup fib lists, in case there's been
+	 * a backup nexthop activation.
 	 */
-	for (ALL_NEXTHOPS(re->nhe->nhg, nexthop)) {
-		if (!nexthop->nh_label) {
-			if (IS_ZEBRA_DEBUG_PW)
-				zlog_debug("%s: unlabeled route for %s",
-					   __func__, pw->ifname);
-			return -1;
+	nhg = rib_get_fib_nhg(re);
+	if (nhg && nhg->nexthop) {
+		for (ALL_NEXTHOPS_PTR(nhg, nexthop)) {
+			if (CHECK_FLAG(nexthop->flags, NEXTHOP_FLAG_RECURSIVE))
+				continue;
+
+			if (CHECK_FLAG(nexthop->flags, NEXTHOP_FLAG_ACTIVE) &&
+			    nexthop->nh_label != NULL) {
+				found_p = true;
+				break;
+			}
 		}
 	}
 
+	if (found_p)
+		return 0;
+
+	nhg = rib_get_fib_backup_nhg(re);
+	if (nhg && nhg->nexthop) {
+		for (ALL_NEXTHOPS_PTR(nhg, nexthop)) {
+			if (CHECK_FLAG(nexthop->flags, NEXTHOP_FLAG_RECURSIVE))
+				continue;
+
+			if (CHECK_FLAG(nexthop->flags, NEXTHOP_FLAG_ACTIVE) &&
+			    nexthop->nh_label != NULL) {
+				found_p = true;
+				break;
+			}
+		}
+	}
+
+	if (!found_p) {
+		if (IS_ZEBRA_DEBUG_PW)
+			zlog_debug("%s: unlabeled route for %s",
+				   __func__, pw->ifname);
+		return -1;
+	}
+
 	return 0;
 }
 
@@ -319,9 +407,7 @@ DEFUN_NOSH (pseudowire_if,
 	const char *ifname;
 	int idx = 0;
 
-	zvrf = vrf_info_lookup(VRF_DEFAULT);
-	if (!zvrf)
-		return CMD_WARNING;
+	zvrf = zebra_vrf_lookup_by_id(VRF_DEFAULT);
 
 	argv_find(argv, argc, "IFNAME", &idx);
 	ifname = argv[idx]->arg;
@@ -351,9 +437,7 @@ DEFUN (no_pseudowire_if,
 	const char *ifname;
 	int idx = 0;
 
-	zvrf = vrf_info_lookup(VRF_DEFAULT);
-	if (!zvrf)
-		return CMD_WARNING;
+	zvrf = zebra_vrf_lookup_by_id(VRF_DEFAULT);
 
 	argv_find(argv, argc, "IFNAME", &idx);
 	ifname = argv[idx]->arg;
@@ -475,9 +559,7 @@ DEFUN (show_pseudowires,
 	struct zebra_vrf *zvrf;
 	struct zebra_pw *pw;
 
-	zvrf = vrf_info_lookup(VRF_DEFAULT);
-	if (!zvrf)
-		return 0;
+	zvrf = zebra_vrf_lookup_by_id(VRF_DEFAULT);
 
 	vty_out(vty, "%-16s %-24s %-12s %-8s %-10s\n", "Interface", "Neighbor",
 		"Labels", "Protocol", "Status");
@@ -498,7 +580,7 @@ DEFUN (show_pseudowires,
 		vty_out(vty, "%-16s %-24s %-12s %-8s %-10s\n", pw->ifname,
 			(pw->af != AF_UNSPEC) ? buf_nbr : "-", buf_labels,
 			zebra_route_string(pw->protocol),
-			(zebra_pw_enabled(pw) && pw->status == PW_STATUS_UP)
+			(zebra_pw_enabled(pw) && pw->status == PW_FORWARDING)
 				? "UP"
 				: "DOWN");
 	}
@@ -506,6 +588,195 @@ DEFUN (show_pseudowires,
 	return CMD_SUCCESS;
 }
 
+static void vty_show_mpls_pseudowire_detail(struct vty *vty)
+{
+	struct zebra_vrf *zvrf;
+	struct zebra_pw *pw;
+	struct route_entry *re;
+	struct nexthop *nexthop;
+	struct nexthop_group *nhg;
+
+	zvrf = zebra_vrf_lookup_by_id(VRF_DEFAULT);
+
+	RB_FOREACH (pw, zebra_pw_head, &zvrf->pseudowires) {
+		char buf_nbr[INET6_ADDRSTRLEN];
+		char buf_nh[100];
+
+		vty_out(vty, "Interface: %s\n", pw->ifname);
+		inet_ntop(pw->af, &pw->nexthop, buf_nbr, sizeof(buf_nbr));
+		vty_out(vty, "  Neighbor: %s\n",
+			(pw->af != AF_UNSPEC) ? buf_nbr : "-");
+		if (pw->local_label != MPLS_NO_LABEL)
+			vty_out(vty, "  Local Label: %u\n", pw->local_label);
+		else
+			vty_out(vty, "  Local Label: %s\n", "-");
+		if (pw->remote_label != MPLS_NO_LABEL)
+			vty_out(vty, "  Remote Label: %u\n", pw->remote_label);
+		else
+			vty_out(vty, "  Remote Label: %s\n", "-");
+		vty_out(vty, "  Protocol: %s\n",
+			zebra_route_string(pw->protocol));
+		if (pw->protocol == ZEBRA_ROUTE_LDP)
+			vty_out(vty, "  VC-ID: %u\n", pw->data.ldp.pwid);
+		vty_out(vty, "  Status: %s \n",
+			(zebra_pw_enabled(pw) && pw->status == PW_FORWARDING)
+			? "Up"
+			: "Down");
+		re = rib_match(family2afi(pw->af), SAFI_UNICAST, pw->vrf_id,
+			       &pw->nexthop, NULL);
+		if (re == NULL)
+			continue;
+
+		nhg = rib_get_fib_nhg(re);
+		for (ALL_NEXTHOPS_PTR(nhg, nexthop)) {
+			snprintfrr(buf_nh, sizeof(buf_nh), "%pNHv",
+				   nexthop);
+			vty_out(vty, "  Next Hop: %s\n", buf_nh);
+			if (nexthop->nh_label)
+				vty_out(vty, "  Next Hop label: %u\n",
+					nexthop->nh_label->label[0]);
+			else
+				vty_out(vty, "  Next Hop label: %s\n",
+					"-");
+		}
+
+		/* Include any installed backups */
+		nhg = rib_get_fib_backup_nhg(re);
+		if (nhg == NULL)
+			continue;
+
+		for (ALL_NEXTHOPS_PTR(nhg, nexthop)) {
+			snprintfrr(buf_nh, sizeof(buf_nh), "%pNHv",
+				   nexthop);
+			vty_out(vty, "  Next Hop: %s\n", buf_nh);
+			if (nexthop->nh_label)
+				vty_out(vty, "  Next Hop label: %u\n",
+					nexthop->nh_label->label[0]);
+			else
+				vty_out(vty, "  Next Hop label: %s\n",
+					"-");
+		}
+	}
+}
+
+static void vty_show_mpls_pseudowire(struct zebra_pw *pw, json_object *json_pws)
+{
+	struct route_entry *re;
+	struct nexthop *nexthop;
+	struct nexthop_group *nhg;
+	char buf_nbr[INET6_ADDRSTRLEN];
+	char buf_nh[100];
+	json_object *json_pw = NULL;
+	json_object *json_nexthop = NULL;
+	json_object *json_nexthops = NULL;
+
+	json_nexthops = json_object_new_array();
+	json_pw = json_object_new_object();
+
+	json_object_string_add(json_pw, "interface", pw->ifname);
+	if (pw->af == AF_UNSPEC)
+		json_object_string_add(json_pw, "neighbor", "-");
+	else {
+		inet_ntop(pw->af, &pw->nexthop, buf_nbr, sizeof(buf_nbr));
+		json_object_string_add(json_pw, "neighbor", buf_nbr);
+	}
+	if (pw->local_label != MPLS_NO_LABEL)
+		json_object_int_add(json_pw, "localLabel", pw->local_label);
+	else
+		json_object_string_add(json_pw, "localLabel", "-");
+	if (pw->remote_label != MPLS_NO_LABEL)
+		json_object_int_add(json_pw, "remoteLabel", pw->remote_label);
+	else
+		json_object_string_add(json_pw, "remoteLabel", "-");
+	json_object_string_add(json_pw, "protocol",
+			       zebra_route_string(pw->protocol));
+	if (pw->protocol == ZEBRA_ROUTE_LDP)
+		json_object_int_add(json_pw, "vcId", pw->data.ldp.pwid);
+	json_object_string_add(
+		json_pw, "Status",
+		(zebra_pw_enabled(pw) && pw->status == PW_FORWARDING) ? "Up"
+								      : "Down");
+	re = rib_match(family2afi(pw->af), SAFI_UNICAST, pw->vrf_id,
+		       &pw->nexthop, NULL);
+	if (re == NULL)
+		goto done;
+
+	nhg = rib_get_fib_nhg(re);
+	for (ALL_NEXTHOPS_PTR(nhg, nexthop)) {
+		json_nexthop = json_object_new_object();
+		snprintfrr(buf_nh, sizeof(buf_nh), "%pNHv", nexthop);
+		json_object_string_add(json_nexthop, "nexthop", buf_nh);
+		if (nexthop->nh_label)
+			json_object_int_add(
+				json_nexthop, "nhLabel",
+				nexthop->nh_label->label[0]);
+		else
+			json_object_string_add(json_nexthop, "nhLabel",
+					       "-");
+
+		json_object_array_add(json_nexthops, json_nexthop);
+	}
+
+	/* Include installed backup nexthops also */
+	nhg = rib_get_fib_backup_nhg(re);
+	if (nhg == NULL)
+		goto done;
+
+	for (ALL_NEXTHOPS_PTR(nhg, nexthop)) {
+		json_nexthop = json_object_new_object();
+		snprintfrr(buf_nh, sizeof(buf_nh), "%pNHv", nexthop);
+		json_object_string_add(json_nexthop, "nexthop", buf_nh);
+		if (nexthop->nh_label)
+			json_object_int_add(
+				json_nexthop, "nhLabel",
+				nexthop->nh_label->label[0]);
+		else
+			json_object_string_add(json_nexthop, "nhLabel",
+					       "-");
+
+		json_object_array_add(json_nexthops, json_nexthop);
+	}
+
+done:
+
+	json_object_object_add(json_pw, "nexthops", json_nexthops);
+	json_object_array_add(json_pws, json_pw);
+}
+
+static void vty_show_mpls_pseudowire_detail_json(struct vty *vty)
+{
+	json_object *json = NULL;
+	json_object *json_pws = NULL;
+	struct zebra_vrf *zvrf;
+	struct zebra_pw *pw;
+
+	zvrf = zebra_vrf_lookup_by_id(VRF_DEFAULT);
+
+	json = json_object_new_object();
+	json_pws = json_object_new_array();
+	RB_FOREACH (pw, zebra_pw_head, &zvrf->pseudowires) {
+		vty_show_mpls_pseudowire(pw, json_pws);
+	}
+	json_object_object_add(json, "pw", json_pws);
+	vty_json(vty, json);
+}
+
+DEFUN(show_pseudowires_detail, show_pseudowires_detail_cmd,
+      "show mpls pseudowires detail [json]$json",
+      SHOW_STR MPLS_STR
+      "Pseudowires\n"
+      "Detailed output\n" JSON_STR)
+{
+	bool uj = use_json(argc, argv);
+
+	if (uj)
+		vty_show_mpls_pseudowire_detail_json(vty);
+	else
+		vty_show_mpls_pseudowire_detail(vty);
+
+	return CMD_SUCCESS;
+}
+
 /* Pseudowire configuration write function. */
 static int zebra_pw_config(struct vty *vty)
 {
@@ -513,9 +784,7 @@ static int zebra_pw_config(struct vty *vty)
 	struct zebra_vrf *zvrf;
 	struct zebra_pw *pw;
 
-	zvrf = vrf_info_lookup(VRF_DEFAULT);
-	if (!zvrf)
-		return 0;
+	zvrf = zebra_vrf_lookup_by_id(VRF_DEFAULT);
 
 	RB_FOREACH (pw, zebra_static_pw_head, &zvrf->static_pseudowires) {
 		vty_out(vty, "pseudowire %s\n", pw->ifname);
@@ -525,8 +794,7 @@ static int zebra_pw_config(struct vty *vty)
 				pw->local_label, pw->remote_label);
 		else
 			vty_out(vty,
-				" ! Incomplete config, specify the static "
-				"MPLS labels\n");
+				" ! Incomplete config, specify the static MPLS labels\n");
 
 		if (pw->af != AF_UNSPEC) {
 			char buf[INET6_ADDRSTRLEN];
@@ -534,12 +802,12 @@ static int zebra_pw_config(struct vty *vty)
 			vty_out(vty, " neighbor %s\n", buf);
 		} else
 			vty_out(vty,
-				" ! Incomplete config, specify a neighbor "
-				"address\n");
+				" ! Incomplete config, specify a neighbor address\n");
 
 		if (!(pw->flags & F_PSEUDOWIRE_CWORD))
 			vty_out(vty, " control-word exclude\n");
 
+		vty_out(vty, "exit\n");
 		vty_out(vty, "!\n");
 		write = 1;
 	}
@@ -568,4 +836,5 @@ void zebra_pw_vty_init(void)
 	install_element(PW_NODE, &pseudowire_control_word_cmd);
 
 	install_element(VIEW_NODE, &show_pseudowires_cmd);
+	install_element(VIEW_NODE, &show_pseudowires_detail_cmd);
 }
